@@ -33,12 +33,13 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/wait.h>
+#include <time.h>
 #include <unistd.h>
 #include "types.h"
 #include "pmain.h"
 
 static char *_cfConfigDir;
-static char *_cfDataDir;
+static const char *_cfDataDir;
 static char *_cfProgramDir;
 
 static char *argv0;
@@ -51,10 +52,43 @@ static int *plScrMode, crashmode;
 static void (**_plSetTextMode)(unsigned char size);
 static void (**_plSetGraphMode)(unsigned char size);
 
-static struct itimerval i[3];
+static inline void eintr_close(int fd)
+{
+	while (close(fd))
+	{
+		if (errno!=EINTR)
+		{
+			perror(__FILE__ " close()");
+			break;
+		}
+	}
+}
 
+static inline void eintr_waitpid(pid_t pid, int *status, int options)
+{
+	while (waitpid(pid, status, options)<0)
+	{
+		if (errno!=EINTR)
+		{
+			perror(__FILE__ " waitpid()");
+			break;
+		}
+	}
+}
 
-static void stopstuff(void)
+static inline void eintr_dup(int fd)
+{
+	while (dup(fd)<0)
+	{
+		if (errno!=EINTR)
+		{
+			perror(__FILE__ " dup()");
+			break;
+		}
+	}
+}
+
+static void stopstuff(struct itimerval *i)
 {
 	struct itimerval z;
 
@@ -63,28 +97,32 @@ static void stopstuff(void)
 	setitimer(ITIMER_VIRTUAL, &z, &i[1]);
 	setitimer(ITIMER_PROF, &z, &i[2]);
 	crashmode=*plScrMode;
-	if (*_plSetTextMode)
-		(*_plSetTextMode)(255);
+	if (_plSetTextMode != NULL)
+		if (*_plSetTextMode != NULL)
+			(*_plSetTextMode)((unsigned char)255);
 }
-static void restartstuff(void)
+static void restartstuff(struct itimerval *i)
 {
 	setitimer(ITIMER_REAL, &i[0], 0);
 	setitimer(ITIMER_VIRTUAL, &i[1], 0);
 	setitimer(ITIMER_PROF, &i[2], 0);
 	if (crashmode==101)	
 	{
-		if (*_plSetGraphMode)
-			(*_plSetGraphMode)(1);
+		if (_plSetGraphMode != NULL)
+			if (*_plSetGraphMode != NULL)
+				(*_plSetGraphMode)((unsigned char)1);
 	} else if (crashmode==100)
 	{
-		if (*_plSetGraphMode)
-			(*_plSetGraphMode)(0);
+		if (_plSetGraphMode != NULL)
+			if (*_plSetGraphMode != NULL)
+				(*_plSetGraphMode)((unsigned char)0);
 	} else
-		if (*_plSetTextMode)
-			(*_plSetTextMode)(crashmode);
+		if (_plSetTextMode != NULL)
+			if (*_plSetTextMode != NULL)
+				(*_plSetTextMode)((unsigned char)crashmode);
 }
 
-static char *locate_ocp_ini_try(char *base)
+static const char *locate_ocp_ini_try(const char *base)
 {
 	static char buffer[256];
 	struct stat st;
@@ -94,16 +132,17 @@ static char *locate_ocp_ini_try(char *base)
 	return NULL;
 }
 
-static char *locate_ocp_ini(void)
+static const char *locate_ocp_ini(void)
 {
-	char *retval;
-	if ((retval=getenv("CPDIR")))
+	const char *retval;
+	const char *temp;
+	if ((temp=getenv("CPDIR")))
 	{
 /*		if (!AllowSymlinked)
 		{
 			fprintf(stderr, "Using $CPDIR when running setuid is not allowed\n");
-		} else*/ if ((retval=locate_ocp_ini_try(retval)))
-			return retval;
+		} else*/if ((retval=locate_ocp_ini_try(temp)))
+				return retval;
 	}
 	if ((retval=locate_ocp_ini_try(PREFIX "/share/ocp" DIR_SUFFIX "/etc/")))
 		return retval;
@@ -124,27 +163,31 @@ static char *locate_ocp_ini(void)
 	return NULL;
 }
 
-static char *locate_ocp_pak_try(char *base)
+static const char *locate_ocp_hlp_try(const char *base)
 {
-	char buffer[256];
+	static char buffer[256];
 	struct stat st;
-	snprintf(buffer, sizeof(buffer), "%s%s", base, "ocp.pak");
+	snprintf(buffer, sizeof(buffer), "%s%s", base, "ocp.hlp");
 	if (!stat(buffer, &st))
-		return base;
+	{
+		snprintf(buffer, sizeof(buffer), "%s", base);
+		return buffer; /* since input data might come from getenv, it is not safe to keep them */
+	}
 	return NULL;
 }
 
-static char *locate_ocp_pak(void)
+static const char *locate_ocp_hlp(void)
 {
-	char *retval;
-	if ((retval=getenv("OCPDIR")))
-		if ((retval=locate_ocp_pak_try(retval)))
+	const char *retval;
+	const char *temp;
+	if ((temp=getenv("OCPDIR")))
+		if ((retval=locate_ocp_hlp_try(temp)))
 			return retval;
-	if ((retval=locate_ocp_pak_try(PREFIX "/share/ocp" DIR_SUFFIX "/")))
+	if ((retval=locate_ocp_hlp_try(PREFIX "/share/ocp" DIR_SUFFIX "/")))
 		return retval;
-	if ((retval=locate_ocp_pak_try(PREFIX "/share/ocp" DIR_SUFFIX "/data/")))
+	if ((retval=locate_ocp_hlp_try(PREFIX "/share/ocp" DIR_SUFFIX "/data/")))
 		return retval;
-	if ((retval=locate_ocp_pak_try(LIBDIR)))
+	if ((retval=locate_ocp_hlp_try(LIBDIR)))
 		return retval;
 	return NULL;
 }
@@ -152,36 +195,36 @@ static char *locate_ocp_pak(void)
 static void reset(void)
 {
 	pid_t pid;
-	char *argv[5];
+	const char *argv[5];
 
 	if (!(pid=fork()))
 	{
 		if (!isatty(2))
 		{
-			close(2);
-			dup(1);
+			eintr_close(2);
+			eintr_dup(1);
 		}
 		argv[1]=NULL;
 		argv[0]="reset";
-		execvp("reset", argv);
+		execvp("reset", (char **)argv);
 		fprintf(stderr, "Failed to exec reset\n");
-		_exit(1);
+		exit(EXIT_FAILURE);
 	} else if (pid>0) /* not an error */
 	{
 		int status;
-		waitpid(pid, &status, 0);
+		eintr_waitpid(pid, &status, 0);
 	}
 
 	if (!(pid=fork()))
 	{
 		argv[1]=NULL;
 		argv[0]="clear";
-		execvp("clear", argv);
-		_exit(1);
+		(void)execvp("clear", (char * const *)argv);
+		exit(EXIT_FAILURE);
 	} else if (pid>0) /* not an error */
 	{
 		int status;
-		waitpid(pid, &status, 0);
+		eintr_waitpid(pid, &status, 0);
 	}
 }
 
@@ -221,13 +264,14 @@ void sigsegv(int signal, struct sigcontext r)
 void sigsegv(int signal)
 #endif
 {
+	struct itimerval i[3];
 #ifdef KICKSTART_GDB
 	char *argv[5];
 	char buffer[32];
 	pid_t pid;
 #endif
 
-	stopstuff();
+	stopstuff(i);
 
 	/* loose setuid stuff before we launch any other external tools */
 	if (getegid()!=getgid())
@@ -236,8 +280,9 @@ void sigsegv(int signal)
 	if (geteuid()!=getuid())
 		seteuid(getuid());
 
-	if (*_plSetTextMode) /* don't reset if we havn't used the screen yet */
-		reset();
+	if(_plSetTextMode)
+		if (*_plSetTextMode) /* don't reset if we havn't used the screen yet */
+			reset();
 #if defined(__linux)
 	dumpcontext(signal, r);
 #else
@@ -252,7 +297,7 @@ void sigsegv(int signal)
 		argv[0]="gdb";
 		argv[1]=argv0;
 		argv[2]="-p";
-		sprintf(buffer, "%d", getpid());
+		snprintf(buffer, sizeof(buffer), "%d", getpid());
 		argv[3]=buffer;
 		argv[4]=NULL;
 	
@@ -260,28 +305,29 @@ void sigsegv(int signal)
 		{
 			if (!isatty(2))
 			{
-				close(2);
-				dup(1);
+				eintr_close(2);
+				eintr_dup(1);
 			}
 			setsid();
 
 			if (dir)
-				chdir(dir);
+				if (chdir(dir))
+					perror(__FILE__ " chdir()");
 			execvp("gdb", argv);
 
 			perror("execvp(gdb)\n");
-			_exit(1);
+			exit(EXIT_FAILURE);
 		} else if (pid>=0) /* not an error */
 		{
 			int status;
-			waitpid(pid, &status, 0);
+			eintr_waitpid(pid, &status, 0);
 		}
 	}
 #endif
 	if (signal!=SIGINT)
-		_exit(signal);
+		exit(signal);
 	else
-		restartstuff();
+		restartstuff(i);
 }
 
 static int cp(const char *src, const char *dst)
@@ -289,24 +335,23 @@ static int cp(const char *src, const char *dst)
 	int _errno;
 	int srcfd, dstfd;
 	char buffer[65536];
-	int retval;
+	ssize_t retval;
 	if ((srcfd=open(src, O_RDONLY))<0)
 		return -1;
 	if ((dstfd=open(dst, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH))<0)
 	{
 		_errno=errno;
-		close(srcfd);
+		eintr_close(srcfd);
 		errno=_errno;
 		return -1;
 	}
-	chown(dst, getuid(), getgid());
-	while ((retval=read(srcfd, buffer, 65536))>0)
+	while ((retval=read(srcfd, buffer, (size_t)65536))>0)
 	{
-		if (write(dstfd, buffer, retval)!=retval)
+		if (write(dstfd, buffer, (size_t)retval)!=retval)
 		{
 			_errno=errno;
-			close(srcfd);
-			close(dstfd);
+			eintr_close(srcfd);
+			eintr_close(dstfd);
 			errno=_errno;
 			return -1;
 		}
@@ -314,28 +359,28 @@ static int cp(const char *src, const char *dst)
 	if (retval<0)
 	{
 		_errno=errno;
-		close(srcfd);
-		close(dstfd);
+		eintr_close(srcfd);
+		eintr_close(dstfd);
 		errno=_errno;
 		return -1;
 	}
-	close(srcfd);
-	close(dstfd);
+	eintr_close(srcfd);
+	eintr_close(dstfd);
 	return 0;
 }
 
 int validate_home(void)
 {
-	char *home=getenv("HOME");
+	const char *home=getenv("HOME");
 	char *temp;
-	char *temp2;
+	const char *temp2;
 	struct stat st;
 	
 	if (!home)
 		home="/root";
-	else if(!strlen(home))
+	else if (!strlen(home))
 		home="/root";
-	else if (*home!='/')
+	else if ((*home)!='/')
 	{
 		fprintf(stderr, "$HOME does not start with a /\n");
 		return -1;
@@ -357,7 +402,6 @@ int validate_home(void)
 				perror("mkdir()");
 				return -1;
 			}
-			chown(temp, getuid(), getgid());
 		} else {
 			perror("stat($HOME)");
 			return -1;
@@ -429,7 +473,7 @@ static void *locate_libocp_try(const char *src, int verbose)
 			if (S_ISLNK(st.st_mode))
 			{
 				fprintf(stderr, "Symlinked libocp" LIB_SUFFIX " is not allowed when running setuid\n");
-				exit(-1);
+				exit(EXIT_FAILURE);
 			}
 		}
 	}
@@ -512,8 +556,6 @@ static int runocp(void *handle, int argc, char *argv[])
 	strcpy(cfConfigDir, _cfConfigDir);
 	strcpy(cfDataDir, _cfDataDir);
 	strcpy(cfProgramDir, _cfProgramDir);
-	free(_cfConfigDir);
-	free(_cfProgramDir);
 
 	return bootup->main(argc, argv);
 }
@@ -559,9 +601,9 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	if (!(_cfDataDir=locate_ocp_pak()))
+	if (!(_cfDataDir=locate_ocp_hlp()))
 	{
-		fprintf(stderr, "Failed to locate ocp.pak..\n");
+		fprintf(stderr, "Failed to locate ocp.hlp..\n");
 		return -1;
 	}
 
@@ -571,6 +613,11 @@ int main(int argc, char *argv[])
 
 	dir=0;
 
+	if (_cfConfigDir)
+		free(_cfConfigDir);
+	if (_cfProgramDir)
+		free(_cfProgramDir);
+
 #ifdef HAVE_DUMA
 	DUMA_delFrame();
 #endif
