@@ -41,6 +41,10 @@ static int handlecounter;
 struct dll_handle loadlist[MAXDLLLIST];
 int loadlist_n;
 
+#ifdef SUPPORT_STATIC_PLUGINS
+DLLEXTINFO_PREFIX struct linkinfostruct staticdlls = {"static", "Compiled in plugins (c) 2009 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+#endif
+
 static char reglist[1024];
 static void parseinfo (char *pi, const char *key)
 {
@@ -221,7 +225,7 @@ int lnkLinkDir(const char *dir)
 	}
 	while ((de=readdir(d)))
 	{
-		int len=strlen(de->d_name);
+		size_t len=strlen(de->d_name);
 		if (len<strlen(LIB_SUFFIX))
 			continue;
 		if (strcmp(de->d_name+len-strlen(LIB_SUFFIX), LIB_SUFFIX))
@@ -229,10 +233,12 @@ int lnkLinkDir(const char *dir)
 		if (files>=1024)
 		{
 			fprintf(stderr, "lnkLinkDir: Too many libraries in directory %s\n", dir);
+			closedir(d);
 			return -1;
 		}
 		filenames[files++]=strdup(de->d_name);
 	}
+	closedir(d);
 	if (!files)
 		return 0;
 #ifdef HAVE_QSORT
@@ -242,7 +248,7 @@ int lnkLinkDir(const char *dir)
 #endif
 	for (n=0;n<files;n++)
 	{
-		if (snprintf(buffer, sizeof(buffer), "%s%s", dir, filenames[n])>=sizeof(buffer))
+		if (snprintf(buffer, sizeof(buffer), "%s%s", dir, filenames[n])>=(signed)(sizeof(buffer)-1))
 		{
 			fprintf(stderr, "lnkLinkDir: path too long %s%s\n", dir, filenames[n]);
 			for (;n<files;n++)
@@ -289,7 +295,8 @@ void lnkFree(const int id)
 		for (i=loadlist_n-1;i>=0;i--)
 		{
 #ifndef NO_DLCLOSE
-			dlclose(loadlist[i].handle);
+			if (loadlist[i].handle) /* this happens for static plugins */
+				dlclose(loadlist[i].handle);
 #endif
 		}
 		loadlist_n=0;
@@ -298,7 +305,8 @@ void lnkFree(const int id)
 			if (loadlist[i].id==id)
 			{
 #ifndef NO_DLCLOSE
-				dlclose(loadlist[i].handle);
+				if (loadlist[i].handle) /* this happens for static plugins */
+					dlclose(loadlist[i].handle);
 #endif
 				memmove(&loadlist[i], &loadlist[i+1], (MAXDLLLIST-i-1)*sizeof(loadlist[0]));
 				loadlist_n--;
@@ -307,11 +315,46 @@ void lnkFree(const int id)
 	}
 }
 
+#ifdef SUPPORT_STATIC_PLUGINS
+static void lnkLoadStatics(void)
+{
+	struct linkinfostruct *iterator = &staticdlls;
+	while (iterator->name)
+	{
+		#ifdef LD_DEBUG
+		fprintf(stderr, "[lnk] Adding static module: \"%s\"\n", iterator->name);
+		#endif
+
+		if (loadlist_n>=MAXDLLLIST)
+		{
+			fprintf(stderr, "[lnk] Too many open shared objects\n");
+			return; /* this is really not reachable, but nice to test... */
+		}
+
+		loadlist[loadlist_n].handle=NULL;
+
+		loadlist[loadlist_n].id=++handlecounter;
+
+		loadlist[loadlist_n].info=iterator;
+		
+		loadlist[loadlist_n].info->size=0;
+
+		loadlist_n++;
+
+		iterator++;
+	}
+	return;
+}
+#endif
+
 void lnkInit(void)
 {
 	loadlist_n=0;
 	handlecounter=0;
 	memset(loadlist, 0, sizeof(loadlist));
+#ifdef SUPPORT_STATIC_PLUGINS
+	lnkLoadStatics();
+#endif
 }
 
 void *lnkGetSymbol(const int id, const char *name)
