@@ -40,7 +40,6 @@
 #include "pmain.h"
 #include "stuff/err.h"
 #include "stuff/poutput.h"
-#include "binfile/binfpak.h"
 #include "plinkman.h"
 #include "psetting.h"
 
@@ -65,7 +64,7 @@ static void plCloseAll(void)
 
 static int cmdhlp(void)
 {
-	if (cfGetProfileString("commandline", "h", 0) || cfGetProfileString("commandline", "?", 0))
+	if (cfGetProfileString("commandline", "h", 0) || cfGetProfileString("commandline", "?", 0) || cfGetProfileString("commandline--", "help", 0))
 	{
 		printf("\nopencp command line help\n");
 		printf("Usage: ocp [<options>]* [@<playlist>]* [<modulename>]* \n");
@@ -142,15 +141,101 @@ static int init_modules(int argc, char *argv[])
 
 	cfConfigSec="defaultconfig";
 
-	if (cfGetProfileInt("version", "epoch", 0, 10)!=20070712)
 	{
+		int epoch = cfGetProfileInt("version", "epoch", 0, 10);
+		if (epoch <= 20060815)
+		{
+			char temp[1024];
 
+			fprintf(stderr, "ocp.ini update (0.1.10) adds devpALSA to [sound] playerdevices=....\n");
+			snprintf(temp, sizeof(temp), "devpALSA %s", cfGetProfileString("sound", "playerdevices", ""));
+			cfSetProfileString("sound", "playerdevices", temp);
+
+			fprintf(stderr, "ocp.ini update (0.1.10) adds [sound] digitalcd=on\n");
+			cfSetProfileBool("sound", "digitalcd", 1);
+
+			fprintf(stderr, "ocp.ini update (0.1.10) adds AY to [fileselector] modextensions=....\n");
+			snprintf(temp, sizeof(temp), "%s AY", cfGetProfileString("fileselector", "modextensions", ""));
+			cfSetProfileString("fileselector", "modextensions", temp);
+
+			fprintf(stderr, "ocp.ini update (0.1.10) adds [devpALSA]\n");
+			cfSetProfileString("devpALSA", "link", "devpalsa");
+			cfSetProfileInt("devpALSA", "keep", 1, 10);
+
+			fprintf(stderr, "ocp.ini update (0.1.10) adds [filetype 37]\n");
+			cfSetProfileInt("filetype 37", "color", 6, 10);
+			cfSetProfileString("filetype 37", "name", "AY");
+			cfSetProfileString("filetype 37", "interface", "plOpenCP");
+			cfSetProfileString("filetype 37", "pllink", "playay");
+			cfSetProfileString("filetype 37", "player", "ayPlayer");
+		}
+
+		if (epoch <= 20070712)
+		{
+			char temp[1024];
+			fprintf(stderr, "ocp.ini update (0.1.13/0.1.14) adds devpCA to [sound] playerdevices=....\n");
+			snprintf(temp, sizeof(temp), "devpCA %s", cfGetProfileString("sound", "playerdevices", ""));
+			cfSetProfileString("sound", "playerdevices", temp);
+
+			fprintf(stderr, "ocp.ini update (0.1.13/0.1.14) adds [devpCA]\n");
+			cfSetProfileString("devpCA", "link", "devpcoreaudio");
+
+			fprintf(stderr, "ocp.ini update (0.1.14) changed [devpOSS] revstereo to off\nn");
+			cfSetProfileBool("devpOSS", "revstereo", 0);
+
+			fprintf(stderr, "ocp.ini update (0.1.14) changed [devsOSS] revstereo to off\nn");
+			cfSetProfileBool("devsOSS", "revstereo", 0);	
+
+			fprintf(stderr, "ocp.ini update (0.1.14) adds [filetype 38]\n");
+			cfSetProfileInt("filetype 38", "color", 6, 10);
+			cfSetProfileString("filetype 38", "name", "FLA");
+			cfSetProfileString("filetype 38", "interface", "plOpenCP");
+			cfSetProfileString("filetype 38", "pllink", "playflac");
+			cfSetProfileString("filetype 38", "player", "flacPlayer");
+		}
+
+		if (epoch <= 20081117)
+		{
+			fprintf(stderr, "ocp.ini update (0.1.17) removes [general] autoload=....\n");
+			cfRemoveEntry("general", "autoload");
+
+			fprintf(stderr, "ocp.ini update (0.1.16/0.1.17) removes [general] link=....\n");
+			cfRemoveEntry("general", "link");
+			fprintf(stderr, "ocp.ini update (0.1.16/0.1.17) removes [defaultconfig] link=....\n");
+			cfRemoveEntry("defaultconfig", "link");
+
+			fprintf(stderr, "ocp.ini update (0.1.16) renames [x11] framebuffer to autodetect\n");
+			cfSetProfileBool("x11", "autodetect", cfGetProfileBool("x11", "framebuffer", 1, 1));
+			cfRemoveEntry("x11", "framebuffer");
+
+			fprintf(stderr, "ocp.ini update (0.1.16) adds [x11] font=1\n");
+			cfSetProfileInt("x11", "font", cfGetProfileInt("x11", "font", 1, 10), 10);
+	
+			fprintf(stderr, "ocp.ini update (0.1.16) adds [x11] xvidmode=on\n");
+			cfSetProfileBool("x11", "xvidmode", cfGetProfileBool("x11", "xvidmode", 1, 1));
+		}
+
+		if (epoch < 20090208)
+		{
+			cfSetProfileInt("version", "epoch", 20090208, 10);
+			cfStoreConfig();
+			if (isatty(2))
+			{
+				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\033[0m\nrm -f ~/.ocp/ocp.ini\n\n");
+			} else {
+				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\nrm -f ~/.ocp/ocp.ini\n\n");
+			}
+			sleep(5);
+		}
+
+	}
+	if (cfGetProfileInt("version", "epoch", 0, 10)!=20090208)
+	{
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\n\033[1m\033[31mWARNING, ocp.ini has changed, please update or remove by doing this:\033[0m\nrm -f ~/.ocp/ocp.ini\n");
-
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20090208\033[0m\n\n");
 		} else {
-			fprintf(stderr,"\n\nWARNING, ocp.ini has changed, please update or remove by doing this:\n\nrm -f ~/.ocp/ocp.ini\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20090208\033[0m\n\n");
 		}
 		sleep(5);
 	}
@@ -161,10 +246,14 @@ static int init_modules(int argc, char *argv[])
 	lnkLink(cfGetProfileString2(cfConfigSec, "defaultconfig", "prelink", ""));
 	lnkLink(cfGetProfileString("general", "prelink", ""));
 
-	if (lnkLinkDir(cfGetProfileString("general", "autoload", ""))<0)
 	{
-		fprintf(stderr, "could not autoload directory: %s\n", cfGetProfileString("general", "autoload", "unset"));
-		return -1;
+		char buffer[PATH_MAX];
+		snprintf(buffer, sizeof(buffer), "%sautoload/", cfProgramDir);
+		if (lnkLinkDir(buffer)<0)
+		{
+			fprintf(stderr, "could not autoload directory: %s\n", buffer);
+			return -1;
+		}
 	}
 
 	if (lnkLink(cfGetProfileString("general", "link", ""))<0)
@@ -179,9 +268,6 @@ static int init_modules(int argc, char *argv[])
 		return -1;
 	}
 
-	if (pakfInit())
-		return -1;
-
 	fprintf(stderr, "running initializers...\n");
 
 	for (i=0;i<loadlist_n;i++)
@@ -215,10 +301,18 @@ static int init_modules(int argc, char *argv[])
 void done_modules(void)
 {
 	plCloseAll();
-	pakfClose();
 	lnkFree(0);
 }
 
+#ifdef GCC_411_RUNTIMECHECK
+int failcheck(signed int source, signed int filter)
+{
+	if ((source>128)&&(filter>0))
+		return 1;
+	return 0;
+}
+#endif
+
 static int _bootup(int argc, char *argv[])
 {
 	int result;
@@ -231,6 +325,24 @@ static int _bootup(int argc, char *argv[])
 		fprintf(stderr, "Ported to Unix by Stian Skjelstad\n");
 	}
 
+#ifdef GCC_411_RUNTIMECHECK
+	fprintf(stderr, "Checking for gcc known 4.1.1 fault - ");
+	{
+		int j;
+		for (j=0;j<256;j++)
+		{
+			signed char j2=(signed char)j;
+			signed int j3=j2;
+			if (failcheck(j, j3))
+			{
+				fprintf(stderr, "failed\nTry to remove any -O flag or to add -fwrapv to CFLAGS and CXXFLAGS and recompile\n");
+				return 0;
+			}
+		}
+	}
+	fprintf(stderr, "pass\n");
+#endif
+
 	if (cfGetConfig(argc, argv))
 		return -1;
 
