@@ -1,24 +1,22 @@
-	/* OpenCP Module Player
-	 * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
-	 *
-	 * CP.INI file and environment reading functions
-	 *
-	 * revision history: (please note changes here)
-	 *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
-	 *    -first release
-	 *  -fd981014   Felix Domke <tmbinc@gmx.net>
-	 *    -Bugfix at cfReadINIFile (first if-block, skips the filename in
-	 *     the commandline, without these, funny errors occured.)
-	 *  -fd981106   Felix Domke    <tmbinc@gmx.net>
-	 *    -edited for new binfile
-	 *  -ss040613   Stian Skjelstad <stian@nixia.no>
-	 *    - rewritten for unix
-	 *    - use argc and argv semantics
-	 *  -ss040825   Stian Skjelstad <stian@nixia.no>
-	 *    - added back the commandline stuff
-	 */
-
-	/*#define NO_CPDLL_IMPORT*/
+/* OpenCP Module Player
+ * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ *
+ * OCP.INI file and environment reading functions
+ *
+ * revision history: (please note changes here)
+ *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ *    -first release
+ *  -fd981014   Felix Domke <tmbinc@gmx.net>
+ *    -Bugfix at cfReadINIFile (first if-block, skips the filename in
+ *     the commandline, without these, funny errors occured.)
+ *  -fd981106   Felix Domke    <tmbinc@gmx.net>
+ *    -edited for new binfile
+ *  -ss040613   Stian Skjelstad <stian@nixia.no>
+ *    - rewritten for unix
+ *    - use argc and argv semantics
+ *  -ss040825   Stian Skjelstad <stian@nixia.no>
+ *    - added back the commandline stuff
+ */
 
 #include "config.h"
 #include <stdio.h>
@@ -174,7 +172,7 @@ static int cfReadINIFile(int argc, char *argv[])
 		}
 
 		commentbuf[0]=0;
-	    	switch (readiniline(keybuf, strbuf, commentbuf, linebuffer))
+		switch (readiniline(keybuf, strbuf, commentbuf, linebuffer))
 		{
 			case 0:
 				if (commentbuf[0]&&(cfINIApps_index>=0))
@@ -234,6 +232,8 @@ static int cfReadINIFile(int argc, char *argv[])
 			{
 				if ((argv[c][1]=='-')&&(!argv[c][2])) /* Unix legacy */
 					break;
+				if (argv[c][1]=='-')
+					continue;
 
 				cfINInApps++;
 				cfINIApps=realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
@@ -293,7 +293,31 @@ static int cfReadINIFile(int argc, char *argv[])
 				cfINIApps[cfINInApps-1].keys[index].linenum=-1;
 				cfINIApps[cfINInApps-1].keys[index].comment=NULL;
 			}
-		
+	
+		cfINInApps++;
+		cfINIApps=realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
+		cfINIApps[cfINInApps-1].app=strdup("CommandLine--");
+		cfINIApps[cfINInApps-1].keys=NULL;
+		cfINIApps[cfINInApps-1].nkeys=0;
+		cfINIApps[cfINInApps-1].linenum=-1;
+		cfINIApps[cfINInApps-1].comment=NULL;
+	
+		for (c=1;c<argc;c++)
+			if ((argv[c][0]=='-')&&(argv[c][1]=='-'))
+			{
+				int index=cfINIApps[cfINInApps-1].nkeys;
+
+				if (!argv[c][2]) /* Unix legacy */
+					break;
+				
+				cfINIApps[cfINInApps-1].nkeys++;
+				cfINIApps[cfINInApps-1].keys=realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
+				cfINIApps[cfINInApps-1].keys[index].key=strdup(argv[c]+2);
+				cfINIApps[cfINInApps-1].keys[index].str=strdup("1");
+				cfINIApps[cfINInApps-1].keys[index].linenum=-1;
+				cfINIApps[cfINInApps-1].keys[index].comment=NULL;
+			}
+	
 		cfINInApps++;
 		cfINIApps=realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
 		cfINIApps[cfINInApps-1].app=strdup("CommandLine_Files");
@@ -304,6 +328,8 @@ static int cfReadINIFile(int argc, char *argv[])
 
 		{
 			int countin=0;
+			int files = 0;
+			int playlists = 0;
 
 			for (c=1;c<argc;c++)
 			{
@@ -319,16 +345,18 @@ static int cfReadINIFile(int argc, char *argv[])
 						continue;
 					}
 				}
-	
-				if (argv[c][0]!='@')
+
 				{
 					int index=cfINIApps[cfINInApps-1].nkeys;
 					char buffer[32];
-					sprintf(buffer, "file%d", cfINIApps[cfINInApps-1].nkeys);
+					if (argv[c][0]!='@')
+						sprintf(buffer, "file%d", files++);
+					else
+						sprintf(buffer, "playlist%d", playlists++);
 					cfINIApps[cfINInApps-1].nkeys++;
 					cfINIApps[cfINInApps-1].keys=realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
 					cfINIApps[cfINInApps-1].keys[index].key=strdup(buffer);
-					cfINIApps[cfINInApps-1].keys[index].str=strdup(argv[c]);
+					cfINIApps[cfINInApps-1].keys[index].str=strdup(argv[c]+(argv[c][0]=='@'?1:0));
 					cfINIApps[cfINInApps-1].keys[index].linenum=-1;
 					cfINIApps[cfINInApps-1].keys[index].comment=NULL;
 				}
@@ -467,6 +495,36 @@ void cfSetProfileBool(const char *app, const char *key, const int str)
 	cfSetProfileString(app, key, (str?"on":"off"));
 }
 
+void cfRemoveEntry(const char *app, const char *key)
+{
+	int i, j;
+	for (i=0; i<cfINInApps; i++)
+		if (!strcasecmp(cfINIApps[i].app, app))
+		{
+			for (j=0; j<cfINIApps[i].nkeys; j++)
+				if (cfINIApps[i].keys[j].key)
+					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
+					{
+						if (cfINIApps[i].keys[j].str)
+							free(cfINIApps[i].keys[j].str);
+						if (cfINIApps[i].keys[j].key)
+							free(cfINIApps[i].keys[j].key);
+						if (cfINIApps[i].keys[j].comment)
+							free(cfINIApps[i].keys[j].comment);
+						memmove(cfINIApps[i].keys + j, cfINIApps[i].keys + j + 1, cfINIApps[i].nkeys * sizeof(cfINIApps[i].keys[0]));
+						cfINIApps[i].nkeys--;
+						if (cfINIApps[i].nkeys)
+						{
+							void *temp = realloc(cfINIApps[i].keys, cfINIApps[i].nkeys*sizeof(cfINIApps[i].keys[0]));
+							if (temp)
+								cfINIApps[i].keys = temp;
+							else
+								fprintf(stderr, __FILE__ ": warning, realloc() failed #1\n");
+						}
+					}
+		}
+}
+
 int cfCountSpaceList(const char *str, int maxlen)
 {
 	int i=0;
@@ -557,7 +615,7 @@ int cfGetConfig(int argc, char *argv[])
 		}
 		strcat(cfTempDir, "/");
 	}
-#if 0
+#ifdef PSETTING_DEBUG
 	{
 		int i, j;
 		char buffer[2+KEYBUF_LEN+1+STRBUF_LEN+COMMENTBUF_LEN+32+1+1];
