@@ -375,6 +375,8 @@ static int plDotsKey(uint16_t key)
 			cpiKeyHelp('N', "Change note dots type");
 			cpiKeyHelp(KEY_PPAGE, "Change note dots range down");
 			cpiKeyHelp(KEY_NPAGE, "Change note dots range up");
+			cpiKeyHelp(KEY_CTRL_PGUP, "Adjust scale up");
+			cpiKeyHelp(KEY_CTRL_PGDN, "Adjust scale down");
 			cpiKeyHelp(KEY_HOME, "Reset note dots range");
 			return 0;
 		case KEY_PPAGE:
@@ -387,16 +389,18 @@ static int plDotsKey(uint16_t key)
 			if (plDotsMiddle>96*256)
 				plDotsMiddle=96*256;
 			break;			
-/* TODO-keys	case 0x8400: //ctrl-pgup
+		case KEY_CTRL_PGUP:
+		/* case 0x7600: //ctrl-pgdn */
+			plDotsScale=(plDotsScale+1)*32/31;
+				if (plDotsScale>256)
+			plDotsScale=256;
+			break;
+		case KEY_CTRL_PGDN:
+		/* case 0x8400: //ctrl-pgup */
 			plDotsScale=plDotsScale*31/32;
 			if (plDotsScale<16)
 				plDotsScale=16;
 			break;
-		case 0x7600: //ctrl-pgdn
-			plDotsScale=(plDotsScale+1)*32/31;
-				if (plDotsScale>256)
-			plDotsScale=256;
-			break;*/
 		/*case 0x4700: //home*/
 		case KEY_HOME:
 			plDotsMiddle=72*256;
@@ -448,7 +452,7 @@ static int plDotsEvent(int ignore)
 	return 1;
 }
 
-static struct cpimoderegstruct plDotsMode = {"dots", dotSetMode, dotDraw, dotIProcessKey, plDotsKey, plDotsEvent};
+static struct cpimoderegstruct plDotsMode = {"dots", dotSetMode, dotDraw, dotIProcessKey, plDotsKey, plDotsEvent CPIMODEREGSTRUCT_TAIL};
 
 void plUseDots(int (*get)(struct notedotsdata *, int))
 {
