@@ -74,9 +74,9 @@ void (*plIdle)(void);
 
 char plMuteCh[MAXLCHAN];
 
-void (*plGetMasterSample)(int16_t *, int len, uint32_t rate, int mode);
-int (*plGetLChanSample)(int ch, int16_t *, int len, uint32_t rate, int opt);
-int (*plGetPChanSample)(int ch, int16_t *, int len, uint32_t rate, int opt);
+void (*plGetMasterSample)(int16_t *, unsigned int len, uint32_t rate, int mode);
+int (*plGetLChanSample)(unsigned int ch, int16_t *, unsigned int len, uint32_t rate, int opt);
+int (*plGetPChanSample)(unsigned int ch, int16_t *, unsigned int len, uint32_t rate, int opt);
 
 unsigned short plNLChan;
 unsigned short plNPChan;
@@ -100,6 +100,8 @@ unsigned long plEscTick;
 static struct cpimoderegstruct *curmode;
 static char curmodehandle[9];
 
+static struct interfacestruct plOpenCP;
+
 void cpiSetGraphMode(int big)
 {
 	plSetGraphMode(big);
@@ -116,7 +118,7 @@ void cpiSetTextMode(int size)
 void cpiDrawGStrings()
 {
 	char *verstr="  opencp v" VERSION;
-	char *author="(c) 1994-2007 Stian Skjelstad ";
+	char *author="(c) 1994-2008 Stian Skjelstad ";
 	char tstr[CONSOLE_MAX_X+1];
 
 #ifdef DEBUG
@@ -397,11 +399,14 @@ static int plmpInit(void)
 
 	cpiKeyHelpReset = cpiResetScreen;
 
+	plRegisterInterface (&plOpenCP);
+
 	return errOk;
 }
 
 static void plmpClose(void)
 {
+	plUnregisterInterface (&plOpenCP);
 	mdbUnregisterReadInfo(&cpiReadInfoReg);
 
 	while (cpiDefModes)
@@ -679,7 +684,7 @@ static interfaceReturnEnum plmpDrawScreen(void)
 	for (mod=cpiModes; mod; mod=mod->next)
 		mod->Event(cpievKeepalive);
 
-	if (plEscTick&&(dos_clock()>(plEscTick+2*DOS_CLK_TCK)))
+	if (plEscTick&&(dos_clock()>(time_t)(plEscTick+2*DOS_CLK_TCK)))
 		plEscTick=0;
 
 	while (ekbhit())
@@ -716,12 +721,17 @@ static interfaceReturnEnum plmpDrawScreen(void)
 				break;
 			case _KEY_ENTER:
 				return interfaceReturnNextManuel;
-			case 'f': case 'F':/* case 0x5200: TODO-keys*/
+			case 'f': case 'F':
+			case KEY_INSERT:
+			/* case 0x5200: //insert */
 				return interfaceReturnCallFs;
-			case 'd': case 'D': case KEY_CTRL_D: /* case 0x6f00: TODO-keys*/
+			case 'd': case 'D': case KEY_CTRL_D:
+			/* case 0x6f00: // alt-f8 TODO-keys*/
 				return interfaceReturnDosShell;
 			case KEY_CTRL_J:
-				return interfaceReturnNextAuto;
+				return interfaceReturnPrevManuel;
+			case KEY_CTRL_K:
+				return interfaceReturnNextManuel;
 			case KEY_CTRL_L:
 				fsLoopMods=!fsLoopMods;
 				break;
@@ -731,15 +741,17 @@ static interfaceReturnEnum plmpDrawScreen(void)
 				fsScrType=plScrType;
 				curmode->SetMode();
 				break;
-			/* TODO-keys case 0x2600: // ALT+l
+			#if 0
+			TODO plLoopPatterns
+			case KEY_ALT_L:
 				plLoopPatterns=!plLoopPatterns;
+			#endif
 			#ifdef DEBUG
 				if(plLoopPatterns)
 					DEBUGSTR("pattern loop enabled");
 				else
 					DEBUGSTR("pattern loop disabled");
 			#endif
-				break;*/
 			#ifdef DOS32 /* TODO*/
 			case 0xF8:
 				Screenshot();
@@ -748,14 +760,17 @@ static interfaceReturnEnum plmpDrawScreen(void)
 			case KEY_ALT_K:
 				cpiKeyHelp(KEY_ESC, "Exit");
 				cpiKeyHelp(_KEY_ENTER, "Next song");
+				cpiKeyHelp(KEY_INSERT, "Open file selected");
 				cpiKeyHelp('f', "Open file selector");
 				cpiKeyHelp('F', "Open file selector");
 				cpiKeyHelp('d', "Open a shell");
 				cpiKeyHelp('D', "Open a shell");
 				cpiKeyHelp(KEY_CTRL_D, "Open a shell");
-				cpiKeyHelp(KEY_CTRL_J, "Next song (forced)");
+				cpiKeyHelp(KEY_CTRL_J, "Prev song (forced)");
+				cpiKeyHelp(KEY_CTRL_K, "Next song (forced)");
 				cpiKeyHelp(KEY_CTRL_L, "Toggle song looping (ALT-C setting)");
 				cpiKeyHelp(KEY_ALT_C, "Open setup dialog");
+				/* cpiKeyHelp(KEY_ALT_L, "Toggle plLoopPatterns"); */
 				/*return 0;*/
 			default:
 				for (mod=cpiModes; mod; mod=mod->next)
@@ -806,7 +821,9 @@ char plNoteStr[132][4]=
 	"C-9","C#9","D-9","D#9","E-9","F-9","F#9","G-9","G#9","A-9","A#9","B-9"
 };
 
-struct interfacestruct plOpenCP = {plmpOpenFile, plmpCallBack, plmpCloseFile};
+static struct interfacestruct plOpenCP = {plmpOpenFile, plmpCallBack, plmpCloseFile, "plOpenCP", NULL};
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"cpiface", "OpenCP Interface (c) 1994-04 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0, LateInit: plmpInit, PreClose: plmpClose};
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"cpiface", "OpenCP Interface (c) 1994-08 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0, LateInit: plmpInit, PreClose: plmpClose};
 /* OpenCP Module Player */
