@@ -542,6 +542,8 @@ static int plStripeKey(uint16_t key)
 		case KEY_ALT_K:
 			cpiKeyHelp(KEY_PPAGE, "Reduce frequency space for graphical spectrum analyzer");
 			cpiKeyHelp(KEY_PPAGE, "Increase frequency space for graphical spectrum analyzer");
+			cpiKeyHelp(KEY_CTRL_PGUP, "Adjust scale down");
+			cpiKeyHelp(KEY_CTRL_PGDN, "Adjust scale up");
 			cpiKeyHelp(KEY_HOME, "Reset settings for graphical spectrum analyzer");
 			cpiKeyHelp(KEY_TAB, "Cycle palette for graphical spectrum analyzer");
 			cpiKeyHelp(KEY_SHIFT_TAB, "Cycle palette for mini graphical spectrum analyzer");
@@ -553,19 +555,21 @@ static int plStripeKey(uint16_t key)
 			plAnalRate=plAnalRate*30/32;
 			plAnalRate=(plAnalRate>=64000)?64000:(plAnalRate<1024)?1024:plAnalRate;
 			break;
-/*TODO-keys	case 0x8400: //ctrl-pgup
-			plAnalScale=plAnalScale*32/31;
+		case KEY_CTRL_PGUP:
+		/* case 0x8400: //ctrl-pgup */
+			plAnalScale=(plAnalScale+1)*32/31;
 			plAnalScale=(plAnalScale>=4096)?4096:(plAnalScale<256)?256:plAnalScale;
-			break;*/
+			break;
 		/*case 0x5100: //pgdn*/
 		case KEY_NPAGE:
 			plAnalRate=plAnalRate*32/30;
 			plAnalRate=(plAnalRate>=64000)?64000:(plAnalRate<1024)?1024:plAnalRate;
 			break;
-/*TODO-keys	case 0x7600: //ctrl-pgdn
+		case KEY_CTRL_PGDN:
+		/* case 0x7600: //ctrl-pgdn */
 			plAnalScale=plAnalScale*31/32;
 			plAnalScale=(plAnalScale>=4096)?4096:(plAnalScale<256)?256:plAnalScale;
-			break;*/
+			break;
 		/*case 0x4700: //home*/
 		case KEY_HOME:
 			plAnalRate=5512;
@@ -652,7 +656,7 @@ static int strEvent(int ev)
 	return 1;
 }
 
-static struct cpimoderegstruct cpiModeGraph = {"graph", strSetMode, strDraw, strIProcessKey, plStripeKey, strEvent};
+static struct cpimoderegstruct cpiModeGraph = {"graph", strSetMode, strDraw, strIProcessKey, plStripeKey, strEvent CPIMODEREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
