@@ -311,6 +311,9 @@ static int InstAProcessKey(uint16_t key)
 			cpiKeyHelp(KEY_HOME, "Scroll to to the first line in instrument viewer");
 			cpiKeyHelp(KEY_END, "Scroll to to the last line in instrument viewer");
 			cpiKeyHelp(KEY_TAB, "Toggle instrument viewer mode");
+			cpiKeyHelp(KEY_SHIFT_TAB, "Toggle instrument viewer mode");
+			cpiKeyHelp(KEY_CTRL_PGUP, "Scroll up a page in the instrument viewer");
+			cpiKeyHelp(KEY_CTRL_PGDN, "Scroll down a page in the instrument viewer");
 			return 0;
 		case 'i': case 'I':
 			plInstType=(plInstType+1)%4;
@@ -324,12 +327,14 @@ static int InstAProcessKey(uint16_t key)
 		case KEY_NPAGE:
 			plInstScroll++;
 			break;
-/* TODO-keys	case 0x8400: //ctrl-pgup
+		case KEY_CTRL_PGUP:
+		/* case 0x8400: //ctrl-pgup */
 			plInstScroll-=plInstHeight;
 			break;
-		case 0x7600: //ctrl-pgdn
+		case KEY_CTRL_PGDN:
+		/* case 0x7600: //ctrl-pgdn */
 			plInstScroll+=plInstHeight;
-			break;*/
+			break;
 		/*case 0x4700: //home*/
 		case KEY_HOME:
 			plInstScroll=0;
@@ -338,12 +343,9 @@ static int InstAProcessKey(uint16_t key)
 		case KEY_END:
 			plInstScroll=plInstLength;
 			break;
-/* TODO-keys	case 0x1700: // alt-i
-			plInsDisplay.Clear();
-			break;*/
 		case KEY_TAB: /* tab */
-/* TODO-keys	case 0x0F00: // shift-tab
-		case 0xA500:*/
+		case KEY_SHIFT_TAB: /* 0x0f00 */
+/* TODO-keys    case 0xA500:  alt-tab */
 			plInstMode=!plInstMode;
 			break;
 		default:
@@ -366,7 +368,7 @@ static int InstEvent(int ev)
 	return 1;
 }
 
-static struct cpitextmoderegstruct cpiTModeInst = {"inst", InstGetWin, InstSetWin, InstDraw, InstIProcessKey, InstAProcessKey, InstEvent};
+static struct cpitextmoderegstruct cpiTModeInst = {"inst", InstGetWin, InstSetWin, InstDraw, InstIProcessKey, InstAProcessKey, InstEvent CPITEXTMODEREGSTRUCT_TAIL};
 
 void plUseInstruments(struct insdisplaystruct *x)
 {
