@@ -154,19 +154,25 @@ static int plScopesKey(uint16_t key)
 			cpiKeyHelp(KEY_NPAGE, "Decrease the frequency space for the phase viewer");
 			cpiKeyHelp(KEY_HOME, "Reset the settings for the phase viewer");
 			cpiKeyHelp(KEY_TAB, "Toggle phase viewer channel-mode");
-			break;
-/* TODO-keys	case 0x8400: //ctrl-pgup
-		//    plOszRate=plOszRate*31/32;
-		//    plOszRate=(plOszRate>=512000)?256000:(plOszRate<2048)?2048:plOszRate;
-			plScopesRatio=plScopesRatio*32/31;
+			cpiKeyHelp(KEY_SHIFT_TAB, "Toggle phase viewer channel-mode");
+			cpiKeyHelp(KEY_ALT_O, "Toggle phase viewer channel-mode");
+			cpiKeyHelp(KEY_CTRL_PGUP, "Adjust scale up");
+			cpiKeyHelp(KEY_CTRL_PGDN, "Adjust scale down");
+			return 0;
+		case KEY_CTRL_PGUP:
+		/* case 0x8400: //ctrl-pgup */
+		/*      plOszRate=plOszRate*31/32; */
+		/*      plOszRate=(plOszRate>=512000)?256000:(plOszRate<2048)?2048:plOszRate; */
+			plScopesRatio=(plScopesRatio+1)*32/31;
 			plScopesRatio=(plScopesRatio>=1024)?1024:(plScopesRatio<64)?64:plScopesRatio;
 			break;
-		case 0x7600: //ctrl-pgdn
-		//    plOszRate=plOszRate*32/31;
-		//    plOszRate=(plOszRate>=256000)?256000:(plOszRate<2048)?2048:plOszRate;
+		case KEY_CTRL_PGDN:
+		/* case 0x7600: //ctrl-pgdn */
+		/*      plOszRate=plOszRate*32/31; */
+		/*      plOszRate=(plOszRate>=256000)?256000:(plOszRate<2048)?2048:plOszRate; */
 			plScopesRatio=plScopesRatio*31/32;
 			plScopesRatio=(plScopesRatio>=1024)?1024:(plScopesRatio<64)?64:plScopesRatio;
-			break;*/
+			break;
 		/*case 0x4900: //pgup*/
 		case KEY_PPAGE:
 			if (plOszChan==2)
@@ -197,9 +203,11 @@ static int plScopesKey(uint16_t key)
 			plOszRate=44100;
 			break;
 		case KEY_TAB: /* tab */
-/* TODO -keys	case 0x0F00: // shift-tab
-		case 0xA500:
-		case 0x1800: // alt-o*/
+		case KEY_SHIFT_TAB:
+		/* case 0x0F00: // shift-tab */
+/* TODO -keys	case 0xA500: // alt-tab */
+		case KEY_ALT_O:
+		/*case 0x1800: // alt-o*/
 			if (plOszChan==2)
 			{
 				plOszMono=!plOszMono;
@@ -425,7 +433,7 @@ static int scoEvent(int ev)
 	return 1;
 }
 
-static struct cpimoderegstruct cpiModePhase = {"phase", scoSetMode, scoDraw, scoIProcessKey, plScopesKey, scoEvent};
+static struct cpimoderegstruct cpiModePhase = {"phase", scoSetMode, scoDraw, scoIProcessKey, plScopesKey, scoEvent CPIMODEREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
