@@ -31,7 +31,7 @@ static int cpiReadMemInfo(struct moduleinfostruct *m, const char *buf, size_t _i
 
 static int cpiReadInfo(struct moduleinfostruct *m, FILE *file, const char *c, size_t _ignore)
 {
-	return 0;
+	return cpiReadMemInfo(m, c, _ignore);
 }
 
-struct mdbreadinforegstruct cpiReadInfoReg = {cpiReadMemInfo, cpiReadInfo};
+struct mdbreadinforegstruct cpiReadInfoReg = {cpiReadMemInfo, cpiReadInfo, 0 MDBREADINFOREGSTRUCT_TAIL};
