@@ -783,7 +783,7 @@ static int trkEvent(int ev)
 	return 1;
 }
 
-static struct cpitextmoderegstruct cpiTModeTrack = {"trak", TrakGetWin, TrakSetWin, TrakDraw, TrakIProcessKey, TrakAProcessKey, trkEvent};
+static struct cpitextmoderegstruct cpiTModeTrack = {"trak", TrakGetWin, TrakSetWin, TrakDraw, TrakIProcessKey, TrakAProcessKey, trkEvent CPITEXTMODEREGSTRUCT_TAIL};
 
 void cpiTrkSetup(const struct cpitrakdisplaystruct *c, int npat)
 {
