@@ -149,9 +149,9 @@ static void SetWin(int xmin, int xwid, int ymin, int ywid)
 static void Draw(int focus)
 {
 	uint16_t buf[CONSOLE_MAX_X];
-	int ll;
+	size_t ll;
 	int i;
-	int barlen;
+	unsigned int barlen;
 	int su, sd;
 
 	memset(buf, 0, sizeof(buf));
@@ -237,8 +237,8 @@ static void Draw(int focus)
 			
 			char   sbuf[CONSOLE_MAX_X], *ptr=&sbuf[0];
 			int    i;
-			int    of;
-			int    po;
+			unsigned int of;
+			unsigned int po;
 			
 			strcpy(sbuf, x.name);
 			for (i=x.val+1, ptr=&sbuf[0]; i && *ptr; ptr++)
@@ -266,9 +266,9 @@ static void Draw(int focus)
 			int p=((x.val-x.min)*(barlen))/(x.max-x.min);       /* range: 0..barlen */
 			int po;
 			
-			p=(p>barlen)?barlen:(p<0)?0:p;
+			p=((unsigned)p>barlen)?barlen:(p<0)?0:p; /* p never get this big */
 			
-			for (po=0; po<barlen; po++)
+			for (po=0; (unsigned)po<barlen; po++) /* barlen and po should never be this big */
 			{
 				int p4=(po<<2)/barlen;                    /* range: 0..4 */
 				if(po<p)
@@ -431,7 +431,7 @@ static int Event(int ev)
 	return 0;
 }
 
-static struct cpitextmoderegstruct cpiVolCtrl={"volctrl", GetWin, SetWin, Draw, IProcessKey, AProcessKey, Event};
+static struct cpitextmoderegstruct cpiVolCtrl={"volctrl", GetWin, SetWin, Draw, IProcessKey, AProcessKey, Event CPITEXTMODEREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
