@@ -84,42 +84,36 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 			continue;
 		}
 
-		if ((dsym=lnkReadInfoReg(n->linkhand, "addprocs")))
-/*		if (*dsym)*/
-			n->addprocs=(struct devaddstruct *)_lnkGetSymbol(dsym);
-		else
-			n->addprocs=NULL;
-
 		bypass=cfGetProfileBool(drvhand, "bypass", 0, 0);
 		n->ihandle=handlfn++;
 		n->keep=cfGetProfileBool(drvhand, "keep", 0, 0);
-		n->dev.port=cfGetProfileInt(drvhand, "port", -1, 16);
-		n->dev.port2=cfGetProfileInt(drvhand, "port2", -1, 16);
+		n->devinfo.port=cfGetProfileInt(drvhand, "port", -1, 16);
+		n->devinfo.port2=cfGetProfileInt(drvhand, "port2", -1, 16);
 /*		n->dev.irq=cfGetProfileInt(drvhand, "irq", -1, 10);
 		n->dev.irq2=cfGetProfileInt(drvhand, "irq2", -1, 10);
 		n->dev.dma=cfGetProfileInt(drvhand, "dma", -1, 10);
 		n->dev.dma2=cfGetProfileInt(drvhand, "dma2", -1, 10);*/
-		n->dev.subtype=cfGetProfileInt(drvhand, "subtype", -1, 10);
-		strncpy(n->dev.path, cfGetProfileString(drvhand, "path", ""), sizeof(n->dev.path));
-		n->dev.path[sizeof(n->dev.path)-1]=0;
-		strncpy(n->dev.mixer, cfGetProfileString(drvhand, "mixer", ""), sizeof(n->dev.mixer));
-		n->dev.mixer[sizeof(n->dev.mixer)-1]=0;
-		n->dev.chan=0;
-		n->dev.mem=0;
-		n->dev.opt=0;
+		n->devinfo.subtype=cfGetProfileInt(drvhand, "subtype", -1, 10);
+		strncpy(n->devinfo.path, cfGetProfileString(drvhand, "path", ""), sizeof(n->devinfo.path));
+		n->devinfo.path[sizeof(n->devinfo.path)-1]=0;
+		strncpy(n->devinfo.mixer, cfGetProfileString(drvhand, "mixer", ""), sizeof(n->devinfo.mixer));
+		n->devinfo.mixer[sizeof(n->devinfo.mixer)-1]=0;
+		n->devinfo.chan=0;
+		n->devinfo.mem=0;
+		n->devinfo.opt=0;
 
 		strcpy(n->name, dev->name);
-		if (n->addprocs)
-			if (n->addprocs->GetOpt)
-				n->dev.opt=n->addprocs->GetOpt(drvhand);
-		n->dev.opt|=cfGetProfileInt(drvhand, "options", 0, 16);
+		if (dev->addprocs)
+			if (dev->addprocs->GetOpt)
+				n->devinfo.opt=dev->addprocs->GetOpt(drvhand);
+		n->devinfo.opt|=cfGetProfileInt(drvhand, "options", 0, 16);
 
 		fprintf(stderr, "%s", n->name);
 		for (i=strlen(n->name); i<32; i++)
 			fprintf(stderr, ".");
 		if (!bypass)
 		{
-			if (!dev->Detect(&n->dev))
+			if (!dev->Detect(&n->devinfo))
 			{
 				fprintf(stderr, " not found: optimize ocp.ini!\n");
 				lnkFree(n->linkhand);
@@ -127,7 +121,7 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 			        continue;
 			}
 		} else
-			n->dev.dev=dev;
+			n->devinfo.devtype=dev;
 
 		if (!n->keep)
 		{
@@ -136,14 +130,14 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 		}
 
 		fprintf(stderr, " (#%d", n->ihandle);
-		if (n->dev.subtype!=-1)
-			fprintf(stderr, " t%d", n->dev.subtype);
-		if (n->dev.port!=-1)
-			fprintf(stderr, " p%x", n->dev.port);
-		if (n->dev.port2!=-1)
-			fprintf(stderr, " q%x", n->dev.port2);
-		if (n->dev.mem)
-			fprintf(stderr, " m%d", (int)(n->dev.mem>>10));
+		if (n->devinfo.subtype!=-1)
+			fprintf(stderr, " t%d", n->devinfo.subtype);
+		if (n->devinfo.port!=-1)
+			fprintf(stderr, " p%x", n->devinfo.port);
+		if (n->devinfo.port2!=-1)
+			fprintf(stderr, " q%x", n->devinfo.port2);
+		if (n->devinfo.mem)
+			fprintf(stderr, " m%d", (int)(n->devinfo.mem>>10));
 		fprintf(stderr, ")\n"); 
 		
 /*		strcpy(str, " (");
@@ -197,4 +191,4 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 	return 1;
 }
 
-struct linkinfostruct dllextinfo = {"devi", "OpenCP Devices Auxiliary Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"devi", "OpenCP Devices Auxiliary Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
