@@ -36,6 +36,8 @@ static struct devinfonode *curplaydev;
 static struct devinfonode *defplaydev;
 
 static struct mdbreaddirregstruct plrReadDirReg;
+static struct interfacestruct plrIntr;
+static struct preprocregstruct plrPreprocess;
 
 static struct devinfonode *getdevstr(struct devinfonode *n, const char *hnd)
 {
@@ -54,11 +56,11 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 		return;
 	if (*curdev)
 	{
-		if ((*curdev)->addprocs)
-			if ((*curdev)->addprocs->Close)
-				(*curdev)->addprocs->Close();
+		if ((*curdev)->devinfo.devtype->addprocs)
+			if ((*curdev)->devinfo.devtype->addprocs->Close)
+				(*curdev)->devinfo.devtype->addprocs->Close();
 		plrProcessKey=0;
-		(*curdev)->dev.dev->Close();
+		(*curdev)->devinfo.devtype->Close();
 		if (!(*curdev)->keep)
 		{
 			lnkFree((*curdev)->linkhand);
@@ -71,7 +73,6 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 	if (dev->linkhand<0)
 	{
 		char lname[22];
-		char *dsym;
 		strncpy(lname,cfGetProfileString(dev->handle, "link", ""),21);
 		dev->linkhand=lnkLink(lname);
 		if (dev->linkhand<0)
@@ -79,29 +80,24 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 			fprintf(stderr, "device load error\n");
 			return;
 		}
-		dev->dev.dev=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
-		if (!dev->dev.dev)
+		dev->devinfo.devtype=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
+		if (!dev->devinfo.devtype)
 		{
 			fprintf(stderr, "device symbol error\n");
 			lnkFree(dev->linkhand);
 			dev->linkhand=-1;
 			return;
 		}
-		dsym=lnkReadInfoReg(dev->linkhand, "addprocs");
-		if (dsym)
-			dev->addprocs=0;
-		else
-			dev->addprocs=(struct devaddstruct *)_lnkGetSymbol(dsym);
 	}
 	fprintf(stderr, "%s selected...\n", dev->name);
-	if (dev->dev.dev->Init(&dev->dev))
+	if (dev->devinfo.devtype->Init(&dev->devinfo))
 	{
-		if (dev->addprocs)
-			if (dev->addprocs->Init)
-				dev->addprocs->Init(dev->handle);
-		if (dev->addprocs)
-			if (dev->addprocs->ProcessKey)
-				plrProcessKey=dev->addprocs->ProcessKey;
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->Init)
+				dev->devinfo.devtype->addprocs->Init(dev->handle);
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->ProcessKey)
+				plrProcessKey=dev->devinfo.devtype->addprocs->ProcessKey;
 		(*curdev)=dev;
 		return;
 	}
@@ -138,6 +134,8 @@ static int playdevinit(void)
 
 	mdbRegisterReadDir(&plrReadDirReg);
 
+	plRegisterInterface (&plrIntr);
+	plRegisterPreprocess (&plrPreprocess);
 
 	dmSETUP=RegisterDrive("setup:");
 
@@ -175,6 +173,9 @@ static void playdevclose(void)
 
 	mdbUnregisterReadDir(&plrReadDirReg);
 
+	plUnregisterInterface (&plrIntr);
+	plUnregisterPreprocess (&plrPreprocess);
+
 	setdevice(&curplaydev, 0);
 	while (plPlayerDevices)
 	{
@@ -198,7 +199,7 @@ static int plrReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	{
 		if (path==dmSETUP->basepath)
 		{
-			if (ml->find(ml, node)<0)
+			if (modlist_find(ml, node)<0)
 			{
 				memset(&m, 0, sizeof(m));
 				m.drive=drive;
@@ -206,7 +207,7 @@ static int plrReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 				strcpy(m.shortname, "DEVICES");
 				m.dirdbfullpath=node;
 				m.flags=MODLIST_FLAG_DIR;
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 			}
 		}
 			
@@ -225,7 +226,7 @@ static int plrReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 /*			if (fsMatchFileName12(m.name, mask))*/
 			{
 				char npath[64];	
-				m.fileref=mdbGetModuleReference(m.name, dev->dev.mem);
+				m.fileref=mdbGetModuleReference(m.name, dev->devinfo.mem);
 				if (m.fileref==0xffffffff)
 					goto out;
 				m.drive=drive;
@@ -238,12 +239,12 @@ static int plrReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 					struct moduleinfostruct mi;
 					mdbGetModuleInfo(&mi, m.fileref);
 					mi.flags1|=MDB_VIRTUAL;
-					mi.channels=dev->dev.chan;
+					mi.channels=dev->devinfo.chan;
 					strcpy(mi.modname, dev->name);
 					mi.modtype=mtDEVp;
 					mdbWriteModuleInfo(m.fileref, &mi);
 				}
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 				dirdbUnref(m.dirdbfullpath);
 			}
 		}
@@ -268,9 +269,10 @@ static void plrPrep(const char *path, struct moduleinfostruct *m, FILE **fp)
 	plrResetDevice();
 }
 
-struct interfacestruct plrIntr = {plrSet, 0, 0};
-static struct mdbreaddirregstruct plrReadDirReg = {plrReadDir};
-
-struct preprocregstruct plrPreprocess = {plrPrep};
-char *dllinfo = "preprocess plrPreprocess";
-struct linkinfostruct dllextinfo = {"plrbase", "OpenCP Player Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: playdevinit, Close: playdevclose};
+static struct mdbreaddirregstruct plrReadDirReg = {plrReadDir MDBREADDIRREGSTRUCT_TAIL};
+static struct interfacestruct plrIntr = {plrSet, 0, 0, "plrIntr" INTERFACESTRUCT_TAIL};
+static struct preprocregstruct plrPreprocess = {plrPrep PREPROCREGSTRUCT_TAIL};
+#ifndef SUPPORT_STATIC_PLUGINS
+char *dllinfo = "";
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"plrbase", "OpenCP Player Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: playdevinit, Close: playdevclose};
