@@ -37,6 +37,8 @@ struct devinfonode *plSamplerDevices;
 static struct devinfonode *cursampdev;
 static struct devinfonode *defsampdev;
 static struct mdbreaddirregstruct smpReadDirReg;
+static struct interfacestruct smpIntr;
+static struct preprocregstruct smpPreprocess;
 
 static struct devinfonode *getdevstr(struct devinfonode *n, const char *hnd)
 {
@@ -83,11 +85,11 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 		return;
 	if (*curdev)
 	{
-		if ((*curdev)->addprocs)
-			if ((*curdev)->addprocs->Close)
-				(*curdev)->addprocs->Close();
+		if ((*curdev)->devinfo.devtype->addprocs)
+			if ((*curdev)->devinfo.devtype->addprocs->Close)
+				(*curdev)->devinfo.devtype->addprocs->Close();
 		smpProcessKey=0;
-		(*curdev)->dev.dev->Close();
+		(*curdev)->devinfo.devtype->Close();
 		if (!(*curdev)->keep)
 		{
 			lnkFree((*curdev)->linkhand);
@@ -100,7 +102,6 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 	if (dev->linkhand<0)
 	{
 		char lname[22];
-		char *dsym;
 		strncpy(lname,cfGetProfileString(dev->handle, "link", ""),21);
 		dev->linkhand=lnkLink(lname);
 		if (dev->linkhand<0)
@@ -108,30 +109,25 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 			fprintf(stderr, "device load error\n");
 			return;
 		}
-		dev->dev.dev=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
-		if (!dev->dev.dev)
+		dev->devinfo.devtype=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
+		if (!dev->devinfo.devtype)
 		{
 			fprintf(stderr, "device symbol error\n");
 			lnkFree(dev->linkhand);
 			dev->linkhand=-1;
 			return;
 		}
-		dsym=lnkReadInfoReg(dev->linkhand, "addprocs");
-		if (dsym)
-			dev->addprocs=0;
-		else
-			dev->addprocs=(struct devaddstruct *)_lnkGetSymbol(dsym);
 	}
 
 	fprintf(stderr, "%s selected...\n", dev->name);
-	if (dev->dev.dev->Init(&dev->dev))
+	if (dev->devinfo.devtype->Init(&dev->devinfo))
 	{
-		if (dev->addprocs)
-			if (dev->addprocs->Init)
-				dev->addprocs->Init(dev->handle);
-		if (dev->addprocs)
-			if (dev->addprocs->ProcessKey)
-				smpProcessKey=dev->addprocs->ProcessKey;
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->Init)
+				dev->devinfo.devtype->addprocs->Init(dev->handle);
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->ProcessKey)
+				smpProcessKey=dev->devinfo.devtype->addprocs->ProcessKey;
 		*curdev=dev;
 		return;
 	}
@@ -171,6 +167,9 @@ static int sampdevinit(void)
 
 	mdbRegisterReadDir(&smpReadDirReg);
 
+	plRegisterInterface (&smpIntr);
+	plRegisterPreprocess (&smpPreprocess);
+
 	dmSETUP=RegisterDrive("setup:");
 
 	if (!strlen(cfGetProfileString2(cfSoundSec, "sound", "samplerdevices", "")))
@@ -224,6 +223,9 @@ static void sampdevclose(void)
 #endif
 	mdbUnregisterReadDir(&smpReadDirReg);
 
+	plUnregisterInterface (&smpIntr);
+	plUnregisterPreprocess (&smpPreprocess);
+
 	setdevice(&cursampdev, 0);
 
 	while (plSamplerDevices)
@@ -248,7 +250,7 @@ static int smpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	{
 		if (path==dmSETUP->basepath)
 		{
-			if (ml->find(ml, node)<0)
+			if (modlist_find(ml, node)<0)
 			{
 				memset(&m, 0, sizeof(m));
 				m.drive=drive;
@@ -256,7 +258,7 @@ static int smpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 				strcpy(m.shortname, "DEVICES");
 				m.dirdbfullpath=node;
 				m.flags=MODLIST_FLAG_DIR;
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 			}
 		}	
 	}
@@ -274,7 +276,7 @@ static int smpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 /*			if (fsMatchFileName12(m.name, mask)) */
 			{
 				char npath[64];
-				m.fileref=mdbGetModuleReference(m.name, dev->dev.mem);
+				m.fileref=mdbGetModuleReference(m.name, dev->devinfo.mem);
 				if (m.fileref==0xffffffff)
 					goto out;
 				m.drive=drive;
@@ -287,12 +289,12 @@ static int smpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 					struct moduleinfostruct mi;
 					mdbGetModuleInfo(&mi, m.fileref);
 					mi.flags1|=MDB_VIRTUAL;
-					mi.channels=dev->dev.chan;
+					mi.channels=dev->devinfo.chan;
 					strcpy(mi.modname, dev->name);
 					mi.modtype=mtDEVs;
 					mdbWriteModuleInfo(m.fileref, &mi);
 				}
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 				dirdbUnref(m.dirdbfullpath);
 			}
 		}
@@ -316,8 +318,10 @@ static void smpPrep(const char *path, struct moduleinfostruct *m, FILE **fp)
 	smpResetDevice();
 }
 
-struct interfacestruct smpIntr = {smpSet, 0, 0};
-static struct mdbreaddirregstruct smpReadDirReg = {smpReadDir};
-struct preprocregstruct smpPreprocess = {smpPrep};
-char *dllinfo = "preprocess smpPreprocess";
-struct linkinfostruct dllextinfo = {"smpbase", "OpenCP Sampler Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: sampdevinit, Close: sampdevclose};
+static struct mdbreaddirregstruct smpReadDirReg = {smpReadDir MDBREADDIRREGSTRUCT_TAIL};
+static struct interfacestruct smpIntr = {smpSet, 0, 0, "smpIntr" INTERFACESTRUCT_TAIL};
+static struct preprocregstruct smpPreprocess = {smpPrep PREPROCREGSTRUCT_TAIL};
+#ifndef SUPPORT_STATIC_PLUGINS
+char *dllinfo = "";
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"smpbase", "OpenCP Sampler Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: sampdevinit, Close: sampdevclose};
