@@ -34,6 +34,8 @@ struct devinfonode *plWaveTableDevices;
 static struct devinfonode *curwavedev;
 static struct devinfonode *defwavedev;
 static struct mdbreaddirregstruct mcpReadDirReg;
+static struct interfacestruct mcpIntr;
+static struct preprocregstruct mcpPreprocess;
 
 static struct devinfonode *getdevstr(struct devinfonode *n, const char *hnd)
 {
@@ -52,11 +54,11 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 		return;
 	if (*curdev)
 	{
-		if ((*curdev)->addprocs)
-			if ((*curdev)->addprocs->Close)
-				(*curdev)->addprocs->Close();
+		if ((*curdev)->devinfo.devtype->addprocs)
+			if ((*curdev)->devinfo.devtype->addprocs->Close)
+				(*curdev)->devinfo.devtype->addprocs->Close();
 		mcpProcessKey=0;
-		(*curdev)->dev.dev->Close();
+		(*curdev)->devinfo.devtype->Close();
 		if (!(*curdev)->keep)
 		{
 			lnkFree((*curdev)->linkhand);
@@ -70,7 +72,6 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 	if (dev->linkhand<0)
 	{
 		char lname[22];
-		char *dsym;
 		strncpy(lname,cfGetProfileString(dev->handle, "link", ""),21);
 		dev->linkhand=lnkLink(lname);
 		if (dev->linkhand<0)
@@ -78,32 +79,25 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 			fprintf(stderr, "device load error\n");
 			return;
 		}
-		dev->dev.dev=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
-		if (!dev->dev.dev)
+		dev->devinfo.devtype=(struct sounddevice *)_lnkGetSymbol(lnkReadInfoReg(dev->linkhand, "driver"));
+		if (!dev->devinfo.devtype)
 		{
 			fprintf(stderr, "device symbol error\n");
 			lnkFree(dev->linkhand);
 			dev->linkhand=-1;
 			return;
 		}
-		dsym=lnkReadInfoReg(dev->linkhand, "addprocs");
-	
-		dev->addprocs=0;	
-		if (dsym)
-			dev->addprocs=0;
-		else
-			dev->addprocs=(struct devaddstruct *)_lnkGetSymbol(dsym);
 	}
 
 	fprintf(stderr, "%s selected...\n", dev->name);
-	if (dev->dev.dev->Init(&dev->dev))
+	if (dev->devinfo.devtype->Init(&dev->devinfo))
 	{
-		if (dev->addprocs)
-			if (dev->addprocs->Init)
-				dev->addprocs->Init(dev->handle);
-		if (dev->addprocs)
-			if (dev->addprocs->ProcessKey)
-				mcpProcessKey=dev->addprocs->ProcessKey;
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->Init)
+				dev->devinfo.devtype->addprocs->Init(dev->handle);
+		if (dev->devinfo.devtype->addprocs)
+			if (dev->devinfo.devtype->addprocs->ProcessKey)
+				mcpProcessKey=dev->devinfo.devtype->addprocs->ProcessKey;
 		*curdev=dev;
 		return;
 	}
@@ -142,6 +136,9 @@ static int wavedevinit(void)
 
 	mdbRegisterReadDir(&mcpReadDirReg);
 
+	plRegisterInterface (&mcpIntr);
+	plRegisterPreprocess (&mcpPreprocess);
+
 	dmSETUP=RegisterDrive("setup:");
 
 	if (!strlen(cfGetProfileString2(cfSoundSec, "sound", "wavetabledevices", "")))
@@ -196,6 +193,9 @@ static void wavedevclose(void)
 
 	mdbUnregisterReadDir(&mcpReadDirReg);
 
+	plUnregisterInterface (&mcpIntr);
+	plUnregisterPreprocess (&mcpPreprocess);
+
 	setdevice(&curwavedev, 0);
 	while (plWaveTableDevices)
 	{
@@ -219,7 +219,7 @@ static int mcpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	{
 		if (path==dmSETUP->basepath)
 		{
-			if (ml->find(ml, node)<0)
+			if (modlist_find(ml, node)<0)
 			{
 				memset(&m, 0, sizeof(m));
 				m.drive=drive;
@@ -227,7 +227,7 @@ static int mcpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 				strcpy(m.shortname, "DEVICES");
 				m.dirdbfullpath=node;
 				m.flags=MODLIST_FLAG_DIR;
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 			}
 		}
 	}
@@ -245,7 +245,7 @@ static int mcpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 /*			if (fsMatchFileName12(m.name, mask))*/
 			{
 				char npath[64];
-				m.fileref=mdbGetModuleReference(m.name, dev->dev.mem);
+				m.fileref=mdbGetModuleReference(m.name, dev->devinfo.mem);
 				if (m.fileref==0xffffffff)
 					goto out;
 				m.drive=drive;
@@ -258,12 +258,12 @@ static int mcpReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 					struct moduleinfostruct mi;
 					mdbGetModuleInfo(&mi, m.fileref);
 					mi.flags1|=MDB_VIRTUAL;
-					mi.channels=dev->dev.chan;
+					mi.channels=dev->devinfo.chan;
 					strcpy(mi.modname, dev->name);
 					mi.modtype=mtDEVw;
 					mdbWriteModuleInfo(m.fileref, &mi);
 				}
-				ml->append(ml, &m);
+				modlist_append(ml, &m);
 				dirdbUnref(m.dirdbfullpath);
 			}
 		}
@@ -290,10 +290,10 @@ static void mcpPrep(const char *path, struct moduleinfostruct *info, FILE **bp)
 }
 
 
-static struct mdbreaddirregstruct mcpReadDirReg = {mcpReadDir};
-
-struct interfacestruct mcpIntr = {mcpSetDev, 0, 0};
-
-struct preprocregstruct mcpPreprocess = {mcpPrep};
-char *dllinfo = "preprocess mcpPreprocess";
-struct linkinfostruct dllextinfo = {"mcpbase", "OpenCP Wavetable Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: wavedevinit, Close: wavedevclose};
+static struct mdbreaddirregstruct mcpReadDirReg = {mcpReadDir MDBREADDIRREGSTRUCT_TAIL};
+static struct interfacestruct mcpIntr = {mcpSetDev, 0, 0, "mcpIntr" INTERFACESTRUCT_TAIL};
+static struct preprocregstruct mcpPreprocess = {mcpPrep PREPROCREGSTRUCT_TAIL};
+#ifndef SUPPORT_STATIC_PLUGINS
+char *dllinfo = "";
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"mcpbase", "OpenCP Wavetable Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: wavedevinit, Close: wavedevclose};
