@@ -1,6 +1,7 @@
 #ifndef _IMSDEV_H
 #define _IMSDEV_H
 
+/* All drivers set these, but they are not used to anything */
 #define SS_DEVTYPE 3
 #define SS_PLAYER 0
 #define SS_SAMPLER 1
@@ -13,9 +14,17 @@ struct sounddevice;
 
 #define DEVICE_NAME_MAX 63
 
+struct devaddstruct
+{
+	uint32_t (*GetOpt)(const char *devinfonode_handle);
+	void (*Init)(const char *devinfonode_handle);
+	void (*Close)();
+	int (*ProcessKey)(uint16_t);
+};
+
 struct deviceinfo
 {
-	struct sounddevice *dev;
+	struct sounddevice *devtype;
 	int16_t port;
 	int16_t port2;
 /*	signed char irq;
@@ -37,6 +46,7 @@ struct sounddevice
 	int (*Detect)(struct deviceinfo *c);
 	int (*Init)(const struct deviceinfo *c);
 	void (*Close)(void);
+	struct devaddstruct *addprocs;
 };
 
 #endif
