@@ -23,13 +23,13 @@ void (*mcpSet)(int ch, int opt, int val);
 int (*mcpGet)(int ch, int opt);
 void (*mcpGetRealVolume)(int ch, int *l, int *r);
 void (*mcpGetRealMasterVolume)(int *l, int *r);
-void (*mcpGetMasterSample)(int16_t *s, int len, uint32_t rate, int opt);
-int (*mcpGetChanSample)(int ch, int16_t *s, int len, uint32_t rate, int opt);
-int (*mcpMixChanSamples)(int *ch, int n, int16_t *s, int len, uint32_t rate, int opt);
+void (*mcpGetMasterSample)(int16_t *s, unsigned int len, uint32_t rate, int opt);
+int (*mcpGetChanSample)(unsigned int ch, int16_t *s, unsigned int len, uint32_t rate, int opt);
+int (*mcpMixChanSamples)(unsigned int *ch, unsigned int n, int16_t *s, unsigned int len, uint32_t rate, int opt);
 
-int mcpMixMaxRate;
-int mcpMixProcRate;
+unsigned int mcpMixMaxRate;
+unsigned int mcpMixProcRate;
 int mcpMixOpt;
-int mcpMixBufSize;
-int mcpMixMax;
+unsigned int mcpMixBufSize;
+unsigned int mcpMixMax;
 int mcpMixPoll;
