@@ -28,7 +28,7 @@
 
 #define MIXBUFLEN 2048
 
-static void (*mixGetMixChannel)(int ch, struct mixchannel *chn, uint32_t rate);
+static void (*mixGetMixChannel)(unsigned int ch, struct mixchannel *chn, uint32_t rate);
 
 static struct mixchannel *channels;
 
@@ -92,13 +92,13 @@ static void mixgetmixch(int ch, struct mixchannel *chn, int rate)
 		return;
 	}
 	if (chn->status&MIX_PLAY16BIT)
-		chn->samp=(void*)((uint32_t)chn->samp>>1);
+		chn->samp=(void*)((unsigned long)chn->samp>>1);
 	if (chn->status&MIX_PLAYFLOAT)
-		chn->samp=(void*)((uint32_t)chn->samp>>2);
+		chn->samp=(void*)((unsigned long)chn->samp>>2);
 	chn->replen=(chn->status&MIX_LOOPED)?(chn->loopend-chn->loopstart):0;
 }
 
-static void putchn(struct mixchannel *chn, int len, int opt)
+static void putchn(struct mixchannel *chn, unsigned int len, int opt)
 {
 	if (!(chn->status&MIX_PLAYING)||(chn->status&MIX_MUTE))
 		return;
@@ -129,14 +129,14 @@ static void putchn(struct mixchannel *chn, int len, int opt)
 	mixPlayChannel(mixbuf, len, chn, opt&mcpGetSampleStereo);
 }
 
-void mixGetMasterSample(int16_t *s, int len, uint32_t rate, int opt)
+void mixGetMasterSample(int16_t *s, unsigned int len, uint32_t rate, int opt)
 {
 	int stereo=(opt&mcpGetSampleStereo)?1:0;
 	int i;
 	for (i=0; i<channum; i++)
 		mixgetmixch(i, &channels[i], rate);
 
-	if (len>(MIXBUFLEN>>stereo))
+	if (len>((unsigned)MIXBUFLEN>>stereo))
 	{
 		memset(s+MIXBUFLEN, 0, ((len<<stereo)-MIXBUFLEN)<<1);
 		len=MIXBUFLEN>>stereo;
@@ -147,11 +147,11 @@ void mixGetMasterSample(int16_t *s, int len, uint32_t rate, int opt)
 	mixClip(s, mixbuf, len<<stereo, amptab, clipmax);
 }
 
-static int mixMixChanSamples(int *ch, int n, int16_t *s, int len, uint32_t rate, int opt)
+static int mixMixChanSamples(unsigned int *ch, unsigned int n, int16_t *s, unsigned int len, uint32_t rate, int opt)
 {
 	int stereo=(opt&mcpGetSampleStereo)?1:0;
 	int ret;
-	int i;
+	unsigned int i;
 
 	if (!n)
 	{
@@ -187,7 +187,7 @@ static int mixMixChanSamples(int *ch, int n, int16_t *s, int len, uint32_t rate,
 	return ret;
 }
 
-int mixGetChanSample(int ch, int16_t *s, int len, uint32_t rate, int opt)
+int mixGetChanSample(unsigned int ch, int16_t *s, unsigned int len, uint32_t rate, int opt)
 {
 	return mixMixChanSamples(&ch, 1, s, len, rate, opt);
 }
@@ -245,7 +245,7 @@ void mixSetAmplify(int amp)
 	calcamptab((amplify*channum)>>11);
 }
 
-int mixInit(void (*getchan)(int ch, struct mixchannel *chn, uint32_t rate), int masterchan, int chn, int amp)
+int mixInit(void (*getchan)(unsigned int ch, struct mixchannel *chn, uint32_t rate), int masterchan, unsigned int chn, int amp)
 {
 	int i,j;
 
