@@ -19,14 +19,14 @@
 #include "stuff/imsrtns.h"
 
 
-int plrRate;
+unsigned int plrRate;
 int plrOpt;
-int (*plrPlay)(void **buf, int *len);
+int (*plrPlay)(void **buf, unsigned int *len);
 void (*plrStop)(void);
 void (*plrSetOptions)(uint32_t rate, int opt);
 int (*plrGetBufPos)(void);
 int (*plrGetPlayPos)(void);
-void (*plrAdvanceTo)(int pos);
+void (*plrAdvanceTo)(unsigned int pos);
 uint32_t (*plrGetTimer)(void);
 void (*plrIdle)(void);
 #ifdef PLR_DEBUG
@@ -45,17 +45,17 @@ static unsigned long buflen;
 
 void plrGetRealMasterVolume(int *l, int *r)
 {
-	int len=samprate/20;
-	int p;
-	int pass2;
+	uint32_t len=samprate/20;
+	int32_t p;
+	int32_t pass2;
 	mixAddAbsfn fn;
 	unsigned long v;
 	
 	if (len>buflen)
 		len=buflen;
 	p=plrGetPlayPos()>>(stereo+bit16);
-	
-	pass2=len-buflen+p;
+
+	pass2=len-(uint32_t)buflen+p;
 
 	if (stereo)
 	{
@@ -94,13 +94,13 @@ void plrGetRealMasterVolume(int *l, int *r)
 	}
 }
 
-void plrGetMasterSample(int16_t *buf, int len, uint32_t rate, int opt)
+void plrGetMasterSample(int16_t *buf, uint32_t len, uint32_t rate, int opt)
 {
 	uint32_t step=umuldiv(samprate, 0x10000, rate);
-	int maxlen;
+	unsigned int maxlen;
 	int stereoout;
-	unsigned long bp;
-	signed long pass2;
+	uint32_t bp;
+	int32_t pass2;
 	mixGetMasterSamplefn fn;
 	
 	if (step<0x1000)
@@ -116,8 +116,10 @@ void plrGetMasterSample(int16_t *buf, int len, uint32_t rate, int opt)
 		len=maxlen;
 	}
 
-	bp=plrGetPlayPos()>>(stereo+bit16);pass2=len-imuldiv(buflen-bp,0x10000,step);
-
+	bp=plrGetPlayPos()>>(stereo+bit16);
+	
+	pass2=len-imuldiv((uint32_t)buflen-bp,0x10000,step);
+	
 	if (bit16)
 		if (stereo)
 			if (!stereoout)
@@ -154,7 +156,7 @@ void plrGetMasterSample(int16_t *buf, int len, uint32_t rate, int opt)
 
 int plrOpenPlayer(void **buf, uint32_t *len, uint32_t bufl)
 {
-	int dmalen;
+	unsigned int dmalen;
 	
 	if (!plrPlay)
 		return 0;
