@@ -41,18 +41,29 @@ static int stereosizefac(int type)
 static void sampto8(struct sampleinfo *s)
 {
 	int i, l;
+	void *newptr;
       	s->type&=~mcpSamp16Bit;
 	s->type|=mcpSampRedBits;
 	l=(s->length+SAMPEND)<<sampsizefac(s->type);
 	for (i=0; i<l; i++)
 		((int8_t *)s->ptr)[i]=((int8_t *)s->ptr)[2*i+1];
-       	s->ptr=realloc(s->ptr,(s->length+SAMPEND)<<sampsizefac(s->type));
+       	newptr=realloc(s->ptr,(s->length+SAMPEND)<<sampsizefac(s->type));
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ " (sampto8): warning, realloc() failed\n");
+	} else
+		s->ptr=newptr;
 }
 
 static void samptomono(struct sampleinfo *s)
 {
 	int i;
 	int l = s->length+SAMPEND;
+	void *newptr;
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (samptomono)\n");
+	fprintf(stderr, __FILE__ ": s->ptr=%p\n", s->ptr);
+#endif
 	s->type&=~mcpSampStereo;
 	s->type|=mcpSampRedStereo;
 	if (s->type&mcpSampFloat)
@@ -64,26 +75,52 @@ static void samptomono(struct sampleinfo *s)
 	else
 		for (i=0; i<l; i++)
 			((int8_t *)s->ptr)[i]=(((int8_t *)s->ptr)[2*i]+((int8_t *)s->ptr)[2*i+1])>>1;
-	s->ptr=realloc(s->ptr,l<<sampsizefac(s->type));
+	newptr=realloc(s->ptr,l<<sampsizefac(s->type));
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": realloced buffer=%p\n", newptr);
+#endif
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ " samptomono(): warning, realloc() failed\n"); /* safe to continue when buffer-shrink failes */
+	} else
+		s->ptr=newptr;
 }
 
 
-static void samptofloat(struct sampleinfo *s)
+static int samptofloat(struct sampleinfo *s)
 {
 	int i;
 	int l=s->length<<sampsizefac(s->type&mcpSampStereo);
 	int l2;
+	void *tmpnew;
 	float *newptr;
 	int extra;
-	
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (samptofloat)\n");
+	fprintf(stderr, __FILE__ ": s->ptr=%p\n", s->ptr);
+#endif
 	s->type|=mcpSampFloat;
 	l2=s->length<<sampsizefac(s->type);
-	s->ptr=realloc(s->ptr,l2);
+	tmpnew=realloc(s->ptr,l2);
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": tmpptr=%p\n", tmpnew);
+#endif
+	if (!tmpnew)
+	{
+		fprintf(stderr, __FILE__ " samptofloat(): error, realloc() failed\n");
+		return 0;
+	}
+	s->ptr=tmpnew;
 	if (s->type&mcpSampStereo)
 		extra=SAMPEND*2;
 	else
 		extra=SAMPEND;
 	newptr = malloc(sizeof(float)*(l+extra));
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ " samptofloat(): error, malloc() failed\n");
+		return 0;
+	}
 	if (s->type&mcpSamp16Bit)
 	{
 		for (i=0;i<l;i++)
@@ -96,10 +133,18 @@ static void samptofloat(struct sampleinfo *s)
 		newptr[l+i]=newptr[l-1];
 	free(s->ptr);
 	s->ptr=newptr;
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": realloced buffer=%p\n", newptr);
+#endif
+	return 1;
 }
 
 static void repairloop(struct sampleinfo *s)
 {
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (repairloop)\n");
+#endif
+
 	if (s->type&mcpSampLoop)
 	{
 		if (s->loopend<=s->loopstart)
@@ -137,15 +182,20 @@ static void repairloop(struct sampleinfo *s)
 
 static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 {
-	int newlen=s->length;
-	int replen=s->loopend-s->loopstart;
-	int sreplen=s->sloopend-s->sloopstart;
+	unsigned int newlen=s->length;
+	unsigned int replen=s->loopend-s->loopstart;
+	unsigned int sreplen=s->sloopend-s->sloopstart;
 	int restricted=0; /* boolean */
 	int toforward=0; /* boolean */
 	int stoforward=0; /* boolean */
-	int expandloop=0;
-	int sexpandloop=0;
-	int c;
+	unsigned int expandloop=0;
+	unsigned int sexpandloop=0;
+	unsigned int c;
+	void *newptr;
+
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (expandsmp), nopingpongloops=%d\n", nopingpongloops);
+#endif
 
 	if ((s->type&mcpSampLoop)&&(s->type&mcpSampSLoop)&&(s->loopend>s->sloopstart)&&(s->sloopend>s->loopstart))
 		restricted=1;
@@ -186,15 +236,30 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 	if (newlen<2)
 		newlen=2;
 
-	s->ptr=realloc(s->ptr, (newlen+SAMPEND)<<sampsizefac(s->type));
-	if (!s->ptr)
+	newptr=realloc(s->ptr, (newlen+SAMPEND)<<sampsizefac(s->type));
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": realloced buffer=%p\n", newptr);
+#endif
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ " expandsmp(): error, realloc() failed\n");
 		return 0;
+	}
+	s->ptr=newptr;
 	if (expandloop)
 	{
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": doing expandloop\n");
+		fprintf(stderr, __FILE__ ": s->length=%d\n", s->length);
+		fprintf(stderr, __FILE__ ": s->loopend=%d\n", s->loopend);
+		fprintf(stderr, __FILE__ ": expandloop=%d\n", expandloop);
+		fprintf(stderr, __FILE__ ": replen=%d\n", replen);
+		fprintf(stderr, __FILE__ ": mcpSampBiDi set? %d\n", s->type&mcpSampBiDi);
+#endif
 		if (sampsizefac(s->type)==2)
 		{
 			float *p=(float *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; c>=0; c--)
+			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -207,7 +272,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 		} else if (sampsizefac(s->type)==1)
 		{
 			int16_t *p=(int16_t *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; c>=0; c--)
+			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -219,7 +284,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 				p[c]=p[c-(replen<<1)];
 		} else {
 			int8_t *p=(int8_t *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; c>=0; c--)
+			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -245,6 +310,14 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 
 	if (sexpandloop)
 	{
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": doing sexpandloop\n");
+		fprintf(stderr, __FILE__ ": s->length=%d\n", s->length);
+		fprintf(stderr, __FILE__ ": s->sloopend=%d\n", s->sloopend);
+		fprintf(stderr, __FILE__ ": sexpandloop=%d\n", sexpandloop);
+		fprintf(stderr, __FILE__ ": sreplen=%d\n", sreplen);
+		fprintf(stderr, __FILE__ ": mcpSampBiDi set? %d\n", s->type&mcpSampBiDi);
+#endif
 		if (sampsizefac(s->type)==2)
 		{
 			int l;
@@ -326,11 +399,20 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 static int repairsmp(struct sampleinfo *s)
 {
 	int i;
+	void *newptr;
 
-	s->ptr=realloc(s->ptr, (s->length+SAMPEND)<<sampsizefac(s->type));
-	if (!s->ptr)
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (repairsmp)\n");
+#endif
+
+	newptr=realloc(s->ptr, (s->length+SAMPEND)<<sampsizefac(s->type));
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ ": (repairsmp) error, realloc() failed\n");
 		return 0;
-	
+	}
+	s->ptr=newptr;
+
 	repairloop(s);
 
 	if (sampsizefac(s->type)==2)
@@ -388,6 +470,12 @@ static void dividefrq(struct sampleinfo *s)
 {
 	int i;
 	int l=s->length>>1;
+	void *newptr;
+
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (dividefrq)\n");
+#endif
+
 	if (sampsizefac(s->type)==2)
 		for (i=0; i<l; i++)
 			((float *)s->ptr)[i]=((float *)s->ptr)[2*i];
@@ -405,7 +493,12 @@ static void dividefrq(struct sampleinfo *s)
 	s->samprate>>=1;
 	s->type|=(s->type&mcpSampRedRate2)?mcpSampRedRate4:mcpSampRedRate2;
 
-	s->ptr=realloc(s->ptr, (s->length+SAMPEND)<<sampsizefac(s->type));
+	newptr=realloc(s->ptr, (s->length+SAMPEND)<<sampsizefac(s->type));
+	if (!newptr)
+	{
+		fprintf(stderr, __FILE__ ": (repairsmp) warning, realloc() failed\n");
+	} else
+		s->ptr=newptr;
 }
 
 static int totalsmpsize(struct sampleinfo *samples, int nsamp, int always16bit)
@@ -450,7 +543,7 @@ static int reduce16(struct sampleinfo *samples, int samplenum, uint32_t *redpars
 		int best=0; /* init to 0 removes warning */
 		uint32_t bestdif=0;
 		for (i=0; i<samplenum; i++)
-			if (curdif<=redpars[i])
+			if ((unsigned)curdif<=redpars[i])
 			{
 				if (!fit||(bestdif>redpars[i]))
 				{
@@ -502,7 +595,7 @@ static int reducestereo(struct sampleinfo *samples, int samplenum, uint32_t *red
 		int best=0; /* init to 0 removes warning */
 		uint32_t bestdif=0;
 		for (i=0; i<samplenum; i++)
-			if (curdif<=redpars[i])
+			if ((unsigned)curdif<=redpars[i])
 			{
 				if (!fit||(bestdif>redpars[i]))
 				{
@@ -579,7 +672,13 @@ static int reducefrq(struct sampleinfo *samples, int samplenum, uint32_t *redpar
 
 static int convertsample(struct sampleinfo *s)
 {
-	int i;
+	unsigned int i;
+
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": (convertsample)\n");
+	if (s->loopstart>=s->loopend)
+		fprintf(stderr, __FILE__ ": s->loopstart>=s->loopend, masking away mcpSampLoop flag if set\n");
+#endif	
 
 	if (s->loopstart>=s->loopend)
 		s->type&=~mcpSampLoop;
@@ -590,7 +689,10 @@ static int convertsample(struct sampleinfo *s)
 	if ((s->type&(mcpSampBigEndian|mcpSamp16Bit))==(mcpSamp16Bit))
 #endif
 	{
-		int l=s->length<<sampsizefac(s->type);
+		unsigned int l=s->length<<sampsizefac(s->type);
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": sampledata is 16bit and endian does not match host system, converting\n");
+#endif		
 		for (i=0; i<l; i+=2)
 		{
 			char tmp=((char *)s->ptr)[i];
@@ -606,6 +708,9 @@ static int convertsample(struct sampleinfo *s)
 	
 	if (s->type&mcpSampDelta)
 	{
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": sampledata is stored as deltas, making linear\n");
+#endif
 		if (s->type&mcpSampStereo)
 		{
 			if (s->type&mcpSamp16Bit)
@@ -643,7 +748,10 @@ static int convertsample(struct sampleinfo *s)
 
 	if (s->type&mcpSampUnsigned)
 	{
-		int l=s->length<<stereosizefac(s->type);
+		unsigned int l=s->length<<stereosizefac(s->type);
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": sampledata is stored unsigned, making it signed\n");
+#endif
 		if (s->type&mcpSamp16Bit)
 		{
 			for (i=0; i<l; i++)
@@ -665,13 +773,31 @@ int mcpReduceSamples(struct sampleinfo *si, int n, long mem, int opt)
 	int samplenum=n;
 
 	int i;
+
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": mcpReduceSamples STARTED\n");
+#endif
 	for (i=0; i<samplenum; i++)
 	{
+#ifdef MCP_DEBUG
+		fprintf(stderr, __FILE__ ": [%d]\n", i);
+#endif
 		if (!convertsample(&samples[i]))
+		{
+#ifdef MCP_DEBUG
+			fprintf(stderr, __FILE__ ": mcpReduceSamples FAILED\n");
+#endif
 			return 0;
+		}
 		repairloop(&samples[i]);
 		if (!expandsmp(&samples[i], opt&mcpRedNoPingPong))
+		{
+#ifdef MCP_DEBUG
+			fprintf(stderr, __FILE__ ": mcpReduceSamples FAILED\n");
+#endif
 			return 0;
+		}
+
 	}
 
 	if (opt&mcpRedToMono)
@@ -692,17 +818,37 @@ int mcpReduceSamples(struct sampleinfo *si, int n, long mem, int opt)
 		if ((opt&mcpRedAlways16Bit)||!reduce16(samples, samplenum, redpars, memmax))
 			if (!reducestereo(samples, samplenum, redpars, memmax))
 				if (!reducefrq(samples, samplenum, redpars, memmax))
+				{
+					free(redpars);
+#ifdef MCP_DEBUG
+					fprintf(stderr, __FILE__ ": mcpReduceSamples FAILED\n");
+#endif
 					return 0;
+				}
 		free(redpars);
 	}
 
 	for (i=0; i<samplenum; i++)
 		if (!repairsmp(&samples[i]))
+		{
+#ifdef MCP_DEBUG
+			fprintf(stderr, __FILE__ ": mcpReduceSamples FAILED\n");
+#endif
 			return 0;
+		}
 
 	if (opt&mcpRedToFloat)
 		for (i=0; i<samplenum; i++)
-			samptofloat(&samples[i]);
+			if (!samptofloat(&samples[i]))
+			{
+#ifdef MCP_DEBUG
+				fprintf(stderr, __FILE__ ": mcpReduceSamples FAILED\n");
+#endif
+				return 0;
+			}
+#ifdef MCP_DEBUG
+	fprintf(stderr, __FILE__ ": mcpReduceSamples DONE\n");
+#endif
 
 	return 1;
 }
