@@ -1,7 +1,7 @@
 /** OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'05 Stian Skjelstad <stian@nixia.no>
  *
- * ALSA Player device
+ * ALSA (Advanced Linux Sound Architecture) Player device
  *
  *  revision history: (please note changes here)
  */
@@ -46,8 +46,8 @@ static struct dmDrive *dmSETUP;
 #define MAX_ALSA_MIXER 256
 static char *playbuf;
 static int buflen; /* in bytes */
-volatile static int kernpos, cachepos, bufpos; /* in bytes */
-volatile static int cachelen, kernlen; /* to make life easier */
+static volatile int kernpos, cachepos, bufpos; /* in bytes */
+static volatile int cachelen, kernlen; /* to make life easier */
 static struct ocpvolstruct mixer_entries[MAX_ALSA_MIXER];
 static int alsa_mixers_n=0;
 
@@ -58,18 +58,18 @@ static int alsa_mixers_n=0;
  *  on getplaypos we use last known kernpos if locked, else update kernpos
  */
 
-volatile static uint32_t playpos; /* how many samples have we done totally */
+static volatile uint32_t playpos; /* how many samples have we done totally */
 static int stereo;
 static int bit16;
 
-volatile static int busy=0;
+static volatile int busy=0;
 uint32_t customfileref=0xffffffff;
 uint32_t custommixerref=0xffffffff;
 
 static int mlDrawBox(void)
 {
 	int mlTop=plScrHeight/2-2;
-	int i;
+	unsigned int i;
 
 	displayvoid(mlTop+1, 5, plScrWidth-10);
 	displayvoid(mlTop+2, 5, plScrWidth-10);
@@ -266,7 +266,7 @@ static void flush(void)
 	busy--;
 }
 /* stolen from devposs */
-static void advance(int pos)
+static void advance(unsigned int pos)
 {
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "advance: oldpos=%d newpos=%d add=%d (busy=%d)\n", bufpos, pos, (pos-bufpos+buflen)%buflen, busy);
@@ -336,17 +336,20 @@ static FILE *alsaSelectMixer(struct modlistentry *entry)
 {
 	char *t;
 	int card;
-	if (!strcmp(entry->name, "none.dev"))
+	if (!strcmp(entry->name, "default.dev"))
+	{
+		strcpy(alsaMixerName, "default");
+	} else if (!strcmp(entry->name, "none.dev"))
 	{
 		strcpy(alsaMixerName, "");
 	} else if (!strcmp(entry->name, "custom.dev"))
 	{
 		int mlTop=mlDrawBox();
 		char str[DEVICE_NAME_MAX+1];
-		int curpos;
-		int cmdlen;
+		unsigned int curpos;
+		unsigned int cmdlen;
 		int insmode=1;
-		int scrolled=0;
+		unsigned int scrolled=0;
 
 		strcpy(str, alsaMixerName);
 
@@ -434,7 +437,7 @@ static FILE *alsaSelectMixer(struct modlistentry *entry)
 			}
 			while ((curpos-scrolled)>=(plScrWidth-10))
 				scrolled+=8;
-			while ((curpos-scrolled)<0)
+			while (((signed)curpos-(signed)scrolled)<0)
 				scrolled-=8;
 		}
 	} else {
@@ -471,10 +474,10 @@ static FILE *alsaSelectPcmOut(struct modlistentry *entry)
 	{
 		int mlTop=mlDrawBox();
 		char str[DEVICE_NAME_MAX+1];
-		int curpos;
-		int cmdlen;
+		unsigned int curpos;
+		unsigned int cmdlen;
 		int insmode=1;
-		int scrolled=0;
+		unsigned int scrolled=0;
 
 		strcpy(str, alsaCardName);
 
@@ -562,7 +565,7 @@ static FILE *alsaSelectPcmOut(struct modlistentry *entry)
 			}
 			while ((curpos-scrolled)>=(plScrWidth-10))
 				scrolled+=8;
-			while ((curpos-scrolled)<0)
+			while (((signed)curpos-(signed)scrolled)<0)
 				scrolled-=8;
 		}
 	} else {
@@ -703,7 +706,7 @@ static int list_devices_for_card(int card, struct modlist *ml, const struct dmDr
 			}
 			entry.adb_ref=0xffffffff;
 			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=alsaSelectPcmOut;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 			dirdbUnref(entry.dirdbfullpath);
 		}
 		cards++;
@@ -726,29 +729,27 @@ static int list_cards(struct modlist *ml, const struct dmDrive *drive, uint32_t
 		struct modlistentry entry;
 		memset(&entry, 0, sizeof(entry));
 
-		if (!mixercaponly)
+		snprintf(entry.shortname, sizeof(entry.shortname), "default.dev");
+		strcpy(entry.name, entry.shortname);
+		entry.drive=drive;
+		entry.dirdbfullpath=dirdbFindAndRef(parent, entry.name);
+		entry.flags=MODLIST_FLAG_FILE|MODLIST_FLAG_VIRTUAL;
+		entry.fileref=mdbGetModuleReference(entry.name, 0);
+		if (entry.fileref!=0xffffffff)
 		{
-			snprintf(entry.shortname, sizeof(entry.shortname), "default.dev");
-			strcpy(entry.name, entry.shortname);
-			entry.drive=drive;
-			entry.dirdbfullpath=dirdbFindAndRef(parent, entry.name);
-			entry.flags=MODLIST_FLAG_FILE|MODLIST_FLAG_VIRTUAL;
-			entry.fileref=mdbGetModuleReference(entry.name, 0);
-			if (entry.fileref!=0xffffffff)
-			{
-				struct moduleinfostruct mi;
-				mdbGetModuleInfo(&mi, entry.fileref);
-				mi.flags1&=~MDB_VIRTUAL;
-				mi.channels=2;
-				snprintf(mi.modname, sizeof(mi.modname), "default output");
-				mi.modtype=mtUnRead;
-				mdbWriteModuleInfo(entry.fileref, &mi);
-			}
-			entry.adb_ref=0xffffffff;
-			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=alsaSelectPcmOut;
-			ml->append(ml, &entry);
-			dirdbUnref(entry.dirdbfullpath);
-		} else {
+			struct moduleinfostruct mi;
+			mdbGetModuleInfo(&mi, entry.fileref);
+			mi.flags1&=~MDB_VIRTUAL;
+			mi.channels=2;
+			snprintf(mi.modname, sizeof(mi.modname), "default output");
+			mi.modtype=mtUnRead;
+			mdbWriteModuleInfo(entry.fileref, &mi);
+		}
+		entry.adb_ref=0xffffffff;
+		entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=mixercaponly?alsaSelectMixer:alsaSelectPcmOut;
+		modlist_append(ml, &entry);
+		dirdbUnref(entry.dirdbfullpath);
+		if (mixercaponly) {
 			snprintf(entry.shortname, sizeof(entry.shortname), "none.dev");
 			strcpy(entry.name, entry.shortname);
 			entry.drive=drive;
@@ -767,7 +768,7 @@ static int list_cards(struct modlist *ml, const struct dmDrive *drive, uint32_t
 			}
 			entry.adb_ref=0xffffffff;
 			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=alsaSelectMixer;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 			dirdbUnref(entry.dirdbfullpath);
 
 		}
@@ -792,7 +793,7 @@ static int list_cards(struct modlist *ml, const struct dmDrive *drive, uint32_t
 		}
 		entry.adb_ref=0xffffffff;
 		entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=mixercaponly?alsaSelectMixer:alsaSelectPcmOut;
-		ml->append(ml, &entry);
+		modlist_append(ml, &entry);
 		dirdbUnref(entry.dirdbfullpath);
 
 	}
@@ -848,7 +849,7 @@ static int list_cards(struct modlist *ml, const struct dmDrive *drive, uint32_t
 				}
 				entry.adb_ref=0xffffffff;
 				entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=alsaSelectMixer;
-				ml->append(ml, &entry);
+				modlist_append(ml, &entry);
 				dirdbUnref(entry.dirdbfullpath);
 			}
 		} else
@@ -893,7 +894,7 @@ static void SetOptions(unsigned int rate, int opt)
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "ALSA snd_pcm_hw_params_any(alsa_pcm, hwparams) = ");
 #endif
-	if ((err=snd_pcm_hw_params_any(alsa_pcm, hwparams)))
+	if ((err=snd_pcm_hw_params_any(alsa_pcm, hwparams))<0) /* we need to check for <0 here, due to a bug in the ALSA PulseAudio plugin */
 	{
 #ifdef ALSA_DEBUG
 		fprintf(stderr, "failed: %s\n", snd_strerror(-err));
@@ -1101,7 +1102,7 @@ static char *alsaDebug(void)
 }
 #endif
 
-static int alsaPlay(void **buf, int *len)
+static int alsaPlay(void **buf, unsigned int *len)
 {
 	if (!alsa_pcm)
 		return 0;
@@ -1225,7 +1226,7 @@ static void alsaOpenDevice(void)
 	}
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "ok\n");
-	fprintf(stderr, "ALSA snd_mixer_attach(mixer, device = \"%s\") = ", device);
+	fprintf(stderr, "ALSA snd_mixer_attach(mixer, device = \"%s\") = ", alsaMixerName);
 #endif
 	if ((err=snd_mixer_attach(mixer, alsaMixerName))<0)
 	{
@@ -1377,10 +1378,10 @@ static int alsaInit(const struct deviceinfo *c)
 static int alsaDetect(struct deviceinfo *card)
 {
 	int cards = list_cards(NULL, NULL, DIRDB_NOPARENT, NULL, 0, 0);
-	card->dev=&plrAlsa;
+	card->devtype=&plrAlsa;
 	snprintf(card->path, sizeof(card->path), "%s", cfGetProfileString("devpALSA", "path", "default"));
 	snprintf(alsaCardName, sizeof(alsaCardName), "%s", card->path);
-	snprintf(card->mixer, sizeof(card->mixer), "%s", cfGetProfileString("devpALSA", "mixer", "hw:0"));
+	snprintf(card->mixer, sizeof(card->mixer), "%s", cfGetProfileString("devpALSA", "mixer", "default"));
 	snprintf(alsaMixerName, sizeof(alsaMixerName), "%s", card->mixer);
 /*	card->irq=-1;
 	card->irq2=-1;
@@ -1422,7 +1423,7 @@ static int alsaReadDir(struct modlist *ml, const struct dmDrive *drive, const ui
 		entry.fileref=0xffffffff;
 		entry.adb_ref=0xffffffff;
 		entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=0;
-		ml->append(ml, &entry);
+		modlist_append(ml, &entry);
 	} else {
 		uint32_t dmpcmout,dmmixer;
 		dmpcmout=dirdbFindAndRef(dmalsa, "PCM.OUT");
@@ -1437,7 +1438,7 @@ static int alsaReadDir(struct modlist *ml, const struct dmDrive *drive, const ui
 			entry.fileref=0xffffffff;
 			entry.adb_ref=0xffffffff;
 			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=0;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 
 			strcpy(entry.shortname, "MIXER");
 			strcpy(entry.name, "MIXER");
@@ -1447,7 +1448,7 @@ static int alsaReadDir(struct modlist *ml, const struct dmDrive *drive, const ui
 			entry.fileref=0xffffffff;
 			entry.adb_ref=0xffffffff;
 			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=0;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 
 		} else if (path==dmpcmout)
 			list_cards(ml, drive, path, mask, opt, 0);
@@ -1508,8 +1509,8 @@ static void __attribute__((destructor))fini(void)
 	}
 }
 
-struct sounddevice plrAlsa={SS_PLAYER, "ALSA device driver", alsaDetect, alsaInit, alsaClose};
+struct sounddevice plrAlsa={SS_PLAYER, "ALSA device driver", alsaDetect, alsaInit, alsaClose, NULL};
 struct ocpvolregstruct volalsa={volalsaGetNumVolume, volalsaGetVolume, volalsaSetVolume};
-static struct mdbreaddirregstruct readdirAlsa = {alsaReadDir};
+static struct mdbreaddirregstruct readdirAlsa = {alsaReadDir MDBREADDIRREGSTRUCT_TAIL};
 char *dllinfo="driver plrAlsa; volregs volalsa";
-struct linkinfostruct dllextinfo = {"devpalsa", "OpenCP Player Device: ALSA (c) 2005-06 Stian Skjelstad", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"devpalsa", "OpenCP Player Device: ALSA (c) 2005-06 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
