@@ -1,3 +1,9 @@
+/** OpenCP Module Player
+ * copyright (c) '06-'09 Stian Skjelstad <stian@nixia.no>
+ *
+ * CoreAudio (Darwin/Mac OS/OSX) Player device
+ */
+
 #include "config.h"
 #include <stdio.h>
 #include <unistd.h>
@@ -26,6 +32,37 @@ volatile static int cachelen, kernlen; /* to make life easier */
 
 volatile static uint32_t playpos; /* how many samples have we done totally */
 
+static const char *OSStatus_to_string(OSStatus status)
+{
+	switch (status)
+ 	{
+   		case kAudioHardwareNoError:
+			return "kAudioHardwareNoError";
+		case kAudioHardwareNotRunningError:
+			return "kAudioHardwareNotRunningError";
+		case kAudioHardwareUnspecifiedError:
+			return "kAudioHardwareUnspecifiedError";
+		case kAudioHardwareUnknownPropertyError:
+			return "kAudioHardwareUnknownPropertyError";
+		case kAudioHardwareBadPropertySizeError:
+			return "kAudioHardwareBadPropertySizeError";
+		case kAudioHardwareIllegalOperationError:
+			return "kAudioHardwareIllegalOperationError";
+		case kAudioHardwareBadDeviceError:
+			return "kAudioHardwareBadDeviceError";
+		case kAudioHardwareBadStreamError:
+			return "kAudioHardwareBadStreamError";
+		case kAudioHardwareUnsupportedOperationError:
+			return "kAudioHardwareUnsupportedOperationError";
+		case kAudioDeviceUnsupportedFormatError:
+			return "kAudioDeviceUnsupportedFormatError";
+		case kAudioDevicePermissionsError:
+			return "kAudioDevicePermissionsError";
+		default:
+			return "unknown";
+	}
+}
+
 /*
 static void print_format(const char* str,AudioStreamBasicDescription *f){
     uint32_t flags=(uint32_t) f->mFormatFlags;
@@ -145,11 +182,8 @@ static void CoreAudioIdle(void)
 {
 }
 
-static void CoreAudioAdvanceTo(int pos)
+static void CoreAudioAdvanceTo(unsigned int pos)
 {
-#ifdef ALSA_DEBUG
-	fprintf(stderr, "advance: oldpos=%d newpos=%d add=%d (busy=%d)\n", bufpos, pos, (pos-bufpos+buflen)%buflen, busy);
-#endif
 	pthread_mutex_lock(&mutex);
 
 	cachelen+=(pos-bufpos+buflen)%buflen;
@@ -191,10 +225,10 @@ static void CoreAudioStop(void)
 
 	status=AudioOutputUnitStop(theOutputUnit);
 	if (status)
-		fprintf(stderr, "AudioOutputUnitStop returned %d\n", (int)status);
+		fprintf(stderr, "[CoreAudio] AudioOutputUnitStop returned %d (%s)\nn", (int)status, OSStatus_to_string(status));
 }
 
-static int CoreAudioPlay(void **buf, int *len)
+static int CoreAudioPlay(void **buf, unsigned int *len)
 {
 	OSErr status;
 
@@ -224,7 +258,7 @@ static int CoreAudioPlay(void **buf, int *len)
 	status=AudioOutputUnitStart(theOutputUnit);
 	if (status)
 	{
-  		fprintf(stderr, "AudioOutputUnitStart returned %d\n", (int)status);
+  		fprintf(stderr, "[CoreAudio] AudioOutputUnitStart returned %d (%s)\n", (int)status, OSStatus_to_string(status));
 		free(*buf);
 		*buf = playbuf = 0;
 		plrGetBufPos = 0;
@@ -260,55 +294,61 @@ static void CoreAudioClose(void)
 
 static int CoreAudioDetect(struct deviceinfo *card)
 {
+#if 0
 	static AudioDeviceID gOutputDeviceID; /* TODO */
+#endif
 
 	AudioStreamBasicDescription inDesc;
 	const int channels=2;
 	OSStatus status;
-	UInt32 propertySize;
+/*	UInt32 propertySize; */
 
 	ComponentDescription desc;
 	Component comp;
 
-	UInt32 maxFrames, size;
+	UInt32 /*maxFrames,*/ size;
 
 	AURenderCallbackStruct renderCallback;
 
+#if 0
 	propertySize = sizeof(gOutputDeviceID);
 	status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,
 	                                  &propertySize,
 	                                  &gOutputDeviceID);
+	if (status) {
+		fprintf(stderr, "[CoreAudio] Could not get default output Device (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
+		goto errorout;
+	}
 
+	if (gOutputDeviceID == kAudioDeviceUnknown)
+	{
+		fprintf(stderr, "[CoreAudio] Unable to find a default output device\n");
+		goto errorout;
+	}
+#endif
 	desc.componentType = kAudioUnitType_Output;
-        desc.componentSubType = kAudioUnitSubType_DefaultOutput;
-        desc.componentManufacturer = kAudioUnitManufacturer_Apple;
-        desc.componentFlags = 0;
-        desc.componentFlagsMask = 0;
+	desc.componentSubType = kAudioUnitSubType_DefaultOutput;
+	desc.componentManufacturer = kAudioUnitManufacturer_Apple;
+	desc.componentFlags = 0;
+	desc.componentFlagsMask = 0;
 
 	comp = FindNextComponent (comp, &desc);
 	if ( !comp ) {
-		fprintf(stderr, "Unable to find the Output Unit component\n");
+		fprintf(stderr, "[CoreAudio] Unable to find the Output Unit component\n");
 		goto errorout;
 	}
 
 	status = OpenAComponent (comp, &theOutputUnit);
 	if (status) {
-		fprintf(stderr, "Unable to open Output Unit component (status = %d)\n", (int)status);
+		fprintf(stderr, "[CoreAudio] Unable to open Output Unit component (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
 		goto errorout;
 	}
 
-	status = AudioUnitInitialize (theOutputUnit);
-	if (status) {
-		fprintf(stderr, "Unable to initialize Output Unit component (status = %d)\n", (int)status);
-		goto errorout_component;
-	}
-	/* ao is now created, the above is needed only ONCE */
-
 	size = sizeof(inDesc);
 	status = AudioUnitGetProperty (theOutputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &inDesc, &size);
 	if (status) {
-		fprintf(stderr, "Unable to get the input (default) format (status = %d)\n", (int)status);
-		goto errorout_uninit;
+		fprintf(stderr, "[CoreAudio] Unable to get the input (default) format (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
+		goto errorout_component;
 	}
 
 	/*print_format("default: ", &inDesc);*/
@@ -319,8 +359,9 @@ static int CoreAudioDetect(struct deviceinfo *card)
 	inDesc.mChannelsPerFrame=channels;
 	inDesc.mBitsPerChannel=16;
 	inDesc.mFormatFlags = kAudioFormatFlagIsSignedInteger|kAudioFormatFlagIsPacked; /* float is possible */
-	inDesc.mFormatFlags |= kAudioFormatFlagIsBigEndian; /* TODO, detect */
-
+#ifdef WORDS_BIGENDIAN
+	inDesc.mFormatFlags |= kAudioFormatFlagIsBigEndian;
+#endif
 	inDesc.mFramesPerPacket=1;
 	inDesc.mBytesPerPacket = inDesc.mBytesPerFrame = inDesc.mFramesPerPacket*channels*(inDesc.mBitsPerChannel/8);
 	
@@ -328,41 +369,36 @@ static int CoreAudioDetect(struct deviceinfo *card)
 
 	status = AudioUnitSetProperty (theOutputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &inDesc, sizeof(inDesc));
 	if (status) {
-		fprintf(stderr, "Unable to set the input format (status = %d)\n", (int)status);
-		goto errorout_uninit;
+		fprintf(stderr, "[CoreAudio] Unable to set the input format (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
+		goto errorout_component;
 	}
 
+#if 0
 	size = sizeof(maxFrames);
 	status = AudioUnitGetProperty (theOutputUnit, kAudioDevicePropertyBufferSize, kAudioUnitScope_Input, 0, &maxFrames, &size);
 	if (status) {
-		fprintf(stderr, "AudioUnitGetProperty returned %d when getting kAudioDevicePropertyBufferSize\n", (int)status);
+		fprintf(stderr, "[CoreAudio] AudioUnitGetProperty returned %d (%s) when getting kAudioDevicePropertyBufferSize\n", (int)status, OSStatus_to_string(status));
 		goto errorout_uninit;
 	}
 
-	fprintf(stderr, "maxFrames=%d\n", (int)maxFrames);
+	fprintf(stderr, "[CoreAudio] maxFrames=%d\n", (int)maxFrames);
+#endif
 
 	renderCallback.inputProc = theRenderProc;
 	renderCallback.inputProcRefCon = 0;
 	status = AudioUnitSetProperty(theOutputUnit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input, 0, &renderCallback, sizeof(AURenderCallbackStruct));
         if (status) {
-		fprintf(stderr, "Unable to set the render callback (status = %d)\n", (int)status);
-		goto errorout_uninit;
+		fprintf(stderr, "[CoreAudio] Unable to set the render callback (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
+		goto errorout_component;
 	}
 
-/*	audio_resume(theOutputUnit);
-
-	for (i=0;i<500;i++)
-	{
-		pthread_mutex_lock(&mutex);
-		usleep(10000);
-		pthread_mutex_unlock(&mutex);
-		usleep(90000);
+	status = AudioUnitInitialize (theOutputUnit);
+	if (status) {
+		fprintf(stderr, "[CoreAudio] Unable to initialize Output Unit component (status = %d (%s))\n", (int)status, OSStatus_to_string(status));
+		goto errorout_component;
 	}
+	/* ao is now created, the above is needed only ONCE */
 
-	AudioUnitUninitialize (theOutputUnit);
-	CloseComponent (theOutputUnit);
-	fprintf(stderr, "All ok\n");
-*/
 	card->dev=&plrCoreAudio;
 	card->port=0;
 	card->port2=0;
@@ -374,8 +410,10 @@ static int CoreAudioDetect(struct deviceinfo *card)
 
 	return 1;
 
+#if 0
 errorout_uninit:
 	AudioUnitUninitialize (theOutputUnit);
+#endif
 errorout_component:
 	CloseComponent (theOutputUnit);
 errorout:
