@@ -61,7 +61,7 @@ rewrite:
 				for (i=0;i<j;i++)
 					d[i]=uint16_little(d[i]);
 			}
-			if (write(file, diskcache, cachepos)!=cachepos)
+			if ((unsigned)write(file, diskcache, cachepos)!=cachepos)
 			{
 				if (errno==EAGAIN)
 					goto rewrite;
@@ -84,7 +84,7 @@ static int getpos(void)
 	return (bufpos+bufrate)%buflen;
 }
 
-static void advance(int pos)
+static void advance(unsigned int pos)
 {
 	busy=1;
 
@@ -141,7 +141,7 @@ static void dwSetOptions(uint32_t rate, int opt)
 	plrOpt=opt;
 }
 
-static int dwPlay(void **buf, int *len)
+static int dwPlay(void **buf, unsigned int *len)
 {
 	char fn[15];
 	int i;
@@ -294,7 +294,7 @@ static void dwClose(void)
 
 static int dwDetect(struct deviceinfo *card)
 {
-	card->dev=&plrDiskWriter;
+	card->devtype=&plrDiskWriter;
 	card->port=-1;
 	card->port2=-1;
 /*	card->irq=-1;
@@ -311,7 +311,6 @@ static int dwDetect(struct deviceinfo *card)
 #include "dev/devigen.h"
 #include "boot/psetting.h"
 
-struct sounddevice plrDiskWriter={SS_PLAYER, "Disk Writer", dwDetect, dwInit, dwClose};
-struct devaddstruct plrDiskAdd = {0, 0, 0, 0};
+struct sounddevice plrDiskWriter={SS_PLAYER, "Disk Writer", dwDetect, dwInit, dwClose, 0};
 char *dllinfo = "driver plrDiskWriter;";
-struct linkinfostruct dllextinfo = {"devpdisk", "OpenCP Player Device: Disk Writer (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"devpdisk", "OpenCP Player Device: Disk Writer (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
