@@ -51,8 +51,8 @@ static int fd_mixer=-1;
 static char *playbuf;
 static int buflen;
 
-volatile static int kernpos, cachepos, bufpos;
-volatile static int cachelen, kernlen; /* to make life easier */
+static volatile int kernpos, cachepos, bufpos;
+static volatile int cachelen, kernlen; /* to make life easier */
 
 /*  playbuf     kernpos  cachepos   bufpos      buflen
  *    |           | kernlen | cachelen |          |
@@ -61,11 +61,11 @@ volatile static int cachelen, kernlen; /* to make life easier */
  *  on getplaypos we use last known kernpos if locked, else update kernpos
  */
 
-volatile static uint32_t playpos; /* how many samples have we done totally */
+static volatile uint32_t playpos; /* how many samples have we done totally */
 static int stereo;
 static int bit16;
 
-volatile static int busy=0;
+static volatile int busy=0;
 
 static int mixer_devmask=0;
 static struct ocpvolstruct mixer_entries[SOUND_MIXER_NRDEVICES];
@@ -287,7 +287,7 @@ static int getbufpos(void)
 	return retval;
 }
 
-static void advance(int pos)
+static void advance(unsigned int pos)
 {
 	if (busy++)
 	{
@@ -407,7 +407,7 @@ static char *ossDebug(void)
 }
 #endif
 
-static int ossPlay(void **buf, int *len)
+static int ossPlay(void **buf, unsigned int *len)
 {
 #ifdef OSS_DEBUG
 	int tmp;
@@ -541,7 +541,7 @@ static int ossDetect(struct deviceinfo *card)
 	int tmp;
 	char *temp;
 
-	card->dev=&plrOSS;
+	card->devtype=&plrOSS;
 	card->port=-1;
 	card->port2=-1;
 	card->subtype=-1;
@@ -553,8 +553,8 @@ static int ossDetect(struct deviceinfo *card)
 #ifdef OSS_DEBUG
 		fprintf(stderr, "devposs: $DSP found\n");
 #endif
-		strncpy(card->path, temp, 128);
-		card->path[127]=0;
+		strncpy(card->path, temp, DEVICE_NAME_MAX);
+		card->path[DEVICE_NAME_MAX-1]=0;
 	} else if (!card->path[0])
 	{
 #ifdef OSS_DEBUG
@@ -567,8 +567,8 @@ static int ossDetect(struct deviceinfo *card)
 #ifdef OSS_DEBUG
 		fprintf(stderr, "devposs: $MIXER found\n");
 #endif
-		strncpy(card->mixer, temp, 128);
-		card->mixer[127]=0;
+		strncpy(card->mixer, temp, DEVICE_NAME_MAX);
+		card->mixer[DEVICE_NAME_MAX-1]=0;
 	}
 #ifdef HAVE_SYS_SOUNDCARD_H
 /* this test will fail with liboss */
@@ -741,9 +741,9 @@ static int volossSetVolume(struct ocpvolstruct *v, int n)
 	return 0;
 }
 
-struct sounddevice plrOSS={SS_PLAYER, "OSS player", ossDetect,  ossInit,  ossClose};
-struct devaddstruct plrOSSAdd = {ossGetOpt, 0, 0, 0};
+static struct devaddstruct plrOSSAdd = {ossGetOpt, 0, 0, 0};
+struct sounddevice plrOSS={SS_PLAYER, "OSS player", ossDetect,  ossInit,  ossClose, &plrOSSAdd};
 struct ocpvolregstruct voloss={volossGetNumVolume, volossGetVolume, volossSetVolume};
 
-char *dllinfo = "driver plrOSS; addprocs plrOSSAdd; volregs voloss";
-struct linkinfostruct dllextinfo = {"devposs", "OpenCP Player Device: OSS (c) '04-'05 Stian Skjelstad", DLLVERSION, 0};
+char *dllinfo = "driver plrOSS; volregs voloss";
+struct linkinfostruct dllextinfo = {"devposs", "OpenCP Player Device: OSS (c) '04-'05 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
