@@ -284,7 +284,7 @@ static int ossDetect(struct deviceinfo *card)
 #endif
 	char *temp;
 
-	card->dev=&smpOSS;
+	card->devtype=&smpOSS;
 	card->port=-1;
 	card->port2=-1;
 	card->subtype=-1;
@@ -296,8 +296,8 @@ static int ossDetect(struct deviceinfo *card)
 #ifdef OSS_DEBUG
 		fprintf(stderr, "devsoss: $DSP found\n");
 #endif
-		strncpy(card->path, temp, 128);
-		card->path[127]=0;
+		strncpy(card->path, temp, DEVICE_NAME_MAX);
+		card->path[DEVICE_NAME_MAX-1]=0;
 	} else if (!card->path[0])
 	{
 #ifdef OSS_DEBUG
@@ -310,8 +310,8 @@ static int ossDetect(struct deviceinfo *card)
 #ifdef OSS_DEBUG
 		fprintf(stderr, "devsoss: $MIXER found\n");
 #endif
-		strncpy(card->mixer, temp, 128);
-		card->mixer[127]=0;
+		strncpy(card->mixer, temp, DEVICE_NAME_MAX);
+		card->mixer[DEVICE_NAME_MAX-1]=0;
 	}
 	if (stat(card->path, &st))
 	{
@@ -369,8 +369,8 @@ static uint32_t ossGetOpt(const char *sec)
 	return opt;
 }
  
-struct sounddevice smpOSS={SS_SAMPLER, "OSS Recorder", ossDetect, ossInit, ossClose};
 struct devaddstruct smpOSSAdd = {ossGetOpt, 0, 0, 0};
+struct sounddevice smpOSS={SS_SAMPLER, "OSS Recorder", ossDetect, ossInit, ossClose, &smpOSSAdd};
 
-char *dllinfo="driver smpOSS; addprocs smpOSSAdd;";
-struct linkinfostruct dllextinfo = {"devsoss", "OpenCP Sampler Device: OSS (c) '04-'05 Stian Skjelstad", DLLVERSION, 0};
+char *dllinfo="driver smpOSS";
+struct linkinfostruct dllextinfo = {"devsoss", "OpenCP Sampler Device: OSS (c) '04-'05 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
