@@ -63,7 +63,7 @@ static int samplenum;
 
 static int16_t (*amptab)[256]; /* signedness is not fixed here */
 static unsigned long clipmax;
-volatile static int clipbusy;
+static volatile int clipbusy;
 #define BARRIER __asm__ __volatile__ ("":);
 
 static unsigned long amplify;
@@ -92,7 +92,7 @@ static int16_t (*interpoltabq2)[16][256][4];
 
 static int16_t *scalebuf=0;
 static int32_t *buf32;
-static int bufpos;
+static uint32_t bufpos;
 static uint32_t buflen;
 static void *plrbuf;
 
@@ -431,7 +431,7 @@ static void mixer(void)
 	bufdeltatot=((buflen+(plrGetBufPos()>>(stereo+bit16))-bufpos)%buflen);
 	if (_pause)
 	{
-		int pass2=((bufpos+bufdeltatot)>buflen)?(bufpos+bufdeltatot-buflen):0;
+		uint32_t pass2=((bufpos+bufdeltatot)>buflen)?(bufpos+bufdeltatot-buflen):0;
 
 		if (bit16)
 		{
@@ -452,7 +452,7 @@ static void mixer(void)
 		pausesamps+=bufdeltatot;
 	} else while (bufdeltatot>0)
 	{
-		int bufdelta=(bufdeltatot>MIXBUFLEN)?MIXBUFLEN:bufdeltatot;
+		uint32_t bufdelta=(bufdeltatot>MIXBUFLEN)?MIXBUFLEN:bufdeltatot;
 
 		if (bufdelta>(buflen-bufpos))
 			bufdelta=buflen-bufpos;
@@ -684,7 +684,7 @@ static void SET(int ch, int opt, int val)
 				stopchan(chn);
 				if (val<0)
 					val=0;
-				if (val>=chn->length)
+				if ((unsigned)val>=chn->length)
 				val=chn->length-1;
 				chn->pos=val;
 				chn->fpos=0;
@@ -804,7 +804,7 @@ static void Idle(void)
 
 
 
-static void GetMixChannel(int ch, struct mixchannel *chn, uint32_t rate)
+static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate)
 	/* Refered to by OpenPlayer to mixInit */
 {
 	struct channel *c=&channels[ch];
@@ -1168,7 +1168,7 @@ static void wmixClose(void)
 
 static int wmixDetect(struct deviceinfo *c)
 {
-	c->dev=&mcpMixer;
+	c->devtype=&mcpMixer;
 	c->port=-1;
 	c->port2=-1;
 /*	c->irq=-1;
@@ -1247,7 +1247,8 @@ static int mixProcKey(unsigned short key)
 	return 0;
 }
 
-struct sounddevice mcpMixer={SS_WAVETABLE|SS_NEEDPLAYER, "Mixer", wmixDetect, wmixInit, wmixClose};
 struct devaddstruct mcpMixAdd = {mixGetOpt, mixrInit, 0, mixProcKey};
-char *dllinfo="driver mcpMixer; addprocs mcpMixAdd";
-struct linkinfostruct dllextinfo = {"devwmix", "OpenCP Wavetable Device: Mixer (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct sounddevice mcpMixer={SS_WAVETABLE|SS_NEEDPLAYER, "Mixer", wmixDetect, wmixInit, wmixClose, &mcpMixAdd};
+char *dllinfo="driver mcpMixer";
+
+struct linkinfostruct dllextinfo = {"devwmix", "OpenCP Wavetable Device: Mixer (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
