@@ -79,7 +79,7 @@ static int     channelnum;
 static struct channel *channels;
 static float   fadedown[2];
 
-static int      bufpos;
+static unsigned int bufpos;
 static uint32_t buflen;
 
 static void (*playerproc)(void);
@@ -341,7 +341,7 @@ static void mixmain(/*int min*/)
 		pausesamps+=bufdeltatot;
 	} else while (bufdeltatot>0)
 	{
-		int bufdelta=(bufdeltatot>MIXF_MIXBUFLEN)?MIXF_MIXBUFLEN:bufdeltatot;
+		unsigned int bufdelta=(bufdeltatot>MIXF_MIXBUFLEN)?MIXF_MIXBUFLEN:bufdeltatot;
 		uint32_t ticks2go;
 		float invt2g;
 		if (bufdelta>(buflen-bufpos))
@@ -625,7 +625,7 @@ static void SET(int ch, int opt, int val)
 				chn->newsamp=1;
 				if (val<0)
 					val=0;
-				if (val>=chn->length)
+				if ((unsigned)val>=chn->length)
 					val=chn->length-1;
 				smpposw[ch]=(float *)(chn->samp)+val;
 				smpposf[ch]=0;
@@ -772,7 +772,7 @@ static void Idle(void)
  * bla, toller hack: es funktioniert. (fd)
  */
 
-static void GetMixChannel(int ch, struct mixchannel *chn, uint32_t rate)
+static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate)
 {
 	struct channel *c=&channels[ch];
 
@@ -1021,7 +1021,7 @@ static void Close()
 
 static int Detect(struct deviceinfo *c)
 {
-	c->dev=&mcpFMixer;
+	c->devtype=&mcpFMixer;
 	c->port=-1;
 	c->port2=-1;
 /*	c->irq=-1;
@@ -1104,7 +1104,8 @@ static int mixfProcKey(uint16_t key)
 	return 0;
 }
 
-struct sounddevice mcpFMixer={SS_WAVETABLE|SS_NEEDPLAYER, "FPU Mixer", Detect, Init, Close};
 struct devaddstruct mcpFMixAdd = {mixfGetOpt, mixfInit, 0, mixfProcKey};
-char *dllinfo="driver mcpFMixer; addprocs mcpFMixAdd";
-struct linkinfostruct dllextinfo = {"devwmixf", "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-04 Tammo Hinrichs, Fabian Giesen", DLLVERSION, 0};
+struct sounddevice mcpFMixer={SS_WAVETABLE|SS_NEEDPLAYER, "FPU Mixer", Detect, Init, Close, &mcpFMixAdd};
+char *dllinfo="driver mcpFMixer";
+
+struct linkinfostruct dllextinfo = {"devwmixf", "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-04 Tammo Hinrichs, Fabian Giesen", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
