@@ -139,7 +139,7 @@ static void playchannels(unsigned short len)
 
 static void timerproc(void)
 {
-	int bufdelta=samprate*TIMERRATE/1193046;
+	unsigned int bufdelta=samprate*TIMERRATE/1193046;
 
 	if (channelnum&&!pause)
 	{
@@ -265,7 +265,7 @@ static void SET(int ch, int opt, int val)
 			break;
 		case mcpCPosition:
 			channels[ch].status&=~NONE_PLAYING;
-			if (val>=channels[ch].length)
+			if ((unsigned)val>=channels[ch].length)
 			{
 				if (channels[ch].status&NONE_LOOPED)
 					val=channels[ch].loopstart;
@@ -337,7 +337,7 @@ static int GET(int ch, int opt)
 }
 
 
-static void GetMixChannel(int ch, struct mixchannel *chn, uint32_t rate)
+static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate)
 {
 	struct channel *c=&channels[ch];
 	chn->samp=c->samp;
@@ -451,7 +451,7 @@ static void Close(void)
 
 static int Detect(struct deviceinfo *c)
 {
-	c->dev=&mcpNone;
+	c->devtype=&mcpNone;
 	c->port=-1;
 	c->port2=-1;
 /*	c->irq=-1;
@@ -464,6 +464,7 @@ static int Detect(struct deviceinfo *c)
 	return 1;
 }
 
-struct sounddevice mcpNone={SS_WAVETABLE, "None", Detect, Init, Close};
+struct sounddevice mcpNone={SS_WAVETABLE, "None", Detect, Init, Close, 0};
 char *dllinfo = "driver mcpNone";
-struct linkinfostruct dllextinfo = {"devwnone", "OpenCP Wavetable Device: None (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+
+struct linkinfostruct dllextinfo = {"devwnone", "OpenCP Wavetable Device: None (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
