@@ -221,7 +221,7 @@ void nonepublic_dwmixa1(void)
 		"  movl %eax, (playmonoi16vol1-4)\n"
 		"  movl %ebx, (playmonoi16int0-4)\n"
 		"  incl %ebx\n"
-		"  movl %ebx, (playmonoi16int0-4)\n"
+		"  movl %ebx, (playmonoi16int1-4)\n"
 		"  decl %ebx\n"
 		"  ret\n"
 	);
@@ -391,7 +391,7 @@ void nonepublic_dwmixa1(void)
 		".long   playmonoi,       monoistepl-4,    monoisteph-4,    monoiramp-4,     dummydd,         monoiendp-4,     0,0\n"
 		".long   playmonoi16,     monoi16stepl-4,  monoi16steph-4,  monoi16ramp-4,   dummydd,         monoi16endp-4,   0,0\n"
 		".long   playstereo,      stereostepl-4,   stereosteph-4,   stereoramp0-4,   stereoramp1-4,   stereoendp-4,    0,0\n"
-		".long   playstereo16,    stereo16stepl-4, stereo16steph-4, stereo16ramp0-4, stereo16ramp1-4, stereo16endp,    0,0\n"
+		".long   playstereo16,    stereo16stepl-4, stereo16steph-4, stereo16ramp0-4, stereo16ramp1-4, stereo16endp-4,  0,0\n"
 		".long   playstereoi,     stereoistepl-4,  stereoisteph-4,  stereoiramp0-4,  stereoiramp1-4,  stereoiendp-4,   0,0\n"
 		".long   playstereoi16,   stereoi16stepl-4,stereoi16steph-4,stereoi16ramp0-4,stereoi16ramp1-4,stereoi16endp-4, 0,0\n"
 
@@ -881,7 +881,7 @@ void nonepublic_dwmixa2(void)
 		"  movb (mixrClip8min-2), %dl\n"
 		"  addl 1024(%ebx, %edx, 2), %eax\n"
 		"  movb %ah, (mixrClip8minv-1)\n"
-		"  movb (mixrClip8max-1), %dl\n"
+		"  movb (mixrClip8max-4), %dl\n"
 		"  movl (%ebx, %edx, 2), %eax\n"
 		"  movb (mixrClip8max-3), %dl\n"
 		"  addl 512(%ebx, %edx, 2), %eax\n"
@@ -920,7 +920,7 @@ void nonepublic_dwmixa2(void)
 		"mixrClip8endp1:\n"
 		"  jb mixrClip8lp\n"
 		"mixrClip8done:\n"
-		"  ret\n"
+		"  jmp mixrClip8out\n"
 
 		"mixrClip8low:\n"
 		"    movb $12, (%edi)\n"
@@ -1036,6 +1036,7 @@ void mixrClip(void *dst, int32_t *src, int len, void *tab, int32_t max, int b16)
 		"  jb mixrCliplp\n"
 		/* jmp mixrClipdone\n" */  
 		"mixrClipdone:"
+		"mixrClip8out:"
 #ifdef __PIC__
 		"popl %%ebx\n"
 #endif
@@ -1078,21 +1079,17 @@ void mixrSetupAddresses(int32_t (*vol)[256], uint8_t (*intr)[256][2])
 	mixrFadeChannelintrtab=intr;
 }
 
+#include <stdio.h>
 void mixrFadeChannel(int32_t *fade, struct channel *ch)
 {
-	int32_t vol0=ch->curvols[0]<<8;
-	int32_t vol1=ch->curvols[1]<<8;
-
 	if (ch->status&MIXRQ_PLAY16BIT)
 	{
-		vol0|=(ch->realsamp.bit16[ch->pos]>>8);
-		vol1|=(ch->realsamp.bit16[ch->pos]>>8);
+		fade[0]+=mixrFadeChannelvoltab[ch->curvols[0]][((uint16_t)ch->realsamp.bit16[ch->pos])>>8];
+		fade[1]+=mixrFadeChannelvoltab[ch->curvols[1]][((uint16_t)ch->realsamp.bit16[ch->pos])>>8];
 	} else {
-		vol0|=ch->realsamp.bit8[ch->pos];
-		vol1|=ch->realsamp.bit8[ch->pos];
+		fade[0]+=mixrFadeChannelvoltab[ch->curvols[0]][(uint8_t)ch->realsamp.bit8[ch->pos]];
+		fade[1]+=mixrFadeChannelvoltab[ch->curvols[1]][(uint8_t)ch->realsamp.bit8[ch->pos]];
 	}
-	*fade+=vol0;
-	*fade+=vol1;
 	ch->curvols[0]=0;
 	ch->curvols[1]=0;
 }
@@ -1107,8 +1104,8 @@ static void playmono(int32_t *buf, uint32_t len, struct channel *chan)
 
 	while (len)
 	{
-		*(buf++)+=mixrFadeChannelvoltab[vol0][chan->realsamp.bit8[pos]];
-		fpos+=chan->step&0x0000ffff;
+		*(buf++)+=mixrFadeChannelvoltab[vol0][(uint8_t)(chan->realsamp.bit8[pos])];
+		fpos+=chan->step&0xffff;
 		if (fpos&0xffff0000)
 		{
 			pos++;
@@ -1129,7 +1126,7 @@ static void playmono16(int32_t *buf, uint32_t len, struct channel *chan)
 
 	while (len)
 	{
-		*(buf++)+=mixrFadeChannelvoltab[vol0][chan->realsamp.bit16[pos]>>8];
+		*(buf++)+=mixrFadeChannelvoltab[vol0][((uint16_t)(chan->realsamp.bit16[pos]))>>8];
 		fpos+=chan->step&0x0000ffff;
 		if (fpos&0xffff0000)
 		{
@@ -1152,8 +1149,8 @@ static void playmonoi(int32_t *buf, uint32_t len, struct channel *chan)
 	while (len)
 	{
 		uint8_t cache=
-			(*mixrFadeChannelintrtab)[(uint8_t)(chan->realsamp.bit8[pos])][0]+
-			(*mixrFadeChannelintrtab)[(uint8_t)(chan->realsamp.bit8[pos+1])][1];
+			mixrFadeChannelintrtab[fpos>>12][((uint8_t)(chan->realsamp.bit8[pos]))][0]+
+			mixrFadeChannelintrtab[fpos>>12][((uint8_t)(chan->realsamp.bit8[pos+1]))][1];
 		*(buf++)+=mixrFadeChannelvoltab[vol0][cache];
 		fpos+=chan->step&0x0000ffff;
 		if (fpos&0xffff0000)
@@ -1177,8 +1174,8 @@ static void playmonoi16(int32_t *buf, uint32_t len, struct channel *chan)
 	while (len)
 	{
 		uint8_t cache = 
-			(*mixrFadeChannelintrtab)[((uint16_t)(chan->realsamp.bit16[pos]))>>8][0]+
-			(*mixrFadeChannelintrtab)[((uint16_t)(chan->realsamp.bit16[pos+1]))>>8][1];
+			mixrFadeChannelintrtab[fpos>>12][((uint16_t)(chan->realsamp.bit16[pos]))>>8][0]+
+			mixrFadeChannelintrtab[fpos>>12][((uint16_t)(chan->realsamp.bit16[pos+1]))>>8][1];
 		*(buf++)+=mixrFadeChannelvoltab[vol0][cache];
 		fpos+=chan->step&0x0000ffff;
 		if (fpos&0xffff0000)
@@ -1258,8 +1255,8 @@ static void playstereoi(int32_t *buf, uint32_t len, struct channel *chan)
 	while (len)
 	{
 		uint8_t cache=
-			(*mixrFadeChannelintrtab)[((uint8_t)(chan->realsamp.bit8[pos]))][0]+
-			(*mixrFadeChannelintrtab)[((uint8_t)(chan->realsamp.bit8[pos+1]))][1];
+			mixrFadeChannelintrtab[fpos>>12][((uint8_t)(chan->realsamp.bit8[pos]))][0]+
+			mixrFadeChannelintrtab[fpos>>12][((uint8_t)(chan->realsamp.bit8[pos+1]))][1];
 		*(buf++)+=mixrFadeChannelvoltab[vol0][cache];
 		*(buf++)+=mixrFadeChannelvoltab[vol1][cache];
 		fpos+=chan->step&0x0000ffff;
@@ -1287,8 +1284,8 @@ static void playstereoi16(int32_t *buf, uint32_t len, struct channel *chan)
 	while (len)
 	{
 		uint8_t cache=
-			(*mixrFadeChannelintrtab)[((uint16_t)(chan->realsamp.bit16[pos]))>>8][0]+
-			(*mixrFadeChannelintrtab)[((uint16_t)(chan->realsamp.bit16[pos+1]))>>8][1];
+			mixrFadeChannelintrtab[fpos>>12][((uint16_t)(chan->realsamp.bit16[pos]))>>8][0]+
+			mixrFadeChannelintrtab[fpos>>12][((uint16_t)(chan->realsamp.bit16[pos+1]))>>8][1];
 		*(buf++)+=mixrFadeChannelvoltab[vol0][cache];
 		*(buf++)+=mixrFadeChannelvoltab[vol1][cache];
 		fpos+=chan->step&0x0000ffff;
@@ -1366,14 +1363,14 @@ mixrPlayChannelbigloop:
 			{
 				if (chan->pos > chan->loopstart)
 				{
-					mypos-=chan->loopstart;
+					mypos -= chan->loopstart;
 					inloop = 1;
 				}
 			}
 		} else {
-			mypos = chan->length - chan->pos - (!!chan->fpos);
 			mystep=chan->step;
 			myfpos=-chan->fpos;
+			mypos = chan->length - chan->pos - (!!chan->fpos);
 			if (chan->status&MIXRQ_LOOPED)
 			{
 				if (chan->pos < chan->loopend)
@@ -1386,18 +1383,19 @@ mixrPlayChannelbigloop:
 		mixrPlayChannelmaxplaylen: /* here we analyze how much we can sample */
 		{
 			uint64_t tmppos;
-			tmppos=(((int64_t)(mypos<<16))|myfpos)+((int64_t)mystep)-1;
+			tmppos=(((uint64_t)(mypos<<16))|myfpos)+((uint32_t)mystep)-1;
 			if ((tmppos>>32)<mystep)
 			{/* this is the safe check to avoid overflow in div */
-				uint32_t tmplen;	
+				uint32_t tmplen;
 				tmplen = tmppos / mystep; /* eax */
 				if (mylen>=tmplen)
 				{
 					mylen=tmplen; /* ecx = eax */
 					if (!inloop)
 					{ /* add padding */
+						dofade=1;
 						chan->status&=~MIXRQ_PLAYING;
-						fillen+=(tmplen-mylen); /* the gap that is left */
+						fillen+=(len-mylen); /* the gap that is left */
 						len=mylen;
 					}
 				}
@@ -1470,11 +1468,11 @@ mixrPlayChannelnoplay:
 	{
 		int64_t tmp64;
 		int32_t tmp32;
-		tmp64=((int64_t)chan->step)*((int64_t)mylen);
-		tmp32=tmp64&0xffff;
-		tmp32+=chan->fpos;
-		chan->fpos=(uint16_t)tmp32;
-		chan->pos+=(tmp64>>16)+(tmp32>0xffff);
+		tmp64=((int64_t)chan->step)*mylen;
+		tmp32=(tmp64&0xffff)+(uint16_t)chan->fpos;
+
+		chan->fpos=tmp32&0xffff;
+		chan->pos+=(tmp64>>16) + (tmp32>0xffff);
 	}
 	chan->curvols[0]+=mylen*ramping[0];
 	chan->curvols[1]+=mylen*ramping[1];
@@ -1483,11 +1481,11 @@ mixrPlayChannelnoplay:
 
 	if (inloop)
 	{
-		uint32_t mypos = chan->pos; /* eax */
+		int32_t mypos = chan->pos; /* eax */
 		if (chan->step<0)
 		{
-			if (mypos>=chan->loopstart)
-				return; /* ?? */
+		 	if (mypos>=(int32_t)chan->loopstart)
+				return;
 			if (!(chan->status&MIXRQ_PINGPONGLOOP))
 			{
 				mypos+=chan->replen;
@@ -1496,14 +1494,12 @@ mixrPlayChannelpong:
 				chan->step=-chan->step;
 				chan->fpos=-chan->fpos;
 				mypos+=!!(chan->fpos);
-				mypos-=mypos;
-				mypos+=chan->loopstart;
-				mypos+=chan->loopstart; /* ?? */
+				mypos=-mypos+chan->loopstart+chan->loopstart;
 			}
 		} else {
 mixrPlayChannelforward2:
 			if (mypos<chan->loopend)
-				return; /* ?? */
+				return;
 			if (!(chan->status&MIXRQ_PINGPONGLOOP))
 			{
 				mypos-=chan->replen;
@@ -1512,9 +1508,7 @@ mixrPlayChannelping:
 				chan->step=-chan->step;
 				chan->fpos=-chan->fpos;
 				mypos+=!!(chan->fpos);
-				mypos-=mypos;
-				mypos+=chan->loopend;
-				mypos+=chan->loopend; /* ?? */
+				mypos=-mypos+chan->loopend+chan->loopend;
 			}
 		}
 mixrPlayChannelloopiflen:
@@ -1560,7 +1554,7 @@ mixrPlayChannelfadechk:
 			return;
 	}
 mixrPlayChannelfade:
-		mixrFadeChannel(fadebuf, chan);
+	mixrFadeChannel(fadebuf, chan);
 #if 0
  #error We need assembler replace here
 	void *routptr;
@@ -2248,19 +2242,18 @@ void mixrClip(void *dst, int32_t *src, int len, void *tab, int32_t max, int b16)
 {
 	if (!b16)
 	{
-		int8_t *_dst=dst;
-		int32_t *_src=(int32_t *)src;
-		const int16_t (*amptab)[256] = tab;
-		const int16_t *mixrClipamp1 = amptab[0];
-		const int16_t *mixrClipamp2 = amptab[1];
-		const int16_t *mixrClipamp3 = amptab[2];
+		uint8_t *_dst=dst;
+		const uint16_t (*amptab)[256] = tab;
+		const uint16_t *mixrClipamp1 = amptab[0];
+		const uint16_t *mixrClipamp2 = amptab[1];
+		const uint16_t *mixrClipamp3 = amptab[2];
 		const int32_t mixrClipmax=max;
 		const int32_t mixrClipmin=-max;
-		const int8_t mixrClipminv =
+		const uint8_t mixrClipminv =
 			(mixrClipamp1[mixrClipmin&0xff]+
 			mixrClipamp2[(mixrClipmin&0xff00)>>8]+
 			mixrClipamp3[(mixrClipmin&0xff0000)>>16])>>8;
-		const int8_t mixrClipmaxv =
+		const uint8_t mixrClipmaxv =
 			(mixrClipamp1[mixrClipmax&0xff]+
 			mixrClipamp2[(mixrClipmax&0xff00)>>8]+
 			mixrClipamp3[(mixrClipmax&0xff0000)>>16])>>8;
@@ -2274,30 +2267,31 @@ void mixrClip(void *dst, int32_t *src, int len, void *tab, int32_t max, int b16)
 				*_dst=mixrClipmaxv;
 			} else {
 				*_dst=
-					(mixrClipamp1[*_src&0xff]+
-					mixrClipamp2[(*_src&0xff00)>>8]+
-					mixrClipamp3[(*_src&0xff0000)>>16])>>8;
+					(mixrClipamp1[*src&0xff]+
+					mixrClipamp2[(*src&0xff00)>>8]+
+					mixrClipamp3[(*src&0xff0000)>>16])>>8;
 			}
-			_src++;
+			src++;
 			_dst++;
+			len--;
 		}
 	} else {
-		int16_t *_dst=dst;
-		int32_t *_src=(int32_t *)src;
-		const int16_t (*amptab)[256] = (void *)tab;
-		const int16_t *mixrClipamp1 = amptab[0];
-		const int16_t *mixrClipamp2 = amptab[1];
-		const int16_t *mixrClipamp3 = amptab[2];
+		uint16_t *_dst=dst;
+		const uint16_t (*amptab)[256] = (void *)tab;
+		const uint16_t *mixrClipamp1 = amptab[0];
+		const uint16_t *mixrClipamp2 = amptab[1];
+		const uint16_t *mixrClipamp3 = amptab[2];
 		const int32_t mixrClipmax=max;
 		const int32_t mixrClipmin=-max;
-		const int16_t mixrClipminv =
-			mixrClipamp1[mixrClipmin&0xff]+
+		const uint16_t mixrClipminv =
+			mixrClipamp1[(mixrClipmin&0xff)]+
 			mixrClipamp2[(mixrClipmin&0xff00)>>8]+
 			mixrClipamp3[(mixrClipmin&0xff0000)>>16];
-		const int16_t mixrClipmaxv =
-			mixrClipamp1[mixrClipmax&0xff]+
+		const uint16_t mixrClipmaxv =
+			mixrClipamp1[(mixrClipmax&0xff)]+
 			mixrClipamp2[(mixrClipmax&0xff00)>>8]+
 			mixrClipamp3[(mixrClipmax&0xff0000)>>16];
+
 		while (len)
 		{
 			if (*src<mixrClipmin)
@@ -2308,11 +2302,11 @@ void mixrClip(void *dst, int32_t *src, int len, void *tab, int32_t max, int b16)
 				*_dst=mixrClipmaxv;
 			} else {
 				*_dst=
-					mixrClipamp1[*_src&0xff]+
-					mixrClipamp2[(*_src&0xff00)>>8]+
-					mixrClipamp3[(*_src&0xff0000)>>16];
+					mixrClipamp1[*src&0xff]+
+					mixrClipamp2[(*src&0xff00)>>8]+
+					mixrClipamp3[(*src&0xff0000)>>16];
 			}
-			_src++;
+			src++;
 			_dst++;
 			len--;
 		}
