@@ -14,7 +14,6 @@
  */
 
 #include "config.h"
-#include <assert.h>
 #include <stdio.h>
 #include "types.h"
 #include "dwmix.h"
@@ -790,7 +789,7 @@ static void playmono(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step/0x10000;
+	int32_t  posadd=chan->step>>16;
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -810,7 +809,7 @@ static void playmono16(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step/0x10000;
+	int32_t  posadd=chan->step>>16;
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -830,7 +829,7 @@ static void playmonoi(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step/0x10000;
+	int32_t  posadd=chan->step>>16;
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -853,7 +852,7 @@ static void playmonoi16(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step/0x10000;
+	int32_t  posadd=chan->step>>16;
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -909,7 +908,7 @@ static void playmonoi216(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step/0x10000;
+	int32_t  posadd=chan->step>>16;
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -982,6 +981,7 @@ void mixqPlayChannel(int16_t *buf, uint32_t len, struct channel *chan, int quiet
 mixqPlayChannel_bigloop:
 	inloop=0;
 	mylen=len;
+
 	if (chan->step)
 	{
 		uint32_t mystep; /* ebx */
@@ -997,8 +997,8 @@ mixqPlayChannel_bigloop:
 			{
 				if (chan->pos>=chan->loopstart)
 				{
-					mypos-=chan->loopstart;
-					inloop=1;
+					mypos -= chan->loopstart;
+					inloop = 1;
 				}
 			}
 		} else {
@@ -1011,7 +1011,7 @@ mixqPlayChannel_bigloop:
 				if (chan->pos<chan->loopend)
 				{
 					mypos -=  chan->length - chan->loopend;
-					inloop=1;
+					inloop = 1;
 				}
 			}
 		}
@@ -1019,7 +1019,7 @@ mixqPlayChannel_bigloop:
 	mixqPlayChannel_maxplaylen:
 		{
 			uint64_t tmppos;/* signed to avoid some warnings */
-			tmppos=(((uint64_t)(mypos<<16))|myfpos)/*+((uint64_t)mystep)-1*/;
+			tmppos=(((uint64_t)(mypos<<16))|myfpos)+((uint32_t)mystep)-1;
 			if ((tmppos>>32)<mystep)
 			{
 				uint32_t tmplen;
@@ -1042,22 +1042,21 @@ mixqPlayChannel_playecx:
 	buf+=mylen;
 	len-=mylen;
 	{
-		int64_t tmp;
-		int32_t tmpf;
-		tmp=((int64_t)chan->step)*((int64_t)(mylen));
-		tmpf=(tmp&0xffff)+chan->fpos;
-		chan->fpos=tmpf&0xffff;
-		chan->pos+=(tmp>>16)+(tmpf>0xffff);
-		assert(chan->pos<chan->length);
+		int64_t tmp64;
+		int32_t tmp32;
+		tmp64=((int64_t)chan->step)*mylen;
+		tmp32=(tmp64&0xffff)+(uint16_t)chan->fpos;
+
+		chan->fpos=tmp32&0xffff;
+		chan->pos+=(tmp64>>16)+(tmp32>0xffff);
 	}
 
 	if (inloop)
 	{
 		uint32_t mypos = chan->pos; /* eax */
-	
 		if (chan->step<0)
 		{
-			if (mypos>=chan->loopstart)
+			if (mypos>=(int32_t)chan->loopstart)
 				return;
 			if (!(chan->status&MIXQ_PINGPONGLOOP))
 			{
@@ -1090,15 +1089,11 @@ mixqPlayChannel_loopiflen:
 			goto mixqPlayChannel_bigloop;
 	}
 mixqPlayChannel_fill:
-	if (!fillen)
-		return;
-	{
-		uint32_t tmplen=chan->length;
-		chan->pos=tmplen;
-	}
+	if (fillen)
 	{
 		int16_t sample;
 		int count;
+		chan->pos=chan->length;
 		if (!(chan->status&MIXQ_PLAY16BIT))
 		{
 			sample=chan->realsamp.bit8[chan->pos]<<8;
