@@ -85,7 +85,7 @@ char adbInit(void)
 	struct adbheader header;
 
 	int old=0;
-	int i;
+	unsigned int i;
 
       	adbDirty=0;
 	adbData=0;
@@ -157,7 +157,7 @@ char adbInit(void)
 			adbData[i].size=uint32_little(oldentry.size);
 		}
 	} else {
-		if (read(f, adbData, adbNum*sizeof(*adbData))!=(adbNum*sizeof(*adbData)))
+		if (read(f, adbData, adbNum*sizeof(*adbData))!=(ssize_t)(adbNum*sizeof(*adbData)))
 		{
 			fprintf(stderr, "EOF\n");
 			/* File is broken / premature EOF */
@@ -203,7 +203,25 @@ void adbUpdate(void)
 	lseek(f, 0, SEEK_SET);
 	memcpy(header.sig, adbsigv2, 16);
 	header.entries=uint32_little(adbNum);
-	write(f, &header, sizeof(header));
+	while (1)
+	{
+		ssize_t res;
+		res = write(f, &header, sizeof(header));
+		if (res < 0)
+		{
+			if (errno==EAGAIN)
+				continue;
+			if (errno==EINTR)
+				continue;
+			fprintf(stderr, __FILE__ " write() to %s failed: %s\n", path, strerror(errno));
+			exit(1);
+		} else if (res != sizeof(header))
+		{
+			fprintf(stderr, __FILE__ " write() to %s returned only partial data\n", path);
+			exit(1);
+		} else
+			break;
+	}
 
 	i=0;
 
@@ -223,7 +241,25 @@ void adbUpdate(void)
 		lseek(f, 20+i*sizeof(*adbData), SEEK_SET);
 		adbData[i].parent=uint32_little(adbData[i].parent);
 		adbData[i].size=uint32_little(adbData[i].size);
-		write(f, adbData+i, (j-i)*sizeof(*adbData));
+		while (1)
+		{
+			ssize_t res;
+			res = write(f, adbData+i, (j-i)*sizeof(*adbData));
+			if (res < 0)
+			{
+				if (errno==EAGAIN)
+					continue;
+				if (errno==EINTR)
+					continue;
+				fprintf(stderr, __FILE__ " write() to %s failed: %s\n", path, strerror(errno));
+				exit(1);
+			} else if (res != (ssize_t)((j-i)*sizeof(*adbData)))
+			{
+				fprintf(stderr, __FILE__ " write() to %s returned only partial data\n", path);
+				exit(1);
+			} else
+				break;
+		}
 		adbData[i].parent=uint32_little(adbData[i].parent);
 		adbData[i].size=uint32_little(adbData[i].size);
 		i=j;
@@ -715,7 +751,7 @@ static int arcReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 			m.modinfo.gen.modtype=mtUnRead; /* TODO mdb etc. */
 #endif
 
-			ml->append(ml, &m);
+			modlist_append(ml, &m);
 			dirdbUnref(m.dirdbfullpath);
 		}
 		if (done==-1)
@@ -724,4 +760,4 @@ static int arcReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	return 1;
 }
 
-struct mdbreaddirregstruct adbReadDirReg = {arcReadDir};
+struct mdbreaddirregstruct adbReadDirReg = {arcReadDir MDBREADDIRREGSTRUCT_TAIL};
