@@ -53,33 +53,29 @@ static int method_supported(int method)
 	return (method==0)||(method==8)||(method==9);
 }
 
-static int decode_partial(char *dest, int _destlen, uint8_t *src, int _srclen, int method)
+static int decode_partial(char *dest, size_t destlen, uint8_t *src, size_t srclen, int method)
 {
 	switch (method)
 	{
 		case 0:
 			{
-				_destlen=_srclen;
-				if (_destlen>sizeof(mdbScanBuf))
-					_destlen=sizeof(mdbScanBuf);
-				memcpy(mdbScanBuf, adbScanBuf, _destlen);
+				if (destlen > srclen )
+					destlen = srclen;
+				memcpy(dest, src, destlen);
 				break;
 			}
 
 		case 8:
 		case 9:
 			{
-				uLong srclen=_srclen;
-				uLongf destlen=_destlen;
-
 				z_stream strm;
 				int res;
 
 				memset(&strm, 0, sizeof(strm));
-				strm.next_in=adbScanBuf;
-				strm.avail_in=srclen;
-				strm.next_out=(Bytef *)mdbScanBuf;
-				strm.avail_out=sizeof(mdbScanBuf);
+				strm.next_in=src;
+				strm.avail_in=(uLong)srclen;
+				strm.next_out=(Bytef *)dest;
+				strm.avail_out=(uLongf)destlen;
 							
 				if ((res=inflateInit2(&strm, -15)))
 				{
@@ -90,11 +86,10 @@ static int decode_partial(char *dest, int _destlen, uint8_t *src, int _srclen, i
 					destlen=strm.total_out;
 					inflateEnd(&strm);
 				}
-				_destlen=destlen;
 				break;
 			}
 	}
-	return _destlen;
+	return destlen;
 }
 
 static int decode_to_fd(char *src, ssize_t srclen, int dest, int method)
@@ -473,7 +468,7 @@ static int adbZIPCall(const int act, const char *apath, const char *fullname, co
 }
 
 
-static struct adbregstruct adbZIPReg = {".ZIP", adbZIPScan, adbZIPCall};
+static struct adbregstruct adbZIPReg = {".ZIP", adbZIPScan, adbZIPCall ADBREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
@@ -485,6 +480,8 @@ static void __attribute__((destructor))done(void)
 	adbUnregister(&adbZIPReg);
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
+#endif
 
-struct linkinfostruct dllextinfo = {"arczip", "OpenCP Archive Reader: .ZIP (c) 1994-06 Niklas Beisert", DLLVERSION, 0};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"arczip", "OpenCP Archive Reader: .ZIP (c) 1994-06 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
