@@ -241,7 +241,7 @@ static int cdReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 			entry.fileref=0xffffffff;
 			entry.adb_ref=0xffffffff;
 			entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=0;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 			dirdbUnref(entry.dirdbfullpath); /* overkill */
 			current=current->next;
 		}
@@ -314,7 +314,7 @@ static int cdReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 								}
 								entry.adb_ref=(int)current; /* nasty hack, but hey.. it is free of use */
 								entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=cdrom_ReadHandle;
-								ml->append(ml, &entry);
+								modlist_append(ml, &entry);
 								dirdbUnref(entry.dirdbfullpath);
 							}
 						}
@@ -345,7 +345,7 @@ static int cdReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 						}
 						entry.adb_ref=(int)current; /* nasty hack, but hey.. it is free of use */
 						entry.Read=0; entry.ReadHeader=0; entry.ReadHandle=cdrom_ReadHandle;
-						ml->append(ml, &entry);
+						modlist_append(ml, &entry);
 						dirdbUnref(entry.dirdbfullpath);
 					}
 				}
@@ -357,6 +357,9 @@ static int cdReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 	return 1;
 }
 
-static struct mdbreaddirregstruct cdReadDirReg = {cdReadDir};
+static struct mdbreaddirregstruct cdReadDirReg = {cdReadDir MDBREADDIRREGSTRUCT_TAIL};
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"cdrom", "OpenCP UNIX audio-cdrom filebrowser (c) '04-'05 Stian Skjelstad", DLLVERSION, 0, Init: cdint, Close: cdclose};
+#endif
+
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"cdrom", "OpenCP UNIX audio-cdrom filebrowser (c) '04-'05 Stian Skjelstad", DLLVERSION, 0, Init: cdint, Close: cdclose, PreInit:0, LateInit:0, PreClose:0, LateClose:0};
