@@ -1,3 +1,9 @@
+/* OpenCP Module Player
+ * copyright (c) '94-'09 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ *
+ * database for storing a tree of filenames
+ */ 
+
 #include "config.h"
 #include <fcntl.h>
 #include <stdio.h>
@@ -268,10 +274,11 @@ void dirdbRef(uint32_t node)
 	dirdbData[node].refcount++;
 }
 
-uint32_t dirdbResolvePathWithBaseAndRef(uint32_t base, char *name /* PATH_MAX + 1 */)
+uint32_t dirdbResolvePathWithBaseAndRef(uint32_t base, const char *name /* PATH_MAX + 1 */)
 {
 	char segment[PATH_MAX+1];
-	char *next, *split;
+	const char *next;
+	char *split;
 	uint32_t retval=base, newretval;
 
 #ifdef DIRDB_DEBUG
@@ -313,10 +320,11 @@ uint32_t dirdbResolvePathWithBaseAndRef(uint32_t base, char *name /* PATH_MAX +
 	return retval;
 }
 
-extern uint32_t dirdbResolvePathAndRef(char *name /*PATH_MAX + 1 */)
+extern uint32_t dirdbResolvePathAndRef(const char *name /*PATH_MAX + 1 */)
 {
 	char segment[PATH_MAX+1];
-	char *next, *split;
+	const char *next;
+	char *split;
 	uint32_t retval=DIRDB_NOPARENT, newretval;
 
 #ifdef DIRDB_DEBUG
@@ -407,7 +415,7 @@ void dirdbGetname(uint32_t node, char *name /*NAME_MAX+1*/)
 	strcpy(name, dirdbData[node].name);
 }
 
-static void dirdbGetFullnameR(uint32_t node, char *name, int *left, int nobase)
+static void dirdbGetFullnameR(uint32_t node, char *name, unsigned int *left, int nobase)
 {
 	if (dirdbData[node].parent!=DIRDB_NOPARENT)
 	{
@@ -432,7 +440,7 @@ errorout:
 
 void dirdbGetFullName(uint32_t node, char *name /* PATH_MAX+1, ends not with a / */, int flags)
 {
-	int i = PATH_MAX;
+	unsigned int i = PATH_MAX;
 	*name=0;
 	if (node>=dirdbNum)
 	{
