@@ -15,8 +15,8 @@ extern void dirdbGetname(uint32_t node, char *name /*NAME_MAX+1*/);
 extern void dirdbUnref(uint32_t node);
 extern void dirdbRef(uint32_t node);
 extern uint32_t dirdbFindAndRef(uint32_t parent, const char *name /* NAME_MAX + 1 */);
-extern uint32_t dirdbResolvePathWithBaseAndRef(uint32_t base, char *name /* PATH_MAX + 1 */);
-extern uint32_t dirdbResolvePathAndRef(char *name /*PATH_MAX + 1 */);
+extern uint32_t dirdbResolvePathWithBaseAndRef(uint32_t base, const char *name /* PATH_MAX + 1 */);
+extern uint32_t dirdbResolvePathAndRef(const char *name /*PATH_MAX + 1 */);
 extern int dirdbInit(void);
 extern void dirdbClose(void);
 extern uint32_t dirdbGetParentAndRef(uint32_t node);
