@@ -22,7 +22,7 @@ static int fsReadMemInfo(struct moduleinfostruct *m, const char *buf, size_t len
   /* check for PLS play list */
 
 	char *b=(char *)buf;
-	int pos=10;
+	unsigned int pos=10;
 	int num=0;
 
 	if (!memcmp(buf,"[playlist]", 10))
@@ -100,4 +100,4 @@ static int fsReadInfo(struct moduleinfostruct *m, FILE *fp, const char *buf, siz
 	return 0;
 }
 
-struct mdbreadinforegstruct fsReadInfoReg = {fsReadMemInfo, fsReadInfo};
+struct mdbreadinforegstruct fsReadInfoReg = {fsReadMemInfo, fsReadInfo, 0 MDBREADINFOREGSTRUCT_TAIL};
