@@ -13,11 +13,22 @@
 #include "boot/plinkman.h"
 #include "mdb.h"
 
-extern struct mdbreadinforegstruct ampegpReadInfoReg, itpReadInfoReg, oggReadInfoReg, gmdReadInfoReg, xmpReadInfoReg, gmiReadInfoReg, wavReadInfoReg;
+extern struct mdbreadinforegstruct
+#ifdef HAVE_MAD
+	ampegpReadInfoReg,
+#endif
+	itpReadInfoReg,
+	oggReadInfoReg,
+	gmdReadInfoReg,
+	xmpReadInfoReg,
+	gmiReadInfoReg,
+	wavReadInfoReg;
 
 static void __attribute__((constructor))init(void)
 {
+#ifdef HAVE_MAD
 	mdbRegisterReadInfo(&ampegpReadInfoReg);
+#endif
 	mdbRegisterReadInfo(&itpReadInfoReg);
 	mdbRegisterReadInfo(&oggReadInfoReg);
 	mdbRegisterReadInfo(&gmdReadInfoReg);
@@ -28,7 +39,9 @@ static void __attribute__((constructor))init(void)
 
 static void __attribute__((destructor))done(void)
 {
+#ifdef HAVE_MAD
 	mdbUnregisterReadInfo(&ampegpReadInfoReg);
+#endif
 	mdbUnregisterReadInfo(&itpReadInfoReg);
 	mdbUnregisterReadInfo(&oggReadInfoReg);
 	mdbUnregisterReadInfo(&gmdReadInfoReg);
@@ -37,5 +50,8 @@ static void __attribute__((destructor))done(void)
 	mdbUnregisterReadInfo(&wavReadInfoReg);
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"fstypes", "OpenCP Module Detection (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+#endif
+
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"fstypes", "OpenCP Module Detection (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
