@@ -9,6 +9,7 @@
  */
 
 #include "config.h"
+#include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -375,7 +376,7 @@ int mdbInit(void)
 	mdbData=malloc(sizeof(struct modinfoentry)*mdbNum);
 	if (!mdbData)
 		return 0;
-	if (read(f, mdbData, mdbNum*sizeof(*mdbData))!=(mdbNum*sizeof(*mdbData)))
+	if (read(f, mdbData, mdbNum*sizeof(*mdbData))!=(signed)(mdbNum*sizeof(*mdbData)))
 	{
 		mdbNum=0;
 		free(mdbData);
@@ -434,8 +435,25 @@ void mdbUpdate(void)
 	lseek(f, 0, SEEK_SET);
 	memcpy(header.sig, mdbsigv1, sizeof(mdbsigv1));
 	header.entries = uint32_little(mdbNum);
-	write(f, &header, sizeof(header));
-	
+	while (1)
+	{
+		ssize_t res;
+		res = write(f, &header, sizeof(header));
+		if (res < 0)
+		{
+			if (errno==EAGAIN)
+				continue;
+			if (errno==EINTR)
+				continue;
+			fprintf(stderr, __FILE__ " write() to %s failed: %s\n", path, strerror(errno));
+			exit(1);
+		} else if (res != sizeof(header))
+		{
+			fprintf(stderr, __FILE__ " write() to %s returned only partial data\n", path);
+			exit(1);
+		} else
+			break;
+	}
 	i=0;
 	while (i<mdbNum)
 	{
@@ -450,7 +468,25 @@ void mdbUpdate(void)
 			else
 				break;
 		lseek(f, 64+i*sizeof(*mdbData), SEEK_SET);
-		write(f, mdbData+i, (j-i)*sizeof(*mdbData));
+		while (1)
+		{
+			ssize_t res;
+			res = write(f, mdbData+i, (j-i)*sizeof(*mdbData));
+			if (res < 0)
+			{
+				if (errno==EAGAIN)
+					continue;
+				if (errno==EINTR)
+					continue;
+				fprintf(stderr, __FILE__ " write() to %s failed: %s\n", path, strerror(errno));
+				exit(1);
+			} else if (res != (signed)((j-i)*sizeof(*mdbData)))
+			{
+				fprintf(stderr, __FILE__ " write() to %s returned only partial data\n", path);
+				exit(1);
+			} else
+				break;
+		}
 		i=j;
 	}
 	lseek(f, 0, SEEK_END);
