@@ -27,9 +27,9 @@
 #include "modlist.h"
 #include "stuff/compat.h"
 
-static void modlist_free(struct modlist *modlist)
+void modlist_free(struct modlist *modlist)
 {
-	int i;
+	unsigned int i;
 	for (i=0;i<modlist->num;i++)
 	{
 		dirdbUnref(modlist->files[i]->dirdbfullpath);
@@ -39,23 +39,26 @@ static void modlist_free(struct modlist *modlist)
 		free(modlist->files);
 	free(modlist);
 }
-static struct modlistentry *modlist_getcur(const struct modlist *modlist) /* does not Ref() */
+
+struct modlistentry *modlist_getcur(const struct modlist *modlist) /* does not Ref() */
 {
-	return modlist->get(modlist, modlist->pos);
+	return modlist_get(modlist, modlist->pos);
 }
 
-static struct modlistentry *modlist_get(const struct modlist *modlist, int index) /* does not Ref() */
+struct modlistentry *modlist_get(const struct modlist *modlist, unsigned int index) /* does not Ref() */
 {
 	if (!(modlist->num))
 		return NULL; /* should never happen */
+	/*
 	if (index<0)
 		return modlist->files[0];
+	*/
 	if (index>=modlist->num)
 		return modlist->files[modlist->num-1];
 	return modlist->files[index];
 }
 
-static void modlist_append(struct modlist *modlist, struct modlistentry *entry)
+void modlist_append(struct modlist *modlist, struct modlistentry *entry)
 {
 	if (!entry)
 		return;
@@ -74,9 +77,21 @@ static void modlist_append(struct modlist *modlist, struct modlistentry *entry)
 	modlist->num++;
 }
 
-static void modlist_remove(struct modlist *modlist, int index, int count)
+void modlist_remove_all_by_path(struct modlist *modlist, uint32_t ref)
 {
-	int i;
+	unsigned int i;
+	for (i=0;i<modlist->num;)
+	{
+		if (modlist->files[i]->dirdbfullpath == ref)
+			modlist_remove(modlist, i, 1);
+		else
+			i++;
+	}
+}
+
+void modlist_remove(struct modlist *modlist, unsigned int index, unsigned int count)
+{
+	unsigned int i;
 	if (index>=modlist->num)
 		return;
 	if (index+count>modlist->num)
@@ -93,17 +108,33 @@ static void modlist_remove(struct modlist *modlist, int index, int count)
 		modlist->max-=50;
 		modlist->files=realloc(modlist->files, modlist->max*sizeof(modlist->files[0]));
 	}
+	if (!modlist->num)
+		modlist->pos = 0;
+	else if (modlist->pos >= modlist->num)
+		modlist->pos = modlist->num-1;
 }
 
-static int modlist_find(struct modlist *modlist, const uint32_t path)
+int modlist_find(struct modlist *modlist, const uint32_t path)
 {
-	int retval;
+	unsigned int retval;
 	for (retval=0;retval<modlist->num;retval++)
 		if (path==modlist->files[retval]->dirdbfullpath)
 			return retval;
 	return -1;
 }
 
+void modlist_swap(struct modlist *modlist, unsigned int index1, unsigned int index2)
+{
+	struct modlistentry *entry;
+	if (index1>=modlist->num)
+		return;
+	if (index2>=modlist->num)
+		return;
+	entry = modlist->files[index1];
+	modlist->files[index1] = modlist->files[index2];
+	modlist->files[index2] = entry;
+}
+
 static const char *fuzzycmp12(const char *dst, const char *src)
 {
 	char DST, SRC;
@@ -123,12 +154,12 @@ static const char *fuzzycmp12(const char *dst, const char *src)
 	return dst;
 }
 
-static int modlist_fuzzyfind(struct modlist *modlist, const char *filename)
+int modlist_fuzzyfind(struct modlist *modlist, const char *filename)
 {
-	int retval=0;
+	unsigned int retval=0;
 	int hitscore=0;
-	int i;
-	int len=strlen(filename);
+	unsigned int i;
+	unsigned int len=strlen(filename);
 	if (!len)
 		return 0;
 	for (i=0;i<modlist->num;i++)
@@ -136,7 +167,7 @@ static int modlist_fuzzyfind(struct modlist *modlist, const char *filename)
 		const char *temp=modlist->files[i]->shortname;
 		const char *diff=fuzzycmp12(temp, filename);
 		int score=diff-temp;
-		if (score==len)
+		if ((unsigned)score==len)
 			return i;
 		else if (score>hitscore)
 		{
@@ -161,12 +192,12 @@ static int mlecmp(const void *a, const void *b)
 	return 0;
 }
 
-static void modlist_sort(struct modlist *modlist)
+void modlist_sort(struct modlist *modlist)
 {
 	qsort(modlist->files, modlist->num, sizeof(*modlist->files), mlecmp);
 }
 
-struct modlist *create_modlist(void)
+struct modlist *modlist_create(void)
 {
 	/* TODO ARCS */
 	/*
@@ -174,26 +205,15 @@ struct modlist *create_modlist(void)
 	*/
 	struct modlist *retval=calloc(sizeof(struct modlist), 1);
 
-	retval->_free=modlist_free;
-	retval->getcur=modlist_getcur;
-	retval->get=modlist_get;
-	retval->append=modlist_append;
-	retval->remove=modlist_remove;
-	retval->find=modlist_find;
-	retval->fuzzyfind=modlist_fuzzyfind;
-	retval->sort=modlist_sort;
-
 	return retval;
 }
 
-
-
-
-
-
-
-
-
+void modlist_append_modlist(struct modlist *target, struct modlist *source)
+{
+	unsigned int i;
+	for (i=0;i<source->num;i++)
+		modlist_append(target, modlist_get(source, i));
+}
 
 void fs12name(char *shortname, const char *source)
 {
