@@ -5,6 +5,7 @@
 
 struct moduleinfostruct;
 extern int fsGetNextFile(char *path, struct moduleinfostruct *info, FILE **); /* info comes from external buffer */
+extern int fsGetPrevFile(char *path, struct moduleinfostruct *info, FILE **); /* info comes from external buffer */
 extern int fsFilesLeft(void);
 extern signed int fsFileSelect(void);
 /* extern char fsAddFiles(const char *);      use the playlist instead..*/
@@ -37,6 +38,8 @@ struct preprocregstruct
 	struct preprocregstruct *next;
 };
 
+#define PREPROCREGSTRUCT_TAIL ,0
+
 struct filehandlerstruct
 {
 	int (*Process)(const char *path, struct moduleinfostruct *m, FILE **f);
@@ -47,6 +50,7 @@ typedef enum {
 	interfaceReturnNextAuto=1,
 	interfaceReturnQuit=2,
 	interfaceReturnNextManuel=3,
+	interfaceReturnPrevManuel=6,
 	interfaceReturnCallFs=4,
 	interfaceReturnDosShell=5
 } interfaceReturnEnum;
@@ -56,7 +60,18 @@ struct interfacestruct
 	int (*Init)(const char *path, struct moduleinfostruct *info, FILE **f);
 	interfaceReturnEnum (*Run)(void);
 	void (*Close)(void);
+	const char *name;
+	struct interfacestruct *next;
 };
+#define INTERFACESTRUCT_TAIL ,0
+
+
+extern void plRegisterInterface(struct interfacestruct *interface);
+extern void plUnregisterInterface(struct interfacestruct *interface);
+extern struct interfacestruct *plFindInterface(const char *name);
+extern struct preprocregstruct *plPreprocess;
+extern void plRegisterPreprocess(struct preprocregstruct *r);
+extern void plUnregisterPreprocess(struct preprocregstruct *r);
 
 #define RD_PUTSUBS 1
 #define RD_ARCSCAN 2
@@ -93,6 +108,7 @@ int fsMatchFileName12(const char *a, const char *b);
 
 extern void fsSetup(void);
 extern void fsRescanDir(void);
+extern void fsForceRemove(const char *path);
 
 struct modlistentry;
 extern int dosfile_Read(struct modlistentry *entry, char **mem, size_t *size); /* used by medialib */
