@@ -33,14 +33,11 @@
 #include "stuff/poutput.h"
 #include "boot/psetting.h"
 
-
-static struct preprocregstruct *plPreprocess = 0;
-
 extern struct mdbreaddirregstruct adbReadDirReg, dosReadDirReg, fsReadDirReg, plsReadDirReg, m3uReadDirReg;
 extern struct mdbreadinforegstruct fsReadInfoReg;
 
-static struct interfacestruct *plintr;
-static struct interfacestruct *nextintr;
+static struct interfacestruct *plintr = 0;
+static struct interfacestruct *nextintr = 0;
 static FILE *thisf=NULL;
 static FILE *nextf=NULL;
 static struct moduleinfostruct nextinfo;
@@ -48,13 +45,33 @@ static struct moduleinfostruct plModuleInfo;
 static char thispath[PATH_MAX+1];
 static char nextpath[PATH_MAX+1];
 
-static char callfs;
+typedef enum
+{
+	DoNotForceCallFS = 0,
+	DoForceCallFS = 1
+} enumForceCallFS;
+
+typedef enum
+{
+	DoNotAutoCallFS = 0,
+	DoAutoCallFS = 1
+} enumAutoCallFS;
+
+typedef enum
+{
+	DoNotForceNext=0,
+	DoForceNext=1,
+	DoForcePrev=2
+} enumForceNext;
+
+static enumAutoCallFS callfs;
 static char firstfile;
 
-static int  stop;
+static interfaceReturnEnum stop;
 
 /* stop: 0 cont
  *       1 next song
+ *       6 prev song
  *       2 quit
  *       3 iface : next song or fs
  *       4 iface : call fs
@@ -64,13 +81,13 @@ static int  stop;
 
 
 /* return values:
- * 0  - no files available (and user hit esc
+ * 0  - no files available (and user hit esc)
  * 1  - we have a new song availble
  * -1 - error occured
  */
 
 int callselector (char *path, struct moduleinfostruct *info, FILE **fi,
-                  char callfs, char forcecall, char forcenext,
+                  enumAutoCallFS callfs, enumForceCallFS forcecall, enumForceNext forcenext,
                   struct interfacestruct **iface)
 {
 	int ret;
@@ -89,9 +106,7 @@ int callselector (char *path, struct moduleinfostruct *info, FILE **fi,
 	{
 		ret=result=0;
 		if ((callfs && !fsFilesLeft())||forcecall)
-		{
 			ret=result=fsFileSelect();
-		}
 
 		if (!fsFilesLeft())
 			return 0;
@@ -105,19 +120,33 @@ int callselector (char *path, struct moduleinfostruct *info, FILE **fi,
 				conSave();
 				break;
 			}
-			if (!fsGetNextFile(tpath, &tmodinfo, &tf))
+			if (forcenext==2)
 			{
-				if (tf)
+				if (!fsGetPrevFile(tpath, &tmodinfo, &tf))
 				{
-					fclose(tf);
-					tf=NULL;
+					if (tf)
+					{
+						fclose(tf);
+						tf=NULL;
+					}
+					conSave();
+					continue;
+				}
+			} else {
+				if (!fsGetNextFile(tpath, &tmodinfo, &tf))
+				{
+					if (tf)
+					{
+						fclose(tf);
+						tf=NULL;
+					}
+					conSave();
+					continue;
 				}
-				conSave();
-				continue;
 			}
 
 			sprintf(secname, "filetype %d", tmodinfo.modtype&0xFF);
-			intr=(struct interfacestruct *)_lnkGetSymbol(cfGetProfileString(secname, "interface", ""));
+			intr=plFindInterface(cfGetProfileString(secname, "interface", ""));
 			hdlr=(struct filehandlerstruct *)_lnkGetSymbol(cfGetProfileString(secname, "handler", ""));
 
 			if (hdlr)
@@ -127,7 +156,7 @@ int callselector (char *path, struct moduleinfostruct *info, FILE **fi,
 
 			conSave();
 			{
-				int i;
+				unsigned int i;
 				for (i=0;i<plScrHeight;i++)
 					displayvoid(i, 0, plScrWidth);
 			}
@@ -146,28 +175,23 @@ int callselector (char *path, struct moduleinfostruct *info, FILE **fi,
 					fclose(tf);
 					tf=NULL;
 				}
+				fsForceRemove(tpath);
 			}
 		}
 		if (ret)
 			conSave();
 	} while (ret);
 
-  return 0;
+	return 0;
 }
 
 static int _fsMain(int argc, char *argv[])
 {
 	conSave();
 
-	callfs=0;
-	stop=0;
+	callfs=DoNotAutoCallFS;
+	stop=interfaceReturnContinue;
 	firstfile=1;
-#if 0
-	plintr=0;
-	thisf=0;
-	nextf=0;
-#endif
-
 
 #ifdef DOS32
   /* TODO.. this can be done on xterm consoles, and proc titles*/
@@ -184,24 +208,26 @@ static int _fsMain(int argc, char *argv[])
 		{
 			uint16_t key=egetch();
 			if ((key==0)||(key==3)||(key==27))
-				stop=2;
+				stop=interfaceReturnQuit;
 		}*/
-		if (stop==2)
+		if (stop==interfaceReturnQuit)
 			break;
 
-		stop=0;
-
 		if (!plintr)
 		{
 			int fsr;
 			conSave();
-			fsr=callselector(nextpath,&nextinfo,&nextf,(callfs||firstfile),0,1,&nextintr);
+			fsr=callselector(nextpath,&nextinfo,&nextf,(callfs||firstfile), (stop==interfaceReturnCallFs)?DoForceCallFS:DoNotForceCallFS, DoForceNext, &nextintr);
 			if (!fsr)
+			{
 				break;
-			else if (fsr==-1);
-				callfs=1;
+			} else if (fsr==-1)
+			{
+				callfs=DoAutoCallFS;
+			}
 			conRestore();
 		}
+		stop=interfaceReturnContinue;
 
 		if (callfs)
 			firstfile=0;
@@ -230,13 +256,14 @@ static int _fsMain(int argc, char *argv[])
 			plintr=nextintr;
 			nextintr=0;
 
-			stop=0;
+			stop=interfaceReturnContinue;
 
 			for (prep=plPreprocess; prep; prep=prep->next)
 				prep->Preprocess(thispath, &plModuleInfo, &thisf);
 
 			if (!plintr->Init(thispath, &plModuleInfo, &thisf))
 			{
+				stop = interfaceReturnCallFs; /* file failed, exit to filebrowser, if we don't do this, we can end up with a freeze if we only have this invalid file in the playlist, optional we could remove the file... */
 				plintr=0;
 			} else {
 				/*
@@ -261,34 +288,46 @@ static int _fsMain(int argc, char *argv[])
 /*			while (ekbhit())
 				egetch();*/
 
-			while (!stop)
+			while (!stop) /* != interfaceReturnContinue */
 			{
 				stop=plintr->Run();
 				switch (stop)
 				{
-					case 1: /* next playlist file (auto) */
-						if (firstfile)
-							stop=2;
-						else
-							stop=callselector(nextpath,&nextinfo,&nextf,callfs,0,1,&nextintr)?1:2;
+					case interfaceReturnContinue: /* 0 */
+					case interfaceReturnQuit:
 						break;
-					case 3: /* next playlist file (man) */
+					case interfaceReturnNextAuto: /* next playlist file (auto) */
+						if (callselector(nextpath,&nextinfo,&nextf, DoAutoCallFS, DoNotForceCallFS, DoNotForceNext, &nextintr)==0)
+						{
+							if (fsFilesLeft())
+							{
+								callselector(nextpath, &nextinfo,&nextf, DoNotAutoCallFS, DoNotForceCallFS, DoForceNext, &nextintr);
+								stop = interfaceReturnNextAuto;
+							} else
+								stop = interfaceReturnQuit;
+						} else
+							stop = interfaceReturnNextAuto;
+						break;
+					case interfaceReturnPrevManuel: /* prev playlist file (man) */
 						if (fsFilesLeft())
-							stop=callselector(nextpath,&nextinfo,&nextf,1,0,1,&nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
+							stop=callselector(nextpath,&nextinfo,&nextf, DoNotAutoCallFS, DoNotForceCallFS, DoForcePrev, &nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
 						else
-							stop=callselector(nextpath,&nextinfo,&nextf,1,0,0,&nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
-						if (stop==-1)
-							callfs=1;
+							stop=callselector(nextpath,&nextinfo,&nextf, DoAutoCallFS, DoNotForceCallFS, DoNotForceNext, &nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
 						break;
-					case 4: /* call fs */
-						callfs=1;
-						stop=callselector(nextpath,&nextinfo,&nextf,1,1,0,&nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
+					case interfaceReturnNextManuel: /* next playlist file (man) */
+						if (fsFilesLeft())
+							stop=callselector(nextpath,&nextinfo,&nextf, DoNotAutoCallFS, DoNotForceCallFS, DoForceNext, &nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
+						else
+							stop=callselector(nextpath,&nextinfo,&nextf, DoAutoCallFS, DoNotForceCallFS, DoNotForceNext, &nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
 						break;
-					case 5: /* dos shell */
+					case interfaceReturnCallFs: /* call fs */
+						stop=callselector(nextpath,&nextinfo,&nextf, DoAutoCallFS, DoForceCallFS, DoNotForceNext, &nextintr)?interfaceReturnNextAuto:interfaceReturnContinue;
+						break;
+					case interfaceReturnDosShell: /* dos shell */
 						plSetTextMode(fsScrType);
 						if (conRestore())
 							break;
-						stop=0;
+						stop=interfaceReturnContinue;
 						plDosShell();
 						conSave();
 /*						while (ekbhit())
@@ -312,33 +351,14 @@ static int _fsMain(int argc, char *argv[])
 	return errOk;
 }
 
-static void plRegisterPreprocess(struct preprocregstruct *r)
-{
-	r->next=plPreprocess;
-	plPreprocess=r;
-}
-
 static int fsint(void)
 {
-	/*const char *sec=cfGetProfileString(cfConfigSec, "fileselsec", "fileselector");*/
-
-	char regname[50];
-	const char *regs=NULL;
-
 	mdbRegisterReadDir(&adbReadDirReg);
 	mdbRegisterReadDir(&dosReadDirReg);
 	mdbRegisterReadDir(&fsReadDirReg);
 	mdbRegisterReadDir(&plsReadDirReg);
 	mdbRegisterReadDir(&m3uReadDirReg);
 	mdbRegisterReadInfo(&fsReadInfoReg);
-	regs=_lnkReadInfoReg("preprocess");
-
-	while (cfGetSpaceListEntry(regname, &regs, 49))
-	{
-		void *reg=_lnkGetSymbol(regname);
-		if (reg)
-			plRegisterPreprocess((struct preprocregstruct *)reg);
-	}
 	
 	fprintf(stderr, "initializing fileselector...\n");
 	if (!fsInit())
@@ -347,8 +367,6 @@ static int fsint(void)
 		return errGen;
 	}
 
-/*	fsAddFiles(cfCommandLine);*/
-
 	return errOk;
 }
 
@@ -568,5 +586,8 @@ static void __attribute__((destructor))done(void)
 		ocpmain = 0;
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"pfilesel", "OpenCP Fileselector (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, PreInit: fsint, LateClose: fsclose};
+#endif
+
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"pfilesel", "OpenCP Fileselector (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, PreInit: fsint, LateClose: fsclose};
