@@ -1,3 +1,9 @@
+/* OpenCP Module Player
+ * copyright (c) '94-'09 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ *
+ * playlist related functions (used by .pls and .m3u parsers)
+ */
+
 #include "config.h"
 #include <fnmatch.h>
 #include <string.h>
@@ -113,6 +119,6 @@ void fsAddPlaylist(struct modlist *ml, const char *path, const char *mask, unsig
 	retval.Read=dosfile_Read;
 	retval.ReadHeader=dosfile_ReadHeader;
 	retval.ReadHandle=dosfile_ReadHandle;
-	ml->append(ml, &retval); 
+	modlist_append(ml, &retval); 
 	dirdbUnref(retval.dirdbfullpath);
 }
