@@ -57,23 +57,23 @@ int main(int argc, char *argv[])
 		off += dir[i].len;
 	}
 
-	if (fwrite("PACK", 4, 1, output)!=1) { perror("fwrite)"); return -1;}
+	if (fwrite("PACK", 4, 1, output)!=1) { perror("fwrite)"); fclose(output); return -1;}
 	off = uint32_little(off);
-	if (fwrite(&off, sizeof(uint32_t), 1, output)!=1) { perror("fwrite)"); return -1;}
+	if (fwrite(&off, sizeof(uint32_t), 1, output)!=1) { perror("fwrite()"); fclose(output); return -1;}
 	off = uint32_little(off);
 	off=nfiles*sizeof(dir[0]);
 	off = uint32_little(off);
-	if (fwrite(&off, sizeof(uint32_t), 1, output)!=1) { perror("fwrite)"); return -1;}
+	if (fwrite(&off, sizeof(uint32_t), 1, output)!=1) { perror("fwrite()"); fclose(output); return -1;}
 
 	for (i=0; i<nfiles; i++)
 	{
 		char *buffer;
 		FILE *file;
-		if (!(file=fopen(dir[i].name, "r"))) { perror("fopen()"); return -1; }
+		if (!(file=fopen(dir[i].name, "r"))) { perror("fopen()"); fclose(output); return -1; }
 		buffer=calloc(dir[i].len, 1);
-		fread(buffer, dir[i].len, 1, file);
+		if (fread(buffer, dir[i].len, 1, file)!=1) { perror("fwrite()"); fclose(output); return -1;}
 		fclose(file);
-		if (fwrite(buffer, dir[i].len, 1, output)!=1) { perror("fwrite)"); return -1;}
+		if (fwrite(buffer, dir[i].len, 1, output)!=1) { perror("fwrite)"); fclose(output); return -1;}
 		free(buffer);
 	}
 
@@ -82,7 +82,7 @@ int main(int argc, char *argv[])
 		dir[i].len = uint32_little(dir[i].len);
 		dir[i].off = uint32_little(dir[i].off);
 	}
-	if (fwrite(dir, sizeof(dir[0]), nfiles, output)!=1) { perror("fwrite)"); return -1;}
+	if (fwrite(dir, sizeof(dir[0]), nfiles, output)!=1) { perror("fwrite)"); fclose(output); return -1;}
 	fclose(output);
 	free(dir);
 	return 0;
