@@ -87,14 +87,13 @@
 #include "boot/plinkman.h"
 #include "stuff/err.h"
 #include "help/cphelper.h"
-#include "binfile/binfpak.h"
 #include "cpiface/cpiface.h"
 #include <curses.h>
 
-static int plWinFirstLine, plWinHeight, plHelpHeight, plHelpScroll;
+static unsigned int plWinFirstLine, plWinHeight, plHelpHeight, plHelpScroll;
 
-static int        Helpfile_ID=1213219663;
-static int        Helpfile_Ver=0x011000;
+static const uint32_t  Helpfile_ID=1213219663;
+static const uint32_t  Helpfile_Ver=0x011000;
 static uint32_t   Helppages;
 static int        HelpfileErr=hlpErrNoFile;
 static helppage  *Page, *curpage;
@@ -122,8 +121,12 @@ static int        link_ind;
 
 static int doReadVersion100Helpfile(FILE *file)
 {
-	int i;
-	fread(&Helppages, sizeof(Helppages), 1, file);
+	unsigned int i;
+	if (fread(&Helppages, sizeof(Helppages), 1, file) != 1)
+	{
+		perror(__FILE__ ": fread failed #1: ");
+		return hlpErrBadFile;
+	}
 	Page=calloc(Helppages, sizeof(Page[0]));
 
 	for (i=0; i<Helppages; i++)
@@ -131,16 +134,43 @@ static int doReadVersion100Helpfile(FILE *file)
 		unsigned char len;
 		
 		memset(Page[i].name, 0, 128);
-		fread(&len, sizeof(len), 1, file);
-		fread(Page[i].name, len, 1, file);
+		if (fread(&len, sizeof(len), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #2: ");
+			return hlpErrBadFile;
+		}
+		if (fread(Page[i].name, len, 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #3: ");
+			return hlpErrBadFile;
+		}
 
     		memset(Page[i].desc, 0, 128);
-		fread(&len, sizeof(len), 1, file);
-		fread(Page[i].desc, len, 1, file);
+		if (fread(&len, sizeof(len), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #4: ");
+			return hlpErrBadFile;
+		}
+
+		if (fread(Page[i].desc, len, 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #5: ");
+			return hlpErrBadFile;
+		}
+
+		if (fread(&Page[i].size, sizeof(Page[i].size), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #6: ");
+			return hlpErrBadFile;
+		}
 
-		fread(&Page[i].size, sizeof(Page[i].size), 1, file);
 		Page[i].size = uint32_little (Page[i].size);
-		fread(&Page[i].lines, sizeof(Page[i].lines), 1, file);
+		if (fread(&Page[i].lines, sizeof(Page[i].lines), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #7: ");
+			return hlpErrBadFile;
+		}
+
 		Page[i].lines = uint32_little (Page[i].lines);
 
 		Page[i].links=NULL;
@@ -150,7 +180,11 @@ static int doReadVersion100Helpfile(FILE *file)
 	for (i=0; i<Helppages; i++)
 	{
 		Page[i].data=calloc(Page[i].size, 1);
-		fread(Page[i].data, Page[i].size, 1, file);
+		if (fread(Page[i].data, Page[i].size, 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #8: ");
+			return hlpErrBadFile;
+		}
 	};
 
 	return hlpErrOk;
@@ -160,9 +194,13 @@ static int doReadVersion110Helpfile(FILE *file)
 {
 	int  *compdatasize;
 	char *inbuf;
-	int i;
+	unsigned int i;
 
-	fread(&Helppages, sizeof(Helppages), 1, file);
+	if (fread(&Helppages, sizeof(Helppages), 1, file) != 1)
+	{
+		perror(__FILE__ ": fread failed #9: ");
+		return hlpErrBadFile;
+	}
 	Helppages = uint32_little (Helppages);
 	Page = calloc(Helppages, sizeof(Page[0]));
 
@@ -173,19 +211,54 @@ static int doReadVersion110Helpfile(FILE *file)
 		unsigned char len;
 
 		memset(Page[i].name, 0, 128);
-		fread(&len, sizeof(len), 1, file);
-		fread(Page[i].name, len, 1, file);
+		if (fread(&len, sizeof(len), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #10: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
+		if (fread(Page[i].name, len, 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #11: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
 
 		memset(Page[i].desc, 0, 128);
-		fread(&len, sizeof(len), 1, file);
-		fread(Page[i].desc, len, 1, file);
+		if (fread(&len, sizeof(len), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #12: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
+		if (fread(Page[i].desc, len, 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #13: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
 
-		fread(&Page[i].size, sizeof(Page[i].size), 1, file);
+		if (fread(&Page[i].size, sizeof(Page[i].size), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #14: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
 		Page[i].size = uint32_little (Page[i].size);
-		fread(&Page[i].lines, sizeof(Page[i].lines), 1, file);
+		if (fread(&Page[i].lines, sizeof(Page[i].lines), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #15: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
 		Page[i].lines = uint32_little (Page[i].lines);
 
-		fread(&compdatasize[i], sizeof(compdatasize[i]), 1, file);
+		if (fread(&compdatasize[i], sizeof(compdatasize[i]), 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #16: ");
+			free(compdatasize);
+			return hlpErrBadFile;
+		}
 		compdatasize[i]=uint32_little(compdatasize[i]);
 		Page[i].links=NULL;
 		Page[i].rendered=NULL;
@@ -196,7 +269,13 @@ static int doReadVersion110Helpfile(FILE *file)
 		uLongf temp=Page[i].size;
 		Page[i].data=calloc(Page[i].size, 1);
 		inbuf=calloc(compdatasize[i], 1);
-		fread(inbuf, compdatasize[i], 1, file);
+		if (fread(inbuf, compdatasize[i], 1, file) != 1)
+		{
+			perror(__FILE__ ": fread failed #17: ");
+			free(compdatasize);
+			free(inbuf);
+			return hlpErrBadFile;
+		}
 		uncompress((Bytef *)Page[i].data, &temp, (Bytef *)inbuf, compdatasize[i]);
 		Page[i].size=temp;
 		free(inbuf);
@@ -211,12 +290,20 @@ static int doReadHelpFile(FILE *file)
 	uint32_t version;
 	uint32_t temp;
 
-	fread(&temp, sizeof(temp), 1, file);
+	if (fread(&temp, sizeof(temp), 1, file) != 1)
+	{
+		perror(__FILE__ ": fread failed #18: ");
+		return hlpErrBadFile;
+	}
 	temp = uint32_little (temp);
 	if (temp!=Helpfile_ID)
 		return hlpErrBadFile;
 
-	fread(&version, sizeof(version), 1, file);
+	if (fread(&version, sizeof(version), 1, file) != 1)
+	{
+		perror(__FILE__ ": fread failed #19: ");
+		return hlpErrBadFile;
+	}
 	version = uint32_little (version);
 
 	if (version>Helpfile_Ver)
@@ -260,11 +347,14 @@ static char plReadHelpExternal(void)
 
 static char plReadHelpPack(void)
 {
+	char path[PATH_MAX];
 	FILE *ref;
+
 	if (Page && (HelpfileErr==hlpErrOk))
 		return 1;
 
-	if ((ref=pakfOpen("ocp.hlp")))
+	snprintf(path, sizeof(path), "%s%s", cfDataDir, "ocp.hlp");
+	if ((ref=fopen(path, "r")))
 	{
 		HelpfileErr=doReadHelpFile(ref);
 		fclose(ref);
@@ -278,7 +368,7 @@ static char plReadHelpPack(void)
 
 helppage *brDecodeRef(char *name)
 {
-	int i;
+	unsigned int i;
 	for (i=0; i<Helppages; i++)
 		if (!strcasecmp(Page[i].name, name))
 			return &Page[i];
@@ -294,7 +384,7 @@ static help_link *firstLinkOnPage(helppage *pg)
 
 static int linkOnCurrentPage(help_link *lnk)
 {
-	int y;
+	unsigned int y;
 
 	if (!lnk)
 		return 0;
@@ -508,17 +598,17 @@ void brSetPage(helppage *page)
 
 void brDisplayHelp(void)
 {
-	int curlinky;
+	unsigned int curlinky;
 	char destbuffer[60];
 	char strbuffer[256];
 	char numbuffer[4];
 	int descxp;
-	int y;
+	unsigned int y;
 	
 	if ((plHelpScroll+plWinHeight)>plHelpHeight)
 		plHelpScroll=plHelpHeight-plWinHeight;
 
-	if (plHelpScroll<0)
+	if ((signed)plHelpScroll<0)
 		plHelpScroll=0;
 
 	if (curlink)
@@ -541,7 +631,7 @@ void brDisplayHelp(void)
 	strcat(strbuffer, "%");
 
 	memset(destbuffer, 0x20, 60);
-	descxp=MAX(0, 59-strlen(strbuffer));
+	descxp=MAX(0, 59-(signed)strlen(strbuffer));
 
 	strncpy(&destbuffer[descxp], strbuffer, 59-descxp);
 	displaystr(plWinFirstLine-1, 20, 0x08, destbuffer, 59);
@@ -575,21 +665,24 @@ void brDisplayHelp(void)
 		for (y=2; y<plWinHeight; y++)
 			displayvoid(y+plWinFirstLine, 0, CONSOLE_MAX_X);
 	} else {
-		uint16_t _void[CONSOLE_MAX_X-80];
-		fillstr(_void, 0, 0, 0, CONSOLE_MAX_X-80);
 		for (y=0; y<plWinHeight; y++)
 		{
+			if ((y+plHelpScroll)>=plHelpHeight)
+			{
+				displayvoid(y+plWinFirstLine, 0, plScrWidth);
+				continue;
+			}
 			if (y!=curlinky)
 			{
 				displaystrattr(y+plWinFirstLine, 0, &curpage->rendered[(y+plHelpScroll)*80], 80);
-				displaystrattr(y+plWinFirstLine, 80, _void, plScrWidth-80);
+				displayvoid(y+plWinFirstLine, 80, plScrWidth-80);
 			} else {
 				int yp=(y+plHelpScroll)*80;
 				int xp;
 			        char dummystr[82];
 				int i, off;
 
-       				 if (curlink->posx!=0)
+       				if (curlink->posx!=0)
 			        {
 					displaystrattr(y+plWinFirstLine, 0, &curpage->rendered[yp], curlink->posx);
 			        };
@@ -608,7 +701,7 @@ void brDisplayHelp(void)
 
 				displaystr(y+plWinFirstLine, curlink->posx, 4, dummystr, curlink->len);
 
-				displaystrattr(y+plWinFirstLine, 80, _void, plScrWidth-80);
+				displayvoid(y+plWinFirstLine, 80, plScrWidth-80);
 
 			        /* and all this just to prevent flickering. ARG! */
 			}
@@ -662,7 +755,7 @@ int brHelpKey(uint16_t key)
 
 				if (link2!=curlink)
 				{
-					if (plHelpScroll-link2->posy>1)
+					if ((signed)(plHelpScroll-link2->posy)>1)
 						plHelpScroll--;
 					else {
 						link_ind=MAX(link_ind-1, 0);
@@ -704,7 +797,7 @@ int brHelpKey(uint16_t key)
 		/*case 0x4900: //pgup*/
 		case KEY_PPAGE:
 			plHelpScroll-=plWinHeight;
-			if (plHelpScroll<0)
+			if ((signed)plHelpScroll<0)
 				plHelpScroll=0;
 
 			if (curpage->linkcount)
@@ -719,7 +812,7 @@ int brHelpKey(uint16_t key)
 
 					for (i=curpage->linkcount-1; i>=0; i--)
 					{
-						int d=ABS(plHelpScroll+plWinHeight-curpage->links[i].posy-1);
+						int d=ABS((signed)plHelpScroll+(signed)plWinHeight-(signed)curpage->links[i].posy-1);
 						if (d<bestd)
 						{
 							bestd=d;
@@ -750,7 +843,7 @@ int brHelpKey(uint16_t key)
 
 				        for (i=0; i<curpage->linkcount; i++)
 				        {
-						int d=ABS(plHelpScroll-curpage->links[i].posy);
+						int d=ABS((signed)plHelpScroll-(signed)curpage->links[i].posy);
 						if (d<bestd)
 						{
 							bestd=d;
@@ -815,7 +908,7 @@ int brHelpKey(uint16_t key)
 	}
 	if ((plHelpScroll+plWinHeight)>plHelpHeight)
 		plHelpScroll=plHelpHeight-plWinHeight;
-	if (plHelpScroll<0)
+	if ((signed)plHelpScroll<0)
 		plHelpScroll=0;
 	return 1;
 }
@@ -850,7 +943,7 @@ static int hlpGlobalInit(void)
 
 void hlpFreePages(void)
 {
-	int i;
+	unsigned int i;
 	for (i=0; i<Helppages; i++)
 	{
 		if (Page[i].data)
@@ -887,5 +980,7 @@ static void hlpGlobalClose(void)
 	hlpFreePages();
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"cphelper", "OpenCP help browser (c) 1998-04 Fabian Giesen", DLLVERSION, 0, Init: hlpGlobalInit, Close: hlpGlobalClose};
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"cphelper", "OpenCP help browser (c) 1998-04 Fabian Giesen", DLLVERSION, 0, Init: hlpGlobalInit, Close: hlpGlobalClose};
