@@ -9,6 +9,7 @@
  */
 
 #include "config.h"
+#include <curses.h>
 #include <stdio.h>
 #include <string.h>
 #include "types.h"
@@ -55,10 +56,10 @@ static int hlpIProcessKey(unsigned short key)
 			cpiKeyHelp('H', "Enable help browser");
 			cpiKeyHelp('?', "Enable help browser");
 			cpiKeyHelp('!', "Enable help browser");
-			cpiKeyHelp(KEY_F1, "Enable help browser");
+			cpiKeyHelp(KEY_F(1), "Enable help browser");
 			return 0;
-		/*case 0x6800: case 0x3b00: TODO-keys*/
-		case 'h': case 'H': case '?': case '!': case KEY_F1:
+		/* case 0x6800: // alt-f1 TODO keys */
+		case 'h': case 'H': case '?': case '!': case KEY_F(1):
 			cpiGetMode(beforehelp);
 			cpiSetMode("coolhelp");
 			break;
@@ -77,11 +78,12 @@ static int plHelpKey(unsigned short key)
 			cpiKeyHelp('H', "Exit help browser");
 			cpiKeyHelp('?', "Exit help browser");
 			cpiKeyHelp('!', "Exit help browser");
-			cpiKeyHelp(KEY_F1, "Exit help browser");
+			cpiKeyHelp(KEY_F(1), "Exit help browser");
 			cpiKeyHelp(KEY_ESC, "Exit help browser");
 			return brHelpKey(key);
-		/*case 0x6800: case 0x3b00: TODO-keys*/
-		case 'h': case 'H': case '?': case '!': case 27: case KEY_F1:
+		/* case 0x6800: // alt-f1 TODO keys */
+		case 'h': case 'H': case '?': case '!': case 27:
+		case KEY_F(1):
 			cpiSetMode(beforehelp);
 			break;
 		default:
@@ -102,7 +104,7 @@ static int hlpEvent(int ev)
 	return 1;
 }
 
-static struct cpimoderegstruct hlpHelpBrowser = {"coolhelp", hlpSetMode, hlpDraw, hlpIProcessKey, plHelpKey, hlpEvent};
+static struct cpimoderegstruct hlpHelpBrowser = {"coolhelp", hlpSetMode, hlpDraw, hlpIProcessKey, plHelpKey, hlpEvent CPIMODEREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
@@ -114,5 +116,7 @@ static void __attribute__((destructor))done(void)
 	cpiUnregisterDefMode(&hlpHelpBrowser);
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"cphlpif", "OpenCP help browser CPIFACE wrapper (c) 1998-04 Fabian Giesen", DLLVERSION, 0};
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"cphlpif", "OpenCP help browser CPIFACE wrapper (c) 1998-04 Fabian Giesen", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
