@@ -36,8 +36,8 @@ static struct dmDrive *dmMEDIALIB;
 
 static int mlDrawBox(void)
 {
-	int mlTop=plScrHeight/2-2;
-	int i;
+	unsigned int mlTop=plScrHeight/2-2;
+	unsigned int i;
 
 	displayvoid(mlTop+1, 5, plScrWidth-10);
 	displayvoid(mlTop+2, 5, plScrWidth-10);
@@ -62,9 +62,9 @@ static int mlDrawBox(void)
 
 static int mlSubScan(const uint32_t dirdbnode, int mlTop)
 {
-	struct modlist *ml = create_modlist();
+	struct modlist *ml = modlist_create();
 	struct modlistentry *mle;
-	int i;
+	unsigned int i;
 	char npath[PATH_MAX+1];
 
 	dirdbGetFullName(dirdbnode, npath, DIRDB_FULLNAME_NOBASE|DIRDB_FULLNAME_ENDSLASH);
@@ -80,7 +80,7 @@ static int mlSubScan(const uint32_t dirdbnode, int mlTop)
 
 	for (i=0;i<ml->num;i++)
 	{
-		mle=ml->get(ml, i);
+		mle=modlist_get(ml, i);
 		if (mle->flags&MODLIST_FLAG_DIR)
 		{
 			if (strcmp(mle->name, ".."))
@@ -99,7 +99,7 @@ static int mlSubScan(const uint32_t dirdbnode, int mlTop)
 			dirdbMakeMdbRef(mle->dirdbfullpath, mle->fileref);
 		}
 	}
-	ml->_free(ml);
+	modlist_free(ml);
 	return 0;
 }
 
@@ -126,15 +126,15 @@ static int mlScan(const uint32_t dirdbnode)
 
 static FILE *mlSourcesAdd(struct modlistentry *entry)
 {
-	int mlTop=mlDrawBox();
+	unsigned int mlTop=mlDrawBox();
 	int editpath=0;
 
 	/* these are for editing the path */
 	char str[PATH_MAX+6];
-	int curpos;
-	int cmdlen;
+	unsigned int curpos;
+	unsigned int cmdlen;
       	int insmode=1;
-	int scrolled=0;
+	unsigned int scrolled=0;
 
 	strcpy(str, "file:/");
 	curpos=strlen(str);
@@ -262,7 +262,7 @@ static FILE *mlSourcesAdd(struct modlistentry *entry)
 		}
 		while ((curpos-scrolled)>=(plScrWidth-10))
 			scrolled+=8;
-		while ((curpos-scrolled)<0)
+		while (((signed)curpos-(signed)scrolled)<0)
 			scrolled-=8;
 	}
 }
@@ -295,13 +295,13 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 		fs12name(entry.shortname, "all");
 		strcpy(entry.name, "all");
 		entry.dirdbfullpath=dmall;
-		ml->append(ml, &entry);
+		modlist_append(ml, &entry);
 	
 		strcpy(entry.shortname, "search");
 		strcpy(entry.name, "search");
 		fs12name(entry.shortname, "search");
 		entry.dirdbfullpath=dmsearch;
-		ml->append(ml, &entry);
+		modlist_append(ml, &entry);
 
 		strcpy(entry.shortname, "addfiles");
 		strcpy(entry.name, "addfiles");
@@ -309,7 +309,7 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 		entry.dirdbfullpath=dmadd;
 		entry.flags=MODLIST_FLAG_FILE|MODLIST_FLAG_VIRTUAL;
 		entry.ReadHandle=mlSourcesAdd;
-		ml->append(ml, &entry);
+		modlist_append(ml, &entry);
 
 		goto out;
 	}
@@ -334,19 +334,19 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 			entry.Read=dosfile_Read;
 			entry.ReadHeader=dosfile_ReadHeader;
 			entry.ReadHandle=dosfile_ReadHandle;
-			ml->append(ml, &entry);
+			modlist_append(ml, &entry);
 /*			dirdbUnref(entry.dirdbfullpath);*/
 		}
 		goto out;
 	}
 	if (path==dmsearch)
 	{
-		int mlTop=mlDrawBox();
+		unsigned int mlTop=mlDrawBox();
 		char str[NAME_MAX];
-		int curpos;
-		int cmdlen;
+		unsigned int curpos;
+		unsigned int cmdlen;
 		int insmode=1;
-		int scrolled=0;
+		unsigned int scrolled=0;
 
 		displaystr(mlTop+1, 5, 0x0b, "Give me something to crunch!!", 29);
 		displaystr(mlTop+3, 5, 0x0b, "-- Finish with enter --", 23);
@@ -428,7 +428,7 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 					goto out;
 				case _KEY_ENTER:
 					{
-						int i, j;
+						unsigned int i, j;
 						char buffer[PATH_MAX+1];
 						int first=1;
 						uint32_t dirdbnode;
@@ -492,7 +492,7 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 								entry.Read=dosfile_Read;
 								entry.ReadHeader=dosfile_ReadHeader;
 								entry.ReadHandle=dosfile_ReadHandle;
-								ml->append(ml, &entry);
+								modlist_append(ml, &entry);
 					/*			dirdbUnref(entry.dirdbfullpath);*/
 							}
 						}
@@ -501,7 +501,7 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 			}
 			while ((curpos-scrolled)>=(plScrWidth-10))
 				scrolled+=8;
-			while ((curpos-scrolled)<0)
+			while (((signed)curpos-(signed)scrolled)<0)
 				scrolled-=8;
 		}
 	}
@@ -526,7 +526,7 @@ static void mlclose(void)
 	mdbUnregisterReadDir(&mlReadDirReg);
 }
 
-static struct mdbreaddirregstruct mlReadDirReg = {mlReadDir};
+static struct mdbreaddirregstruct mlReadDirReg = {mlReadDir MDBREADDIRREGSTRUCT_TAIL};
 
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"medialib", "OpenCP medialib (c) 2005 Stian Skjelstad", DLLVERSION, 0, Init: mlint, Close: mlclose};
+struct linkinfostruct dllextinfo = {"medialib", "OpenCP medialib (c) 2005 Stian Skjelstad", DLLVERSION, 0, Init: mlint, Close: mlclose, PreInit:0, LateInit: 0, PreClose:0, LateClose: 0};
