@@ -133,7 +133,7 @@ static void audio_pcm_s16(int16_t *dest, unsigned int nsamples, uint16_t *source
 {
 	unsigned int len;
 	int ls, rs;
-	unsigned short xormask=0x0000;
+	uint16_t xormask=0x0000;
 
 	len = nsamples;
 	if (srnd)
@@ -148,7 +148,7 @@ static void audio_pcm_s16(int16_t *dest, unsigned int nsamples, uint16_t *source
 		PANPROC;
 		*dest = rs;
 		dest ++;
-		*dest = (signed short)ls^xormask;
+		*dest = (int16_t)ls^xormask;
 		dest ++;
 	}
 }
@@ -236,7 +236,7 @@ write_dat:
 /* from main.c */
 static void mem_init(int track)
 {
-	static unsigned char intz[]=
+	static uint8_t intz[]=
 	{
 		0xf3,		/* di */
 		0xcd,0,0,	/* call init */
@@ -245,7 +245,7 @@ static void mem_init(int track)
 		0x76,		/* halt */
 		0x18,0xfa	/* jr loop */
 	};
-	static unsigned char intnz[]=
+	static uint8_t intnz[]=
 	{
 		0xf3,		/* di */
 		0xcd,0,0,	/* call init */
@@ -256,7 +256,7 @@ static void mem_init(int track)
 		0x18,0xf7	/* jr loop */
 	};
 	int init,ay_1st_block,ourinit,interrupt;
-	unsigned char *ptr;
+	uint8_t *ptr;
 	int addr,len,ofs;
 	
 #define GETWORD(x) (((*(x))<<8)|(*(x+1)))
@@ -1009,7 +1009,7 @@ int __attribute__ ((visibility ("internal"))) ayOpenPlayer(FILE *file)
 	srnd=0;
 	aySetVolume(64, 0, 64, 0);
 /*	aySetAmplify(amplify);   TODO */
-	buf16=malloc(sizeof(unsigned short)*buflen*2);
+	buf16=malloc(sizeof(uint16_t)*buflen*2);
 	if (!buf16)
 	{
 		if(aydata.tracks)
@@ -1116,7 +1116,7 @@ void __attribute__ ((visibility ("internal"))) ayPause(unsigned char p)
       	inpause=p;
 }
 
-void __attribute__ ((visibility ("internal"))) aySetSpeed(unsigned short sp)
+void __attribute__ ((visibility ("internal"))) aySetSpeed(uint16_t sp)
 {
       	if (sp<32)
 		sp=32;
