@@ -1,3 +1,9 @@
+/* OpenCP Module Player
+ * copyright (c) '05-'09 Stian Skjelstad <stian@nixia.no>
+ *
+ * .ay file type detection routines for the file selector
+ */ 
+
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
@@ -18,7 +24,7 @@ static int ayReadMemInfo(struct moduleinfostruct *m, const char *buf, size_t len
 	int miscptr;
 	const char *miscstr;
 
-	int left;
+	unsigned int left;
 
 /*	uint8_t tracks, default_track;*/
 
@@ -110,6 +116,6 @@ static void __attribute__((destructor))done(void)
 }
 
 
-static struct mdbreadinforegstruct ayReadInfoReg = {ayReadMemInfo, ayReadInfo, ayEvent};
+static struct mdbreadinforegstruct ayReadInfoReg = {ayReadMemInfo, ayReadInfo, ayEvent MDBREADINFOREGSTRUCT_TAIL};
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"aytype", "OpenCP AY Detection (c) 2005 Stian Skjelstad", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"aytype", "OpenCP AY Detection (c) 2005 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
