@@ -72,31 +72,31 @@ __attribute__ ((visibility ("internal"))) int sound_stereo_ay_narrow=0;	/* (AY s
 
 static int sound_framesiz;
 
-static unsigned int ay_tone_levels[16];
+static uint32_t ay_tone_levels[16];
 
-static signed short *sound_buf;
+static int16_t *sound_buf;
 static int sound_oldpos,sound_fillpos,sound_oldval,sound_oldval_orig;
 
 /* foo_subcycles are fixed-point with low 16 bits as fractional part.
  * The other bits count as the chip does.
  */
-static unsigned int ay_tone_tick[3],ay_tone_high[3],ay_noise_tick;
-static unsigned int ay_tone_subcycles,ay_env_subcycles;
-static unsigned int ay_env_internal_tick,ay_env_tick;
-static unsigned int ay_tick_incr;
-static unsigned int ay_tone_period[3],ay_noise_period,ay_env_period;
+static uint32_t ay_tone_tick[3],ay_tone_high[3],ay_noise_tick;
+static uint32_t ay_tone_subcycles,ay_env_subcycles;
+static uint32_t ay_env_internal_tick,ay_env_tick;
+static uint32_t ay_tick_incr;
+static uint32_t ay_tone_period[3],ay_noise_period,ay_env_period;
 
 static int beeper_last_subpos=0;
 
 /* AY registers */
 /* we have 16 so we can fake an 8910 if needed (XXX any point?) */
-static unsigned char sound_ay_registers[16];
+static uint8_t sound_ay_registers[16];
 
 struct ay_change_tag
   {
-  unsigned long tstates;
-  unsigned short ofs;
-  unsigned char reg,val;
+  uint32_t tstates;
+  uint16_t ofs;
+  uint8_t reg,val;
   };
 
 static struct ay_change_tag ay_change[AY_CHANGE_MAX];
@@ -166,7 +166,7 @@ if(!sound_stereo)
 /*sound_enabled=1;*/
 sound_framesiz=sound_freq/50;
 
-if((sound_buf=malloc(sizeof(signed short)*sound_framesiz*(sound_stereo+1)))==NULL)
+if((sound_buf=malloc(sizeof(int16_t)*sound_framesiz*(sound_stereo+1)))==NULL)
   {
   sound_end();
   return(0);
@@ -219,7 +219,7 @@ void __attribute__ ((visibility ("internal"))) sound_end(void)
 
 
 /* write sample to buffer as pseudo-stereo */
-static void sound_write_buf_pstereo(signed short *out,int c)
+static void sound_write_buf_pstereo(int16_t *out,int c)
 {
 int bl=(c-pstereobuf[pstereopos])/2;
 int br=(c+pstereobuf[pstereopos])/2;
@@ -317,14 +317,14 @@ static int env_first=1,env_rev=0,env_counter=15;
 int tone_level[3];
 int mixer,envshape;
 int f,g,level,count;
-signed short *ptr;
+int16_t *ptr;
 struct ay_change_tag *change_ptr=ay_change;
 int changes_left=ay_change_count;
 int reg,r;
 int is_low,is_on;
 int chan1,chan2,chan3;
 int frametime=ay_tsmax*50;
-unsigned int tone_count,noise_count;
+uint32_t tone_count,noise_count;
 
 /* convert change times to sample offsets */
 for(f=0;f<ay_change_count;f++)
@@ -605,7 +605,7 @@ CLOCK_RESET(AY_CLOCK_CPC);
 int __attribute__ ((visibility ("internal"))) sound_frame(int really_play)
 {
 static int silent_level=-1;
-signed short *ptr;
+int16_t *ptr;
 int f,silent,chk;
 int fulllen=sound_framesiz*(sound_stereo+1);
 
@@ -651,7 +651,7 @@ silent_level=sound_buf[fulllen-1];
 if(fading)
   {
   if(fadetime<=0)
-    memset(sound_buf,0,fulllen*sizeof(signed short)),silent=1;
+    memset(sound_buf,0,fulllen*sizeof(int16_t)),silent=1;
   else
     {
     ptr=sound_buf;
@@ -712,7 +712,7 @@ fadetime=fadetotal=fadetime_in_sec*sound_freq;
 
 void __attribute__ ((visibility ("internal"))) sound_beeper(int on)
 {
-signed short *ptr;
+int16_t *ptr;
 int newpos;
 int subpos;
 int val,subval;
