@@ -16,15 +16,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
+#include "types.h"
 #include "main.h"
 #include "z80.h"
 
 #define parity(a) (partable[a])
 
-static unsigned char partable[256]={
+static uint8_t partable[256]={
       4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
       0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
       0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
@@ -43,14 +45,14 @@ static unsigned char partable[256]={
       4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4
    };
 
-static unsigned char a, f, b, c, d, e, h, l;
-static unsigned char r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
-static unsigned short pc;
-static unsigned short ix, iy, sp;
-static unsigned int radjust;
-static unsigned int ixoriy, new_ixoriy;
-static unsigned int intsample;
-static unsigned char op;
+static uint8_t a, f, b, c, d, e, h, l;
+static uint8_t r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
+static uint16_t pc;
+static uint16_t ix, iy, sp;
+static uint32_t radjust;
+static uint32_t ixoriy, new_ixoriy;
+static uint32_t intsample;
+static uint8_t op;
 static int interrupted;
 
 void __attribute__ ((visibility ("internal"))) ay_z80_init(unsigned char *data,unsigned char *stacketc)
