@@ -18,12 +18,12 @@
 
 #define instr(opcode,cycles) case opcode: {ay_tstates+=cycles
 #define HLinstr(opcode,cycles,morecycles) \
-                             case opcode: {unsigned short addr; \
+                             case opcode: {uint16_t addr; \
                                 ay_tstates+=cycles; \
                                 if(ixoriy==0)addr=hl; \
                                 else ay_tstates+=morecycles, \
                                    addr=(ixoriy==1?ix:iy)+ \
-                                        (signed char)fetch(pc),\
+                                        (int8_t)fetch(pc),\
                                    pc++
 #define endinstr             }; break
 
@@ -47,36 +47,36 @@
                                          f|=(var&0xa8)|((var==127)<<2)|\
                                             ((!var)<<6)\
                                        )
-#define swap(x,y) {unsigned char t=x; x=y; y=t;}
+#define swap(x,y) {uint8_t t=x; x=y; y=t;}
 #define addhl(hi,lo) /* 16-bit add */ if(!ixoriy){\
-                      unsigned short t;\
+                      uint16_t t;\
                       l=t=l+(lo);\
                       f=(f&0xc4)|(((t>>=8)+(h&0x0f)+((hi)&0x0f)>15)<<4);\
                       h=t+=h+(hi);\
                       f|=(h&0x28)|(t>>8);\
                    }\
-                   else do{unsigned long t=(ixoriy==1?ix:iy);\
+                   else do{uint32_t t=(ixoriy==1?ix:iy);\
                       f=(f&0xc4)|(((t&0xfff)+((hi<<8)|lo)>0xfff)<<4);\
                       t+=(hi<<8)|lo;\
                       if(ixoriy==1)ix=t; else iy=t;\
                       f|=((t>>8)&0x28)|(t>>16);\
                    } while(0)
-#define adda(x,c) /* 8-bit add */ do{unsigned short y;\
-                      unsigned char z=(x);\
+#define adda(x,c) /* 8-bit add */ do{uint16_t y;\
+                      uint8_t z=(x);\
                       y=a+z+(c);\
                       f=(y&0xa8)|(y>>8)|(((a&0x0f)+(z&0x0f)+(c)>15)<<4)|\
                         (((~a^z)&0x80&(y^a))>>5);\
                       f|=(!(a=y))<<6;\
                    } while(0)
-#define suba(x,c) /* 8-bit subtract */ do{unsigned short y;\
-                      unsigned char z=(x);\
+#define suba(x,c) /* 8-bit subtract */ do{uint16_t y;\
+                      uint8_t z=(x);\
                       y=(a-z-(c))&0x1ff;\
                       f=(y&0xa8)|(y>>8)|(((a&0x0f)<(z&0x0f)+(c))<<4)|\
                         (((a^z)&0x80&(y^a))>>5)|2;\
                       f|=(!(a=y))<<6;\
                    } while(0)
-#define cpa(x) /* 8-bit compare */ do{unsigned short y;\
-                      unsigned char z=(x);\
+#define cpa(x) /* 8-bit compare */ do{uint16_t y;\
+                      uint8_t z=(x);\
                       y=(a-z)&0x1ff;\
                       f=(y&0xa8)|(y>>8)|(((a&0x0f)<(z&0x0f))<<4)|\
                         (((a^z)&0x80&(y^a))>>5)|2|((!y)<<6);\
@@ -94,7 +94,7 @@
                       f=(a&0xa8)|((!a)<<6)|parity(a);\
                    } while(0)
 
-#define jr /* execute relative jump */ do{int j=(signed char)fetch(pc);\
+#define jr /* execute relative jump */ do{int j=(int8_t)fetch(pc);\
                       pc+=j+1;\
                       ay_tstates+=5;\
                    } while(0)
@@ -276,7 +276,7 @@ instr(33,10);
 endinstr;
 
 instr(34,16);
-   {unsigned short addr=fetch2(pc);
+   {uint16_t addr=fetch2(pc);
     pc+=2;
     if(!ixoriy)store2b(addr,h,l);
     else if(ixoriy==1)store2(addr,ix);
@@ -292,7 +292,7 @@ endinstr;
 
 instr(36,4);
    if(ixoriy==0)inc(h);
-   else{unsigned char t;
+   else{uint8_t t;
       t=(ixoriy==1?ix:iy)>>8;
       inc(t);
       if(ixoriy==1)ix=(ix&0xff)|(t<<8);
@@ -302,7 +302,7 @@ endinstr;
 
 instr(37,4);
    if(ixoriy==0)dec(h);
-   else{unsigned char t;
+   else{uint8_t t;
       t=(ixoriy==1?ix:iy)>>8;
       dec(t);
       if(ixoriy==1)ix=(ix&0xff)|(t<<8);
@@ -317,7 +317,7 @@ endinstr;
 
 instr(39,4);
    {
-      unsigned char incr=0, carry=cy;
+      uint8_t incr=0, carry=cy;
       if((f&0x10) || (a&0x0f)>9) incr=6;
       if((f&1) || (a>>4)>9) incr|=0x60;
       if(f&2)suba(incr,0);
@@ -341,7 +341,7 @@ instr(41,11);
 endinstr;
 
 instr(42,16);
-  {unsigned short addr=fetch2(pc);
+  {uint16_t addr=fetch2(pc);
    pc+=2;
    if(!ixoriy){
       l=fetch(addr);
@@ -360,7 +360,7 @@ endinstr;
 
 instr(44,4);
    if(!ixoriy)inc(l);
-   else {unsigned char t;
+   else {uint8_t t;
       t=(ixoriy==1?ix:iy);
       inc(t);
       if(ixoriy==1)ix=(ix&0xff00)|t;
@@ -370,7 +370,7 @@ endinstr;
 
 instr(45,4);
    if(!ixoriy)dec(l);
-   else {unsigned char t;
+   else {uint8_t t;
       t=(ixoriy==1?ix:iy);
       dec(t);
       if(ixoriy==1)ix=(ix&0xff00)|t;
@@ -399,7 +399,7 @@ instr(49,10);
 endinstr;
 
 instr(50,13);
-  {unsigned short addr=fetch2(pc);
+  {uint16_t addr=fetch2(pc);
    pc+=2;
    store(addr,a);
   }
@@ -410,14 +410,14 @@ instr(51,6);
 endinstr;
 
 HLinstr(52,11,8);
-  {unsigned char t=fetch(addr);
+  {uint8_t t=fetch(addr);
    inc(t);
    store(addr,t);
   }
 endinstr;
 
 HLinstr(53,11,8);
-  {unsigned char t=fetch(addr);
+  {uint8_t t=fetch(addr);
    dec(t);
    store(addr,t);
   }
@@ -442,7 +442,7 @@ instr(57,11);
 endinstr;
 
 instr(58,13);
-  {unsigned short addr=fetch2(pc);
+  {uint16_t addr=fetch2(pc);
    pc+=2;
    a=fetch(addr);
   }
@@ -1108,7 +1108,7 @@ instr(0xda,10);
 endinstr;
 
 instr(0xdb,11);
-   {unsigned short t;
+   {uint16_t t;
       a=t=ay_in(a,fetch(pc));
       ay_tstates+=t>>8;
       pc++;
@@ -1152,18 +1152,18 @@ endinstr;
 
 instr(0xe3,19);
    if(!ixoriy){
-      unsigned short t=fetch2(sp);
+      uint16_t t=fetch2(sp);
       store2b(sp,h,l);
       l=t;
       h=t>>8;
    }
    else if(ixoriy==1){
-      unsigned short t=fetch2(sp);
+      uint16_t t=fetch2(sp);
       store2(sp,ix);
       ix=t;
    }
    else{
-      unsigned short t=fetch2(sp);
+      uint16_t t=fetch2(sp);
       store2(sp,iy);
       iy=t;
    }
