@@ -69,7 +69,7 @@ static int cd_fd;
 /* TODO, add circular rip buffer */
 static struct cdrom_read_audio rip_ioctl;
 static unsigned char rip_ioctl_buf[CD_FRAMESIZE_RAW*BUFFER_SLOTS];
-static int rip_pcm_left;
+static unsigned int rip_pcm_left;
 
 static void cdIdler(void)
 {
@@ -190,7 +190,8 @@ void __attribute__ ((visibility ("internal"))) cdIdle(void)
 					cdbufpos-=cdbuflen;
 			}
 		} else { /* re-sample intil we don't have more target-buffer or source-buffer */
-			int32_t c0, c1, c2, c3, ls, rs, vm1,v1,v2, wp1, wp2, wpm1;
+			int32_t c0, c1, c2, c3, ls, rs, vm1,v1,v2, wpm1;
+			uint32_t wp1, wp2;
 /*			if ((bufdelta-=2)<0) bufdelta=0;  by my meening, this should be in place   TODO stian */
 			for (i=0; i<bufdelta; i++)
 			{
