@@ -12,6 +12,7 @@
  */
 
 #include "config.h"
+#include <curses.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -193,15 +194,19 @@ static int cdaProcessKey(uint16_t key)
 			cpiKeyHelp('t', "Toggle sector view mode");
 			cpiKeyHelp(KEY_DOWN, "Jump back (small)");
 			cpiKeyHelp(KEY_UP, "Jump forward (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump forward (big)");
 			cpiKeyHelp(KEY_LEFT, "Jump back");
 			cpiKeyHelp(KEY_RIGHT, "Jump forward");
 			cpiKeyHelp(KEY_HOME, "Jump to start of track");
 			cpiKeyHelp('<', "Jump track back");
 			cpiKeyHelp('>', "Jump track forward");
-			cpiKeyHelp(KEY_F9, "Pitch speed down");
-			cpiKeyHelp(KEY_F11, "Pitch speed down");
-			cpiKeyHelp(KEY_F10, "Pitch speed up");
-			cpiKeyHelp(KEY_F12, "Pitch speed up");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump track back");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump track forward");
+			cpiKeyHelp(KEY_F(9), "Pitch speed down");
+			cpiKeyHelp(KEY_F(11), "Pitch speed down");
+			cpiKeyHelp(KEY_F(10), "Pitch speed up");
+			cpiKeyHelp(KEY_F(12), "Pitch speed up");
 			if (plrProcessKey)
 				plrProcessKey(key);
 			return 0;
@@ -250,33 +255,37 @@ static int cdaProcessKey(uint16_t key)
 				break;
 			}
 			for (i=1; i<=cdpTrackNum; i++)
-				if (newpos<(cdpTrackStarts[i]-basesec))
+				if (newpos<(signed)(cdpTrackStarts[i]-basesec))
 					break;
 			newpos=cdpTrackStarts[i-1]-basesec;
 			setnewpos=1;
 			break;
-/* TODO-keys	case 0x8D00: //ctrl-up
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
 			newpos-=60*75;
 			setnewpos=1;
 			break;
-		case 0x9100: //ctrl-down
+		case KEY_CTRL_DOWN:
+		/*case 0x9100: //ctrl-down */
 			newpos+=60*75;
 			setnewpos=1;
-			break;*/
-		case '<' /* CTRL_LEFT TODO-keys */:
+			break;
+		case KEY_CTRL_LEFT:
+		case '<':
 			if (!cdpPlayMode)
 				break;
 			for (i=2; i<=cdpTrackNum; i++)
-				if (newpos<(cdpTrackStarts[i]-basesec))
+				if (newpos<(signed)(cdpTrackStarts[i]-basesec))
 					break;
 			newpos=cdpTrackStarts[i-2]-basesec;
 			setnewpos=1;
 			break;
-		case '>' /* CTRL_RIGH TODO-keys */:
+		case KEY_CTRL_RIGHT:
+		case '>':
 			if (!cdpPlayMode)
 				break;
 			for (i=1; i<=cdpTrackNum; i++)
-				if (newpos<(cdpTrackStarts[i]-basesec))
+				if (newpos<(signed)(cdpTrackStarts[i]-basesec))
 					break;
 			newpos=cdpTrackStarts[i]-basesec;
 			setnewpos=1;
@@ -285,14 +294,14 @@ static int cdaProcessKey(uint16_t key)
 			newpos=0;
 			setnewpos=1;
 			break;*/
-		case KEY_F9:
-		case KEY_F11:
+		case KEY_F(9):
+		case KEY_F(11):
 			if ((speed-=finespeed)<16)
 				speed=16;
 			cdSetSpeed(speed);
 			break;
-		case KEY_F10:
-		case KEY_F12:
+		case KEY_F(10):
+		case KEY_F(12):
 			if ((speed+=finespeed)>2048)
 				speed=2048;
 			cdSetSpeed(speed);
@@ -344,7 +353,7 @@ static int cdaLooped(void)
 	{
 		if (newpos<0)
 			newpos=0;
-		if (newpos>=length)
+		if (newpos>=(signed)length)
 		{
 			if (fsLoopMods)
 				newpos=0;
@@ -428,4 +437,4 @@ static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 
 struct cpifaceplayerstruct cdaPlayer = {cdaOpenFile, cdaCloseFile};
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"playcda", "OpenCP CDA Player (c) 1995-04 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playcda", "OpenCP CDA Player (c) 1995-04 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
