@@ -49,9 +49,9 @@ static uint32_t  flacbufread; /* actually this is the write head, in samples */
 static uint32_t  flacbufpos;  /* read pos, in samples*/
 static uint32_t  flacbuffpos; /* read fine-pos.. when flacbufrate has a fraction */
 /* source info from stream */
-static int flacrate; /* this is the source rate */
+static unsigned int flacrate; /* this is the source rate */
 static int flacstereo; /* this we can ignore I think */
-static int flac_max_blocksize;
+static unsigned int flac_max_blocksize;
 static int flacbits=0;
 
 static uint64_t  flaclastpos;
@@ -105,7 +105,7 @@ static FLAC__SeekableStreamDecoderReadStatus read_callback (
 static FLAC__StreamDecoderReadStatus read_callback (
 	const FLAC__StreamDecoder *decoder,
 	FLAC__byte buffer[],
-	unsigned int *bytes,
+	size_t *bytes,
 	void *client_data)
 #endif
 {
@@ -151,7 +151,7 @@ static FLAC__StreamDecoderWriteStatus write_callback (
 	size of buffer =  frame->header.channels * frame->header.blocksize * frame->header.bits_per_sample / 8
 	buffer is a 2D array with [channel][sample] as index
 */
-	int i;
+	unsigned int i;
 	unsigned short xormask=0x0000;
 
 	if (frame->header.number_type==FLAC__FRAME_NUMBER_TYPE_FRAME_NUMBER)
@@ -258,10 +258,14 @@ static FLAC__StreamDecoderTellStatus tell_callback(
 	FLAC__uint64 *absolute_byte_offset,
 	void *client_data)
 {
-	if ((*absolute_byte_offset=ftell(flacfile))<0)
+	long temp = ftell(flacfile);
+	if (temp<0)
 		return FLAC__STREAM_DECODER_TELL_STATUS_ERROR;
 	else
+	{
+		*absolute_byte_offset = temp;
 		return FLAC__STREAM_DECODER_TELL_STATUS_OK;
+	}
 }
 #endif
 
@@ -500,7 +504,7 @@ void __attribute__ ((visibility ("internal"))) flacClosePlayer(void)
 	decoder = NULL;
 }
 
-void static flacIdler(void)
+static void flacIdler(void)
 {
 	/*
 	fprintf(stderr, "flacIdler\n");
@@ -634,7 +638,8 @@ void __attribute__ ((visibility ("internal"))) flacIdle(void)
 			flacbufpos=(flacbufpos+w)%flacbuflen;
 		}
 	} else { /* re-sample intil we don't have more target-buffer or source-buffer */
-		int32_t c0, c1, c2, c3, ls, rs, vm1, v1, v2, wp1, wp2, wpm1;
+		int32_t c0, c1, c2, c3, ls, rs, vm1, v1, v2, wpm1;
+		uint32_t wp1, wp2;
 		eof_eof=0;
 		for (i=0; i<bufdelta; i++)
 		{
