@@ -74,9 +74,9 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 	uint16_t *ordlist=0;
 	struct sampleinfo **smps=0;
 	struct gmdsample **msmps=0;
-	int *instsampnum;
+	unsigned int *instsampnum;
 	
-	int i,j,t;
+	unsigned int i,j,t;
 
 	uint8_t namelen;
 	unsigned char shadowedby[256];
@@ -90,16 +90,19 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 
 	mpReset(m);
 
-	fread(&sig, 8, 1, file);
+	if (fread(&sig, 8, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #1\n");
 	if (!memcmp(sig, "Extreme", 7))
 		return errFormOldVer;
 	if (memcmp(sig, "AMShdr\x1A", 7))
 		return errFormSig;
 
-	fread(m->name, sig[7], 1, file);
+	if (fread(m->name, sig[7], 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #2\n");
 	m->name[sig[7]]=0;
 
-	fread(&filever, sizeof(filever), 1, file);
+	if (fread(&filever, sizeof(filever), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #3\n");
 	filever = uint16_little (filever);
 	if ((filever!=0x201)&&(filever!=0x202))
 		return errFormOldVer;
@@ -122,7 +125,8 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 			 *  \\------- MIDI channels are used in tune. */
 		} oldhdr;
 		
-		fread(&oldhdr, sizeof(oldhdr), 1, file);
+		if (fread(&oldhdr, sizeof(oldhdr), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #4\n");
 		hdr.ins = oldhdr.ins;
 		hdr.pat = uint16_little (oldhdr.pat);
 		hdr.pos = uint16_little (oldhdr.pos);
@@ -130,7 +134,8 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		hdr.speed = oldhdr.speed;
 		hdr.flags = (oldhdr.flags&0xC0)|0x20;
 	} else {
-		fread(&hdr, sizeof(hdr), 1, file);
+		if (fread(&hdr, sizeof(hdr), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #5\n");
 
 		hdr.pat = uint16_little (hdr.pat);
 		hdr.pos = uint16_little (hdr.pos);
@@ -206,17 +211,20 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		msmps[i]=0;
 		shadowedby[i]=0;
 
-		fread(&namelen, sizeof(namelen), 1, file);
+		if (fread(&namelen, sizeof(namelen), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #6\n");
 		if (namelen>=(sizeof(ip->name-1)))
 		{
 			fprintf(stderr, "AMS: Instrument name too long\n");
 			retval=errFormStruc;
 			goto safeout;
 		}
-		fread(ip->name, namelen, 1, file);
+		if (fread(ip->name, namelen, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #7\n");
 		ip->name[namelen]=0;
 
-		fread(&smpnum, sizeof(smpnum), 1, file);
+		if (fread(&smpnum, sizeof(smpnum), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #8\n");
 		instsampnum[i]=smpnum;
 
 		if (!smpnum)
@@ -233,22 +241,26 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		memset(msmps[i], 0, sizeof(**msmps)*smpnum);
 		memset(smps[i], 0, sizeof(**smps)*smpnum);
 
-		fread(samptab, sizeof(samptab), 1, file);
+		if (fread(samptab, sizeof(samptab), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #9\n");
 		for (j=0; j<3; j++)
 		{
-			fread(&envs[j], 5, 1, file);
+			if (fread(&envs[j], 5, 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #10\n");
 			if (envs[j].points>64)
 			{
 				fprintf(stderr, "AMS: Too many points in envelope\n");
 				retval=errFormStruc;
 				goto safeout;
 			}
-			fread(envs[j].data, envs[j].points*3, 1, file);
+			if (fread(envs[j].data, envs[j].points*3, 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #11\n");
 		}
 
 		vibsweep=0;
 
-		fread(&shadowinst, sizeof(shadowinst), 1, file);
+		if (fread(&shadowinst, sizeof(shadowinst), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #12\n");
 		if (filever==0x201)
 		{
 			vibsweep=shadowinst;
@@ -256,9 +268,11 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		}
 		shadowedby[i]=shadowinst;
 
-		fread(&volfade, sizeof(volfade), 1, file);
+		if (fread(&volfade, sizeof(volfade), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #13\n");
 		volfade = uint16_little (volfade);
-		fread(&envflags, sizeof(envflags), 1, file);
+		if (fread(&envflags, sizeof(envflags), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #14\n");
 		envflags = uint16_little (envflags);
 
 		pchint=(volfade>>12)&3;
@@ -391,21 +405,27 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 			sp->pchenv=0xFFFF;
 			sp->volfade=0xFFFF;
 
-			fread(&namelen, sizeof(namelen), 1, file);
+			if (fread(&namelen, sizeof(namelen), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #15\n");
 			if (namelen>=(sizeof(sp->name)-1))
 			{
 				fprintf(stderr, "AMS: Sample name too long\n");
 				retval=errFormStruc;
 				goto safeout;
 			}
-			fread(sp->name, namelen, 1, file);
+			if (fread(sp->name, namelen, 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #16\n");
 			sp->name[namelen]=0;
-			fread(&sip->length, sizeof(sip->length), 1, file);
+			if (fread(&sip->length, sizeof(sip->length), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #17\n");
+
+			sip->length = uint32_little (sip->length);
 
 			if (!sip->length)
 				continue;
 
-			fread(&amssmp, sizeof(amssmp), 1, file);
+			if (fread(&amssmp, sizeof(amssmp), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #18\n");
 			amssmp.loopstart = uint32_little (amssmp.loopstart);
 			amssmp.loopend   = uint32_little (amssmp.loopend);
 			amssmp.samprate  = uint16_little (amssmp.samprate);
@@ -418,9 +438,9 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 
 			sp->volfade=volfade;
 			sp->pchint=pchint;
-			sp->volenv=m->envelopes[3*i+0].env?(3*i+0):-1;
-			sp->panenv=m->envelopes[3*i+1].env?(3*i+1):-1;
-			sp->pchenv=m->envelopes[3*i+2].env?(3*i+2):-1;
+			sp->volenv=m->envelopes[3*i+0].env?(3*(signed)i+0):-1;
+			sp->panenv=m->envelopes[3*i+1].env?(3*(signed)i+1):-1;
+			sp->pchenv=m->envelopes[3*i+2].env?(3*(signed)i+2):-1;
 
 			sip->loopstart=amssmp.loopstart;
 			sip->loopend=amssmp.loopend;
@@ -429,26 +449,31 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		}
 	}
 
-	fread(&namelen, sizeof(unsigned char), 1, file);
+	if (fread(&namelen, sizeof(unsigned char), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #19\n");
 	if (namelen>=((sizeof(m->composer)-1)))
 	{
 		fprintf(stderr, "AMS: Composer name too long\n");
 		retval=errFormStruc;
 		goto safeout;
 	}
-	fread(m->composer, namelen, 1, file);
+	if (fread(m->composer, namelen, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #20\n");
 	m->composer[namelen]=0;
 	for (i=0; i<32; i++)
 	{
-		fread(&namelen, sizeof(unsigned char), 1, file);
+		if (fread(&namelen, sizeof(unsigned char), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #21\n");
 		fseek(file, namelen, SEEK_CUR);
 	}
 
-	fread(&packlen, sizeof(uint32_t), 1, file);
+	if (fread(&packlen, sizeof(uint32_t), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #22\n");
 	packlen = uint32_little (packlen);
 	fseek(file, packlen-4, SEEK_CUR);
 
-	fread(ordlist, 2*hdr.pos, 1, file);
+	if (fread(ordlist, 2*hdr.pos, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #23\n");
 
 	for (i=0; i<m->ordnum; i++)
 		m->orders[i]=(ordlist[i]<hdr.pat)?ordlist[i]:hdr.pat;
@@ -488,11 +513,15 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		uint16_t len;
 
 
-		fread(&patlen, sizeof(uint32_t), 1, file);
+		if (fread(&patlen, sizeof(uint32_t), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #24\n");
 		patlen = uint32_little(patlen);
-		fread(&maxrow, sizeof(uint8_t), 1, file);
-		fread(&chan, sizeof(uint8_t), 1, file);
-		fread(&namelen, sizeof(uint8_t), 1, file);
+		if (fread(&maxrow, sizeof(uint8_t), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #25\n");
+		if (fread(&chan, sizeof(uint8_t), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #26\n");
+		if (fread(&namelen, sizeof(uint8_t), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #27\n");
 
 		if (namelen>=(sizeof(patname)-1))
 		{
@@ -503,7 +532,8 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 		
 		patlen-=3+namelen;
 
-		fread(patname, namelen, 1, file);
+		if (fread(patname, namelen, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #28\n");
 		patname[namelen]=0;
 		maxcmd=chan>>5;
 		chan&=0x1F;
@@ -529,7 +559,8 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 				goto safeout;
 			}
 		}
-		fread(buffer, patlen, 1, file);
+		if (fread(buffer, patlen, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #29\n");
 
 		for (i=0; i<chan; i++)
 		{
@@ -1044,11 +1075,14 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 			if (!sip->length)
 				continue;
 
-			fread(&packlena, sizeof(uint32_t), 1, file);
+			if (fread(&packlena, sizeof(uint32_t), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #30\n");
 			packlena = uint32_little (packlena);
-			fread(&packlenb, sizeof(uint32_t), 1, file);
+			if (fread(&packlenb, sizeof(uint32_t), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #31\n");
 			packlenb = uint32_little (packlenb);
-			fread(&packbyte, sizeof(uint8_t), 1, file);
+			if (fread(&packbyte, sizeof(uint8_t), 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #32\n");
 
 			packb=malloc(sizeof(unsigned char)*(((packlenb>packlena)?packlenb:packlena)+16));
 			smpp=malloc(sizeof(unsigned char)*(packlena+16));
@@ -1057,7 +1091,8 @@ static int _mpLoadAMS(struct gmdmodule *m, FILE *file)
 				retval=errAllocMem;
 				goto safeout;
 			}
-			fread(packb, packlenb, 1, file);
+			if (fread(packb, packlenb, 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #33\n");
 			
 			p1=p2=0;
 
@@ -1130,7 +1165,7 @@ safeout:
 		free(ordlist);
 	if (temptrack)
 		free(temptrack);
-	if (buffer);
+	if (buffer)
 		free(buffer);
 
 	for (i=0; i<m->instnum; i++)
@@ -1155,4 +1190,4 @@ safeout:
 
 struct gmdloadstruct mpLoadAMS = { _mpLoadAMS };
 
-struct linkinfostruct dllextinfo = {"gmdlams", "OpenCP Module Loader: *.669 (c) 1994-04 Niklas Beisert", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"gmdlams", "OpenCP Module Loader: *.669 (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
