@@ -118,7 +118,7 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		uint8_t pan[32];
 	} mdlhead;
 
-	int i,j,k;
+	unsigned int i,j,k;
 
 	uint8_t ordtab[256];
 	uint8_t patnum;
@@ -131,38 +131,78 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 	uint8_t (*patdata)[256][6];
 	uint8_t *temptrack;
 	int tpos;
-	int smpsav;
+	uint8_t smpsav;
 	uint8_t packtype[255];
 
-	int inssav;
-	struct gmdsample **msmps;
-	int *inssampnum;
-	int maxins;
-	int smpnum;
+	uint8_t inssav;
+	struct gmdsample **msmps = 0;
+	unsigned int *inssampnum = 0;
+	unsigned int maxins;
+	unsigned int smpnum;
+
+	void FreeResources(void)
+	{
+		int j;
+		if (msmps)
+		{
+			for (j=0; j<255; j++)
+			{
+				if (msmps[j])
+					free(msmps[j]);
+			}
+			free(msmps);
+			msmps=0;
+		}
+		if (inssampnum)
+		{
+			free(inssampnum);
+			inssampnum=0;
+		}
+	}
 
       	mpReset(m);
 
-	fread(&waste1, sizeof(uint32_t), 1, file);
+	if (fread(&waste1, sizeof(uint32_t), 1, file)!=1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #1\n");
+		return errFormStruc;
+	}
 	waste1 = uint32_little (waste1);
 	if (waste1!=0x4C444D44)
 		return errFormSig;
 
-	fread(&waste3, sizeof(uint8_t), 1, file);
+	if (fread(&waste3, sizeof(uint8_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #2\n");
+		return errFormStruc;
+	}
 	if ((waste3&0x10)!=0x10)
 	{
 		fprintf(stderr, "Sorry, the file version is too old (load and resave it in DigiTrakker please)\n");
 		return errFormSig;
 	}
 
-	fread(&waste2, sizeof(uint16_t), 1, file);
+	if (fread(&waste2, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #3\n");
+		return errFormStruc;
+	}
 	waste2 = uint16_little (waste2);
 	if (waste2!=0x4E49)
 		return errFormStruc;
 
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #4\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little(blklen);
 
-	fread(&mdlhead, sizeof(mdlhead), 1, file);
+	if (fread(&mdlhead, sizeof(mdlhead), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #5\n");
+		return errFormStruc;
+	}
 	mdlhead.ordnum = uint16_little (mdlhead.ordnum);
 	mdlhead.repstart = uint16_little (mdlhead.repstart);
 
@@ -171,7 +211,7 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			break;
 	m->channum=i;
 	memcpy(m->name, mdlhead.name, 31);
-	m->name[32]=0;
+	m->name[31]=0;
 	memcpy(m->composer, mdlhead.composer, 20);
 	m->composer[20]=0;
 	m->ordnum=mdlhead.ordnum;
@@ -182,21 +222,41 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 	if (mdlhead.ordnum>256)
 		return errFormSupp;
 
-	fread(ordtab, mdlhead.ordnum, 1, file);
+	if (fread(ordtab, mdlhead.ordnum, 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #6\n");
+		return errFormStruc;
+	}
 	fseek(file, 8*m->channum, SEEK_CUR); /* channames */
 	fseek(file, blklen-8*m->channum-91-mdlhead.ordnum, SEEK_CUR);
 
-	fread(&blktype, sizeof(uint16_t), 1, file);
+	if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #7\n");
+		return errFormStruc;
+	}
 	blktype = uint16_little (blktype);
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #8\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little (blklen);
 
 	if (blktype==0x454D)
 	{
 		fseek(file, blklen, SEEK_CUR);
-		fread(&blktype, sizeof(uint16_t), 1, file);
+		if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #9\n");
+			return errFormStruc;
+		}
 		blktype = uint16_little (blktype);
-		fread(&blklen, sizeof(uint32_t), 1, file);
+		if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #10\n");
+			return errFormStruc;
+		}
 		blklen = uint32_little (blklen);
 /* songmessage; every line is closed with the CR-char (13). A
  * 0-byte stands at the end of the whole text.
@@ -205,8 +265,11 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 
 	if (blktype!=0x4150)
 		return errFormStruc;
-	fread(&patnum, sizeof(uint8_t), 1, file);
-
+	if (fread(&patnum, sizeof(uint8_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #11\n");
+		return errFormStruc;
+	}
 	m->patnum=patnum+1;
 	m->tracknum=patnum*(m->channum+1)+1;
 
@@ -217,27 +280,55 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 	{
 		uint8_t chnn;
 		int i;
-		fread(&chnn, sizeof(uint8_t), 1, file);
+		if (fread(&chnn, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #12\n");
+			return errFormStruc;
+		}
 
-		fread(&m->patterns[j].patlen, sizeof(uint8_t), 1, file);
+		if (fread(&m->patterns[j].patlen, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #13\n");
+			return errFormStruc;
+		}
 		m->patterns[j].patlen++;
 
-		fread(m->patterns[j].name, 16, 1, file);
+		if (fread(m->patterns[j].name, 16, 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #14\n");
+			return errFormStruc;
+		}
 		m->patterns[j].name[16]=0;
 		memset(m->patterns[j].tracks, 0, 32*2);
-		fread(m->patterns[j].tracks, 2*chnn, 1, file);
+		if (fread(m->patterns[j].tracks, 2*chnn, 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #15\n");
+			return errFormStruc;
+		}
 		for(i=0;i<chnn;i++)
 			m->patterns[j].tracks[i] = uint16_little (m->patterns[j].tracks[i]);
 	}
 
-	fread(&waste2, sizeof(uint16_t), 1, file);
+	if (fread(&waste2, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #16\n");
+		return errFormStruc;
+	}
 	waste2 = uint16_little (waste2);
 	if (waste2!=0x5254)
 		return errFormStruc;
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #17\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little (blklen);
 
-	fread(&ntracks, sizeof(uint16_t), 1, file);
+	if (fread(&ntracks, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #18\n");
+		return errFormStruc;
+	}
 	ntracks = uint16_little (ntracks);
 
 	trackends=malloc(sizeof(uint8_t *)*ntracks+1);
@@ -256,10 +347,18 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 	for (i=0; i<ntracks; i++)
 	{
 		uint16_t l=0;
-		fread(&l, sizeof(uint16_t), 1, file);
+		if (fread(&l, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #19\n");
+			return errFormStruc;
+		}
 		l = uint16_little (l);
 		trackptrs[1+i]=trackbuf+tpos;
-		fread(trackbuf+tpos, l, 1, file);
+		if (fread(trackbuf+tpos, l, 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #20\n");
+			return errFormStruc;
+		}
 		tpos+=l;
 		trackends[1+i]=trackbuf+tpos;
 	}
@@ -296,7 +395,7 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 						row+=p>>2;
 						break;
 					case 1:
-						for (k=0; k<((p>>2)+1); k++)
+						for (k=0; k<((p>>2)+(unsigned)1); k++)
 							memcpy(patdata[i][row+k], patdata[i][row-1], 6);
 						row+=p>>2;
 						break;
@@ -552,7 +651,7 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		{
 			uint8_t *cp=tp+2;
 
-			int q;
+			unsigned int q;
 
 			if (!row&&(j==ordtab[0]))
 			{
@@ -678,32 +777,45 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 	free(trackbuf);
 	free(patdata);
 
-	fread(&waste2, sizeof(uint16_t), 1, file);
+	if (fread(&waste2, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #21\n");
+		return errFormStruc;
+	}
 	waste2 = uint16_little (waste2);
 	if (waste2!=0x4949)
 		return errFormStruc;
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #22\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little (blklen);
 
 	inssav=0;
-	fread(&inssav, sizeof(uint8_t), 1, file); /* works on x86 atleast */
+	if (fread(&inssav, sizeof(uint8_t), 1, file) != 1) /* works on x86 atleast */
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #23\n");
+		return errFormStruc;
+	}
 
 	m->instnum=255;
 	m->modsampnum=0;
 	m->envnum=192;
 
 /*  envelope **envs=new envelope *[255]; */
-	msmps=malloc(sizeof(struct gmdsample *)*255);
-	inssampnum=malloc(sizeof(int)*255);
+	msmps=calloc(sizeof(struct gmdsample *), 255);
+	inssampnum=calloc(sizeof(int), 255);
 /*  int *insenvnum=new int [255]; */
 	if (/*!envs||!insenvnum||*/!inssampnum||!msmps||!mpAllocInstruments(m, m->instnum))
+	{
+		FreeResources();
 		return errAllocMem;
+	}
 
 	maxins=0;
 
-	memset(msmps, 0, 4*255);
 /*  memset(envs, 0, 4*255); */
-	memset(inssampnum, 0, 4*255);
 /*  memset(insenvnum, 0, 4*255); */
 
 	for (j=0; j<inssav; j++)
@@ -713,18 +825,36 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		int note;
 
 		
-		fread(&insnum, sizeof(uint8_t), 1, file);
+		if (fread(&insnum, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #24\n");
+			FreeResources();
+			return errFormStruc;
+		}
 		insnum--;
 		
 		ip=&m->instruments[insnum];
 
-		fread(&inssampnum[j], sizeof(uint8_t), 1, file);
-		fread(ip->name, 32, 1, file);
+		if (fread(&inssampnum[j], sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #25\n");
+			FreeResources();
+			return errFormStruc;
+		}
+		if (fread(ip->name, 32, 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #26\n");
+			FreeResources();
+			return errFormStruc;
+		}
 		ip->name[31]=0;
 		msmps[j]=malloc(sizeof(struct gmdsample)*inssampnum[j]);
 /*    envs[insnum]=new envelope [inssampnum[j]]; */
 	    	if (!msmps[j]/*||!envs[insnum]*/)
+		{
+			FreeResources();
 			return errAllocMem;
+		}
 
 		note=0;
 		for (i=0; i<inssampnum[j]; i++)
@@ -747,7 +877,12 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			} mdlmsmp;
 			struct gmdsample *sp;
 		
-			fread(&mdlmsmp, sizeof(mdlmsmp), 1, file);
+			if (fread(&mdlmsmp, sizeof(mdlmsmp), 1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #27\n");
+				FreeResources();
+				return errFormStruc;
+			}
 			mdlmsmp.fadeout = uint16_little (mdlmsmp.fadeout);
 			if ((mdlmsmp.highnote+12)>128)
 				mdlmsmp.highnote=128-12;
@@ -825,24 +960,34 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  *    for (i=0; i<insenvnum[j]; i++)
  *      memcpy(&m.envelopes[envnum++], &envs[j][i], sizeof (*m.envelopes));*/
-		free(msmps[j]);
 /*    delete envs[j]; */
 	}
-	free(msmps);
+	FreeResources();
 /*  delete envs;*/
-	free(inssampnum);
 /*  delete insenvnum;*/
 
-	fread(&blktype, sizeof(uint16_t), 1, file);
+	if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #28\n");
+		return errFormStruc;
+	}
 	blktype = uint16_little (blktype);
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #29\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little (blklen);
 
 	if (blktype==0x4556)
 	{
 		uint8_t envnum;
 		
-		fread(&envnum, sizeof(uint8_t), 1, file);
+		if (fread(&envnum, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #30\n");
+			return errFormStruc;
+		}
 		for (i=0; i<envnum; i++)
 		{
 			struct __attribute__((packed))
@@ -855,7 +1000,11 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			struct gmdenvelope *e;
 			int k,l;
 		
-			fread(&env, sizeof(env), 1, file);
+			if (fread(&env, sizeof(env), 1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #31\n");
+				return errFormStruc;
+			}
 			if (env.env[0][0]!=1)
 				continue;
 			e=&m->envelopes[env.num];
@@ -898,16 +1047,28 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			}
 		}
 
-		fread(&blktype, sizeof(uint16_t), 1, file);
+		if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #32\n");
+			return errFormStruc;
+		}
 		blktype = uint16_little (blktype);
-		fread(&blklen, sizeof(uint32_t), 1, file);
+		if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #33\n");
+			return errFormStruc;
+		}
 		blklen = uint32_little (blklen);
 	}
 
 	if (blktype==0x4550)
 	{
 		uint8_t envnum;
-		fread(&envnum, sizeof(uint8_t), 1, file);
+		if (fread(&envnum, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #34\n");
+			return errFormStruc;
+		}
 		for (i=0; i<envnum; i++)
 		{
 			struct __attribute__((packed))
@@ -920,7 +1081,11 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			struct gmdenvelope *e;
 			int k,l;
 
-			fread(&env, sizeof(env),1, file);
+			if (fread(&env, sizeof(env),1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #35\n");
+				return errFormStruc;
+			}
 			if (env.env[0][0]!=1)
 				continue;
 			e=&m->envelopes[64+env.num];
@@ -963,16 +1128,28 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			}
 		}
 
-		fread(&blktype, sizeof(uint16_t), 1, file);
+		if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #36\n");
+			return errFormStruc;
+		}
 		blktype = uint16_little (blktype);
-		fread(&blklen, sizeof(uint32_t), 1, file);
+		if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #37\n");
+			return errFormStruc;
+		}
 		blklen = uint32_little (blklen);
 	}
 
 	if (blktype==0x4546)
 	{
 		uint8_t envnum;
-		fread(&envnum, sizeof(uint8_t), 1, file);
+		if (fread(&envnum, sizeof(uint8_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #38\n");
+			return errFormStruc;
+		}
 
 		for (i=0; i<envnum; i++)
 		{
@@ -987,7 +1164,12 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			struct gmdenvelope *e;
 			int k,l;
 		
-			fread(&env, sizeof(env), 1, file);
+			if (fread(&env, sizeof(env), 1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #39\n");
+				return errFormStruc;
+			}
+
 			if (env.env[0][0]!=1)
 				continue;
 			e=&m->envelopes[128+env.num];
@@ -1030,9 +1212,17 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			}
 		}
 
-		fread(&blktype, sizeof(uint16_t), 1, file);
+		if (fread(&blktype, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #40\n");
+			return errFormStruc;
+		}
 		blktype = uint16_little (blktype);
-		fread(&blklen, sizeof(uint32_t), 1, file);
+		if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #41\n");
+			return errFormStruc;
+		}
 		blklen = uint32_little (blklen);
 	}
 
@@ -1040,8 +1230,11 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		return errFormStruc;
 
 	smpsav=0;
-	fread(&smpsav, sizeof(uint8_t), 1, file); /* works on x86 */
-
+	if (fread(&smpsav, sizeof(uint8_t), 1, file) != 1) /* works on x86 */
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #42\n");
+		return errFormStruc;
+	}
 	memset(packtype, 0xFF, 255);
 
 	for (i=0; i<smpsav; i++)
@@ -1062,7 +1255,11 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		struct sampleinfo *sip;
 
 		
-		fread(&mdlsmp, sizeof(mdlsmp), 1, file);
+		if (fread(&mdlsmp, sizeof(mdlsmp), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #43\n");
+			return errFormStruc;
+		}
 		mdlsmp.rate = uint32_little (mdlsmp.rate);
 		mdlsmp.len = uint32_little (mdlsmp.len);
 		mdlsmp.loopstart = uint32_little (mdlsmp.loopstart);
@@ -1092,11 +1289,19 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 		packtype[mdlsmp.num-1]=(mdlsmp.opt>>2)&3;
 	}
 
-	fread(&waste2, sizeof(uint16_t), 1, file);
+	if (fread(&waste2, sizeof(uint16_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #44\n");
+		return errFormStruc;
+	}
 	waste2 = uint16_little (waste2);
 	if (waste2!=0x4153)
 		return errFormStruc;
-	fread(&blklen, sizeof(uint32_t), 1, file);
+	if (fread(&blklen, sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #45\n");
+		return errFormStruc;
+	}
 	blklen = uint32_little (blklen);
 
 	for (i=0; i<255; i++)
@@ -1119,18 +1324,30 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 				
 		if (packtype[i]==0)
 		{
-			fread(sip->ptr, sip->length<<bit16, 1, file);
+			if (fread(sip->ptr, sip->length<<bit16, 1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #46\n");
+				return errFormStruc;
+			}
 			continue;
 		}
 
-		fread(&packlen, sizeof(uint32_t), 1, file);
+		if (fread(&packlen, sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #47\n");
+			return errFormStruc;
+		}
 		packlen = uint32_little (packlen);
 		packbuf=malloc(sizeof(uint8_t)*(packlen+4));
 
 		if (!packbuf)
 			return errAllocMem;
-		fread(packbuf, packlen, 1, file);
-
+		if (fread(packbuf, packlen, 1, file) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread() failed #48\n");
+			free(packbuf);
+			return errFormStruc;
+		}
 		bitnum=8;
 		ibuf=packbuf;
 		bitlen = packlen;
@@ -1162,7 +1379,6 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 			else
 				((uint16_t *)sip->ptr)[j]=(dlt<<8)|lowbyte;
 		}
-
 		free(packbuf);
 	}
 
@@ -1171,4 +1387,4 @@ static int _mpLoadMDL(struct gmdmodule *m, FILE *file)
 
 struct gmdloadstruct mpLoadMDL = { _mpLoadMDL };
 
-struct linkinfostruct dllextinfo = {"gmdlmdl", "OpenCP Module Loader: *.MDL (c) 1994-04 Niklas Beisert", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"gmdlmdl", "OpenCP Module Loader: *.MDL (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
