@@ -40,19 +40,31 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 		char pan[32];
 	} header;
 
-	int i, t;
+	unsigned int i, t;
 
 	uint8_t orders[128];
 
 	struct gmdpattern *pp;
 	uint32_t filetracks;
-	uint8_t *temptrack;
-	uint8_t *tbuffer;
-	uint16_t (*trackseq)[32];
+	uint8_t *temptrack=0;
+	uint8_t *tbuffer=0;
+	uint16_t (*trackseq)[32]=0;
+
+	int safeout(int err)
+	{
+		if (temptrack)
+			free(temptrack);
+		if (tbuffer)
+			free(tbuffer);
+		if (trackseq)
+			free(trackseq);
+		return err;
+	}
 
 	mpReset(m);
 
-	fread(&header, 66, 1, file);
+	if (fread(&header, 66, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #1\n");
 
 	if ((header.sig&0xFFFFFF)!=0x4D544D)
 		return errFormSig;
@@ -94,7 +106,8 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 			uint8_t volume;
 			uint8_t attr; /* 1=16 bit */
 		} mi;
-		fread(&mi, sizeof(mi), 1, file);
+		if (fread(&mi, sizeof(mi), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #2\n");
 
 		if (mi.length<4)
 			mi.length=0;
@@ -133,7 +146,8 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 		sip->type=((mi.loopend)?mcpSampLoop:0)|((mi.attr&1)?mcpSamp16Bit:0)|mcpSampUnsigned;
 	}
 
-	fread(orders, 128, 1, file);
+	if (fread(orders, 128, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #3\n");
 
 	for (pp=m->patterns, t=0; t<m->patnum; pp++, t++)
 	{
@@ -149,11 +163,13 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 	tbuffer=malloc(sizeof(uint8_t)*(192*header.trknum+192));
 	trackseq=malloc(sizeof(uint16_t)*(header.patnum+1)*32);
 	if (!tbuffer||!temptrack||!trackseq)
-		return errAllocMem;
+		return safeout(errAllocMem);
 
 	memset(tbuffer, 0, 192);
-	fread(tbuffer+192, 192*header.trknum, 1, file);
-	fread(trackseq, 64*(header.patnum+1), 1, file);
+	if (fread(tbuffer+192, 192*header.trknum, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #4\n");
+	if (fread(trackseq, 64*(header.patnum+1), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #5\n");
 
 	for (t=0; t<=header.patnum; t++)
 	{
@@ -367,7 +383,7 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 				trk->ptr=malloc(sizeof(uint8_t)*len);
 				trk->end=trk->ptr+len;
 				if (!trk->ptr)
-					return errAllocMem;
+					return safeout(errAllocMem);
 				memcpy(trk->ptr, temptrack, len);
 			}
 		}
@@ -435,7 +451,7 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 			trk->ptr=malloc(sizeof(uint8_t)*len);
 			trk->end=trk->ptr+len;
 			if (!trk->ptr)
-				return errAllocMem;
+				return safeout(errAllocMem);
 			memcpy(trk->ptr, temptrack, len);
 		}
 	}
@@ -443,6 +459,9 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 	free(temptrack);
 	free(tbuffer);
 	free(trackseq);
+	temptrack=0;
+	tbuffer=0;
+	trackseq=0;
 
       	if (header.comlen&&!(header.comlen%40))
 	{
@@ -458,7 +477,8 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 			int16_t xxx;
 
 			m->message[t]=m->message[0]+t*41;
-			fread(m->message[t], 40, 1, file);
+			if (fread(m->message[t], 40, 1, file) != 1)
+				fprintf(stderr, __FILE__ ": warning, read failed #6\n");
 			for (xxx=0; xxx<40; xxx++)
 				if (!m->message[t][xxx])
 					m->message[t][xxx]=' ';
@@ -481,7 +501,8 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 		sip->ptr=malloc(sizeof(char)*(l+16));
 		if (!sip->ptr)
 			return errAllocMem;
-		fread(sip->ptr, l, 1, file);
+		if (fread(sip->ptr, l, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #7\n");
 	}
 
 	return errOk;
@@ -489,4 +510,4 @@ static int _mpLoadMTM(struct gmdmodule *m, FILE *file)
 
 struct gmdloadstruct mpLoadMTM = { _mpLoadMTM };
 
-struct linkinfostruct dllextinfo = {"gmdlmtm", "OpenCP Module Loader: *.MTM (c) 1994-04 Niklas Beisert", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"gmdlmtm", "OpenCP Module Loader: *.MTM (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
