@@ -27,15 +27,24 @@ static inline void putcmd(uint8_t **p, uint8_t c, uint8_t d)
 static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 {
 
-	int16_t t;
+	uint16_t t;
 	uint8_t orders[256];
 	uint16_t patpara[129];
-	int i,j;
+	unsigned int i,j;
 	struct gmdpattern *pp;
 	uint32_t inspos[256];
 	uint16_t bufSize;
-	uint8_t *buffer;
-	uint8_t *temptrack;
+	uint8_t *buffer = 0;
+	uint8_t *temptrack = 0;
+
+	int safeout(int err)
+	{
+		if (buffer)
+			free(buffer);
+		if (temptrack)
+			free(temptrack);
+		return err;
+	}
 
 	struct __attribute__((packed))
 	{
@@ -51,7 +60,8 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 
 	mpReset(m);
 
-	fread(&hdr, sizeof(hdr), 1, file);
+	if (fread(&hdr, sizeof(hdr), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #1\n");
 	if (memcmp(hdr.magic, "PTMF", 4))
 		return errFormSig;
 	hdr.type = uint16_little(hdr.type);
@@ -74,12 +84,14 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 	m->options=MOD_S3M;
 	m->loopord=0;
 
-	fread(orders, 256, 1, file);
+	if (fread(orders, 256, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #2\n");
 
 	if (!m->patnum)
 		return errFormMiss;
 
-	fread(patpara, 256, 1, file);
+	if (fread(patpara, 256, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #3\n");
 
 	if (!mpAllocInstruments(m, m->instnum)||!mpAllocTracks(m, m->tracknum)||!mpAllocPatterns(m, m->patnum)||!mpAllocSamples(m, m->sampnum)||!mpAllocModSamples(m, m->modsampnum)||!mpAllocOrders(m, m->ordnum))
 		return errAllocMem;
@@ -129,7 +141,8 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 			int32_t magic;
 		} sins;
 
-		fread(&sins, sizeof(sins), 1, file);
+		if (fread(&sins, sizeof(sins), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #4\n");
 		sins.samprate  = uint16_little (sins.samprate);
 		sins.d1        = uint16_little (sins.d1);
 		sins.offset    = uint32_little (sins.offset);
@@ -185,7 +198,7 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 	buffer=malloc(sizeof(uint8_t)*bufSize);
 	temptrack=malloc(sizeof(uint8_t)*2000);
 	if (!temptrack||!buffer)
-		return errAllocMem;
+		return safeout(errAllocMem);
 
 	for (t=0; t<hdr.pats; t++)
 	{
@@ -206,9 +219,10 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 			free(buffer);
 			buffer=malloc(sizeof(uint8_t)*bufSize);
 			if (!buffer)
-				return errAllocMem;
+				return safeout(errAllocMem);
 		}
-		fread(buffer, patSize, 1, file);
+		if (fread(buffer, patSize, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #5\n");
 
 		for (j=0; j<m->channum; j++)
 		{
@@ -482,7 +496,7 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 				trk->ptr=malloc(sizeof(uint8_t)*len);
 				trk->end=trk->ptr+len;
 				if (!trk->ptr)
-					return errAllocMem;
+					return safeout(errAllocMem);
 				memcpy(trk->ptr, temptrack, len);
 			}
 		}
@@ -506,7 +520,7 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 			uint8_t c=*bp++;
 			uint8_t command;
 			uint8_t data;
-			int curchan;
+			unsigned int curchan;
 
 			if (!c)
 			{
@@ -581,12 +595,14 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 			trk->ptr=malloc(sizeof(uint8_t)*len);
 			trk->end=trk->ptr+len;
 			if (!trk->ptr)
-				return errAllocMem;
+				return safeout(errAllocMem);
 			memcpy(trk->ptr, temptrack, len);
 		}
 	}
 	free(buffer);
 	free(temptrack);
+	buffer=0;
+	temptrack=0;
 
       	for (i=0; i<m->instnum; i++)
 	{
@@ -606,7 +622,8 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 		sip->ptr=malloc(sizeof(uint8_t)*(slen+16));
 		if (!sip->ptr)
 			return errAllocMem;
-		fread(sip->ptr, slen, 1, file);
+		if (fread(sip->ptr, slen, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #6\n");
 		x=0;
 		for (j=0; j<slen; j++)
 			((unsigned char *)sip->ptr)[j]=x+=((unsigned char*)sip->ptr)[j];
@@ -617,4 +634,4 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 
 struct gmdloadstruct mpLoadPTM = { _mpLoadPTM };
 
-struct linkinfostruct dllextinfo = {"gmdlptm", "OpenCP Module Loader: *.PTM (c) 1994-04 Niklas Beisert", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"gmdlptm", "OpenCP Module Loader: *.PTM (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
