@@ -30,7 +30,7 @@ static inline void putcmd(uint8_t **p, uint8_t c, uint8_t d)
 
 static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 {
-	int t,i;
+	unsigned int t,i;
 	uint8_t orders[256];
 	uint16_t inspara[256];
 	uint16_t patpara[256];
@@ -61,10 +61,11 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 	mpReset(m);
 
 #ifdef S3M_LOAD_DEBUG
-	fprintf(stderr, "Reading header: %d bytes\n", sizeof(hdr));
+	fprintf(stderr, "Reading header: %d bytes\n", (int)sizeof(hdr));
 #endif
 
-	fread(&hdr, sizeof(hdr), 1, file);
+	if (fread(&hdr, sizeof(hdr), 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #1\n");
 	hdr.d1      = uint16_little (hdr.d1);
 	hdr.orders  = uint16_little (hdr.orders);
 	hdr.ins     = uint16_little (hdr.ins);
@@ -108,10 +109,11 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		m->options|=MOD_MODPAN;
 
 #ifdef S3M_LOAD_DEBUG
-	fprintf(stderr, "Reading pattern orders, %d bytes\n", m->patnum);
+	fprintf(stderr, "Reading pattern orders, %d bytes\n", (int)m->patnum);
 #endif
-	fread(orders, m->patnum, 1, file);
-      	for (t=m->patnum-1; t>=0; t--)
+	if (fread(orders, m->patnum, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #2\n");
+      	for (t=m->patnum-1; (signed)t>=0; t--)
 	{
 		if (orders[t]<254)
 			break;
@@ -142,21 +144,19 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 	m->endord=t;
 
 #ifdef S3M_LOAD_DEBUG
-	fprintf(stderr, "Reading instruments-parameters, %d bytes\n", m->instnum*2);
+	fprintf(stderr, "Reading instruments-parameters, %d bytes\n", (int)m->instnum*2);
 #endif
-	fread(inspara, m->instnum*2, 1, file);
-	{
-		for (i=0;i<(m->instnum*2);i++)
-			inspara[i] = uint16_little (inspara[i]);
-	}
+	if (fread(inspara, m->instnum*2, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #2\n");
+	for (i=0;i<(m->instnum*2);i++)
+		inspara[i] = uint16_little (inspara[i]);
 #ifdef S3M_LOAD_DEBUG
-	fprintf(stderr, "Reading pattern-parameters, %d bytes\n", hdr.pats*2);
+	fprintf(stderr, "Reading pattern-parameters, %d bytes\n", (int)hdr.pats*2);
 #endif
-	fread(patpara, hdr.pats*2, 1, file);
-	{
-		for (i=0;i<(hdr.pats*2);i++)
-			patpara[i] = uint16_little (patpara[i]);
-	}
+	if (fread(patpara, hdr.pats*2, 1, file) != 1)
+		fprintf(stderr, __FILE__ ": warning, read failed #3\n");
+	for (i=0;i<(hdr.pats*(unsigned)2);i++)
+		patpara[i] = uint16_little (patpara[i]);
 
 	/*  hdr.mm|=0x80; */
 	for (i=0; i<32; i++)
@@ -164,9 +164,10 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 	if (hdr.dp==0xFC)
 	{
 #ifdef S3M_LOAD_DEBUG
-		fprintf(stderr, "Reading default panning, %d bytes\n", 32);
+		fprintf(stderr, "Reading default panning, 32 bytes\n");
 #endif
-		fread(defpan, 32, 1, file);
+		if (fread(defpan, 32, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #4\n");
 	}
 	for (i=0; i<32; i++)
 		defpan[i]=(defpan[i]&0x20)?((defpan[i]&0xF)*0x11):((hdr.mm&0x80)?((hdr.channels[i]&8)?0xCC:0x33):0x80);
@@ -204,7 +205,7 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		struct gmdinstrument *ip;
 		struct gmdsample *sp;
 		struct sampleinfo *sip;
-		int j;
+		unsigned int j;
 
 		struct __attribute__((packed))
 		{
@@ -226,13 +227,14 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		} sins;
 
 #ifdef S3M_LOAD_DEBUG
-		fprintf(stderr, "Seeking to (%d/%d)  %d\n", i, m->instnum, ((int32_t)inspara[i])*16);
+		fprintf(stderr, "Seeking to (%d/%d)  %d\n", i, (int)m->instnum, (int)((int32_t)inspara[i])*16);
 #endif
 		fseek(file, ((int32_t)inspara[i])*16, SEEK_SET);
 #ifdef S3M_LOAD_DEBUG
-		fprintf(stderr, "Reading %d bytes of instrument header\n", sizeof(sins));
+		fprintf(stderr, "Reading %d bytes of instrument header\n", (int)sizeof(sins));
 #endif
-		fread(&sins, sizeof(sins), 1, file);
+		if (fread(&sins, sizeof(sins), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #5\n");
 		sins.sampptr   = uint16_little (sins.sampptr);
 		sins.length    = uint32_little (sins.length);
 		sins.loopstart = uint32_little (sins.loopstart);
@@ -242,7 +244,7 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		if ((sins.magic!=0x53524353)&&(sins.magic!=0))
 		{
 #ifdef S3M_LOAD_DEBUG
-			fprintf(stderr, "S3M: Invalid magic on sample (%08x)\n", sins.magic);
+			fprintf(stderr, "S3M: Invalid magic on sample (%08x)\n", (int)sins.magic);
 #endif
 			return errFormStruc;
 		}
@@ -293,7 +295,6 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #ifdef S3M_LOAD_DEBUG
 		fprintf(stderr, "S3M: Out of mem (2.1)\n");
 #endif
-		mpFree(m);
 		return errAllocMem;
 	}
 	if (!(temptrack=malloc(sizeof(uint8_t)*2000)))
@@ -302,7 +303,6 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		fprintf(stderr, "S3M: Out of mem (2)\n");
 #endif
 		free(buffer);
-		mpFree(m);
 		return errAllocMem;
 	}
 
@@ -311,7 +311,7 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 	for (t=0; t<hdr.pats; t++)
 	{
 		uint16_t patsize;
-		int16_t j;
+		uint16_t j;
 		uint8_t *tp;
 		uint8_t *bp;
 		uint8_t *cp;
@@ -320,7 +320,8 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		uint16_t len;
 
 		fseek(file, patpara[t]*16, SEEK_SET);
-		fread(&patsize, sizeof(uint16_t), 1, file);
+		if (fread(&patsize, sizeof(uint16_t), 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #6\n");
 		patsize = uint16_little (patsize);
 		if (patsize>bufsize)
 		{
@@ -333,12 +334,12 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #endif
 				free(buffer);
 				free(temptrack);
-				mpFree(m);
 				return errAllocMem;
 			}
 			buffer=new;
 		}
-		fread(buffer, patsize, 1, file);
+		if (fread(buffer, patsize, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #7\n");
 
 		for (j=0; j<m->channum; j++)
 		{
@@ -605,7 +606,6 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #endif
 					free(buffer);
 					free(temptrack);
-					mpFree(m);
 					return errAllocMem;
 				}
 				memcpy(trk->ptr, temptrack, len);
@@ -631,7 +631,7 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 		{
 			uint8_t command=0;
 			uint8_t data=0;
-			int curchan;
+			unsigned int curchan;
 
 			uint8_t c=*bp++;
 			if (!c)
@@ -718,7 +718,6 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #endif
 				free(buffer);
 				free(temptrack);
-				mpFree(m);
 				return errAllocMem;
 			}
 			memcpy(trk->ptr, temptrack, len);
@@ -744,13 +743,13 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #ifdef S3M_LOAD_DEBUG
 			fprintf(stderr, "S3M: Out of mem (7)?\n");
 #endif
-			mpFree(m);
 			return errAllocMem;
 		}
-		fread(sip->ptr, l, 1, file);
+		if (fread(sip->ptr, l, 1, file) != 1)
+			fprintf(stderr, __FILE__ ": warning, read failed #8\n");
 	}
 
-	for (i=m->channum-1; i>=0; i--)
+	for (i=m->channum-1; (signed)i>=0; i--)
 	{
 		if (chanused[i])
 			break;
@@ -762,7 +761,6 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 #ifdef S3M_LOAD_DEBUG
 		fprintf(stderr, "S3M: No channels left after optimize\n");
 #endif
-		mpFree(m);
 		return errFormMiss;
 	}
 
@@ -771,4 +769,4 @@ static int _mpLoadS3M(struct gmdmodule *m, FILE *file)
 
 struct gmdloadstruct mpLoadS3M = { _mpLoadS3M };
 
-struct linkinfostruct dllextinfo = {"gmdls3m", "OpenCP Module Loader: *.669 (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"gmdls3m", "OpenCP Module Loader: *.669 (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
