@@ -86,8 +86,8 @@ struct trackdata
 	uint8_t lastvolsl;
 	uint8_t lastpitchsl;
 	int8_t glissando;
-	int newpos;
-	int newposend;
+	uint_fast32_t newpos;
+	uint_fast32_t newposend;
 	int newdir;
 	int newloop;
 	int newinst;
@@ -193,7 +193,7 @@ static int quewpos;
 static int quelen;
 
 #define HPITCHMIN (6848>>6)
-#define HPITCHMAX ((unsigned long)6848<<6)
+#define HPITCHMAX ((int32_t)6848<<6)
 #define EPITCHMIN -72*256
 #define EPITCHMAX 96*256
 
@@ -375,7 +375,7 @@ static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
 
 	if (nte!=-1)
 	{
-		int pos=0;
+		uint_fast32_t pos=0;
 
 		t->nteval=nte;
 		t->notehit=1;
@@ -1126,7 +1126,7 @@ static void PlayTick(void)
 		td->finalvol=td->vol;
 		td->finalpan=td->pan;
 		td->finalpitch=td->pitch;
-		td->newpos=-1;
+		td->newpos=(uint_fast32_t)-1;
 		td->newposend=0;
 		td->newdir=-1;
 		td->newloop=-1;
@@ -1201,7 +1201,7 @@ static void PlayTick(void)
 					tdata[i].num=i;
 					tdata[i].chanvol=0xFF;
 					tdata[i].finalpan=tdata[i].pan=(i&1)?255:0;
-					tdata[i].newpos=-1;
+					tdata[i].newpos=(uint_fast32_t)-1;
 					tdata[i].newloop=-1;
 					tdata[i].newdir=-1;
 					tdata[i].newinst=-1;
@@ -1393,7 +1393,7 @@ static void PlayTick(void)
 			}
 
 			td->vibsweeppos+=fs->vibsweep;
-			if (td->vibsweeppos<0)
+			if (td->vibsweeppos<0) /* currently not needed, check is compiled out anyways */
 				td->vibsweeppos=0;
 			if (td->vibsweeppos>0x10000)
 				td->vibsweeppos=0x10000;
@@ -1458,9 +1458,9 @@ static void PlayTick(void)
 		if (td->newinst!=-1)
 			if (td->phys!=-1)
 				mcpSet(td->phys, mcpCInstrument, td->newinst);
-		if (td->newpos!=-1)
+		if (td->newpos!=(uint_fast32_t)-1)
 		{
-			uint32_t l;
+			uint_fast32_t l;
 			const struct sampleinfo *sm;
 			if (td->phys==-1)
 			{
@@ -1744,7 +1744,7 @@ void __attribute__ ((visibility ("internal"))) mpLockPat(int st)
 		lockpattern=-1;
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetChanSample(int ch, int16_t *buf, int len, uint32_t rate, int opt)
+int __attribute__ ((visibility ("internal"))) mpGetChanSample(unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
 {
 	if (tdata[ch].phys==-1)
 	{
