@@ -247,17 +247,23 @@ static int gmdProcessKey(unsigned short key)
 	switch (key)
 	{
 		case KEY_ALT_K:
+			cpiKeyHelp(KEY_ALT_L, "Pattern lock toggle");
 			cpiKeyHelp('p', "Start/stop pause with fade");
 			cpiKeyHelp('P', "Start/stop pause with fade");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump back (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump forward (small)");
 			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
-			cpiKeyHelp('<', "Jump to previous track");
-			cpiKeyHelp('>', "Jump to next track");
+			cpiKeyHelp('<', "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump back (big)");
+			cpiKeyHelp('>', "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump forward (big)");
 			mcpSetProcessKey(key);
 			if (mcpProcessKey)
 				mcpProcessKey(key);
 			return 0;
 #if 0
-		case 0x1900: /* TODO keys */
+		case KEY_ALT_P:
+		/* case 0x1900: alt-p */
 			while (!dkbhit())
 			{
 				if (mcpIdle)
@@ -280,8 +286,8 @@ static int gmdProcessKey(unsigned short key)
 			mcpSet(-1, mcpMasterPause, plPause^=1);
 			plChanChanged=1;
 			break;
-/* TODO-keys
-		case 0x7700: //ctrl-home
+/*
+		case 0x7700: //ctrl-home TODO keys
 			gmdInstClear();
 
 			mpSetPosition(0, 0);
@@ -292,27 +298,32 @@ static int gmdProcessKey(unsigned short key)
 				starttime=dos_clock();
 			break;*/
 		case '<':
-		/*		case 0x7300: //ctrl-left  todo keys */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			mpGetPosition(&pat, &row);
 			mpSetPosition(pat-1, 0);
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			mpGetPosition(&pat, &row);
 			mpSetPosition(pat+1, 0);
 			break;
-/*		case 0x8D00: // ctrl-up
-			mpGetPosition(pat, row);
+		case KEY_CTRL_UP:
+		/*case 0x8D00: // ctrl-up */
+			mpGetPosition(&pat, &row);
 			mpSetPosition(pat, row-8);
 			break;
-		case 0x9100: //ctrl-down
-			mpGetPosition(pat, row);
+		case KEY_CTRL_DOWN:
+		/*case 0x9100: //ctrl-down */
+			mpGetPosition(&pat, &row);
 			mpSetPosition(pat, row+8);
 			break;
-		case 0x2600:
+		case KEY_ALT_L:
+		/* case 0x2600: alt-l */
 			patlock=!patlock;
 			mpLockPat(patlock);
-			break;*/
+			break;
 		default:
 			if (mcpSetProcessKey(key))
 				return 1;
@@ -351,7 +362,7 @@ static int gmdLooped(void)
 
 static int gmdOpenFile(const char *path, struct moduleinfostruct *info, FILE *file)
 {
-	int i;
+	unsigned int i;
 	int retval;
 	char _modname[NAME_MAX+1];
 	char _modext[NAME_MAX+1];
@@ -467,4 +478,4 @@ static int gmdOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 struct cpifaceplayerstruct gmdPlayer = {gmdOpenFile, gmdCloseFile};
 
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"playgmd", "OpenCP General Module Player (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playgmd", "OpenCP General Module Player (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
