@@ -20,7 +20,7 @@
 void __attribute__ ((visibility ("internal"))) mpOptimizePatLens(struct gmdmodule *m)
 {
 	uint8_t *lastrows=malloc(sizeof(uint8_t)*m->patnum);
-	int i;
+	unsigned int i;
 
 	if (!lastrows)
 		return;
@@ -36,7 +36,7 @@ void __attribute__ ((visibility ("internal"))) mpOptimizePatLens(struct gmdmodul
 		while (t.ptr<t.end)
 		{
 			uint8_t row=*t.ptr;
-			int16_t newpat=-1;
+			uint16_t newpat=0xffff;
 			uint8_t newrow=0;
 			uint8_t *end;
 			t.ptr+=2;
@@ -49,12 +49,12 @@ void __attribute__ ((visibility ("internal"))) mpOptimizePatLens(struct gmdmodul
 						newrow=0;
 						break;
 					case cmdBreak:
-						if (newpat==-1)
+						if (newpat==0xffff)
 							newpat=i+1;
 						newrow=t.ptr[1];
 						break;
 				}
-			if (newpat!=-1)
+			if (newpat!=0xffff)
 			{
 				while ((newpat<m->ordnum)&&(m->orders[newpat]==0xFFFF))
 					newpat++;
@@ -91,7 +91,7 @@ void __attribute__ ((visibility ("internal"))) mpOptimizePatLens(struct gmdmodul
 
 void __attribute__ ((visibility ("internal"))) mpReduceInstruments(struct gmdmodule *m)
 {
-	int i,j;
+	unsigned int i,j;
 	char *inptr;
 	for (i=0; i<m->modsampnum; i++)
 	{
@@ -111,7 +111,7 @@ void __attribute__ ((visibility ("internal"))) mpReduceInstruments(struct gmdmod
 				ins->samples[j]=0xFFFF;
 	}
 
-	for (i=m->instnum-1; i>=0; i--)
+	for (i=m->instnum-1; (signed)i>=0; i--)
 	{
 		struct gmdinstrument *ins=&m->instruments[i];
 		for (j=0; j<128; j++)
@@ -160,7 +160,7 @@ void __attribute__ ((visibility ("internal"))) mpReduceMessage(struct gmdmodule
 int __attribute__ ((visibility ("internal"))) mpReduceSamples(struct gmdmodule *m)
 {
 	uint16_t *rellist=malloc(sizeof(uint16_t)*m->sampnum);
-	int i,n;
+	unsigned int i,n;
 
 	if (!rellist)
 		return 0;
@@ -188,7 +188,7 @@ int __attribute__ ((visibility ("internal"))) mpReduceSamples(struct gmdmodule *
 
 void __attribute__ ((visibility ("internal"))) mpRemoveText(struct gmdmodule *m)
 {
-	int i;
+	unsigned int i;
 
 	*m->name=0;
 	*m->composer=0;
@@ -222,7 +222,7 @@ void mpReset(struct gmdmodule *m)
 
 void mpFree(struct gmdmodule *m)
 {
-	int i;
+	unsigned int i;
 
 	if (m->envelopes)
 		for (i=0; i<m->envnum; i++)
@@ -250,7 +250,7 @@ void mpFree(struct gmdmodule *m)
 
 int mpAllocInstruments(struct gmdmodule *m, int n)
 {
-	int i;
+	unsigned int i;
 
 	m->instnum=n;
 	m->instruments=malloc(sizeof(struct gmdinstrument)*m->instnum);
@@ -314,7 +314,7 @@ int mpAllocOrders(struct gmdmodule *m, int n)
 
 int mpAllocModSamples(struct gmdmodule *m, int n)
 {
-	int i;
+	unsigned int i;
 	
 	m->modsampnum=n;
 	m->modsamples=malloc(sizeof(struct gmdsample)*m->modsampnum);
