@@ -10,6 +10,7 @@
 
 #include "config.h"
 #include <errno.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "types.h"
@@ -268,14 +269,23 @@ static int loadFFF(FILE *file)
 
 	/* pass one: read the .FFF file */
 	if (fread(&hd, sizeof(hd), 1, file)!=1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #1\n");
 		return 0;
+	}
 	FFF_CHUNK_HEADER_endian (&hd);
 
       	if (memcmp(hd.id, "FFFF", 4))
+	{
+		fprintf(stderr, __FILE__ ": invalid header\n");
 		return 0;
+	}
 
 	if (fread(&hd, sizeof(hd), 1, file)!=1)
+	{
+		fprintf(stderr, __FILE__ ": fread() failed #2\n");
 		return 0;
+	}
 	FFF_CHUNK_HEADER_endian (&hd);
 
 	while (1) /*(strncmp(hd.id, "FFFF", 4)) ryg, i don't understand your code... (fd)  me too :) (ryg) */
@@ -298,30 +308,30 @@ static int loadFFF(FILE *file)
 			l->next=envp_list;
 			envp_list=l;
 
-			if (fread(&l->chunk->id.id,         sizeof(uint32_t), 1, file)!=1) return 0;
+			if (fread(&l->chunk->id.id,         sizeof(uint32_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #3\n"); return 0;}
 			l->chunk->id.id = uint32_little (l->chunk->id.id);
-			if (fread(&l->chunk->num_envelopes, sizeof(uint8_t),  1, file)!=1) return 0;
-			if (fread(&l->chunk->flags,         sizeof(uint8_t),  1, file)!=1) return 0;
-			if (fread(&l->chunk->mode,          sizeof(uint8_t),  1, file)!=1) return 0;
-			if (fread(&l->chunk->indtype,       sizeof(uint8_t),  1, file)!=1) return 0;
+			if (fread(&l->chunk->num_envelopes, sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #4\n"); return 0;}
+			if (fread(&l->chunk->flags,         sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #5\n"); return 0;}
+			if (fread(&l->chunk->mode,          sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #6\n"); return 0;}
+			if (fread(&l->chunk->indtype,       sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #7\n"); return 0;}
 
 			l->chunk->records=calloc(sizeof(struct FFF_ENVELOPE_RECORD), l->chunk->num_envelopes);
 			lcr=l->chunk->records;
 
 			for (i=0; i<l->chunk->num_envelopes; i++)
 			{
-				if (fread(&lcr[i].nattack,        sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lcr[i].nattack,        sizeof(uint16_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #8\n"); return 0;}
 				lcr[i].nattack = int16_little (lcr[i].nattack);
-				if (fread(&lcr[i].nrelease,       sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lcr[i].nrelease,       sizeof(uint16_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #9\n"); return 0;}
 				lcr[i].nrelease = int16_little (lcr[i].nrelease);
-				if (fread(&lcr[i].sustain_offset, sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lcr[i].sustain_offset, sizeof(uint16_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #10\n"); return 0;}
 				lcr[i].sustain_offset = uint16_little (lcr[i].sustain_offset);
-				if (fread(&lcr[i].sustain_rate,   sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lcr[i].sustain_rate,   sizeof(uint16_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #11\n"); return 0;}
 				lcr[i].sustain_rate = uint16_little (lcr[i].sustain_rate);
-				if (fread(&lcr[i].release_rate,   sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lcr[i].release_rate,   sizeof(uint16_t), 1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #12\n"); return 0;}
 				lcr[i].release_rate = uint16_little (lcr[i].release_rate);
-				if (fread(&lcr[i].hirange,        sizeof(uint8_t),  1, file)!=1) return 0;
-				if (fread(&lcr[i].d0,             sizeof(uint8_t),  1, file)!=1) return 0;
+				if (fread(&lcr[i].hirange,        sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #13\n"); return 0;}
+				if (fread(&lcr[i].d0,             sizeof(uint8_t),  1, file)!=1) {fprintf(stderr, __FILE__ ": fread() failed #14\n"); return 0;}
 
 				if (lcr[i].nattack)
 					lcr[i].attack_points=calloc(sizeof(struct FFF_ENVELOPE_POINT), lcr[i].nattack);
@@ -335,13 +345,19 @@ static int loadFFF(FILE *file)
 				if (lcr[i].nattack)
 				{
 					if (fread(lcr[i].attack_points, sizeof(struct FFF_ENVELOPE_POINT)*lcr[i].nattack, 1, file)!=1)
+					{
+						fprintf(stderr, __FILE__ ": fread() failed #15\n");
 						return 0;
+					}
 					FFF_ENVELOPE_POINT_endian(lcr[i].attack_points, lcr[i].nattack);
 				}
 				if (lcr[i].nrelease)
 				{
 					if (fread(lcr[i].release_points, sizeof(struct FFF_ENVELOPE_POINT)*lcr[i].nrelease, 1, file)!=1)
+					{
+						fprintf(stderr, __FILE__ ": fread() failed #16\n");
 						return 0;
+					}
 					FFF_ENVELOPE_POINT_endian(lcr[i].release_points, lcr[i].nrelease);
 				}
 			}
@@ -353,7 +369,11 @@ static int loadFFF(FILE *file)
 
 			matched=1;
 
-			fread(dummy, sizeof(dummy), 1, file);
+			if (fread(dummy, sizeof(dummy), 1, file) != 1)
+			{
+				fprintf(stderr, __FILE__ ": fread() failed #17\n");
+				return 0;
+			}
 #ifdef DEBUG
 			fprintf(stderr, "[FFF] skipped program\n");
 #endif
@@ -378,23 +398,23 @@ static int loadFFF(FILE *file)
 			ptch_list=l;
 			c=l->chunk;
 
-			if (fread(&c->id.id, sizeof(uint32_t), 1, file)!=1) return 0;
+			if (fread(&c->id.id,         sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #17\n"); return 0;}
 			c->id.id = uint32_little (c->id.id);
-			if (fread(&c->nlayers, sizeof(uint16_t), 1, file)!=1) return 0;
+			if (fread(&c->nlayers,       sizeof(uint16_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #18\n"); return 0;}
 			c->nlayers = int16_little (c->nlayers);
-			if (fread(&c->layer_mode, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->excl_mode, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->excl_group, sizeof(uint16_t), 1, file)!=1) return 0;
+			if (fread(&c->layer_mode,    sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #19\n"); return 0;}
+			if (fread(&c->excl_mode,     sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #20\n"); return 0;}
+			if (fread(&c->excl_group,    sizeof(uint16_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #21\n"); return 0;}
 			c->excl_group = int16_little (c->excl_group);
-			if (fread(&c->effect1, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->effect1_depth, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->effect2, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->effect2_depth, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->bank, sizeof(uint8_t), 1, file)!=1) return 0;
-			if (fread(&c->program, sizeof(uint8_t), 1, file)!=1) return 0;
+			if (fread(&c->effect1,       sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #22\n"); return 0;}
+			if (fread(&c->effect1_depth, sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #23\n"); return 0;}
+			if (fread(&c->effect2,       sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #24\n"); return 0;}
+			if (fread(&c->effect2_depth, sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #25\n"); return 0;}
+			if (fread(&c->bank,          sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #26\n"); return 0;}
+			if (fread(&c->program,       sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #27\n"); return 0;}
 
 			/* actually, iw_layer isn't used on disk, but it's there (fd) yup (ryg) */
-			fread(&dummy, sizeof(uint32_t), 1, file);
+			if (fread(&dummy,            sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #28\n"); return 0;}
 
 #ifdef DEBUG
 			fprintf(stderr, "[FFF] %d:%d loading, %d layers.\n", c->bank, c->program, c->nlayers);
@@ -410,7 +430,7 @@ static int loadFFF(FILE *file)
 
 				lr=&c->iw_layer[i];
 
-				if (fread(&lrhd, sizeof(lrhd), 1, file)!=1) return 0;
+				if (fread(&lrhd, sizeof(lrhd), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #29\n"); return 0;}
 				FFF_CHUNK_HEADER_endian (&lrhd);
 
 				/* sachmal ryg, wielange warste schon auf als du DAS hier gecoded hast? merke:
@@ -423,30 +443,30 @@ static int loadFFF(FILE *file)
 					return 0;
 				}
 
-				if (fread(&lr->id.id, sizeof(uint32_t), 1, file)!=1) return 0;
+				if (fread(&lr->id.id,          sizeof(uint32_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #30\n"); return 0;}
 				lr->id.id = uint32_little(lr->id.id);
-				if (fread(&lr->nwaves, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->flags, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->high_range, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->low_range, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->pan, sizeof(uint8_t), 1, file)!=1) return 0;
-                                if (fread(&lr->pan_freq_scale, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->tremolo, sizeof(struct FFF_LFO), 1, file)!=1) return 0;
+				if (fread(&lr->nwaves,         sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #31\n"); return 0;}
+				if (fread(&lr->flags,          sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #32\n"); return 0;}
+				if (fread(&lr->high_range,     sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #33\n"); return 0;}
+				if (fread(&lr->low_range,      sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #34\n"); return 0;}
+				if (fread(&lr->pan,            sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #35\n"); return 0;}
+                                if (fread(&lr->pan_freq_scale, sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #36\n"); return 0;}
+				if (fread(&lr->tremolo,        sizeof(struct FFF_LFO), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #37\n"); return 0;}
 				FFF_LFO_endian (&lr->tremolo);
-				if (fread(&lr->vibrato, sizeof(struct FFF_LFO), 1, file)!=1) return 0;
+				if (fread(&lr->vibrato,        sizeof(struct FFF_LFO), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #38\n"); return 0;}
 				FFF_LFO_endian (&lr->vibrato);
-				if (fread(&lr->velocity_mode, sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->attenuation,  sizeof(uint8_t), 1, file)!=1) return 0;
-				if (fread(&lr->freq_scale, sizeof(uint16_t), 1, file)!=1) return 0;
+				if (fread(&lr->velocity_mode,  sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #39\n"); return 0;}
+				if (fread(&lr->attenuation,    sizeof(uint8_t),        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #40\n"); return 0;}
+				if (fread(&lr->freq_scale,     sizeof(uint16_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #41\n"); return 0;}
 				lr->freq_scale = int16_little (lr->freq_scale);
-				if (fread(&lr->freq_center, sizeof(uint16_t), 1, file)!=1) return 0;
-				if (fread(&lr->layer_event, sizeof(uint16_t), 1, file)!=1) return 0;
-				if (fread(&lr->penv.id, sizeof(uint32_t), 1, file)!=1) return 0;
+				if (fread(&lr->freq_center,    sizeof(uint16_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #42\n"); return 0;}
+				if (fread(&lr->layer_event,    sizeof(uint16_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #43\n"); return 0;}
+				if (fread(&lr->penv.id,        sizeof(uint32_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #44\n"); return 0;}
 				lr->penv.id = uint32_little (lr->penv.id);
-				if (fread(&lr->venv.id, sizeof(uint32_t), 1, file)!=1) return 0;
+				if (fread(&lr->venv.id,        sizeof(uint32_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #45\n"); return 0;}
 				lr->venv.id = uint32_little (lr->venv.id);
 
-				fread(&dummy, sizeof(uint32_t), 1, file);
+				if (fread(&dummy,              sizeof(uint32_t),       1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #46\n"); return 0;}
 
 				lr->waves=calloc(sizeof(struct FFF_WAVE_CHUNK), lr->nwaves);
 
@@ -460,7 +480,7 @@ static int loadFFF(FILE *file)
 
 					wv=&lr->waves[j];
 
-					if (fread(&wvhd, sizeof(wvhd), 1, file)!=1) return 0;
+					if (fread(&wvhd, sizeof(wvhd), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #47\n"); return 0;}
 					FFF_CHUNK_HEADER_endian (&wvhd);
 
 					if (strncmp(wvhd.id, "WAVE", 4))
@@ -470,26 +490,26 @@ static int loadFFF(FILE *file)
 						return 0;
 					};
 
-					if (fread(&wv->id.id, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->id.id,        sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #48\n"); return 0;}
 					wv->id.id = uint32_little (wv->id.id);
-					if (fread(&wv->size, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->size,         sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #49\n"); return 0;}
 					wv->size = uint32_little (wv->size);
-					if (fread(&wv->start, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->start,        sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #50\n"); return 0;}
 					wv->start = uint32_little (wv->start);
-					if (fread(&wv->loopstart, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->loopstart,    sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #51\n"); return 0;}
 					wv->loopstart = uint32_little (wv->loopstart);
-					if (fread(&wv->loopend, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->loopend,      sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #52\n"); return 0;}
 					wv->loopend = uint32_little (wv->loopend);
-					if (fread(&wv->m_start, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->m_start,      sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #53\n"); return 0;}
 					wv->m_start = uint32_little (wv->m_start);
-					if (fread(&wv->sample_ratio, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->sample_ratio, sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #54\n"); return 0;}
 					wv->sample_ratio = uint32_little (wv->sample_ratio);
-					if (fread(&wv->attenuation, sizeof(uint8_t), 1, file)!=1) return 0;
-					if (fread(&wv->low_note, sizeof(uint8_t), 1, file)!=1) return 0;
-					if (fread(&wv->high_note, sizeof(uint8_t), 1, file)!=1) return 0;
-					if (fread(&wv->format, sizeof(uint8_t), 1, file)!=1) return 0;
-					if (fread(&wv->m_format, sizeof(uint8_t), 1, file)!=1) return 0;
-					if (fread(&wv->data_id.id, sizeof(uint32_t), 1, file)!=1) return 0;
+					if (fread(&wv->attenuation,  sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #55\n"); return 0;}
+					if (fread(&wv->low_note,     sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #56\n"); return 0;}
+					if (fread(&wv->high_note,    sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #57\n"); return 0;}
+					if (fread(&wv->format,       sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #58\n"); return 0;}
+					if (fread(&wv->m_format,     sizeof(uint8_t),  1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #59\n"); return 0;}
+					if (fread(&wv->data_id.id,   sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #60\n"); return 0;}
 					wv->data_id.id = uint32_little (wv->data_id.id);
 					fprintf(stderr, "wave %d loaded.. (%d bytes, playable from %x to %x)\n", i, wv->size, wv->low_note, wv->high_note);
 				}
@@ -509,9 +529,9 @@ static int loadFFF(FILE *file)
 			l->chunk=calloc(sizeof(struct FFF_DATA_CHUNK), 1);
 			l->next=data_list;
 			data_list=l;
-			if (fread(&l->chunk->id.id, sizeof(uint32_t), 1, file)!=1) return 0;
+			if (fread(&l->chunk->id.id,       sizeof(uint32_t), 1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #61\n"); return 0;}
 			l->chunk->id.id = uint32_little (l->chunk->id.id);
-			if (fread(&l->chunk->filename[0], hd.size-4, 1, file)!=1) return 0;
+			if (fread(&l->chunk->filename[0], hd.size-4,        1, file)!=1) { fprintf(stderr, __FILE__ ": fread() failed #62\n"); return 0;}
 		}
 
 		if (!matched)
@@ -857,13 +877,13 @@ static int loadpatchFFF(struct minstrument *ins,
 		{
 			int8_t *temp=calloc(wave->size, 1);
 			uint16_t *d=(uint16_t *)(*smps)[smp].ptr;
-			int i;
-			fread(temp, wave->size, 1, dat);
+			unsigned int i;
+			if (fread(temp, wave->size, 1, dat)!=1) { fprintf(stderr, __FILE__ ": fread() failed #63\n"); free(temp); return 0;}
 			for (i=0; i<wave->size; i++)
 				((int16_t*)d)[i]=ulaw2linear(temp[i]);
 			free(temp);
 		} else
-			fread((*smps)[smp].ptr, wave->size*((wave->format&1)?1:2), 1, dat);
+			if (fread((*smps)[smp].ptr, wave->size*((wave->format&1)?1:2), 1, dat) != 1) { fprintf(stderr, __FILE__ ": fread() failed #64\n"); return 0;}
 
 		fclose(dat);
 	}
