@@ -97,7 +97,8 @@ static int loadpatchFreePats( struct minstrument *ins,
                               struct sampleinfo **smps,
                               uint16_t           *samplenum)
 {
-      	int file, retval;
+      	FILE *file;
+	int retval;
 	char path[PATH_MAX+NAME_MAX+1];
 
 	ins->sampnum=0;
@@ -110,7 +111,7 @@ static int loadpatchFreePats( struct minstrument *ins,
 	}
 
 	snprintf(path, sizeof(path), "%s%s", fpdir, midInstrumentNames[program]);
-	if ((file=open(path, O_RDONLY))<0)
+	if ((file=fopen(path, "r"))==NULL)
 	{
 		fprintf(stderr, "[freepats] '%s': %s\n", path, strerror(errno));
 		return errFileMiss;
@@ -118,7 +119,7 @@ static int loadpatchFreePats( struct minstrument *ins,
 
 	fprintf(stderr, "[freepats] loading file %s\n", path);
 	retval=loadpatchPAT(file, ins, program, sampused, smps, samplenum);
-	close(file);
+	fclose(file);
 	if (retval)
 		fprintf(stderr, "Invalid PAT file\n");
 	return retval;
@@ -131,7 +132,8 @@ static int addpatchFreePats( struct minstrument *ins,
                              struct sampleinfo  *sip,
                              uint16_t           *samplenum)
 {
-	int file, retval;
+	FILE *file;
+	int retval;
 	char path[PATH_MAX+NAME_MAX];
 
 	if (!*midInstrumentNames[program])
@@ -140,7 +142,7 @@ static int addpatchFreePats( struct minstrument *ins,
 		return errFileMiss;
 	}
 	snprintf(path, sizeof(path), "%s%s", fpdir, midInstrumentNames[program]);
-	if ((file=open(path, O_RDONLY))<0)
+	if ((file=fopen(path, "r"))==NULL)
 	{
 		fprintf(stderr, "[freepats] '%s': %s\n", path, strerror(errno));
 		return errFileMiss;
@@ -148,7 +150,7 @@ static int addpatchFreePats( struct minstrument *ins,
 
 	fprintf(stderr, "[freepats] loading file %s\n", path);
 	retval=addpatchPAT(file, ins, program, sn, sampnum, sip, samplenum);
-	close(file);
+	fclose(file);
 	if (retval)
 		fprintf(stderr, "Invalid PAT file\n");
 	return retval;
