@@ -52,7 +52,7 @@ static int16_t getnote(uint32_t frq)  /* frq=freq*1000, res=(oct*12+note)*256 (a
 }
 
 static 
-int loadsamplePAT( int                 file,
+int loadsamplePAT( FILE               *file,
                    struct minstrument *ins,
                    uint8_t             j,
                    uint8_t             vox,
@@ -68,7 +68,11 @@ int loadsamplePAT( int                 file,
 	int q;
 	uint8_t *smpp;
 
-	read(file, &sh, sizeof(sh));
+	if (fread(&sh, sizeof(sh), 1, file)!=1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #1\n");
+		return errFileRead;
+	}
 	PATCHDATA_endian(&sh);
 	bit16=!!(sh.modes&1);
 	if (bit16)
@@ -106,7 +110,7 @@ int loadsamplePAT( int                 file,
 		if (i==highnote)
 		{
 			fprintf(stderr, "[*.PAT loader] i==highnote\n");
-			lseek(file, sh.wave_size<<bit16, SEEK_CUR);
+			fseek(file, sh.wave_size<<bit16, SEEK_CUR);
 			return 1;
 		}
 		memset(ins->note+lownote, j, highnote-lownote);
@@ -157,7 +161,7 @@ int loadsamplePAT( int                 file,
 
 	if (!(smpp=calloc(sip->length<<bit16, 1)))
 		return errAllocMem;
-	if (read(file, smpp, sip->length<<bit16)!=(sip->length<<bit16))
+	if (fread(smpp, 1, sip->length<<bit16, file)!=(sip->length<<bit16))
 		fprintf(stderr, "[*.PAT loader] premature EOF (warning)\n");
 	sip->ptr=smpp;
 	s->handle=(*samplenum)++;
@@ -165,7 +169,7 @@ int loadsamplePAT( int                 file,
 }
 
 int __attribute__ ((visibility ("internal")))
-loadpatchPAT( int file,
+loadpatchPAT( FILE               *file,
               struct minstrument *ins,
               uint8_t             program,
               uint8_t            *sampused,
@@ -182,7 +186,11 @@ loadpatchPAT( int file,
 	ins->sampnum=0;
 	*ins->name=0;
 
-	read(file, &ph, sizeof(ph));
+	if (fread(&ph, sizeof(ph), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #2\n");
+		return errFileRead;
+	}
 	PATCHHEADER_endian(&ph);
 	if (memcmp(ph.header, "GF1PATCH110", 12))
 	{
@@ -195,7 +203,11 @@ loadpatchPAT( int file,
 		return errFormStruc;
 	}
 
-	read(file, &ih, sizeof(ih));
+	if (fread(&ih, sizeof(ih), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #3\n");
+		return errFileRead;
+	}
 	INSTRUMENTDATA_endian(&ih);
 	if (ih.layers>1)
 	{
@@ -212,7 +224,11 @@ loadpatchPAT( int file,
 		snprintf(ins->name, sizeof(ins->name), "%s", name);
 	}
 
-	read(file, &lh, sizeof(lh));
+	if (fread(&lh, sizeof(lh), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #4\n");
+		return errFileRead;
+	}
 	LAYERDATA_endian(&lh);
 	if (!(ins->samples=calloc(sizeof(struct msample), lh.samples)))
 		return errAllocMem;
@@ -250,7 +266,7 @@ loadpatchPAT( int file,
 }
 
 int __attribute__ ((visibility ("internal")))
-addpatchPAT ( int                 file,
+addpatchPAT (FILE               *file,
              struct minstrument *ins,
              uint8_t             program,
              uint8_t             sn,
@@ -266,7 +282,11 @@ addpatchPAT ( int                 file,
 
 	s=&ins->samples[sn];
 
-      	read(file, &ph, sizeof(ph));
+	if (fread(&ph, sizeof(ph), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #5\n");
+		return errFileRead;
+	}
 	PATCHHEADER_endian(&ph);
 	if (memcmp(ph.header, "GF1PATCH110", 12))
 	{
@@ -278,7 +298,11 @@ addpatchPAT ( int                 file,
 		fprintf(stderr, "[*.PAT loader] Invalid number of instruments\n");
 		return errFormStruc;
 	}
-	read(file, &ih, sizeof(ih));
+	if (fread(&ih, sizeof(ih), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #6\n");
+		return errFileRead;
+	}
 	INSTRUMENTDATA_endian(&ih);
 	if (ih.layers<1)
 	{
@@ -319,7 +343,11 @@ addpatchPAT ( int                 file,
 		return 0;
 	}
 
-	read(file, &lh, sizeof(lh));
+	if (fread(&lh, sizeof(lh), 1, file) != 1)
+	{
+		fprintf(stderr, "[*.PAT loader] fread failed #7\n");
+		return errFileRead;
+	}
 	LAYERDATA_endian(&lh);
 	if (lh.samples!=1)
 	{
