@@ -193,7 +193,7 @@ static void drawchannel(uint16_t *buf, int len, int i)
 	}
 }
 
-void gmiChanSetup(const struct midifile *mid)
+void __attribute__ ((visibility ("internal"))) gmiChanSetup(const struct midifile *mid)
 {
 	plChanMInstr=mid->instruments;
 	plUseChannels(drawchannel);
