@@ -44,7 +44,7 @@ static char currentmodext[_MAX_EXT+1];
 static uint32_t starttime;
 static uint32_t pausetime;
 
-static struct midifile mid={0};
+static struct midifile mid={0,0,0,0,0,{0},0,0,0,0};
 
 static void gmiDrawGStrings(uint16_t (*buf)[CONSOLE_MAX_X])
 {
@@ -102,8 +102,12 @@ static int gmiProcessKey(unsigned short key)
 			cpiKeyHelp('p', "Start/stop pause");
 			cpiKeyHelp('P', "Start/stop pause");
 			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
-			cpiKeyHelp('<', "Jump to previous track");
-			cpiKeyHelp('>', "Jump to next track");
+			cpiKeyHelp('<', "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump back (big)");
+			cpiKeyHelp('>', "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump back (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump forward (small)");
 			mcpSetProcessKey(key);
 			if (mcpProcessKey)
 				mcpProcessKey(key);
@@ -117,8 +121,8 @@ static int gmiProcessKey(unsigned short key)
 			mcpSet(-1, mcpMasterPause, plPause^=1);
 			plChanChanged=1;
 			break;
-			/* TODO keys
-		case 0x7700: //ctrl-home
+/*
+		case 0x7700: //ctrl-home TODO keys
 			gmiClearInst();
 			midSetPosition(0);
 
@@ -126,24 +130,26 @@ static int gmiProcessKey(unsigned short key)
 				starttime=pausetime;
 			else
 				starttime=dos_clock();
-			break;*/
-/*TODO Keys
- 		case 0x7300: //ctrl-left */
+			break;
+*/
+		case KEY_CTRL_LEFT:
+ 		/* case 0x7300: //ctrl-left */
 		case '<':
 			midSetPosition(midGetPosition()-(mid.ticknum>>5));
 			break;
-/*TODO Keys
-		case 0x7400: //ctrl-right*/
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right*/
 		case '>':
 			midSetPosition(midGetPosition()+(mid.ticknum>>5));
 			break;
-/* TODO keys
-		case 0x8D00: // ctrl-up
+		case KEY_CTRL_UP:
+		/*case 0x8D00: //ctrl-up */
 			midSetPosition(midGetPosition()-(mid.ticknum>>8));
 			break;
-		case 0x9100: //ctrl-down
+		case KEY_CTRL_DOWN:
+		/*case 0x9100: //ctrl-down */
 			midSetPosition(midGetPosition()+(mid.ticknum>>8));
-			break;*/
+			break;
 		default:
 			if (mcpSetProcessKey(key))
 				return 1;
@@ -272,4 +278,4 @@ static int gmiOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 }
 
 struct cpifaceplayerstruct gmiPlayer = {gmiOpenFile, gmiCloseFile};
-struct linkinfostruct dllextinfo = {"playgmi", "OpenCP GUSPatch Midi Player (c) 1994-99 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playgmi", "OpenCP GUSPatch Midi Player (c) 1994-08 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
