@@ -9,9 +9,6 @@
  */
 
 #include "config.h"
-#include "types.h"
-#include "gmipat.h"
-#include "gmiplay.h"
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -20,6 +17,9 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include "types.h"
+#include "gmipat.h"
+#include "gmiplay.h"
 #include "stuff/err.h"
 
 #define DirStackSize 5
@@ -158,7 +158,8 @@ static int loadpatchTimidity( struct minstrument *ins,
                               uint16_t           *samplenum)
 {
 	int i;
-      	int file=-1, retval;
+      	FILE *file=NULL;
+	int retval;
 	char path[PATH_MAX+NAME_MAX+1];
 
 	ins->sampnum=0;
@@ -172,17 +173,17 @@ static int loadpatchTimidity( struct minstrument *ins,
 	for (i=DirectoryStackIndex-1;i>=0;i--)
 	{
 		snprintf(path, sizeof(path), "%s/%s.pat", DirectoryStack[i], midInstrumentNames[program]);
-		if ((file=open(path, O_RDONLY))>=0)
+		if ((file=fopen(path, "r"))!=NULL)
 			break;
 	}
-	if (file<0)
+	if (!file)
 	{
 		fprintf(stderr, "[timidity] '%s': failed to open file\n", midInstrumentNames[program]);
 		return errFileMiss;
 	}
 	fprintf(stderr, "[timidity] loading file %s\n", path);
 	retval=loadpatchPAT(file, ins, program, sampused, smps, samplenum);
-	close(file);
+	fclose(file);
 	if (retval)
 		fprintf(stderr, "Invalid PAT file\n");
 	return retval;
@@ -195,7 +196,8 @@ static int addpatchTimidity( struct minstrument *ins,
                              struct sampleinfo  *sip,
                              uint16_t           *samplenum)
 {
-	int file=-1, retval;
+	FILE *file=NULL;
+	int retval;
 	char path[PATH_MAX+NAME_MAX];
 	int i;
 
@@ -207,10 +209,10 @@ static int addpatchTimidity( struct minstrument *ins,
 	for (i=DirectoryStackIndex-1;i>=0;i--)
 	{
 		snprintf(path, sizeof(path), "%s/%s.pat", DirectoryStack[i], midInstrumentNames[program]);
-		if ((file=open(path, O_RDONLY))>=0)
+		if ((file=fopen(path, "r"))!=NULL)
 			break;
 	}
-	if (file<0)
+	if (!file)
 	{
 		fprintf(stderr, "[timidity] '%s': failed to open file\n", midInstrumentNames[program]);
 		return errFileMiss;
@@ -218,7 +220,7 @@ static int addpatchTimidity( struct minstrument *ins,
 
 	fprintf(stderr, "[timidity] loading file %s\n", path);
 	retval=addpatchPAT(file, ins, program, sn, sampnum, sip, samplenum);
-	close(file);
+	fclose(file);
 	if (retval)
 		fprintf(stderr, "Invalid PAT file\n");
 	return retval;
