@@ -12,6 +12,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -20,7 +21,6 @@
 #include "types.h"
 #include "gmipat.h"
 #include "gmiplay.h"
-#include "binfile/binfpak.h"
 #include "boot/psetting.h"
 #include "stuff/compat.h"
 #include "stuff/err.h"
@@ -36,29 +36,18 @@ static int loadpatchUltra(struct minstrument *ins,
       	int retval;
 	ins->sampnum=0;
 	*ins->name=0;
-	if (!midInstrumentPath[0])
 	{
-		FILE *f=pakfOpen(midInstrumentNames[program]);
-		if (!f)
-		{
-			fprintf(stderr, "[ultradir] {CP.PAK}/%s not found\n", midInstrumentNames[program]);
-			return errFileMiss;
-		}
-		fprintf(stderr, "[ultradir] loading {CP.PAK}/%s\n", midInstrumentNames[program]);
-		retval=loadpatchPAT(fileno(f), ins, program, sampused, smps, samplenum);
-		fclose(f);
-	} else {
-		int file;
+		FILE *file;
 		char path[PATH_MAX+NAME_MAX];
 		snprintf(path, sizeof(path), "%s%s", midInstrumentPath, midInstrumentNames[program]);
-		if ((file=open(path, O_RDONLY))<0)
+		if ((file=fopen(path, "r"))==NULL)
 		{
 			fprintf(stderr, "[ultradir] '%s': %s\n", path, strerror(errno));
 			return errFileMiss;
 		}
 		fprintf(stderr, "[ultradir] loading %s\n", path);
 		retval=loadpatchPAT(file, ins, program, sampused, smps, samplenum);
-		close(file);
+		fclose(file);
 	}
 	if (retval)
 		fprintf(stderr, "[ultradir] Invalid PAT file\n");
@@ -73,30 +62,19 @@ static int addpatchUltra( struct minstrument *ins,
                           uint16_t           *samplenum)
 {
 	int retval;
-	if (!midInstrumentPath[0])
 	{
-		FILE *f=pakfOpen(midInstrumentNames[program]);
-		if (!f)
-		{
-			fprintf(stderr, "[ultradir] {CP.PAK}/%s not found\n", midInstrumentNames[program]);
-			return errFileMiss;
-		}
-		fprintf(stderr, "[ultradir] loading {CP.PAK}/%s\n", midInstrumentNames[program]);
-		retval=addpatchPAT(fileno(f), ins, program, sn, sampnum, sip, samplenum);
-		fclose(f);
-	} else {
-		int file;
+		FILE *file;
 		char path[PATH_MAX+NAME_MAX];
 
 		snprintf(path, sizeof(path), "%s%s", midInstrumentPath, midInstrumentNames[program]);
-		if ((file=open(path, O_RDONLY))<0)
+		if ((file=fopen(path, "r"))==NULL)
 		{
 			fprintf(stderr, "[ultradir] '%s': %s\n", path, strerror(errno));
 			return errFileMiss;
 		}
 		fprintf(stderr, "[ultradir] loading %s\n", path);
 		retval=addpatchPAT(file, ins, program, sn, sampnum, sip, samplenum);
-		close(file);
+		fclose(file);
 	}
 	if (retval)
 		fprintf(stderr, "[ultradir] Invalid PAT file\n");
@@ -131,12 +109,6 @@ int __attribute__ ((visibility ("internal"))) midInitUltra(void)
 		fprintf(stderr, "[ultradir] parsing %s\n", path);
 		goto got_ultrasound_ini;
 	}
-	if ((inifile=pakfOpen("ULTRASND.INI")))
-	{
-		fprintf(stderr, "[ultradir] parsing {CP.DAT}/ULTRASND.INI");
-		midInstrumentPath[0]=0;
-		goto got_ultrasound_ini;
-	}
 
 	fprintf(stderr, "[ultradir] failed to locate ULTRASND.INI\n");
 	return 0;
@@ -148,8 +120,16 @@ got_ultrasound_ini:
 	fseek(inifile, 0, SEEK_SET);
 
 	if (!(buf=calloc(len+1, 1)))
+	{
+		fprintf(stderr, "[ultradir] calloc() failed\n");
 		return 0;
-	fread(buf, len, 1, inifile);
+	}
+	if (fread(buf, len, 1, inifile) != 1)
+	{
+		fprintf(stderr, "[ultradir] fread() failed\n");
+		free(buf);
+		return 0;
+	}	
   	buf[len]=0;
 	fclose(inifile);
 
