@@ -91,7 +91,11 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 	fseek(file, 0, SEEK_SET);
 	
-	fread(&hdr, sizeof(hdr), 1, file);
+	if (fread(&hdr, sizeof(hdr), 1, file) != 1)
+	{
+		fprintf(stderr, "[IT]: fread() failed #1\n");
+		return errFileRead;
+	}
 	hdr.sig     = uint32_little (hdr.sig);
 	hdr.philite = uint16_little (hdr.philite);
 	hdr.nords   = uint16_little (hdr.nords);
@@ -105,6 +109,21 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 	hdr.msglen  = uint16_little (hdr.msglen);
 	hdr.msgoff  = uint32_little (hdr.msgoff);
 	hdr._d3     = uint32_little (hdr._d3);
+#ifdef IT_LOAD_DEBUG
+	fprintf(stderr, __FILE__ ": hdr.sig %d\n", (int)hdr.sig);
+	fprintf(stderr, __FILE__ ": hdr.philite %d\n", (int)hdr.philite);
+	fprintf(stderr, __FILE__ ": hdr.nords %d\n", (int)hdr.nords);
+	fprintf(stderr, __FILE__ ": hdr.nins %d\n", (int)hdr.nins);
+	fprintf(stderr, __FILE__ ": hdr.nsmps %d\n", (int)hdr.nsmps);
+	fprintf(stderr, __FILE__ ": hdr.npats %d\n", (int)hdr.npats);
+	fprintf(stderr, __FILE__ ": hdr.cwtv %d\n", (int)hdr.cwtv);
+	fprintf(stderr, __FILE__ ": hdr.cmwt %d\n", (int)hdr.cmwt);
+	fprintf(stderr, __FILE__ ": hdr.flags %d\n", (int)hdr.flags);
+	fprintf(stderr, __FILE__ ": hdr.special %d\n", (int)hdr.special);
+	fprintf(stderr, __FILE__ ": hdr.msglen %d\n", (int)hdr.msglen);
+	fprintf(stderr, __FILE__ ": hdr.msgoff %d\n", (int)hdr.msgoff);
+	fprintf(stderr, __FILE__ ": hdr._d3 %d\n", (int)hdr._d3);
+#endif
 	if (hdr.sig!=0x4D504D49)
 	{
 		fprintf(stderr, "IT: Invalid header\n");
@@ -139,10 +158,27 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 	signedsamp=!(hdr.cwtv<0x202);
 	this->deltapacked=(hdr.cmwt==0x215);
 
-	fread(ords, hdr.nords*sizeof(uint8_t), 1, file);
-	fread(insoff, hdr.nins*sizeof(uint32_t), 1, file);
-	fread(sampoff, hdr.nsmps*sizeof(uint32_t), 1, file);
-	fread(patoff, hdr.npats*sizeof(uint32_t), 1, file);
+	if (fread(ords, hdr.nords*sizeof(uint8_t), 1, file) != 1)
+	{
+		fprintf(stderr, "[IT]: fread() failed #2\n");
+		return errFileRead;
+	}
+	if (hdr.nins)
+		if (fread(insoff, hdr.nins*sizeof(uint32_t), 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #3 (hdr.nins=%d)\n", hdr.nins);
+			return errFileRead;
+		}
+	if (fread(sampoff, hdr.nsmps*sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, "[IT]: fread() failed #4\n");
+		return errFileRead;
+	}
+	if (fread(patoff, hdr.npats*sizeof(uint32_t), 1, file) != 1)
+	{
+		fprintf(stderr, "[IT]: fread() failed #5\n");
+		return errFileRead;
+	}
 	for (i=0;i<hdr.nins;i++)
 		insoff[i] = uint32_little (insoff[i]);
 	for (i=0;i<hdr.nsmps;i++)
@@ -175,10 +211,18 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 	{
 		uint16_t usage;
 		char dummy[8];
-		fread(&usage, sizeof(usage), 1, file);
+		if (fread(&usage, sizeof(usage), 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #6\n");
+			return errFileRead;
+		}
 		usage = uint16_little (usage);
 		for (i=0; i<usage; i++)
-			fread(dummy, 8, 1, file);
+			if (fread(dummy, 8, 1, file) != 1)
+			{
+				fprintf(stderr, "[IT]: fread() failed #7\n");
+				return errFileRead;
+			}
 	}
 
 
@@ -188,7 +232,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 		if (!(this->midicmds=malloc(sizeof(char *)*(9+16+128))))
 		{
-			fprintf(stderr, __FILE__ ": malloc(%d) failed #1\n", sizeof(char *)*(9+16+128));
+			fprintf(stderr, __FILE__ ": malloc(%d) failed #1\n", (int)(sizeof(char *)*(9+16+128)));
 			return errAllocMem;
 		}
 
@@ -196,13 +240,17 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 		for (i=0; i<(9+16+128); i++)
 		{
-			fread(cmd, 32, 1, file);
+			if (fread(cmd, 32, 1, file) != 1)
+			{
+				fprintf(stderr, "[IT]: fread() failed #8\n");
+				return errFileRead;
+			}
 			for ( n=0; n<32; n++ )
 				if (!cmd[n])
 					cmd[n]=32;
 			if (!(this->midicmds[i]=malloc(sizeof(char)*(strlen(cmd)+1))))
 			{
-				fprintf(stderr, __FILE__ ": malloc(%d) failed #2\n", sizeof(char)*(strlen(cmd)+1));
+				fprintf(stderr, __FILE__ ": malloc(%d) failed #2\n", (int)(sizeof(char)*(strlen(cmd)+1)));
 				return errAllocMem;
 			}
 			strcpy(this->midicmds[i], cmd);
@@ -217,12 +265,17 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 		if (!(msg=malloc(sizeof(char)*(hdr.msglen+1))))
 		{
-			fprintf(stderr, __FILE__ ": malloc(%d) failed #3\n", sizeof(char)*(hdr.msglen+1));
+			fprintf(stderr, __FILE__ ": malloc(%d) failed #3\n", (int)(sizeof(char)*(hdr.msglen+1)));
 			return errAllocMem;
 		}
 		linect=1;
 		fseek(file, hdr.msgoff, SEEK_SET);
-		fread(msg, hdr.msglen, 1, file);
+		if (fread(msg, hdr.msglen, 1, file) !=1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #9\n");
+			free(msg);
+			return errFileRead;
+		}
 		msg[hdr.msglen]=0;
 		for (i=0; i<hdr.msglen; i++)
 		{
@@ -233,7 +286,8 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		}
 		if (!(this->message=calloc(sizeof(char *), linect+1)))
 		{
-			fprintf(stderr, __FILE__ ": calloc(%d) failed #4\n", sizeof(char *)*linect+1);
+			fprintf(stderr, __FILE__ ": calloc(%d) failed #4\n", (int)(sizeof(char *)*linect+1));
+			free(msg);
 			return errAllocMem;
 		}
 		*this->message=msg;
@@ -276,7 +330,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 	if (!(this->orders=malloc(sizeof(uint16_t)*this->nord)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #5\n", sizeof(uint16_t)*this->nord);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #5\n", (int)(sizeof(uint16_t)*this->nord));
 		return errAllocMem;
 	}
 
@@ -285,12 +339,12 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 	if (!(this->patlens=malloc(sizeof(uint16_t)*this->npat)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #6\n", sizeof(uint16_t)*this->npat);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #6\n", (int)(sizeof(uint16_t)*this->npat));
 		return errAllocMem;
 	}
 	if (!(this->patterns=malloc(sizeof(uint8_t*)*this->npat)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #7\n", sizeof(uint8_t*)*this->npat);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #7\n", (int)(sizeof(uint8_t*)*this->npat));
 		return errAllocMem;
 	}
 			
@@ -299,7 +353,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 	if (!this->patterns[this->npat-1])
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #8\n", sizeof(uint8_t)*64);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #8\n", (int)(sizeof(uint8_t)*64));
 		return errAllocMem;
 	}
 
@@ -325,7 +379,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 			this->patlens[k]=64;
 			if (!(this->patterns[k]=malloc(sizeof(uint8_t)*this->patlens[k])))
 			{
-				fprintf(stderr, __FILE__ ": malloc(%d) failed #9\n", sizeof(uint8_t)*this->patlens[k]);
+				fprintf(stderr, __FILE__ ": malloc(%d) failed #9\n", (int)(sizeof(uint8_t)*this->patlens[k]));
 				return errAllocMem;
 			}
 			memset(this->patterns[k], 0, this->patlens[k]);
@@ -333,25 +387,37 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		}
 		fseek(file, patoff[k], SEEK_SET);
 
-		fread(&patlen, sizeof(uint16_t), 1, file);
+		if (fread(&patlen, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #10\n");
+			return errFileRead;
+		}
 		patlen = uint16_little (patlen);
-		fread(&patrows, sizeof(uint16_t), 1, file);
+		if (fread(&patrows, sizeof(uint16_t), 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #11\n");
+			return errFileRead;
+		}
 		patrows = uint16_little (patrows);
 		fseek(file, 4, SEEK_CUR);
 
 		if (!(patbuf=malloc(sizeof(uint8_t)*patlen)))
 		{
-			fprintf(stderr, __FILE__ ": malloc(%d) failed #10\n", sizeof(uint8_t)*patlen);
+			fprintf(stderr, __FILE__ ": malloc(%d) failed #10\n", (int)(sizeof(uint8_t)*patlen));
 			return errAllocMem;
 		}
 
-		fread(patbuf, patlen, 1, file);
-
+		if (fread(patbuf, patlen, 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #12\n");
+			free(patbuf);
+			return errFileRead;
+		}
 		this->patlens[k]=patrows;
 		if (!(this->patterns[k]=malloc(sizeof(uint8_t)*((6*64+1)*this->patlens[k]))))
 		{
 			free(patbuf);
-			fprintf(stderr, __FILE__ ": malloc(%d) failed #11\n", sizeof(uint8_t)*((6*64+1)*this->patlens[k]));
+			fprintf(stderr, __FILE__ ": malloc(%d) failed #11\n", (int)(sizeof(uint8_t)*((6*64+1)*this->patlens[k])));
 			return errAllocMem;
 		}
 
@@ -406,7 +472,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 			if (!(new=realloc(this->patterns[k], wp-this->patterns[k])))
 			{
 				/* we loose the pointer for-ever here */
-				fprintf(stderr, __FILE__ ": realloc(%d) failed #12\n", wp-this->patterns[k]);
+				fprintf(stderr, __FILE__ ": realloc(%d) failed #12\n", (int)(wp-this->patterns[k]));
 				return errAllocMem;
 			}
 			this->patterns[k]=(uint8_t *)new;
@@ -425,12 +491,12 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 
 	if (!(this->sampleinfos=malloc(sizeof(struct it_sampleinfo)*this->nsampi)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #13\n", sizeof(struct it_sampleinfo)*this->nsampi);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #13\n", (int)(sizeof(struct it_sampleinfo)*this->nsampi));
 		return errAllocMem;
 	}
 	if (!(this->samples=malloc(sizeof(struct it_sample)*this->nsamp)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #14\n", sizeof(struct it_sample)*this->nsamp);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #14\n", (int)(sizeof(struct it_sample)*this->nsamp));
 		return errAllocMem;
 	}
 	memset(this->sampleinfos, 0, this->nsampi*sizeof(struct it_sampleinfo));
@@ -468,7 +534,11 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		struct it_sample *sp=&this->samples[i];
 
 		fseek(file, sampoff[i], SEEK_SET);
-		fread(&shdr, sizeof(shdr), 1, file);
+		if (fread(&shdr, sizeof(shdr), 1, file) != 1)
+		{
+			fprintf(stderr, "[IT]: fread() failed #13\n");
+			return errFileRead;
+		}
 		shdr.sig        = uint32_little (shdr.sig);
 		shdr.length     = uint32_little (shdr.length);
 		shdr.loopstart  = uint32_little (shdr.loopstart);
@@ -535,14 +605,19 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 				decompress16(file, sip->ptr, sip->length, this->deltapacked);
 			else
 				decompress8(file, sip->ptr, sip->length, this->deltapacked);
-		} else
-			fread(sip->ptr, sip->length<<((sip->type&mcpSamp16Bit)?1:0), 1, file);	
+		} else {
+			if (fread(sip->ptr, sip->length<<((sip->type&mcpSamp16Bit)?1:0), 1, file) != 1)
+			{
+				fprintf(stderr, "[IT]: fread() failed #14\n");
+				return errFileRead;
+			}
+		}
 	}
 
 	this->ninst=(hdr.flags&4)?hdr.nins:hdr.nsmps;
 	if (!(this->instruments=malloc(sizeof(struct it_instrument)*this->ninst)))
 	{
-		fprintf(stderr, __FILE__ ": malloc(%d) failed #16\n", sizeof(struct it_instrument)*this->ninst);
+		fprintf(stderr, __FILE__ ": malloc(%d) failed #16\n", (int)(sizeof(struct it_instrument)*this->ninst));
 		return errAllocMem;
 	}
 	memset(this->instruments, 0, this->ninst*sizeof(*this->instruments));
@@ -599,7 +674,11 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		{
 			int i, j;
 			fseek(file, insoff[k], SEEK_SET);
-			fread(&ihdr, sizeof(ihdr), 1, file);
+			if (fread(&ihdr, sizeof(ihdr), 1, file) != 1)
+			{
+				fprintf(stderr, "[IT]: fread() failed #15\n");
+				return errFileRead;
+			}
 			ihdr.sig     = uint32_little (ihdr.sig);
 			ihdr.fadeout = uint16_little (ihdr.fadeout);
 			ihdr.tver    = uint16_little (ihdr.tver);
@@ -619,7 +698,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 				ihdr.keytab[i][1]=k+1;
 			}
 			memcpy(ihdr.name, this->samples[k].name, 26);
-			ihdr.name[26]=0;
+			ihdr.name[25]=0;
 			ihdr.dfp=0x80;
 			ihdr.gbv=128;
 		}
