@@ -243,7 +243,7 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 			c->offset=(c->offset&0xF00)|cmd[4];
 		p->newsamp=s->handle;
 		p->newpos=c->offset<<8;
-		if (p->newpos>=this->sampleinfos[s->handle].length)
+		if (p->newpos>=(signed)this->sampleinfos[s->handle].length)
 			p->newpos=this->sampleinfos[s->handle].length-16;
 	} else if (c->fnotehit)
 	{
@@ -1809,7 +1809,7 @@ int __attribute__ ((visibility ("internal"))) getrealpos(struct itplayer *this)
 int __attribute__ ((visibility ("internal"))) getchansample(struct itplayer *this, int ch, int16_t *buf, int len, uint32_t rate, int opt)
 {
 	int i,n;
-	int chn[64];
+	unsigned int chn[64];
 	n=0;
 	for (i=0; i<this->npchan; i++)
 		if (this->pchannels[i].lch==ch)
