@@ -35,8 +35,8 @@
 #define _MAX_FNAME 8
 #define _MAX_EXT 4
 
-__attribute__ ((visibility ("internal"))) struct itplayer itplayer = {0}; /* dirty export - stian */
-static struct it_module mod = {{0}};
+__attribute__ ((visibility ("internal"))) struct itplayer itplayer = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+static struct it_module mod = {{0},0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,{0},{0},0,0,0,0,0};
 
 static struct it_instrument *insts;
 static struct it_sample *samps;
@@ -55,7 +55,7 @@ void itpInstClear();
 
 static int itpProcessKey(uint16_t key)
 {
-/*	int row;*/
+	int row;
 	int pat, p;
 
 	switch (key)
@@ -64,8 +64,12 @@ static int itpProcessKey(uint16_t key)
 			cpiKeyHelp('p', "Start/stop pause with fade");
 			cpiKeyHelp('P', "Start/stop pause with fade");
 			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
-			cpiKeyHelp('<', "Jump to previous track");
-			cpiKeyHelp('>', "Jump to next track");
+			cpiKeyHelp('<', "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump back (big)");
+			cpiKeyHelp('>', "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump back (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump forward (small)");
 			mcpSetProcessKey(key);
 			if (mcpProcessKey)
 				mcpProcessKey(key);
@@ -78,40 +82,44 @@ static int itpProcessKey(uint16_t key)
 			mcpSet(-1, mcpMasterPause, plPause^=1);
 			plChanChanged=1;
 			break;
-	/* TODO keys
-		case 0x7700: //ctrl-home
+/*
+		case 0x7700: //ctrl-home TODO keys
 			itpInstClear();
 			itplayer.setpos(0, 0);
 			if (plPause)
 				starttime=pausetime;
 			else
 				starttime=dos_clock();
-			break;*/
+			break;
+*/
 		case '<':
-		/*		case 0x7300: //ctrl-left  todo keys */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			p=getpos(&itplayer);
 			pat=p>>16;
 			setpos(&itplayer, pat-1, 0);
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			p=getpos(&itplayer);
 			pat=p>>16;
 			setpos(&itplayer, pat+1, 0);
 			break;
-/*		case 0x8D00: // ctrl-up
-			p=itplayer.getpos();
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
+			p=getpos(&itplayer);
 			pat=p>>16;
 			row=(p>>8)&0xFF;
-			itplayer.setpos(pat, row-8);
+			setpos(&itplayer, pat, row-8);
 			break;
-		case 0x9100: //ctrl-down
-			p=itplayer.getpos();
+		case KEY_CTRL_DOWN:
+		/* case 0x9100: //ctrl-down */
+			p=getpos(&itplayer);
 			pat=p>>16;
 			row=(p>>8)&0xFF;
-			itplayer.setpos(pat, row+8);
+			setpos(&itplayer, pat, row+8);
 			break;
-			*/
 		default:
 			if (mcpSetProcessKey(key))
 				return 1;
@@ -529,7 +537,7 @@ static void itpMute(int i, int m)
 	mutechan(&itplayer, i, m);
 }
 
-static int itpGetLChanSample(int ch, int16_t *buf, int len, uint32_t rate, int opt)
+static int itpGetLChanSample(unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
 {
 	return getchansample(&itplayer, ch, buf, len, rate, opt);
 }
@@ -627,4 +635,4 @@ static int itpOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 }
 
 struct cpifaceplayerstruct itpPlayer = {itpOpenFile, itpCloseFile};
-struct linkinfostruct dllextinfo = {"playit", "OpenCP IT Player (c) 1997-04 Tammo Hinrichs, Niklas Beisert", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playit", "OpenCP IT Player (c) 1997-08 Tammo Hinrichs, Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
