@@ -59,4 +59,4 @@ static int itpReadInfo(struct moduleinfostruct *m, FILE *fp, const char *bf, siz
 	return 0;
 }
 
-struct mdbreadinforegstruct itpReadInfoReg = {itpReadMemInfo, itpReadInfo};
+struct mdbreadinforegstruct itpReadInfoReg = {itpReadMemInfo, itpReadInfo, 0 MDBREADINFOREGSTRUCT_TAIL};
