@@ -14,7 +14,6 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <id3tag.h>
 #include <mad.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,7 +42,7 @@ static int srnd;
 static struct mad_stream stream;
 static struct mad_frame frame;
 static struct mad_synth synth;
-static int data_in_synth;
+static unsigned int data_in_synth;
 int __attribute__ ((visibility ("internal"))) mpeg_Bitrate; /* bitrate of the last decoded frame */
 static int mpegrate; /* samp-rate in the mpeg */
 static int mpegstereo;
@@ -173,6 +172,21 @@ static void audio_pcm_s16(unsigned char *_data, unsigned int nsamples, mad_fixed
 		}
 }
 
+static inline uint_fast32_t id3_tag_query(const unsigned char *data, uint_fast32_t length)
+{
+	if (length>=3)
+		if ((data[0]=='T')&&(data[1]=='A')&&(data[2]=='G'))
+			return 128;
+	if (length>=10)
+		if ((data[0]=='I')&&(data[1]=='D')&&(data[2]=='3')&&(data[3]!=0xff)&&(data[4]!=0xff))
+		{
+			uint_fast32_t size = (data[6]<<21)|(data[7]<<14)|(data[8]<<7)|(data[9]);
+			if (data[5]&0x10) /* do we have a footer? */
+				size+=10; /* size of footer */
+			return size+10; /* size of header */
+		}
+	return 0; /* we can't tell */
+}
 
 static int stream_for_frame(void)
 {
@@ -197,7 +211,7 @@ static int stream_for_frame(void)
 	{
 		if(stream.buffer==NULL || stream.error==MAD_ERROR_BUFLEN)
 		{
-			int len, target;
+			uint32_t len, target;
 			if (stream.next_frame)
 			{
 				if (GuardPtr)
@@ -393,7 +407,8 @@ void __attribute__ ((visibility ("internal"))) mpegIdle(void)
 					mpegbufpos-=mpegbuflen;
 			}
 		} else { /* re-sample intil we don't have more target-buffer or source-buffer */
-			int32_t c0, c1, c2, c3, ls, rs, vm1,v1,v2, wp1, wp2, wpm1;
+			int32_t c0, c1, c2, c3, ls, rs, vm1,v1,v2, wpm1;
+			uint32_t wp1, wp2;
 			for (i=0; i<bufdelta; i++)
 			{
 				wpm1=mpegbufpos-4; if (wpm1<0) wpm1+=mpegbuflen;
@@ -951,10 +966,10 @@ int32_t __attribute__ ((visibility ("internal"))) mpegGetPos(void)
 {
 	return datapos;
 }
-void __attribute__ ((visibility ("internal"))) mpegSetPos(int32_t pos)
+void __attribute__ ((visibility ("internal"))) mpegSetPos(uint32_t pos)
 {
-	if (pos<0)
-		pos=0;
+	/*if (pos<0)
+		pos=0;*/
 	if (pos>fl)
 		pos=fl;
 	newpos=pos;
