@@ -35,8 +35,30 @@ static char *mpstyles[]=
 	"Bass", "Soul"
 };
 
+#ifdef ID3_DEBUG
+static const char *hex = "0123456789abcdef";
+#endif
+
 static void handle_T__(const uint8_t *p, uint_fast32_t size, char *target, int targetsize)
 {
+#ifdef ID3_DEBUG
+	char *temp_p = malloc((size-1)*4+1);
+	int i, j=0;
+	for (i=1;i<size;i++)
+	{
+		if ((p[i]>=32)||(p[i]<=127))
+			temp_p[j++]=p[i];
+		else {
+			temp_p[j++]='\\';
+			temp_p[j++]='x';
+			temp_p[j++]=hex[p[i]>>4];
+			temp_p[j++]=hex[p[i]&15];
+		}
+	}
+	temp_p[j++]=0;
+	fprintf(stderr, "handle_T__(*p=(charset=%d;data=\"%s\"), size=%d, target=%p, targetsize=%d\n", p[0], temp_p, size, target, targetsize);
+	free(temp_p);
+#endif
 	target[0]=0;
 	if (size<1)
 		return;
@@ -47,10 +69,46 @@ static void handle_T__(const uint8_t *p, uint_fast32_t size, char *target, int t
 
 static void handle_T___(const uint8_t *p, uint_fast32_t size, char *target, int targetsize)
 {
+#ifdef ID3_DEBUG
+	char *temp_p = malloc((size-1)*4+1);
+	int i, j=0;
+	for (i=1;i<size;i++)
+	{
+		if ((p[i]>=32)||(p[i]<=127))
+			temp_p[j++]=p[i];
+		else {
+			temp_p[j++]='\\';
+			temp_p[j++]='x';
+			temp_p[j++]=hex[p[i]>>4];
+			temp_p[j++]=hex[p[i]&15];
+		}
+	}
+	temp_p[j++]=0;
+	fprintf(stderr, "handle_T___(*p=(charset=%d;data=\"%s\"), size=%d, target=%p, targetsize=%d) ", p[0], temp_p, size, target, targetsize);
+	fprintf(stderr, "forwarding to handle_T__\n");
+	free(temp_p);
+#endif
 	handle_T__(p, size, target, targetsize);
 }
 static void parseid3v1(struct moduleinfostruct *m, const unsigned char *tag)
 {
+#ifdef ID3_DEBUG
+	char temp_tag[128*4+1];
+	int i, j=0;
+	for (i=1;i<128;i++)
+	{
+		if ((tag[i]>=32)||(tag[i]<=127))
+			temp_tag[j++]=tag[i];
+		else {
+			temp_tag[j++]='\\';
+			temp_tag[j++]='x';
+			temp_tag[j++]=hex[tag[i]>>4];
+			temp_tag[j++]=hex[tag[i]&15];
+		}
+	}
+	temp_tag[j++]=0;
+	fprintf(stderr, "parseid3v1(m=%p, tag=%s) ", m, temp_tag);
+#endif
 	if (!memcmp(tag, "TAG", 3))
 	{
 		if (memcmp(tag+3, "                              ", 30))
@@ -61,6 +119,9 @@ static void parseid3v1(struct moduleinfostruct *m, const unsigned char *tag)
 			m->modname[30]=0;
 			while (strlen(m->modname)&&(m->modname[strlen(m->modname)-1]==' '))
 				m->modname[strlen(m->modname)-1]=0;*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "m->modname=\"%s\"\n", m->modname);
+#endif
 		}
 		if (memcmp(tag+33, "                              ", 30))
 		{
@@ -70,6 +131,9 @@ static void parseid3v1(struct moduleinfostruct *m, const unsigned char *tag)
 			m->composer[30]=0;
 			while (strlen(m->composer)&&(m->composer[strlen(m->composer)-1]==' '))
 		      		m->composer[strlen(m->composer)-1]=0;*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "m->composer=\"%s\"\n", m->composer);
+#endif
 		}
 		if (memcmp(tag+63, "                              ", 30)||memcmp(tag+97, "                              ", 30))
 		{
@@ -81,15 +145,26 @@ static void parseid3v1(struct moduleinfostruct *m, const unsigned char *tag)
 			m->comment[62]=0;
 			while (strlen(m->comment)&&(m->comment[strlen(m->comment)-1]==' '))
 		      		m->comment[strlen(m->comment)-1]=0;
+#ifdef ID3_DEBUG
+			fprintf(stderr, "m->comment=\"%s\"\n", m->comment);
+#endif
 		}
 		if (tag[127]<43)
+		{
 			strcpy(m->style, mpstyles[tag[127]]);
+#ifdef ID3_DEBUG
+			fprintf(stderr, "m->style=\"%s\"\n", m->style);
+#endif
+		}
 		if (memcmp(tag+93, "    ", 4))
 		{
 			char i[5];
 			memcpy(i, tag+93, 4);
 			i[4]=0;
 			m->date=atoi(i)<<16;
+#ifdef ID3_DEBUG
+			fprintf(stderr, "m->date=%d\n", m->date);
+#endif
 		}
 	}
 }
@@ -98,9 +173,16 @@ static inline uint_fast32_t unsync(uint8_t *src, uint_fast32_t orgsize)
 {
 	uint_fast32_t retval = 0;
 	uint8_t *dst, *end = src+orgsize-1;
-
+#ifdef ID3_DEBUG
+	fprintf(stderr, "unsync(src=%p, orgsize=%d) =", src, orgsize);
+#endif
 	if (!orgsize)
+	{
+#ifdef ID3_DEBUG
+		fprintf(stderr, "0\n");
+#endif
 		return 0;
+	}
 
 	for (dst=src;(src+1)<end;src++)
 	{
@@ -115,28 +197,74 @@ static inline uint_fast32_t unsync(uint8_t *src, uint_fast32_t orgsize)
 	*(dst++)=*src;
 	retval++;
 
+#ifdef ID3_DEBUG
+	fprintf(stderr, "%d\n", retval);
+#endif
 	return retval;
 }
 
 static int parseid3v20(struct moduleinfostruct *m, const uint8_t *id3v2header, uint8_t *id3v2data, uint_fast32_t len)
 {
+#ifdef ID3_DEBUG
+	fprintf(stderr, "parseid3v20(m, id3v2header = %p (not used), id3v3data = %p, len = %d\n", id3v2header, id3v2data, len);
+#endif
 	while (1)
 	{
 		uint_fast32_t size;
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v20, iterate, len = %d\n", len);
+#endif
 		if (len<1)
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 len<1, return\n");
+#endif
 			return 1; /* out of data */
+	 	}
 		if (id3v2data[0]==0)
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 padding reached, return\n");
+#endif
 			return 0; /* padding reached */
+		}
 		if (len<6)
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 len<6, return\n");
+#endif
 			return 1; /* out of data */
+		}
 		size = (id3v2data[3]<<16)|(id3v2data[4]<<8)|(id3v2data[5]);
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v20 size=%06x (computed from id3v2data[3]=%02x id3v2data[4]=%02x id3v2data[5]=%02x)\n", size, id3v2data[3], id3v2data[4], id3v2data[5]);
+#endif
 		if (len<(size+6))
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 len<(size+6), return\n");
+#endif
 			return 1; /* out of data */
-
+		}
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v20 tag=\"%c%c%c\"\n", id3v2data[0], id3v2data[1], id3v2data[2]);
+#endif
 		if (!memcmp(id3v2data, "TP1", 3))
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 store into m->composer (size=%d)\n", sizeof(m->composer));
+#endif
 			handle_T__(id3v2data+6, size, m->composer, sizeof(m->composer));
-		else if (!memcmp(id3v2data, "TT2", 3))
+		} else if (!memcmp(id3v2data, "TT2", 3))
+		{
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v20 store into m->modname (size=%d)\n", sizeof(m->modname));
+#endif
 			handle_T__(id3v2data+6, size, m->modname, sizeof(m->modname));
+		}
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v20 skipping 6+size into id3v2data/len\n");
+#endif
 		len-=(size+6);
 		id3v2data+=(size+6);
 	}
@@ -144,52 +272,100 @@ static int parseid3v20(struct moduleinfostruct *m, const uint8_t *id3v2header, u
 
 static int parseid3v23(struct moduleinfostruct *m, const uint8_t *id3v2header, uint8_t *id3v2data, uint_fast32_t len)
 {
-/*	fprintf(stderr, "parseid3v23 reached with len=%d\n", (int)len);*/
+#ifdef ID3_DEBUG
+	fprintf(stderr, "parseid3v23(m, id3v2header = %p (not used), id3v3data = %p, len = %d\n", id3v2header, id3v2data, len);
+#endif
 	while (1)
 	{
 		uint_fast32_t size;
-/*		{
-			int i;
-			for (i=0;i<10;i++)
-				fprintf(stderr, "%02x ", id3v2data[i]);
-			fprintf(stderr, "\n");
-		}*/
+
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v23, iterate, len = %d\n", len);
+#endif
 		if (len<1)
 		{
-		/*	fprintf(stderr, "out of data\n");*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 len<1, return\n");
+#endif
 			return 1; /* out of data */
-		}
+	 	}
 		if (id3v2data[0]==0)
 		{
-			/*fprintf(stderr, "padding reached\n");*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 padding reached, return\n");
+#endif
 			return 0; /* padding reached */
 		}
 		if (len<10)
 		{
-			/*fprintf(stderr, "len<10\n");*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 len<10, return\n");
+#endif
 			return 1; /* out of data */
 		}
+
 		size = (id3v2data[4]<<24)|(id3v2data[5]<<16)|(id3v2data[6]<<8)|(id3v2data[7]);
-		/*fprintf(stderr, "%02x %02x %02x %02x -> %d\n", id3v2data[4], id3v2data[5], id3v2data[6], id3v2data[7], (int)size);*/
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v23 size=%08x (computed from id3v2data[4]=%02x id3v2data[5]=%02x id3v2data[6]=%02x id3v2data[7]=%02x)\n", size, id3v2data[4], id3v2data[5], id3v2data[6], id3v2data[7]);
+#endif
 		if (len<(size+10))
 		{
-			/*fprintf(stderr, "len<(size+10)   len=%d size=%d\n", (int)len, (int)size);*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 len<(size+10), return\n");
+#endif	
 			return 1; /* out of data */
 		}
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v23 flags: id3v2data[8]=%02x id3v2data[9]=%02x\n", id3v2data[8], id3v2data[9]);
+#endif
 		if ((id3v2data[8]&0x8f)||(id3v2data[9]&0xfc))
 		{ /* TODO, we now silently ignore entries with flags, so zlib compressed tag-pages will fail */
-			/*fprintf(stderr, "TODO\n");*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 unsupported flag detected (like zlib compression etc.)\n");
+#endif
 		} else {
 			int offset=10;
+			uint_fast32_t _size = size;
+
 			if (id3v2data[9]&0x02)
-				size = unsync(id3v2data+offset, size);
+			{
+				_size = unsync(id3v2data+offset, size);
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v23 unsync bit set, size updated to %d\n", size);
+#endif
+			}
 			if (id3v2data[9]&0x01)
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v23 size-validation check, ignoring, skipping 4 bytes\n");
+#endif
 				offset+=4; /* we ignore the size-validation check here */
+				if (_size < 4)
+					_size = 0;
+				else
+					_size -= 4;
+			}
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v23 tag=\"%c%c%c%c\"\n", id3v2data[0], id3v2data[1], id3v2data[2], id3v2data[3]);
+#endif
+	
 			if (!memcmp(id3v2data, "TPE1", 4))
-				handle_T___(id3v2data+offset, size, m->composer, sizeof(m->composer));
-			else if (!memcmp(id3v2data, "TIT2", 4))
-				handle_T___(id3v2data+offset, size, m->modname, sizeof(m->modname));
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v23 storing into m->composer\n");
+#endif
+				handle_T___(id3v2data+offset, _size, m->composer, sizeof(m->composer));
+			} else if (!memcmp(id3v2data, "TIT2", 4))
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v23 storing into m->modname\n");
+#endif
+				handle_T___(id3v2data+offset, _size, m->modname, sizeof(m->modname));
+			}
 		}
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v23 skipping 10+size into id3v2data/len\n");
+#endif
 		len-=(size+10);
 		id3v2data+=(size+10);
 	}
@@ -199,19 +375,33 @@ static int parseid3v23(struct moduleinfostruct *m, const uint8_t *id3v2header, u
 /* returns 1 on ok */
 static int parseid3v2(struct moduleinfostruct *m, const uint8_t *id3v2header, uint8_t *id3v2data, uint_fast32_t len)
 {
+#ifdef ID3_DEBUG
+	fprintf(stderr, "parseid3v2(m, id3v2header = %p, id3v3data = %p, len = %d\n", id3v2header, id3v2data, len);
+	fprintf(stderr, "// parseid3v2 tag version 2.%d\n", id3v2header[3]);
+#endif
+
 	if ((id3v2header[5]&0x80))
 	{
-		/*fprintf(stderr, "unsync\n");*/
 		len = unsync(id3v2data, len);
+#ifdef ID3_DEBUG
+		fprintf(stderr, "// parseid3v2 global unsync bit set, new len = %d\n", len);
+#endif
 	}
 	/*fprintf(stderr, "Version 2.%d\n", id3v2header[3]);*/
 	if (id3v2header[3]<4)
 	{
 		if (id3v2header[5]&0x40)
 		{
-			/*fprintf(stderr, "Removing extended header\n");*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v2 TAG version lower than 2.4, and extended tag flaged, trying to skip it (len is fixed to 10 bytes)\n");
+#endif
 			if (len<10)
-				return 0; /* failed to parse */
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v2 to little data to skip, return \n");
+#endif
+				return 1; /* failed to parse */
+			}
 			len-=10;
 			id3v2data+=10;
 		}
@@ -219,19 +409,36 @@ static int parseid3v2(struct moduleinfostruct *m, const uint8_t *id3v2header, ui
 		if (id3v2header[5]&0x40)
 		{
 			uint_fast32_t esize;
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v2 TAG version greater or equal to 2.4, and extended tag flaged, trying  to skip it\n");
+#endif
 			/*fprintf(stderr, "Removing extended header\n");*/
 			if (len<6)
-				return 0; /* failed to parse */
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v2 to little data available to calculate extended header length (len<6), return\n");
+#endif
+				return 1; /* failed to parse */
+			}
 			esize = (id3v2data[0]<<21)|(id3v2data[1]<<14)|(id3v2data[2]<<7)|(id3v2data[3]);
-			/*fprintf(stderr, "esize=%d\n", (int)esize);*/
+#ifdef ID3_DEBUG
+			fprintf(stderr, "// parseid3v2 extended header size=%08x, calculated from id3v2data[0]=%02x, id3v2data[1]=%02x, id3v2data[2]=%02x, id3v2data=%02x\n", esize, id3v2data[0], id3v2data[1], id3v2data[2], id3v2data[3]);
+#endif
 			if (len<esize)
-				return 0; /* failed to parse */
+			{
+#ifdef ID3_DEBUG
+				fprintf(stderr, "// parseid3v2 skipping extended header failed, to little data available (len<size), return\n");
+#endif
+				return 1; /* failed to parse */
+			}
 			len-=esize;
 			id3v2data+=esize;
 		}
 	}
 
-	/* skip 10 bytes extended header if any */
+#ifdef ID3_DEBUG
+	fprintf(stderr, "// parseid3v2, calling sub-parser based on version\n");
+#endif
 	if (id3v2header[3]<=2)
 		return parseid3v20(m, id3v2header, id3v2data, len);
 	else
@@ -643,7 +850,11 @@ static int ampegpReadInfo(struct moduleinfostruct *m, FILE *f, const char *buf,
 		{
 			uint8_t id3v1[128];
 			fseeko(f, -128, SEEK_CUR);
-			fread(id3v1, 128, 1, f);
+			if (fread(id3v1, 128, 1, f)!=1)
+			{
+				fseeko(f, 0, SEEK_SET);
+				return 0;
+			}
 			if ((id3v1[0]=='T')&&(id3v1[1]=='A')&&(id3v1[2]=='G'))
 			{
 				m->modtype=mtMPx;
@@ -657,7 +868,11 @@ static int ampegpReadInfo(struct moduleinfostruct *m, FILE *f, const char *buf,
 		{
 			uint8_t id3v2header[10];
 			fseeko(f, -10, SEEK_CUR);
-			fread(id3v2header, 10, 1, f);
+			if (fread(id3v2header, 10, 1, f)!=1)
+			{
+				fseeko(f, 0, SEEK_SET);
+				return 0;
+			}
 			if ((id3v2header[0]=='I')&&(id3v2header[1]=='D')&&(id3v2header[2]=='3')&&(id3v2header[3]!=0xff)&&(id3v2header[4]!=0xff))
 			{
 				uint8_t *id3v2data;
@@ -668,8 +883,8 @@ static int ampegpReadInfo(struct moduleinfostruct *m, FILE *f, const char *buf,
 				fseeko(f, -(size), SEEK_CUR);
 
 				id3v2data = malloc(size);
-				fread(id3v2data, 10, 1, f);
-				parseid3v2(m, id3v2header, id3v2data, size);
+				if (fread(id3v2data, 10, 1, f)==1)
+					parseid3v2(m, id3v2header, id3v2data, size);
 				free(id3v2data);
 
 				fseeko(f, -(size+10), SEEK_CUR);
@@ -748,4 +963,4 @@ static int ampegpReadInfo(struct moduleinfostruct *m, FILE *f, const char *buf,
 	return 1;
 }
 
-struct mdbreadinforegstruct ampegpReadInfoReg = {ampegpReadMemInfo, ampegpReadInfo};
+struct mdbreadinforegstruct ampegpReadInfoReg = {ampegpReadMemInfo, ampegpReadInfo, 0 MDBREADINFOREGSTRUCT_TAIL};
