@@ -55,7 +55,7 @@ static uint32_t oggpos;
 static uint32_t ogglen;
 static uint8_t *oggbuf=NULL;
 static uint32_t oggbuflen;
-static int32_t oggbufpos;
+static uint32_t oggbufpos;
 static uint32_t oggbuffpos;
 static int32_t oggbufread;
 static uint32_t oggbufrate;
