@@ -1,3 +1,11 @@
+/* OpenCP Module Player
+ * copyright (c) '05-'09 Stian Skjelstad <stian@nixia.no>
+ *
+ * OPLPlay interface routines
+ */ 
+
+
+
 #include "config.h"
 #include <fcntl.h>
 #include <math.h>
@@ -229,8 +237,8 @@ static void drawchannel(uint16_t *buf, int len, int i) /* TODO */
         unsigned char tcol=st?0x08:0x0F;
         unsigned char tcold=st?0x08:0x07;
 
-	char *waves4 []= {"sine", "half", "2x  ", "saw "};
-	char *waves16[]= {"sine curves     ", "half sine curves", "positiv sines   ", "sawtooth        "};
+	const char *waves4 []= {"sine", "half", "2x  ", "saw "};
+	const char *waves16[]= {"sine curves     ", "half sine curves", "positiv sines   ", "sawtooth        "};
 
 	switch (len)
 	{
@@ -338,6 +346,13 @@ static int oplProcessKey(uint16_t key)
 
 	switch (key)
 	{
+		case KEY_ALT_K:
+			cpiKeyHelp('p', "Start/stop pause with fade");
+			cpiKeyHelp('P', "Start/stop pause with fade");
+			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
+			if (plrProcessKey)
+				plrProcessKey(key);
+			return 0;
 		case 'p': case 'P':
 			startpausefade();
 			break;
@@ -350,15 +365,17 @@ static int oplProcessKey(uint16_t key)
 			oplPause(plPause);
 			break;
 #if 0
-			/* TODO keys
-		case 0x8D00: //ctrl-up
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
 			mpegSetPos(mpegGetPos()-mpegrate);
 			break;
-		case 0x9100: //ctrl-down
+		case KEY_CTRL_DOWN:
+		/* case 0x9100: //ctrl-down */
 			mpegSetPos(mpegGetPos()+mpegrate);
 			break;*/
 		case '<':
-		/*		case 0x7300: //ctrl-left  todo keys */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			{
 				int skip=mpeglen>>5;
 				if (skip<128*1024)
@@ -367,7 +384,8 @@ static int oplProcessKey(uint16_t key)
 			}
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			{
 				int skip=mpeglen>>5;
 				if (skip<128*1024)
@@ -375,7 +393,7 @@ static int oplProcessKey(uint16_t key)
 				mpegSetPos(mpegGetPos()+skip);
 			}
 			break;
-		case 0x7700: //ctrl-home
+		case 0x7700: //ctrl-home TODO keys
 			mpegSetPos(0);
 			break;
 	#endif
@@ -440,7 +458,12 @@ static int oplOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 			res=fread(buffer, 1, sizeof(buffer), file);
 			if (res<=0)
 				break;
-			write(fd, buffer, res);
+			if (write(fd, buffer, res)!=res)
+			{
+				perror(__FILE__ ": write failed: ");
+				unlink(_path);
+				return -1;
+			}
 		}
 	}
 	close(fd);
@@ -475,5 +498,5 @@ static int oplOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 extern "C"
 {
   cpifaceplayerstruct oplPlayer = {oplOpenFile, oplCloseFile};
-  struct linkinfostruct dllextinfo = {"playopl", "OpenCP AdPlug (OPL) Player (c) 2005 Stian Skjelstad", 0, 0};
+  struct linkinfostruct dllextinfo = {"playopl", "OpenCP AdPlug (OPL) Player (c) 2005 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
 }
