@@ -1,3 +1,9 @@
+/* OpenCP Module Player
+ * copyright (c) '05-'09 Stian Skjelstad <stian@nixia.no>
+ *
+ * opl-compatible file type detection routines for the file selector
+ */ 
+
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
@@ -61,7 +67,7 @@ static void oplEvent(int event)
 	}
 }
 
-static struct mdbreadinforegstruct oplReadInfoReg = {oplReadMemInfo, oplReadInfo, oplEvent};
+static struct mdbreadinforegstruct oplReadInfoReg = {oplReadMemInfo, oplReadInfo, oplEvent MDBREADINFOREGSTRUCT_TAIL};
 
 static void __attribute__((constructor))init(void)
 {
@@ -74,6 +80,6 @@ static void __attribute__((destructor))done(void)
 }
 
 extern "C" {
-	char *dllinfo = "";
-	struct linkinfostruct dllextinfo = {"opltype", "OpenCP OPL Detection (c) 2005 Stian Skjelstad", DLLVERSION, 0};
+	const char *dllinfo = "";
+	struct linkinfostruct dllextinfo = {"opltype", "OpenCP OPL Detection (c) 2005 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
 }
