@@ -12,6 +12,7 @@
 //    -use compatible timing, and not cputime/clock()
 
 #include "config.h"
+#include <curses.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -234,23 +235,23 @@ static void drawlongvolbar(uint16_t *buf, int, unsigned char st)
 }
 
 
-static char *waves4[]={"    ","tri ","saw ","trsw","puls","trpu","swpu","tsp ",
-                       "nois","????","????","????","????","????","????","????"};
+static const char *waves4[]={"    ","tri ","saw ","trsw","puls","trpu","swpu","tsp ",
+                             "nois","????","????","????","????","????","????","????"};
 
-static char *waves16[]={"                ","triangle        ","sawtooth        ",
-                       "tri + saw       ","pulse           ","triangle + pulse",
-                       "sawtooth + pulse","tri + saw + puls","noise           ",
-                       "invalid         ","invalid         ","invalid         ",
-                       "invalid         ","invalid         ","invalid         ",
-                       "invalid         "};
+static const char *waves16[]={"                ","triangle        ","sawtooth        ",
+                              "tri + saw       ","pulse           ","triangle + pulse",
+                              "sawtooth + pulse","tri + saw + puls","noise           ",
+                              "invalid         ","invalid         ","invalid         ",
+                              "invalid         ","invalid         ","invalid         ",
+                              "invalid         "};
 
-static char *filters3[]={"   ","low","bnd","b+l","hgh","h+l","h+b","hbl"};
-static char *filters12[]={"","low pass","band pass","low + band","high pass",
-                         "band notch","high + band","all pass"};
+static const char *filters3[]={"   ","low","bnd","b+l","hgh","h+l","h+b","hbl"};
+static const char *filters12[]={"","low pass","band pass","low + band","high pass",
+                                "band notch","high + band","all pass"};
 
-static char *fx2[]={"  ","sy","ri","rs"};
-static char *fx7[]={"","sync","ringmod","snc+rng"};
-static char *fx11[]={"","sync","ringmod","sync + ring"};
+static const char *fx2[]={"  ","sy","ri","rs"};
+static const char *fx7[]={"","sync","ringmod","snc+rng"};
+static const char *fx11[]={"","sync","ringmod","sync + ring"};
 
 static void drawchannel(uint16_t *buf, int len, int i)
 {
@@ -426,6 +427,36 @@ static int sidpProcessKey(uint16_t key)
 	char csg;
 	switch (key)
 	{
+		case KEY_ALT_K:
+			cpiKeyHelp('p', "Start/stop pause");
+			cpiKeyHelp('P', "Start/stop pause");
+			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
+			cpiKeyHelp('<', "Previous track");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Previous track");
+			cpiKeyHelp('>', "Next track");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Next track");
+			cpiKeyHelp(KEY_BACKSPACE, "Toggle filter");
+			cpiKeyHelp('-', "Decrease volume (small)");
+			cpiKeyHelp('+', "Increase volume (small)");
+			cpiKeyHelp('/', "Move balance left (small)");
+			cpiKeyHelp('*', "Move balance right (small)");
+			cpiKeyHelp(',', "Move panning against normal (small)");
+			cpiKeyHelp('.', "Move panning against reverse (small)");
+			cpiKeyHelp(KEY_F(2), "Decrease volume");
+			cpiKeyHelp(KEY_F(3), "Increase volume");
+			cpiKeyHelp(KEY_F(4), "Toggle surround on/off");
+			cpiKeyHelp(KEY_F(5), "Move panning against normal");
+			cpiKeyHelp(KEY_F(6), "Move panning against reverse");
+			cpiKeyHelp(KEY_F(7), "Move balance left");
+			cpiKeyHelp(KEY_F(8), "Move balance right");
+			/*cpiKeyHelp(KEY_F(9), "Decrease pitch speed");
+			cpiKeyHelp(KEY_F(10), "Decrease pitch speed");*/
+			cpiKeyHelp(KEY_F(11), "Toggle chip version");
+			cpiKeyHelp(KEY_F(12), "Toggle video (PAL/NTCS)");
+			if (plrProcessKey)
+				plrProcessKey(key);
+			return 0;
+
 		case 'p': case 'P': case KEY_CTRL_P:
 			if (plPause)
 				starttime=starttime+dos_clock()-pausetime;
@@ -434,14 +465,17 @@ static int sidpProcessKey(uint16_t key)
 			plPause=!plPause;
 			sidpPause(plPause);
 			break;
-			/* TODO keys
-		case 0x8D00: //ctrl-up
+#if 0
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
 			break;
-		case 0x9100: //ctrl-down
-			break;*/
-			
+		case KEY_CTRL_DOWN:
+		/* case 0x9100: //ctrl-down */
+			break;
+#endif			
 		case '<':
-		/*		case 0x7300: //ctrl-left  TODO KEYS */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			csg=globinfo.currentSong-1;
 			if (csg)
 			{
@@ -451,7 +485,8 @@ static int sidpProcessKey(uint16_t key)
 			sidpGetGlobInfo(globinfo);
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			csg=globinfo.currentSong+1;
 			if (csg<=globinfo.songs)
 			{
@@ -465,7 +500,7 @@ static int sidpProcessKey(uint16_t key)
 			starttime=dos_clock();
 			sidpGetGlobInfo(globinfo);
 			break;*/
-		case KEY_BS: //backspace
+		case KEY_BACKSPACE: //backspace
 			sidpToggleFilter();
 			break;
 		case '-':
@@ -498,75 +533,75 @@ static int sidpProcessKey(uint16_t key)
 				pan=64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F2:
+		case KEY_F(2):
 			if ((vol-=8)<0)
 				vol=0;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F3:
+		case KEY_F(3):
 			if ((vol+=8)>64)
 				vol=64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F4:
+		case KEY_F(4):
 			sidpSetVolume(vol, bal, pan, srnd=srnd?0:2);
 			break;
-		case KEY_F5:
+		case KEY_F(5):
 			if ((pan-=16)<-64)
 				pan=-64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F6:
+		case KEY_F(6):
 			if ((pan+=16)>64)
 				pan=64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F7:
+		case KEY_F(7):
 			if ((bal-=16)<-64)
 				bal=-64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F8:
+		case KEY_F(8):
 			if ((bal+=16)>64)
 				bal=64;
 			sidpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F9:
+		case KEY_F(9):
 			break;
-		case KEY_F10:
+		case KEY_F(10):
 			break;
-		case KEY_F11:
+		case KEY_F(11):
 			sidpToggleSIDVersion();
 			sidpGetGlobInfo(globinfo);
 			break;
-		case KEY_F12:
+		case KEY_F(12):
 			sidpToggleVideo();
 			sidpGetGlobInfo(globinfo);
 			break;
-			/* TODO keys
-		case 0x5f00: // ctrl f2
+			/*
+		case 0x5f00: // ctrl f2 TODO keys
 			if ((amp-=4)<4)
 				amp=4;
 			sidpSetAmplify(1024*amp);
 			break;
-		case 0x6000: // ctrl f3
+		case 0x6000: // ctrl f3 TODO keys
 			if ((amp+=4)>508)
 				amp=508;
 			sidpSetAmplify(1024*amp);
 			break;
-		case 0x8900: // ctrl f11
+		case 0x8900: // ctrl f11 TODO keys
 			break;
-		case 0x6a00:
+		case 0x6a00: // alt-f3 TODO keys
 			normalize();
 			break;
-		case 0x6900:
+		case 0x6900: // alt-f2 TODO keys
 			set.pan=pan;
 			set.bal=bal;
 			set.vol=vol;
 			set.amp=amp;
 			set.srnd=srnd;
 			break;
-		case 0x6b00:
+		case 0x6b00: // alt-f4 TODO keys
 			pan=64;
 			bal=0;
 			vol=64;
@@ -641,5 +676,5 @@ static int sidpOpenFile(const char *path, struct moduleinfostruct *info, FILE *s
 extern "C"
 {
 	cpifaceplayerstruct sidPlayer = {sidpOpenFile, sidpCloseFile};
-	struct linkinfostruct dllextinfo = {"playsid", "OpenCP SID Player (c) 1993-04 Michael Schwendt, Tammo Hinrichs", 0, 0};
+	struct linkinfostruct dllextinfo = {"playsid", "OpenCP SID Player (c) 1993-04 Michael Schwendt, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
 }
