@@ -29,13 +29,13 @@ static uint8_t signedout;
 static uint32_t samprate;
 static uint8_t reversestereo;
 
-static uint16_t *buf16;
+static uint16_t *buf16=0;
 static uint32_t bufpos;
 static uint32_t buflen;
 static void *plrbuf;
 
-static uint16_t *cliptabl;
-static uint16_t *cliptabr;
+static uint16_t *cliptabl=0;
+static uint16_t *cliptabr=0;
 static uint32_t amplify;
 static uint32_t voll,volr;
 static int pan;
@@ -55,7 +55,7 @@ static uint32_t waverate;
 static uint32_t wavepos;
 static uint32_t wavelen;
 static uint32_t waveoffs;
-static uint8_t *wavebuf;
+static uint8_t *wavebuf=0;
 static uint32_t wavebuflen;
 static uint32_t wavebufpos;
 static uint32_t wavebuffpos;
@@ -70,6 +70,58 @@ static int pause;
 
 static int clipbusy=0;
 
+#ifdef WAVE_DEBUG
+static const char *compression_code_str(uint_fast16_t code)
+{
+	switch (code)
+	{
+		case 1:
+			return "PCM/uncompressed";
+		case 2:
+			return "Microsoft ADPCM";
+		case 3:
+			return "Floating point PCM";
+		case 5:
+			return "Digispeech CVSD / IBM PS/2 Speech Adapter (Motorola MC3418)";
+		case 6:
+			return "ITU G.711 a-law";
+		case 7:
+			return "ITU G.711 mu-law";
+		case 0x10:
+			return "OKI ADPCM";
+		case 0x11:
+			return "DVI ADPCM";
+		case 0x15:
+			return "Digispeech DIGISTD";
+		case 0x16:
+			return "Digispeech DigiFix";
+		case 0x17:
+			return "IMA ADPCM";
+		case 0x20:
+			return "ITU G.723 ADPCM (Yamaha)";
+		case 0x22:
+			return "DSP Group TrueSpeech";
+		case 0x31:
+			return "GSM6.10";
+		case 0x49:
+			return "GSM 6.10";
+		case 0x64:
+			return "ITU G.721 ADPCM";
+		case 0x70:
+			return "Lernout & Hauspie CELP";
+		case 0x72:
+			return "Lernout & Hauspie SBC";
+		case 0x80:
+			return "MPEG";
+		case 65535:
+			return "Experimental";
+		case 0:
+		default:
+			return "Unknown";
+	}
+}
+#endif
+
 static void calccliptab(int32_t ampl, int32_t ampr)
 {
 	int i;
@@ -130,7 +182,7 @@ static void timerproc(void)
 	uint32_t bufdelta;
 	uint32_t pass2;
 	int quietlen;
-	int toloop;
+	uint32_t toloop;
 
 	if (clipbusy++)
 	{
@@ -152,7 +204,7 @@ static void timerproc(void)
 	bufdelta=(buflen+bufplayed-bufpos)%buflen;
 	if (wavebuflen!=wavelen)
 	{
-		int towrap=imuldiv((((wavebuflen+wavebufread-wavebufpos-1)%wavebuflen)>>(wavestereo+wave16bit)), 65536, wavebufrate);
+		uint32_t towrap=(unsigned)imuldiv((((wavebuflen+wavebufread-wavebufpos-1)%wavebuflen)>>(wavestereo+wave16bit)), 65536, wavebufrate);
 		if (bufdelta>towrap)
 			/*quietlen=bufdelta-towrap;*/
 			bufdelta=towrap;
@@ -161,7 +213,7 @@ static void timerproc(void)
 	if (pause)
 		quietlen=bufdelta;
 
-	toloop=imuldiv(((bufloopat-wavebufpos)>>(wave16bit+wavestereo)), 65536, wavebufrate);
+	toloop=(unsigned)imuldiv(((bufloopat-wavebufpos)>>(wave16bit+wavestereo)), 65536, wavebufrate);
 	if (looped)
 		toloop=0;
 
@@ -179,7 +231,7 @@ static void timerproc(void)
 
 	if (bufdelta)
 	{
-		int i;
+		unsigned int i;
 		if ((bufpos+bufdelta)>buflen)
 			pass2=bufpos+bufdelta-buflen;
 		else
@@ -190,7 +242,8 @@ static void timerproc(void)
 		{
 			if (wavestereo)
 			{
-				int32_t wpm1, wp1, wp2, c0, c1, c2, c3, ls, rs, vm1,v1,v2;
+				int32_t wpm1, c0, c1, c2, c3, ls, rs, vm1,v1,v2;
+				unsigned wp1, wp2;
 				for (i=0; i<bufdelta; i++)
 				{
 
@@ -238,7 +291,8 @@ static void timerproc(void)
 						wavebufpos-=wavebuflen;
 				}
 			} else { /* wavestereo */
-				int32_t wpm1, wp1, wp2, c0, c1, c2, c3, vm1,v1,v2;
+				int32_t wpm1, c0, c1, c2, c3, vm1,v1,v2;
+				uint32_t wp1, wp2;
 				for (i=0; i<bufdelta; i++)
 				{
 
@@ -408,10 +462,8 @@ void __attribute__ ((visibility ("internal"))) wpIdle(void)
 uint8_t __attribute__ ((visibility ("internal"))) wpOpenPlayer(FILE *wav, int tostereo, int tolerance)
 {
 	uint32_t temp;
-	int32_t fmtlen;
+	uint32_t fmtlen;
 	uint16_t sh;
-
-	int res;
 	
 	if (!plrPlay)
 		return 0;
@@ -432,71 +484,198 @@ uint8_t __attribute__ ((visibility ("internal"))) wpOpenPlayer(FILE *wav, int to
 	wavefile=wav;
 	fseek(wavefile, 0, SEEK_SET);
 
-	fread(&temp, sizeof(temp), 1, wavefile);
-	if (temp!=int32_little(0x46464952))
+	if (fread(&temp, sizeof(temp), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #1\n");
+		return 0;
+	}
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": comparing header for RIFF: 0x%08x 0x%08x\n", temp, uint32_little(0x46464952));
+#endif
+	if (temp!=uint32_little(0x46464952))
 		return 0;
 
-	fread(&temp, sizeof(temp), 1, wavefile);
+	if (fread(&temp, sizeof(temp), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #2\n");
+		return 0;
+	}
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": ignoring next 32bit: 0x%08x\n", temp);
+#endif
 
-	fread(&temp, sizeof(temp), 1, wavefile);
-	if (temp!=int32_little(0x45564157))
+	if (fread(&temp, sizeof(temp), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #3\n");
+		return 0;
+	}
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": comparing next header for WAVE: 0x%08x 0x%08x\n", temp, uint32_little(0x45564157));
+#endif
+	if (temp!=uint32_little(0x45564157))
 		return 0;
 
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": going to locate \"fmt \" header\n");
+#endif
 	while (1)
 	{
-		fread(&temp, sizeof(temp), 1, wavefile);
-		if (temp==int32_little(0x20746D66))
-			break;
-		if(feof(wavefile))
+		if (fread(&temp, sizeof(temp), 1, wavefile) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread failed #4\n");
 			return 0;
-		fread(&temp, sizeof(temp), 1, wavefile);
-		temp = int32_little(temp);
+		}
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ":  checking 0x%08x 0x%08x\n", temp, uint32_little(0x20746d66));
+#endif
+		if (temp==uint32_little(0x20746D66))
+			break;
+		if (fread(&temp, sizeof(temp), 1, wavefile) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread failed #5\n");
+			return 0;
+		}
+		temp = uint32_little(temp);
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ": failed, skiping next %d bytes\n", temp);
+#endif
 		fseek(wavefile, temp, SEEK_CUR);
 	}
-	res=fread(&fmtlen, sizeof(int32_t), 1, wavefile);
-	fmtlen = int32_little(fmtlen);
+	if (fread(&fmtlen, sizeof(fmtlen), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #6\n");
+		return 0;
+	}
+	fmtlen = uint32_little(fmtlen);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": fmtlen=%d (must be bigger or equal to 16)\n", fmtlen);
+#endif
 	if (fmtlen<16)
 		return 0;
-	res=fread(&sh, sizeof(uint16_t), 1, wavefile);
-	sh = int16_little (sh);
-	if ((sh!=1))
+	if (fread(&sh, sizeof(uint16_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #7\n");
 		return 0;
+	}
+	sh = uint16_little (sh);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": compression code (only 1/pcm is supported): %d %s\n", sh, compression_code_str(sh));
+#endif
+	if ((sh!=1))
+	{
+		fprintf(stderr, __FILE__ ": not uncomressed raw pcm data\n");
+		return 0;
+	}
 
-	fread(&sh, sizeof(uint16_t), 1, wavefile);
-	sh = int16_little (sh);
+	if (fread(&sh, sizeof(uint16_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #8\n");
+		return 0;
+	}
+	sh = uint16_little (sh);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": number of channels: %d\n", (int)sh);
+#endif
+	if ((sh==0)||(sh>2))
+	{
+		fprintf(stderr, __FILE__ ": unsupported number of channels: %d\n", sh);
+		return 0;
+	}
 	wavestereo=(sh==2);
 
-	fread(&waverate, sizeof(uint32_t), 1, wavefile);
-	waverate = int32_little (waverate);
-	fread(&temp, sizeof(uint32_t), 1, wavefile);
-	fread(&sh, sizeof(uint16_t), 1, wavefile);
-	
-	fread(&sh, sizeof(uint16_t), 1, wavefile);
-	sh = int16_little (sh);
+	if (fread(&waverate, sizeof(uint32_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #9\n");
+		return 0;
+	}
+	waverate = uint32_little (waverate);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": waverate %d\n", (int)waverate);
+#endif
+
+	if (fread(&temp, sizeof(uint32_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #10\n");
+		return 0;
+	}
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": average number of bytes per second: %d\n", (int)(uint32_little(temp)));
+#endif
+
+	if (fread(&sh, sizeof(uint16_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #11\n");
+		return 0;
+	}
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": block align: %d\n", (int)(uint16_little(sh)));
+#endif
+
+	if (fread(&sh, sizeof(uint16_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #12\n");
+		return 0;
+	}
+	sh = uint16_little (sh);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": bits per sample: %d\n", (int)sh);
+#endif
+	if ((sh!=8)&&(sh!=16))
+	{
+		fprintf(stderr, __FILE__ ": unsupported bits per sample: %d\n", (int)sh);
+		return 0;
+	}
 	wave16bit=(sh==16);
 	fseek(wavefile, fmtlen-16, SEEK_CUR);
 
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": going to locate \"data\" header\n");
+#endif
 	while (1)
 	{
-		fread(&temp, sizeof(uint32_t), 1, wavefile);
-		if (temp==int32_little(0x61746164))
-			break;
-		if(feof(wavefile))
+		if (fread(&temp, sizeof(uint32_t), 1, wavefile) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread failed #13\n");
 			return 0;
-
-		fread(&temp, sizeof(uint32_t), 1, wavefile);
-		temp = int32_little (temp);
+		}
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ":  checking 0x%08x 0x%08x\n", temp, uint32_little(0x61746164));
+#endif
+		if (temp==uint32_little(0x61746164))
+			break;
+		if (fread(&temp, sizeof(uint32_t), 1, wavefile) != 1)
+		{
+			fprintf(stderr, __FILE__ ": fread failed #14\n");
+			return 0;
+		}
+		temp = uint32_little (temp);
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ": failed, skiping next %d bytes\n", temp);
+#endif
 		fseek(wavefile, temp, SEEK_CUR);
 	}
 
-	fread(&wavelen, sizeof(uint32_t), 1, wavefile);
-	wavelen = int32_little (wavelen);
+	if (fread(&wavelen, sizeof(uint32_t), 1, wavefile) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #15\n");
+		return 0;
+	}
+	wavelen = uint32_little (wavelen);
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ": datalength: %d\n", (int)wavelen);
+#endif	
 	waveoffs=ftell(wavefile);
+#ifdef WAVE_DEBUG
+	fprintf(stderr, __FILE__ ": waveoffs: %d\n", waveoffs);
+#endif
 	/*rawwave.open(*wavefile, waveoffs, wavelen);*/
-	fseek(wavefile, waveoffs, SEEK_CUR);
+	/*fseek(wavefile, waveoffs, SEEK_CUR); */
 
 	if (!wavelen)
+	{
+		fprintf(stderr, __FILE__ ": no data\n");
 		return 0;
+	}
 	wavebuflen=1024*1024;
 	if (wavebuflen>wavelen)
 	{
@@ -517,7 +696,11 @@ uint8_t __attribute__ ((visibility ("internal"))) wpOpenPlayer(FILE *wav, int to
 	wavebuffpos=0;
 	wavebufread=0;
 
-	fread(wavebuf, 1, wavebuflen, rawwave);
+	if (fread(wavebuf, wavebuflen, 1, rawwave) != 1)
+	{
+		fprintf(stderr, __FILE__ ": fread failed #16\n");
+		return 0;
+	}
 	wavepos=wavebuflen;
 
 	plrSetOptions(waverate, (convtostereo||wavestereo)?(PLR_STEREO|PLR_16BIT):PLR_16BIT);
@@ -567,13 +750,25 @@ void __attribute__ ((visibility ("internal"))) wpClosePlayer(void)
 {
 	active=0;
 
+#ifdef WAVE_DEBUG
+		fprintf(stderr, __FILE__ ": Freeing resources\n");
+#endif
+
 	pollClose();
 
 	plrClosePlayer();
-	free(wavebuf);
-	free(buf16);
-	free(cliptabl);
-	free(cliptabr);
+	if (wavebuf)
+		free(wavebuf);
+	if (buf16)
+		free(buf16);
+	if (cliptabl)
+		free(cliptabl);
+	if (cliptabr)
+		free(cliptabr);
+	wavebuf=0;
+	buf16=0;
+	cliptabl=0;
+	cliptabr=0;
 /*	fclose(rawwave);*/
 }
 
@@ -633,7 +828,7 @@ void __attribute__ ((visibility ("internal"))) wpGetInfo(struct waveinfo *i)
 	i->bit16=wave16bit;
 }
 
-void __attribute__ ((visibility ("internal"))) wpSetPos(int32_t pos)
+void __attribute__ ((visibility ("internal"))) wpSetPos(uint32_t pos)
 {
 	pos=((pos<<(wave16bit+wavestereo))+wavelen)%wavelen;
 	if (wavelen==wavebuflen)
