@@ -11,6 +11,7 @@
  */
 
 #include "config.h"
+#include <curses.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
@@ -200,25 +201,29 @@ static int wavProcessKey(unsigned short key)
 			cpiKeyHelp('p', "Start/stop pause");
 			cpiKeyHelp('P', "Start/stop pause");
 			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
-			cpiKeyHelp('<', "Jump back");
-			cpiKeyHelp('>', "Jump forward");
+			cpiKeyHelp('<', "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump back (big)");
+			cpiKeyHelp('>', "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump back (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump forward (small)");
 			cpiKeyHelp('-', "Decrease volume (small)");
 			cpiKeyHelp('+', "Increase volume (small)");
 			cpiKeyHelp('/', "Move balance left (small)");
 			cpiKeyHelp('*', "Move balance right (small)");
 			cpiKeyHelp(',', "Move panning against normal (small)");
 			cpiKeyHelp('.', "Move panning against reverse (small)");
-			cpiKeyHelp(KEY_F2, "Decrease volume");
-			cpiKeyHelp(KEY_F3, "Increase volume");
-			cpiKeyHelp(KEY_F4, "Toggle surround on/off");
-			cpiKeyHelp(KEY_F5, "Move panning against normal");
-			cpiKeyHelp(KEY_F6, "Move panning against reverse");
-			cpiKeyHelp(KEY_F7, "Move balance left");
-			cpiKeyHelp(KEY_F8, "Move balance right");
-			cpiKeyHelp(KEY_F9, "Decrease pitch speed");
-			cpiKeyHelp(KEY_F11, "Decrease pitch speed");
-			cpiKeyHelp(KEY_F10, "Increase pitch speed");
-			cpiKeyHelp(KEY_F12, "Increase pitch speed");
+			cpiKeyHelp(KEY_F(2), "Decrease volume");
+			cpiKeyHelp(KEY_F(3), "Increase volume");
+			cpiKeyHelp(KEY_F(4), "Toggle surround on/off");
+			cpiKeyHelp(KEY_F(5), "Move panning against normal");
+			cpiKeyHelp(KEY_F(6), "Move panning against reverse");
+			cpiKeyHelp(KEY_F(7), "Move balance left");
+			cpiKeyHelp(KEY_F(8), "Move balance right");
+			cpiKeyHelp(KEY_F(9), "Decrease pitch speed");
+			cpiKeyHelp(KEY_F(11), "Decrease pitch speed");
+			cpiKeyHelp(KEY_F(10), "Increase pitch speed");
+			cpiKeyHelp(KEY_F(12), "Increase pitch speed");
 			if (plrProcessKey)
 				plrProcessKey(key);
 			return 0;
@@ -231,23 +236,29 @@ static int wavProcessKey(unsigned short key)
 			plPause=!plPause;
 			wpPause(plPause);
 			break;
-/* TODO-keys	case 0x8D00: //ctrl-up
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
 			wpSetPos(wpGetPos()-waverate);
 			break;
-		case 0x9100: //ctrl-down
+		case KEY_CTRL_DOWN:
+		/* case 0x9100: //ctrl-down */
 			wpSetPos(wpGetPos()+waverate);
-			break;*/
+			break;
 		case '<':
-		/*		case 0x7300: //ctrl-left  todo keys */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			wpSetPos(wpGetPos()-(wavelen>>5));
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			wpSetPos(wpGetPos()+(wavelen>>5));
 			break;
-/*		case 0x7700: //ctrl-home
+/*
+		case 0x7700: //ctrl-home TODO keys
 			wpSetPos(0);
-			break;*/
+			break;
+*/
 		case '-':
 			if (vol>=2)
 				vol-=2;
@@ -278,68 +289,69 @@ static int wavProcessKey(unsigned short key)
 				pan=64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-	      	case KEY_F2: /*f2*/
+	      	case KEY_F(2): /*f2*/
 			if ((vol-=8)<0)
 				vol=0;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F3: /*f3*/
+		case KEY_F(3): /*f3*/
 			if ((vol+=8)>64)
 				vol=64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F4: /*f4*/
+		case KEY_F(4): /*f4*/
 			wpSetVolume(vol, bal, pan, srnd=srnd?0:2);
 			break;
-		case KEY_F5: /*f5*/
+		case KEY_F(5): /*f5*/
 			if ((pan-=16)<-64)
 				pan=-64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F6: /*f6*/
+		case KEY_F(6): /*f6*/
 			if ((pan+=16)>64)
 				pan=64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F7: /*f7*/
+		case KEY_F(7): /*f7*/
 			if ((bal-=16)<-64)
 				bal=-64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F8: /*f8*/
+		case KEY_F(8): /*f8*/
 			if ((bal+=16)>64)
 				bal=64;
 			wpSetVolume(vol, bal, pan, srnd);
 			break;
-		case KEY_F9: /*f9*/
-		case KEY_F11: /*f11*/
+		case KEY_F(9): /*f9*/
+		case KEY_F(11): /*f11*/
 			if ((speed-=finespeed)<16)
 				speed=16;
 			wpSetSpeed(speed);
 			break;
-		case KEY_F10: /*f10*/
-		case KEY_F12: /*f12*/
+		case KEY_F(10): /*f10*/
+		case KEY_F(12): /*f12*/
 			if ((speed+=finespeed)>2048)
 				speed=2048;
 			wpSetSpeed(speed);
 			break;
-/* TODO-keys	case 0x5f00: // ctrl f2
+/*
+		case 0x5f00: // ctrl f2 TODO keys
 			if ((amp-=4)<4)
 				amp=4;
 			wpSetAmplify(1024*amp);
 			break;
-		case 0x6000: // ctrl f3
+		case 0x6000: // ctrl f3 TODO keys
 			if ((amp+=4)>508)
 				amp=508;
 			wpSetAmplify(1024*amp);
 			break;
-		case 0x8900: // ctrl f11
+		case 0x8900: // ctrl f11 TODO keys
 			finespeed=(finespeed==8)?1:8;
 			break;
-		case 0x6a00:
+		case 0x6a00: // alt-f3 TODO keys
 			normalize();
 			break;
-		case 0x6900:
+		case 0x6900: // alt-f2 TODO keys
 			set.pan=pan;
 			set.bal=bal;
 			set.vol=vol;
@@ -347,7 +359,7 @@ static int wavProcessKey(unsigned short key)
 			set.amp=amp;
 			set.srnd=srnd;
 			break;
-		case 0x6b00:
+		case 0x6b00: // alt-f4 TODO keys
 			pan=64;
 			bal=0;
 			vol=64;
@@ -431,4 +443,4 @@ static int wavOpenFile(const char *path, struct moduleinfostruct *info, FILE *wa
 }
 
 struct cpifaceplayerstruct wavPlayer = {wavOpenFile, wavCloseFile};
-struct linkinfostruct dllextinfo = {"playwav", "OpenCP Wave Player (c) 1994-99 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playwav", "OpenCP Wave Player (c) 1994-99 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
