@@ -47,9 +47,9 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 	
 	struct sampleinfo **smps = 0;
 	struct xmpsample **msmps = 0;
-	int *instsmpnum = 0;
+	unsigned int *instsmpnum = 0;
 
-	int i,j,k;
+	unsigned int i,j,k;
 
 	int cleanup(int retval)
 	{
@@ -170,7 +170,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 	m->patterns[head2.npat]=calloc(sizeof(uint8_t), 64*head2.nchan*5);
 	if (!m->patterns[head2.npat])
 	{
-		fprintf(stderr, __FILE__ ": malloc failed #1 (size=%d)\n", sizeof(uint8_t)*64*head2.nchan*5);
+		fprintf(stderr, __FILE__ ": malloc failed #1 (size=%d)\n", (int)sizeof(uint8_t)*64*head2.nchan*5);
 		return cleanup(errAllocMem);
 	}
 
@@ -202,7 +202,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 		m->patterns[i]=calloc(sizeof(uint8_t), pathead.rows*head2.nchan*5);
 		if (!m->patterns[i])
 		{
-			fprintf(stderr, __FILE__ ": malloc failed #3 (i=%d/%d, size=%d)\n", i, head2.npat, sizeof(uint8_t)*pathead.rows*head2.nchan*5);
+			fprintf(stderr, __FILE__ ": malloc failed #3 (i=%d/%d, size=%d)\n", i, head2.npat, (int)sizeof(uint8_t)*pathead.rows*head2.nchan*5);
 			return cleanup(errAllocMem);
 		}
 		if (!pathead.patdata)
@@ -210,7 +210,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 		pbp=pbuf=malloc(sizeof(uint8_t)*pathead.patdata);
 		if (!pbuf)
 		{
-			fprintf(stderr, __FILE__ ": malloc failed #4 (i=%d/%d, size=%d)\n", i, head2.npat, sizeof(uint8_t)*pathead.patdata);
+			fprintf(stderr, __FILE__ ": malloc failed #4 (i=%d/%d, size=%d)\n", i, head2.npat, (int)sizeof(uint8_t)*pathead.patdata);
 			return cleanup(errAllocMem);
 		}
 		if (fread(pbuf, pathead.patdata, 1, file)!=1)
@@ -219,7 +219,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 			return cleanup(errFormStruc);
 		}
 		cur=(uint8_t *)(m->patterns[i]);
-		for (j=0; j<(pathead.rows*head2.nchan); j++)
+		for (j=0; j<((unsigned int)pathead.rows*head2.nchan); j++)
 		{
 			uint8_t pack=(*pbp&0x80)?(*pbp++):0x1F;
 			for (k=0; k<5; k++)
@@ -280,7 +280,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 			memset(&ins1, 0, sizeof(ins1));
 		} else if (ins1.size<sizeof(ins1))
 		{
-			fprintf(stderr, __FILE__ ": Warning, ins1.size<=sizeof(ins1), reading %d more bytes and zeroing last %d bytes\n", ins1.size-sizeof(ins1.size), sizeof(ins1)-(ins1.size-sizeof(ins1.size)));
+			fprintf(stderr, __FILE__ ": Warning, ins1.size<=sizeof(ins1), reading %d more bytes and zeroing last %d bytes\n", (int)(ins1.size-(int)sizeof(ins1.size)), (int)(sizeof(ins1)-(ins1.size-sizeof(ins1.size))));
 			if (fread(/*no need to take pointer of array */ins1.name, ins1.size-sizeof(ins1.size), 1, file)!=1)
 			{
 				fprintf(stderr, __FILE__ ": fread failed #5.2\n");
@@ -320,7 +320,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 			memset(&ins2, 0, sizeof(ins2));
 		} else if (ins1.size<=(sizeof(ins1)+sizeof(ins2)))
 		{
-			fprintf(stderr, __FILE__ ": Warning, ins2.size<=(sizeof(ins1)+sizeof(ins2)), reading %d bytes and zeroing last %d bytes\n", ins1.size-sizeof(ins1), sizeof(ins2)-(ins1.size-sizeof(ins1)));
+			fprintf(stderr, __FILE__ ": Warning, ins2.size<=(sizeof(ins1)+sizeof(ins2)), reading %d bytes and zeroing last %d bytes\n", (int)(ins1.size-(int)sizeof(ins1)), (int)(sizeof(ins2)-(ins1.size-sizeof(ins1))));
 			if (fread(&ins2, ins1.size-sizeof(ins1), 1, file)!=1)
 			{
 				fprintf(stderr, __FILE__ ": fread failed #6.1\n");
@@ -408,7 +408,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 		msmps[i]=calloc(sizeof(struct xmpsample), ins1.samp);
 		if (!smps[i]||!msmps[i])
 		{
-			fprintf(stderr, __FILE__ ": malloc failed #5 (i=%d/%d, %p(%d) %p(%d))\n", i, m->ninst, smps[i], sizeof(struct sampleinfo)*ins1.samp, msmps[i], sizeof(struct xmpsample)*ins1.samp);
+			fprintf(stderr, __FILE__ ": malloc failed #5 (i=%d/%d, %p(%d) %p(%d))\n", i, m->ninst, smps[i], (int)sizeof(struct sampleinfo)*ins1.samp, msmps[i], (int)sizeof(struct xmpsample)*ins1.samp);
 			return cleanup(errAllocMem);
 		}
 
@@ -426,7 +426,7 @@ int __attribute__ ((visibility ("internal"))) xmpLoadModule(struct xmodule *m, F
 			env[0].env=malloc(sizeof(uint8_t)*(ins2.venv[ins2.vnum-1][0]+1));
 			if (!env[0].env)
 			{
-				fprintf(stderr, __FILE__ ": malloc failed #6 (size=%d)\n", sizeof(uint8_t)*(ins2.venv[ins2.vnum-1][0]+1));
+				fprintf(stderr, __FILE__ ": malloc failed #6 (size=%d)\n", (int)sizeof(uint8_t)*(ins2.venv[ins2.vnum-1][0]+1));
 				return cleanup(errAllocMem);
 			}
 			p=0;
@@ -472,7 +472,7 @@ bail1:
 			env[1].env=malloc(sizeof(uint8_t)*(ins2.penv[ins2.pnum-1][0]+1));
 			if (!env[1].env)
 			{
-				fprintf(stderr, __FILE__ ": malloc failed #7 (size=%d)\n", sizeof(uint8_t)*(ins2.penv[ins2.pnum-1][0]+1));
+				fprintf(stderr, __FILE__ ": malloc failed #7 (size=%d)\n", (int)sizeof(uint8_t)*(ins2.penv[ins2.pnum-1][0]+1));
 				return cleanup(errAllocMem);
 			}
 			p=0;
@@ -582,7 +582,7 @@ bail2:
 			sip->ptr=malloc(sizeof(uint8_t)*(l+528));
 			if (!sip->ptr)
 			{
-				fprintf(stderr, __FILE__ ": malloc failed #8 (i=%d, j=%d/%d size=%d)\n", i, j, ins1.samp, sizeof(uint8_t)*(l+528));
+				fprintf(stderr, __FILE__ ": malloc failed #8 (i=%d, j=%d/%d size=%d)\n", i, j, ins1.samp, (int)sizeof(uint8_t)*(l+528));
 				return cleanup(errAllocMem);
 			}
 			if (fread(sip->ptr, l, 1, file)!=1)
@@ -599,7 +599,7 @@ bail2:
 	m->sampleinfos=malloc(sizeof(struct sampleinfo)*m->nsampi);
 	if (!m->samples||!m->sampleinfos)
 	{
-		fprintf(stderr, __FILE__ ": malloc failed #9 (%p(%d) %p(%d))\n", m->samples, sizeof(struct xmpsample)*m->nsamp, m->sampleinfos, sizeof(struct sampleinfo)*m->nsampi);
+		fprintf(stderr, __FILE__ ": malloc failed #9 (%p(%d) %p(%d))\n", m->samples, (int)sizeof(struct xmpsample)*m->nsamp, m->sampleinfos, (int)sizeof(struct sampleinfo)*m->nsampi);
 		return cleanup(errAllocMem);
 	}
 
