@@ -112,9 +112,9 @@ struct channel
 	int chSyncTime;
 	int delayfreq;
 	
-	int nextstop;
-	int nextsamp;
-	int nextpos;
+	unsigned int nextstop;
+	unsigned int nextsamp;
+	unsigned int nextpos;
 	struct xmpsample *cursamp;
 
 	int evpos0;
@@ -799,12 +799,16 @@ static void xmpPlayTick(void)
 				case xmpCmdEnvPos:
 					ch->chVolEnvPos=ch->chPanEnvPos=procdat;
 					ch->fx=xfxEnvPos;
-					if (ch->cursamp->volenv<nenv)
-						if (ch->chVolEnvPos>envelopes[ch->cursamp->volenv].len)
-							ch->chVolEnvPos=envelopes[ch->cursamp->volenv].len;
-					if (ch->cursamp->panenv<nenv)
-						if (ch->chPanEnvPos>envelopes[ch->cursamp->panenv].len)
-							ch->chPanEnvPos=envelopes[ch->cursamp->panenv].len;
+					if (ch->cursamp)
+					{
+						if (ch->cursamp->volenv<nenv)
+							if (ch->chVolEnvPos>envelopes[ch->cursamp->volenv].len)
+								ch->chVolEnvPos=envelopes[ch->cursamp->volenv].len;
+						if (ch->cursamp->panenv<nenv)
+							if (ch->chPanEnvPos>envelopes[ch->cursamp->panenv].len)
+								ch->chPanEnvPos=envelopes[ch->cursamp->panenv].len;
+					} else
+						fprintf(stderr, __FILE__ " CmdEnvPos ch->cursamp not set\n");
 					break;
 				case xmpCmdPanSlide:
 					if (procdat)
@@ -1308,9 +1312,9 @@ static void xmpPlayTick(void)
 
 		if (ch->nextstop)
 			mcpSet(i, mcpCStatus, 0);
-		if (ch->nextsamp!=-1)
+		if (ch->nextsamp!=(unsigned)-1)
 			mcpSet(i, mcpCInstrument, ch->nextsamp);
-		if (ch->nextpos!=-1)
+		if (ch->nextpos!=(unsigned)-1)
 		{
 			mcpSet(i, mcpCPosition, ch->nextpos);
 			mcpSet(i, mcpCLoop, 1);
@@ -1419,7 +1423,7 @@ void __attribute__ ((visibility ("internal"))) xmpSetPos(int ord, int row)
 	realpos=(curord<<16)|(currow<<8);
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetLChanSample(int ch, int16_t *b, int len, uint32_t rate, int opt)
+int __attribute__ ((visibility ("internal"))) xmpGetLChanSample(unsigned int ch, int16_t *b, unsigned int len, uint32_t rate, int opt)
 {
        	return mcpGetChanSample(ch, b, len, rate, opt);
 }
