@@ -113,7 +113,7 @@ static void dopausefade(void)
 
 static int xmpProcessKey(uint16_t key)
 {
-	/* int row; */
+	int row;
 	int pat, p;
 
 	switch (key)
@@ -122,8 +122,12 @@ static int xmpProcessKey(uint16_t key)
 			cpiKeyHelp('p', "Start/stop pause with fade");
 			cpiKeyHelp('P', "Start/stop pause with fade");
 			cpiKeyHelp(KEY_CTRL_P, "Start/stop pause");
-			cpiKeyHelp('<', "Jump to previous track");
-			cpiKeyHelp('>', "Jump to next track");
+			cpiKeyHelp('<', "Jump back (big)");
+			cpiKeyHelp(KEY_CTRL_LEFT, "Jump back (big)");
+			cpiKeyHelp('>', "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_RIGHT, "Jump forward (big)");
+			cpiKeyHelp(KEY_CTRL_UP, "Jump back (small)");
+			cpiKeyHelp(KEY_CTRL_DOWN, "Jump forward (small)");
 			mcpSetProcessKey(key);
 			if (mcpProcessKey)
 				mcpProcessKey(key);
@@ -140,7 +144,7 @@ static int xmpProcessKey(uint16_t key)
 			plChanChanged=1;
 			break;
 #if 0
-		case 0x7700: /* TODO keys ctrl-home */
+		case 0x7700: //ctrl-home TODO keys
 			xmpInstClear();
 			xmpSetPos(0, 0);
 			if (plPause)
@@ -150,30 +154,33 @@ static int xmpProcessKey(uint16_t key)
 			break;
 #endif
 		case '<':
-		/*		case 0x7300: //ctrl-left  todo keys */
+		case KEY_CTRL_LEFT:
+		/* case 0x7300: //ctrl-left */
 			p=xmpGetPos();
 			pat=p>>8;
 			xmpSetPos(pat-1, 0);
 			break;
 		case '>':
-		/*	case 0x7400: //ctrl-right    TODO KEYS */
+		case KEY_CTRL_RIGHT:
+		/* case 0x7400: //ctrl-right */
 			p=xmpGetPos();
 			pat=p>>8;
 			xmpSetPos(pat+1, 0);
 			break;
-/* TODO keys
-		case 0x8D00: // ctrl-up
+		case KEY_CTRL_UP:
+		/* case 0x8D00: //ctrl-up */
 			p=xmpGetPos();
 			pat=p>>8;
 			row=p&0xFF;
 			xmpSetPos(pat, row-8);
 			break;
-		case 0x9100: //ctrl-down
+		case KEY_CTRL_DOWN:
+		/* case 0x9100: //ctrl-down */
 			p=xmpGetPos();
 			pat=p>>8;
 			row=p&0xFF;
 			xmpSetPos(pat, row+8);
-			break;*/
+			break;
 		default:
 			if (mcpSetProcessKey(key))
 				return 1;
@@ -695,4 +702,4 @@ static int xmpOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 }
 
 struct cpifaceplayerstruct xmpPlayer = {xmpOpenFile, xmpCloseFile};
-struct linkinfostruct dllextinfo = {"playxm", "OpenCP XM/MOD Player (c) 1995-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+struct linkinfostruct dllextinfo = {"playxm", "OpenCP XM/MOD Player (c) 1995-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
