@@ -20,7 +20,7 @@
 
 void __attribute__ ((visibility ("internal"))) xmpFreeModule(struct xmodule *m)
 {
-	int i;
+	unsigned int i;
 	if (m->sampleinfos)
 		for (i=0; i<m->nsampi; i++)
 			free(m->sampleinfos[i].ptr);
@@ -42,7 +42,7 @@ void __attribute__ ((visibility ("internal"))) xmpFreeModule(struct xmodule *m)
 void __attribute__ ((visibility ("internal"))) xmpOptimizePatLens(struct xmodule *m)
 {
 	uint8_t *lastrows=malloc(sizeof(uint8_t)*m->npat);
-	int i,j,k;
+	unsigned int i,j,k;
 
 	if (!lastrows)
 		return;
@@ -56,8 +56,8 @@ void __attribute__ ((visibility ("internal"))) xmpOptimizePatLens(struct xmodule
 		first=0;
 		for (j=0; j<m->patlens[m->orders[i]]; j++)
 		{
-			int neword=-1;
-			int newrow=newrow; /* supress warning.. neword needs to be set first, and that will set newrow aswell */
+			unsigned int neword=(unsigned)-1;
+			unsigned int newrow=newrow; /* supress warning.. neword needs to be set first, and that will set newrow aswell */
 			for (k=0; k<m->nchan; k++)
 				switch (m->patterns[m->orders[i]][m->nchan*j+k][3])
 				{
@@ -66,12 +66,12 @@ void __attribute__ ((visibility ("internal"))) xmpOptimizePatLens(struct xmodule
 						newrow=0;
 						break;
 					case xmpCmdBreak:
-						if (neword==-1)
+						if (neword==(unsigned)-1)
 							neword=i+1;
 						newrow=m->patterns[m->orders[i]][m->nchan*j+k][4];
 						break;
 				}
-			if (neword!=-1)
+			if (neword!=(unsigned)-1)
 			{
 				while ((neword<m->nord)&&(m->orders[neword]==0xFFFF))
 					neword++;
