@@ -139,8 +139,8 @@ void _splitpath(const char *src, char *drive, char *path, char *file, char *ext)
 
 void _makepath(char *dst, const char *drive, const char *path, const char *file, const char *ext)
 {
-	int left=PATH_MAX;
-	int cache;
+	unsigned int left=PATH_MAX;
+	unsigned int cache;
 	*dst=0;
 	if (drive)
 	{
@@ -222,4 +222,4 @@ void *memmem(const void *haystack, size_t haystacklen,
 }
 #endif
 
-struct linkinfostruct dllextinfo = {"compat", "OpenCP DOS compatible function-wrappers (c) '04-'05 Stian Skjelstad", DLLVERSION, 0};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"compat", "OpenCP DOS compatible function-wrappers (c) '04-'05 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
