@@ -30,7 +30,9 @@
 #ifdef HAVE_X11
 #include "poutput-x11.h"
 #endif
+#ifdef HAVE_FRAMEBUFFER
 #include "poutput-vcsa.h"
+#endif
 
 static void reset_api(void);
 static void (*console_clean)(void)=NULL;
@@ -93,7 +95,7 @@ static int console_init(void)
 			} else if (!strcmp(driver, "x11"))
 			{
 #ifdef HAVE_X11
-				if (!x11_init())
+				if (!x11_init(1))
 				{
 					console_clean=x11_done;
 					return 0;
@@ -105,7 +107,7 @@ static int console_init(void)
 				return -1;
 			} else if (!strcmp(driver, "vcsa"))
 			{
-#ifdef __linux
+#ifdef HAVE_FRAMEBUFFER
 				memset(_stdin, 0, sizeof(_stdin));
 				if (readlink("/proc/self/fd/0", _stdin, sizeof(_stdin)-1)<0)
 				if (readlink("/dev/fd/0", _stdin, sizeof(_stdin)-1)<0)
@@ -157,7 +159,7 @@ static int console_init(void)
 	{
 #ifdef HAVE_X11
 		fprintf(stderr, "stdout and stdin does not come from the same device, trying just X\n");
-		if (!x11_init())
+		if (!x11_init(0))
 		{
 			console_clean=x11_done;
 			return 0;
@@ -250,6 +252,7 @@ static int console_init(void)
 			break;
 	}
 
+#ifdef HAVE_FRAMEBUFFER
 	if (test_vcsa)
 	{
 		if (!vcsa_init(st.st_rdev&0xff))
@@ -260,13 +263,15 @@ static int console_init(void)
 			return 0;
 		}
 	}
+#endif
+
 #elif defined(HAVE_X11)
 	test_x11=1;
 #endif
 
 #ifdef HAVE_X11
 	if (test_x11)
-		if (!x11_init())
+		if (!x11_init(0))
 		{
 			console_clean=x11_done;
 			return 0;
@@ -411,7 +416,13 @@ static int __egetch(void)
 #endif
 	return 0;
 }
-
+static int __validkey(uint16_t key)
+{
+#ifdef CONSOLE_DEBUG
+	fprintf(stderr, "validkey not implemented in this console driver\n");
+#endif
+	return 0;
+}
 static void __drawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 #ifdef CONSOLE_DEBUG
@@ -469,6 +480,20 @@ static void __plDosShell(void)
 	fprintf(stderr, "plDosShell not implemented in this console driver\n");
 #endif
 }
+static void __plDisplaySetupTextMode(void)
+{
+#ifdef CONSOLE_DEBUG
+	fprintf(stderr, "plDisplaySetupTextMode not implemented in this console driver\n");
+#endif
+}
+static const char *__plGetDisplayTextModeName(void)
+{
+#ifdef CONSOLE_DEBUG
+	fprintf(stderr, "plGetDisplayTextModeName not implemented in this console driver\n");
+#endif
+	return "unknown";
+}
+
 
 static void reset_api(void)
 {
@@ -479,6 +504,9 @@ static void reset_api(void)
 	_displaystrattrdi=__displaystrattrdi;
 	_displayvoid=__displayvoid;
 
+	_plDisplaySetupTextMode=__plDisplaySetupTextMode;
+	_plGetDisplayTextModeName=__plGetDisplayTextModeName;
+
 	_plSetGraphMode=__plSetGraphMode;
 	_gdrawchar=__gdrawchar;
 	_gdrawchart=__gdrawchart;
@@ -493,6 +521,7 @@ static void reset_api(void)
 
 	_ekbhit=__ekbhit;
 	_egetch=__egetch;
+	_validkey=__validkey;
 
 	_drawbar=__drawbar;
 	_idrawbar=__idrawbar;
@@ -508,6 +537,7 @@ static void reset_api(void)
 	_plDosShell=__plDosShell;
 	_vga13=NULL;
 }
-
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"poutput", "OpenCP Output Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0, Init: console_init, Close: console_done};
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"poutput", "OpenCP Output Routines (c) 1994-08 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0, Init: console_init, Close: console_done};
