@@ -29,10 +29,10 @@ int mcpGetFreq6848(int note)
   return umulshr16(umulshr16(umulshr16(hnotetab6848[((note+0x8000)>>12)&0xF],notetab[(note>>8)&0xF]*2),finetab[(note>>4)&0xF]*2),xfinetab[note&0xF]*2);
 }
 
-int mcpGetNote8363(int frq)
+int mcpGetNote8363(unsigned int frq)
 {
   int16_t x;
-  int i;
+  unsigned int i;
   for (i=0; i<15; i++)
     if (hnotetab8363[i+1]<frq)
       break;
@@ -54,10 +54,10 @@ int mcpGetNote8363(int frq)
   return -x-i;
 }
 
-int mcpGetNote6848(int frq)
+int mcpGetNote6848(unsigned int frq)
 {
   int16_t x;
-  int i;
+  unsigned int i;
   for (i=0; i<15; i++)
     if (hnotetab6848[i+1]<frq)
       break;
