@@ -25,6 +25,7 @@ static int ring_head, ring_tail;
 
 static int(*kbhit)(void);
 static int(*getnextchar)(void);
+static int ___valid_key(uint16_t key);
 
 void ___setup_key(int(*_kbhit)(void), int(*_getch)(void))
 {
@@ -34,6 +35,7 @@ void ___setup_key(int(*_kbhit)(void), int(*_getch)(void))
 	getnextchar=_getch;
 	_ekbhit=___peek_key;
 	_egetch=___pop_key;
+	_validkey=___valid_key;
 }
 
 void ___push_key(uint16_t key)
@@ -118,6 +120,9 @@ int ___peek_key(void)
 		ring_tail=(ring_tail+1)%BUFFER_LEN;
 		key_len--;
 	}
+#ifdef KEYBOARD_DEBUG
+	fprintf(stderr, "escapelevel=%d key_len=%d\n", escapelevel, key_len);
+#endif
 	if (escapelevel&&key_len)
 	{
 		if ((((ring_buffer[ring_tail]>='a') && (ring_buffer[ring_tail]<='z')) || (ring_buffer[ring_tail] == 10) || (ring_buffer[ring_tail] == 13)) && (key_len==1) )
@@ -178,12 +183,12 @@ int ___peek_key(void)
 			default:
 				retval=KEY_BACKSPACE;
 				break;
-/*			case 0x5A:
+			case 0x5A:
 				retval=KEY_CTRL_UP;
 				break;
 			case 0x5B:
 				retval=KEY_CTRL_DOWN;
-				break;*/
+				break;
 			case 0x5C:
 				retval=KEY_CTRL_RIGHT;
 				break;
@@ -202,19 +207,19 @@ int ___peek_key(void)
 		switch (upcode)
 		{
 			case 'A'+256:
-				retval=KEY_F1;
+				retval=KEY_F(1);
 				break;
 			case 'B'+256:
-				retval=KEY_F2;
+				retval=KEY_F(2);
 				break;
 			case 'C'+256:
-				retval=KEY_F3;
+				retval=KEY_F(3);
 				break;
 			case 'D'+256:
-				retval=KEY_F4;
+				retval=KEY_F(4);
 				break;
 			case 'E'+256:
-				retval=KEY_F5;
+				retval=KEY_F(5);
 				break;
 			default:
 				retval=0;
@@ -226,25 +231,25 @@ int ___peek_key(void)
 		switch (upcode)
 		{
 			case 17:
-				retval=KEY_F6;
+				retval=KEY_F(6);
 				break;
 			case 18:
-				retval=KEY_F7;
+				retval=KEY_F(7);
 				break;
 			case 19:
-				retval=KEY_F8;
+				retval=KEY_F(8);
 				break;
 			case 20:
-				retval=KEY_F9;
+				retval=KEY_F(9);
 				break;
 			case 21:
-				retval=KEY_F10;
+				retval=KEY_F(10);
 				break;
 			case 23:
-				retval=KEY_F11;
+				retval=KEY_F(11);
 				break;
 			case 24:
-				retval=KEY_F12;
+				retval=KEY_F(12);
 				break;
 
 			case 'A'+256:
@@ -358,9 +363,9 @@ int ___peek_key(void)
 /*			case 'n'+512:
 				retval=KEY_ALT_N;
 				break;*/
-/*			case 'o'+512:
+			case 'o'+512:
 				retval=KEY_ALT_O;
-				break;*/
+				break;
 			case 'p'+512:
 				retval=KEY_ALT_P;
 				break;
@@ -411,6 +416,9 @@ int ___peek_key(void)
 			case 240:
 				retval=KEY_ALT_P;
 				break;
+			case 239:
+				retval=KEY_ALT_O;
+				break;
 			case 233:
 				retval=KEY_ALT_I;
 				break;
@@ -428,7 +436,7 @@ int ___peek_key(void)
 				break;
 			case 127:
 			case 8:
-				retval=KEY_BS;
+				retval=KEY_BACKSPACE;
 				break;
 		}
 	}
@@ -449,3 +457,147 @@ int ___peek_key(void)
 	return retval;
 }
 
+int ___valid_key(uint16_t key)
+{
+	switch (key)
+	{
+		case KEY_ESC:
+		case KEY_NPAGE:
+		case KEY_PPAGE:
+		case KEY_HOME:
+		case KEY_END:
+		case KEY_TAB:
+		case _KEY_ENTER:
+		case KEY_DOWN:
+		case KEY_UP:
+		case KEY_LEFT:
+		case KEY_RIGHT:
+		case KEY_ALT_A:
+		case KEY_ALT_B:
+		case KEY_ALT_C:
+		case KEY_ALT_E:
+		case KEY_ALT_G:
+		case KEY_ALT_I:
+		case KEY_ALT_K:
+		case KEY_ALT_L:
+		case KEY_ALT_M:
+		case KEY_ALT_O:
+		case KEY_ALT_P:
+		case KEY_ALT_R:
+		case KEY_ALT_S:
+		case KEY_ALT_X:
+		case KEY_ALT_Z:
+		case KEY_BACKSPACE:
+		case KEY_F(1):
+		case KEY_F(2):
+		case KEY_F(3):
+		case KEY_F(4):
+		case KEY_F(5):
+		case KEY_F(6):
+		case KEY_F(7):
+		case KEY_F(8):
+		case KEY_F(9):
+		case KEY_F(10):
+		case KEY_F(11):
+		case KEY_F(12):
+		case KEY_DELETE:
+		case KEY_INSERT:
+		case 'a':
+		case 'b':
+		case 'c':
+		case 'd':
+		case 'e':
+		case 'f':
+		case 'g':
+		case 'h':
+		case 'i':
+		case 'j':
+		case 'k':
+		case 'l':
+		case 'm':
+		case 'n':
+		case 'o':
+		case 'p':
+		case 'q':
+		case 'r':
+		case 's':
+		case 't':
+		case 'u':
+		case 'v':
+		case 'w':
+		case 'x':
+		case 'y':
+		case 'z':
+		case 'A':
+		case 'B':
+		case 'C':
+		case 'D':
+		case 'E':
+		case 'F':
+		case 'G':
+		case 'H':
+		case 'I':
+		case 'J':
+		case 'K':
+		case 'L':
+		case 'M':
+		case 'N':
+		case 'O':
+		case 'P':
+		case 'Q':
+		case 'R':
+		case 'S':
+		case 'T':
+		case 'U':
+		case 'V':
+		case 'W':
+		case 'X':
+		case 'Y':
+		case 'Z':
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		case '8':
+		case '9':
+		case '0':
+		case '/':
+		case '*':
+		case '-':
+		case '+':
+		case '\\':
+		case '\'':
+		case ',':
+		case '.':
+		case '?':
+		case '!':
+		case '>':
+		case '<':
+		case KEY_ALT_ENTER:
+			return 1;
+
+		default:
+			fprintf(stderr, __FILE__ ": unknown key 0x%04x\n", (int)key);
+		case KEY_SHIFT_TAB:
+		case KEY_CTRL_P:
+		case KEY_CTRL_D:
+		case KEY_CTRL_H:
+		case KEY_CTRL_J:
+		case KEY_CTRL_L:
+		case KEY_CTRL_Q:
+		case KEY_CTRL_S:
+		case KEY_CTRL_Z:
+		case KEY_CTRL_BS:
+		case KEY_CTRL_UP:
+		case KEY_CTRL_DOWN:
+		case KEY_CTRL_LEFT:
+		case KEY_CTRL_RIGHT:
+		case KEY_CTRL_PGUP:
+		case KEY_CTRL_PGDN:
+		case KEY_CTRL_ENTER:
+			return 0;
+	}
+}
