@@ -77,7 +77,15 @@ static void set_kernel_sizes(int x, int y)
 static void set_plScrType(void)
 {
 	lseek(vgafd, 0, SEEK_SET);
-	read(vgafd, &scrn, sizeof(scrn));
+	while (read(vgafd, &scrn, sizeof(scrn))<0)
+	{
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		fprintf(stderr, __FILE__ " read() failed #1\n");
+		exit(1);
+	}
 
 	set_kernel_sizes(scrn.cols, scrn.lines);
 
@@ -341,12 +349,12 @@ void displayvoid(unsigned short y, unsigned short x, unsigned short len)
 void idrawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 	unsigned char buf[60];
-	int16_t i;
+	unsigned int i;
 	char *scrptr;
-	int16_t yh1=(yh+2)/3;
-	int16_t yh2=(yh+yh1+1)/2;
+	uint16_t yh1=(yh+2)/3;
+	uint16_t yh2=(yh+yh1+1)/2;
 
-	if (hgt>((yh*16)-4))
+	if (hgt>((unsigned int)(yh*16)-4))
 	  hgt=(yh*16)-4;
 
 	scrptr=vgatextram+(2*x+(yb-yh+1)*plScrRowBytes);
@@ -387,12 +395,12 @@ void idrawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 void drawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 	unsigned char buf[60];
-	int16_t i;
+	unsigned int i;
 	char *scrptr=vgatextram+(2*x+yb*plScrRowBytes);
-	int16_t yh1=(yh+2)/3;
-	int16_t yh2=(yh+yh1+1)/2;
+	uint16_t yh1=(yh+2)/3;
+	uint16_t yh2=(yh+yh1+1)/2;
 
-	if (hgt>((yh*16)-4))
+	if (hgt>((unsigned int)(yh*16)-4))
 	  hgt=(yh*16)-4;
 	for (i=0; i<yh; i++)
 	{
@@ -429,7 +437,7 @@ void drawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 
 static void plSetTextMode(unsigned char x)
 {
-	int i;
+	unsigned int i;
 
 #ifdef VCSA_DEBUG
 	fprintf(stderr, "vcsa: set mode %d\n", x);
@@ -488,7 +496,15 @@ static int conRestore(void)
 		return 0;
 	tcsetattr(0, TCSANOW, &orgterm);
 	lseek(vgafd, 0, SEEK_SET);
-	write(vgafd, consoleram, vgamemsize+4);
+	while (write(vgafd, consoleram, vgamemsize+4) < 0)
+	{
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		fprintf(stderr, __FILE__ " write() failed #1\n");
+		exit(1);
+	}
 	conactive=0;
 	return 0;
 }
@@ -499,7 +515,15 @@ static void conSave(void)
 		return;
 	fflush(stderr);
 	lseek(vgafd, 0, SEEK_SET);
-	read(vgafd, consoleram, vgamemsize+4);
+	while (read(vgafd, consoleram, vgamemsize+4) < 0)
+	{
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		fprintf(stderr, __FILE__ " read() failed #2\n");
+		exit(1);
+	}
 	tcsetattr(0, TCSANOW, &ocpterm);
 	conactive=1;
 }
@@ -511,7 +535,15 @@ static int ekbhit(void)
 	if (!plScrMode)
 	{
 		lseek(vgafd, 4, SEEK_SET);
-		write(vgafd, vgatextram, vgamemsize);
+		while (write(vgafd, vgatextram, vgamemsize) < 0)
+		{
+			if (errno==EAGAIN)
+				continue;
+			if (errno==EINTR)
+				continue;
+			fprintf(stderr, __FILE__ " write() failed #2\n");
+			exit(1);
+		}
 	}
 	set.fd=0;
 	set.events=POLLIN;
@@ -569,7 +601,15 @@ static void setcur(unsigned char y, unsigned char x)
 	scrn.x=x;
 	scrn.y=y;
 	lseek(vgafd, 0, SEEK_SET);
-	write(vgafd, &scrn, sizeof(scrn));
+	while (write(vgafd, &scrn, sizeof(scrn)) < 0)
+	{
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		fprintf(stderr, __FILE__ " write() failed #3\n");
+		exit(1);
+	}
 }
 
 int vcsa_init(int minor)
@@ -595,7 +635,15 @@ int vcsa_init(int minor)
 		return -1;
 #endif
 
-	read(vgafd, &scrn, sizeof(scrn));
+	while (read(vgafd, &scrn, sizeof(scrn)) < 0)
+	{
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		fprintf(stderr, __FILE__ " read() failed #3\n");
+		exit(1);
+	}
 
 	plScrHeight = scrn.lines;
 	plScrWidth = scrn.cols;
