@@ -31,6 +31,21 @@
 unsigned char plpalette[256];
 int plScrLineBytes;
 
+void make_title(char *part)
+{
+	uint16_t sbuf[CONSOLE_MAX_X];
+	char *verstr="opencp v" VERSION;
+
+	fillstr(sbuf, 0, 0x30, 0, CONSOLE_MAX_X);
+	writestring(sbuf, 2, 0x30, verstr, strlen(verstr));
+	if (plScrWidth<100)
+		writestring(sbuf, plScrWidth-58, 0x30, part, strlen(part));
+	else
+		writestring(sbuf, (plScrWidth-strlen(part))/2, 0x30, part, strlen(part));
+	writestring(sbuf, plScrWidth-28, 0x30, "(c) '94-'07 Stian Skjelstad", 27);
+	displaystrattr(0, 0, sbuf, plScrWidth);
+}
+
 char *convnum(unsigned long num, char *buf, unsigned char radix, unsigned short len, char clip0)
 {
 	int i;
