@@ -25,13 +25,15 @@
 #define idrawbar(x, yb, yh, hgt, c) (_idrawbar(x, yb, yh, hgt, c))
 #define gupdatepal(c,r,g,b) (_gupdatepal(c,r,g,b))
 #define gflushpal() (_gflushpal())
- 
+#define plDisplaySetupTextmode() (_plDisplaySetupText())
+#define plGetDisplayTextModeName() (_plGetDisplayTextModeName())
 #define Screenshot() (_Screenshot())
 #define TextScreenshot(scrType) (_TextScreenshot(scrType))
 #define RefreshScreen() (_RefreshScreen())
 
 #define ekbhit() (_ekbhit())
 #define egetch() (_egetch())
+#define validkey(k) (_validkey(k))
 
 #define setcur(y, x) _setcur(y, x)
 #define setcurshape(shape) _setcurshape(shape)
@@ -63,13 +65,14 @@ extern void fillstr(uint16_t *buf, const unsigned short ofs, const unsigned char
 
 enum { vidNorm, vidVESA};
 
-extern short plScrHeight;               /* How many textlines can we currently fit. Undefined for wurfel-mode */
-extern short plScrWidth;                /* How many characters can we currently fir on a line */
+extern unsigned int plScrHeight;               /* How many textlines can we currently fit. Undefined for wurfel-mode */
+extern unsigned int plScrWidth;                /* How many characters can we currently fir on a line */
 extern char plVidType;                  /* vidNorm for textmode only, or vidVESA for graphical support also */
 extern unsigned char plScrType;         /* Last set textmode */
 extern int plScrMode;                   /* If we are in graphical mode, this value is set to either 13 (for wurfel), 100 for 640x480 or 101 for 1024x768 */
 extern char *plVidMem;                  /* This points to the current selected bank, and should atleast provide 64k of available bufferspace */
 extern int plScrLineBytes;              /* How many bytes does one line from plVidMem use (can be padded) */
+extern void make_title(char *part);
 
 #ifdef _CONSOLE_DRIVER
 extern unsigned char plpalette[256];
