@@ -49,5 +49,7 @@ static int ssInit(void)
   return errOk;
 }
 
+#ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"sets", "OpenCP Sound Settings Auxiliary Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: ssInit};
+#endif
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"sets", "OpenCP Sound Settings Auxiliary Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: ssInit};
