@@ -71,7 +71,7 @@ void tmSetNewRate(int rate)
 {
 }
 
-int  tmGetTimer(void)
+time_t tmGetTimer(void)
 {
 	return dos_clock(); /* this is somewhat off, but we don't care */
 }
@@ -270,7 +270,7 @@ void tmSetNewRate(int timerval)
 	setitimer(ITIMER_REAL, &spec, NULL);
 }
 
-int tmGetTimer(void)
+time_t tmGetTimer(void)
 {
 	unsigned long tm = tmTimerRate+tmTicker;
 
