@@ -84,10 +84,10 @@ int x11_connect(void)
 
 	if ( (mDisplay = XOpenDisplay(NULL)) == NULL )
 	{
-		fprintf(stderr, "[x11]: can't connect to X server %s\n", XDisplayName(NULL));
+		fprintf(stderr, "[x11] can't connect to X server %s\n", XDisplayName(NULL));
 		return -1;
 	}
-	fprintf(stderr, "[x11]: X is online\n");
+	fprintf(stderr, "[x11] X is online\n");
 
 	mScreen=DefaultScreen(mDisplay);
 
